/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
/**
 * Extends the dijit tree to support item level actions in the tree node. In addition,
 * if the model item can supports rich hovers, the corresponding tree item adds a
 * rich hover on the tree item's name display.
 * 
 */
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.navigator.ActionableTree");

dojo.require("dijit.Tree");
dojo.require("jazz.ui.ResourceLink");
dojo.require("dijit.form.CheckBox");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.horizontalTree.ActionRegistry");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.ItemLevelAction");
dojo.require("com.ibm.asq.common.web.ui.internal.util.BidiUtil");

(function() {

var ItemLevelAction = com.ibm.asq.common.web.ui.internal.widgets.ItemLevelAction;

// IE/Chrome/FF require a width to be set on the tree node label DIV to do CSS ellapsis text clipping
var ActiveClipping = dojo.isIE || dojo.isChrome || dojo.isFF >= 4;

var BidiUtil = com.ibm.asq.common.web.ui.internal.util.BidiUtil;

dojo.declare(
    "com.ibm.asq.common.web.ui.internal.widgets.navigator.ActionableTreeNode",
    dijit._TreeNode,
{
    templateString: '<div class="dijitTreeNode" role="presentation">' +
                    '  <div dojoAttachPoint="rowNode" class="dijitTreeRow" role="presentation" style="overflow:hidden; padding-left: 0;"' + // hide overflow to use ellipsis
                    '       dojoAttachEvent="onmouseenter:_onMouseEnter, onmouseleave:_onMouseLeave, ondijitclick:_onClick, ondblclick:_onDblClick">' +
                    '     <div class="dijitInline" dojoAttachPoint="indentNode"></div>' +
                    '     <img src="${_blankGif}" title="" alt="" dojoAttachPoint="expandoNode" class="dijitTreeExpando" role="presentation">' +
                    '     <span dojoAttachPoint="expandoNodeText" class="dijitExpandoText" role="presentation"></span>' +
                    '     <span dojoAttachPoint="contentNode" class="dijitTreeContent asq-common-tree-content" role="presentation">' + // consider the preceding img offset for the width 
                    '       <img src="${_blankGif}" title="" alt="" dojoAttachPoint="iconNode" class="dijitTreeIcon" role="presentation">' +
                    '       <div dojoAttachPoint="labelNode" class="dijitTreeLabel tree-item-label-ellipsis" role="treeitem" tabindex="-1" dojoAttachEvent="onfocus:_onLabelFocus"' + 
                    '             waiState="selected-false"></div>' +
                    '     </span>' +
                    '  <span dojoAttachPoint="stateIconNode" class="tree-item-action-float-right-state" role="presentation" style="display: none;"></span>' +
                    '  </div>' +
                    '  <div dojoAttachPoint="containerNode" class="dijitTreeContainer" role="presentation" style="display: none;"></div>' +
                    '</div>',

    loadingLabel: MessageUtil.getMessage(MessageBundle.COMMON, "labelExpandCollapse"),
    iconsOffsetWidth: 18,  // width of this.expandoNode, see templateString
    isRemote: false,
    
    postCreate: function()
    {
        this.inherited("postCreate", arguments);
        var actionRegistry = this.tree.actionRegistry;
        
        // figure out the margin offset for clipping the label
        var iconWidth = dojo.style(this.iconNode, "width");
        if (isNaN(iconWidth)) {
            iconWidth = iconWidth.replace("px", "");
        }
        if (iconWidth){
            this.iconsOffsetWidth += iconWidth;
        }
        dojo.style(this.contentNode, "marginRight", this.iconsOffsetWidth + "px");

        // show the check box in the tree item.
        if(this.checkbox) {
            this.checkboxNode = new dijit.form.CheckBox({
                style: "veritcal-align:middle",
                onClick: dojo.hitch(this, "_onCheckboxClick")
            });
            dojo.place(this.checkboxNode.domNode, this.iconNode, "before");
        }
        // for now use tooltip to show full text of clipped label
        this.labelNode.title = this.label;
        
        //On FF 6 and below versions, ellipsis is not supported, use standard clipping
        if(dojo.isFF < 6 && this.isRemote){
        	dojo.removeClass(this.labelNode, "tree-item-label-ellipsis");
			dojo.addClass(this.labelNode, "tree-item-label-ff3-nobind");
			dojo.style(this.labelNode, "display", "inline-block");
        }
        
        // We will initialize the label's width, don't need to trigger
        // label width update when we add the state icon attribute.
        this.setStateIconClass();
        if (actionRegistry && !this.item.supportsRichHover) {
            this._installItemAction(actionRegistry);
        }
        // initialize the label's width to provide clipping.
        this.updateLabelClipWidth(this.widthHint, false);
    },
    
    setStateIconClass: function(){
    	 if (this.item.stateIconClass) {
             this.set("stateIcon", this.item.stateIconClass, /*no update width*/false);
         }
    },
    
    updateLabelClipWidth: function(widthHint, recurse) {
        this.widthHint = widthHint;
        var stateIconWidth = this.stateIconClass ? 22 : 0;
        var actionWidth = this.actionNode ? 22 : 0;
        var totalOffset = stateIconWidth + actionWidth + this.iconsOffsetWidth;
        
        if (this.widthHint) {
            // calculate the indent pixel width based on this tree node's level. Need to subtract
            // that from the width hint to calculate the absolute width for the label to do the clipping.
            var indent = (Math.max(this.indent, 0) * this.tree._nodePixelIndent);
            this.labelNode.style.width = (this.widthHint - indent - this.iconsOffsetWidth - stateIconWidth - 30) + 'px';
            
            // if it's recurse, update each of the children.
            if (recurse) {
                dojo.forEach(this.getChildren(), function(child){
                    child.updateLabelClipWidth(widthHint, recurse);
                });
            }
        }
        else {
            // no active width adjustment, but we'll need to adjust the margin right of the 
            // content node to clip in non active width clipping case.
            dojo.style(this.contentNode, "marginRight", totalOffset + "px");
        }
    },
    
    _installItemAction: function(actionRegistry)
    {
        var showExpandAction = this.tree.showExpandAction;
        var actionDesc = actionRegistry.getContextActions(this.tree.model, this.item);
        
        if (!actionDesc && showExpandAction) {
            actionDesc = {
                actions: [{
                    label: this.loadingLabel,
                    callback: dojo.hitch(this, "_selfExpandCollapse"),
                    defaultAction: true
                }]
            };
        }

        if (actionDesc) {
            this.actionDesc = actionDesc;
            
            // create an hover menu to provide item level action.
            // the menu popup will be handled by the hover menu. We'll
            // just have to supply the menu to it. No caching of the
            // menu means the menu will be cleaned up when it's pop down.
            this._actionMenu = new ItemLevelAction({
                actionIconClass: "item-actions-menu-image",
                menuProvider: dojo.hitch(this, "_actionMenuProvider"),
                onClick: dojo.hitch(this, function(evt) {
                    this._doDefaultAction();
                    this._fixSelection();
                }),
                parentTreeNode: this
            });
            this.actionNode = this._actionMenu.domNode;
            dojo.addClass(this.actionNode, this.actionMenuHoverClass || "tree-item-action-hoverMenu");

            this._showAction(false);
            this.rowNode.appendChild(this.actionNode);
            if(dojo.isIE && this.stateIconClass){
            	//reposition the actionNode in IE if we have a State Icon clss
            	dojo.style(this.actionNode, "marginRight", "-40px");
            }
        }
    },
    
    _doDefaultAction: function()
    {
        var actions = this.actionDesc.actions;
        var len = actions.length;             
        for(var i = 0; i < len; i++) {
            var actionItem = actions[i];
            if (actionItem.defaultAction) {
                actionItem.callback(this.item.parent, this.item);
                return;
            }
        }
    },
    
    // Since we can have a hover menu, the onClick of the tree node is not trigger if the click
    // is on the menu icon.  However, the DnD controller that tracks mouse events will set the
    // tree node as selected.  We need to fix the selection by "clicking" the tree node when
    // the click is on the menu.
    //
    _fixSelection: function() {
        var treeSelectedNode = this.tree.selectedItem;
        var treeSelectedNodeId = treeSelectedNode ? treeSelectedNode.itemId : null;
        if (treeSelectedNodeId && !treeSelectedNode.selected && treeSelectedNodeId == this.item.itemId) {
            // proceed only if the tree selection is the same node and not selected from our click handler
            this.tree.onClick(this.item);
        }
    },
    
    _selfExpandCollapse: function() 
    {
        this._inSelfExpand = true;
        this.tree._onExpandoClick({node:this});    
        this._inSelfExpand = false;
    },
    
    _ie7getTopoffset: function(nodeTop){
        var offsetTop = 0;
        var temp = this.getParent();
        
        while (temp) {
            offsetTop += temp.domNode.offsetTop;
            temp = temp.getParent();
        }
        
        return nodeTop + offsetTop;
    },
    
    _placeAction: function()
    {
        var pos = dojo.coords(this.domNode, /*includeScroll*/ true);
        var rightMostNode =  this.stateIconClass ? this.stateIconNode : this.labelNode;
        var labelPos = dojo.coords(rightMostNode, /*includeScroll*/ true);
        var labelRight = labelPos.l + labelPos.w;
        var eighth = Math.round(pos.w / 8);
        
        // initialization case. no dimension yet. 
        if(pos.w == 0) {
            return;
        }
        // start at quarter. Anything over, we increment by an eighth.
        var left = eighth * 2;
        while (labelRight > left) {
            left += eighth;
        }
        // Make sure we're at most place the action at the end.
        left = Math.min(pos.w - 24, left);
        
        // for IE7, we've to calculate the offset top. The coords only returns the
        // relative position of the node.
        var topPos = ((dojo.isIE == 7) ? this._ie7getTopoffset(pos.t) : pos.t) + 2;
        dojo.style(this._actionMenu.domNode, {
            left: left + "px",
            top: topPos + "px"
        });    
    },
    
    _showAction: function(show) {
        if (this._actionMenu) {
            if (show && !this._actionMenu._isActive) {
                if (this.actionMenuPlacement == 'align') {
                    this._placeAction();
                }
                else if(ActiveClipping) {
                    var offsetRight = this.stateIconClass ? -40 : 0;
                    dojo.style(this.actionNode, 'marginRight', offsetRight);
                }
            }
            this._actionMenu.setActive(show);
        }
    },
    
    // Prevent nodes from being highlighted when they are hovered over. Otherwise it's easy to confuse
    // hover with selection
    _onMouseEnter: function(evt){
        this.inherited(arguments);
        
        var focusStack = dijit.focus.get('activeStack') || [];
        var lastIndex = focusStack.length -1;
        if (!evt && focusStack[lastIndex] !== this.id) {
        	return;
        }
        // keep track of when we are mouse in and out. Don't 
        // dismiss the action if the mouse is still on the row.
        this.mouseHover = true;
        this._showAction(true);
        if(evt && evt.currentTarget) {
        	dojo.addClass(evt.currentTarget ,"dijitTreeRowHover");
        }        
    },

    _onMouseLeave: function(evt){        
        // keep track of when we are mouse in and out. Don't 
        // dismiss the action if the mouse is still on the row.
        this.mouseHover = false;

        // only remove the hover when we're really away from the item.
        // if we're moved away because of the menu popup, still show the highlight.
        // When the menu is closed, we'll remove the action highlight.
        if(this._actionMenu && this._actionMenu.isMenuOpen()) {
            return;
        }  
        
        this.inherited(arguments);
        this._showAction(false);
        if(evt && evt.currentTarget) {
        	dojo.removeClass(evt.currentTarget ,"dijitTreeRowHover");
        }        
        dojo.removeClass(this.rowNode, "onMoveTarget");
    },
    
    // allow keyboard access to have the mouse over effect to show
    // the action menu icon
    onFocus: function() {
        this._onMouseEnter();
    },
	
    onBlur: function() {
        this._onMouseLeave();
    },
	
    // Remove hover highlight when a new node is selected
    // 
    _onLabelFocus: function(evt){
        this.inherited(arguments);  
        
        // ignore self expanding. This will cause our label
        // to be selected. but we don't want the action to be
        // shown after the mouse is away.
        // also don't want to show the action if a node is programmatically
        // selected not from user mouse over
        if(!this._inSelfExpand && this.mouseHover)     
            this._showAction(true);         
    },

    // Execute an action if a right click action is selected
    onContextActionInvoke: function(action, modelitem)
    {
        if(action && action.callback && dojo.isFunction(action.callback)) {
            action.callback(modelitem.parent, modelitem, this);
        }                   
    },

    // Create the context menu based on the action registry, also setup enablements if they are defined 
    _actionMenuProvider: function(){
        var actionRegistry = this.tree.actionRegistry;
        var model = this.tree.model;
        var actionDesc = actionRegistry.getContextActions(model, this.item) || this.actionDesc;
        var menu = null;
        
        if (actionDesc) {
            menu = actionRegistry.getContextMenu(actionDesc, dojo.hitch(this, function(action){
                // delegate the action handling to our creator.
                this.onContextActionInvoke(action, this.item);
            }));
            dojo.forEach(menu.getChildren(), dojo.hitch(this, function(item){
                var action = item.action;
                var enabled = true;
                
                // get the action's enablement state, if one is specified.
                if (action && action.enablementCallback && dojo.isFunction(action.enablementCallback)) {
                    enabled = action.enablementCallback(model, this.item);
                    item.set("disabled", !enabled);
                }
            }));
            // clear the highlight when the menu is closed.
            menu.onClose = dojo.hitch(this, function(){
                // unhighlight the tree row & action.
                if (!this.mouseHover) {
                    setTimeout(dojo.hitch(this, "_onMouseLeave"), 0);
                }
            });
        }
        return menu;
    },
    
    // override superclass to add rich hovers
    setChildItems: function(items) {
        var scid = this.inherited(arguments);

        for (var i=0; i<items.length; i++) {
            var item = items[i];
            if (item.supportsRichHover) {
                var childNode = this.tree.getNodesByItem(item)[0];
                
                // use the ResourceLink's label since it is updated dynamically
                var anchor = document.createElement("a");
                anchor.href= item.artifact.enclosure;
// TODO: open content in same window until user pref is implemented to support open in external window/tab
//                anchor.target= "_blank";
                dojox.xml.parser.removeChildren(childNode.labelNode);
                childNode.labelNode.appendChild(anchor);
                
                // use the ResourceLink's icon since it is updated dynamically
                var span = document.createElement("span");
                childNode.contentNode.replaceChild(span, childNode.iconNode);
                
                var resourceLinkArgs= { 
                    uri: item.artifact.enclosure,
                    presentationUri: item.artifact.enclosure,
                    label: item.name,
                    iconNode: span
                };
                new jazz.ui.ResourceLink(resourceLinkArgs, anchor);
            }
        }
        return scid;
    },
    
    clickChildren : function(parent, isChecked){
    	var children = parent.getChildren();
    	if (children && children.length > 0) {
    		var length = children.length;
    		for(var i = 0; i< length; i++){
				var checkboxNode = children[i] ? children[i].checkboxNode : null;
				if (checkboxNode) {
					checkboxNode.set("checked", isChecked);
					// changing the checkbox attr doesn't generate the callback
					// call it explicitly.
					this.tree._onTreeNodeChecked(children[i].item ,isChecked);
				}
				this.clickChildren(children[i], isChecked);
    		}
		}
    },
    
    _updateItemClasses: function(item){    	    	
    	 if (item.stateIconClass) {
             this.set("stateIcon", item.stateIconClass, /*no update width*/false);
         } else {
        	 this.set("stateIcon", "", /*no update width*/false);
         }    	    	
    	this.inherited('_updateItemClasses', arguments);
    },
    
    // @override
    //      to auto check the parent's checkbox if it's checkable node
    _onClick: function(evt){
		//We don't want to treat the enter key as a click.
		var isEnterKey = (evt.keyCode && evt.keyCode == dojo.keys.ENTER);
		if (!isEnterKey) {
			if (this.checkbox) {
				//This can also get called when the checkbox is clicked directly. 
				//we don't want to modify the checked state of the current row's checkbox in that case.
				//So set the state and traverse the tree starting from either the current row (this) or this row's parent.
				var parent = this._checkBoxClicked ? this.getParent() : this;
				var isChecked = false;
				if(this && this.checkboxNode){
					isChecked = this.checkboxNode.get("checked");
				}
				this.clickChildren(this, isChecked);
				while (parent) {
					var checkboxNode = parent ? parent.checkboxNode : null;
					if (checkboxNode) {
						if(!this._checkBoxClicked){
							checkboxNode.set("checked", isChecked);
						}else{
							var children = parent.getChildren();
							var childrenCheckState = true;
					    	if (children && children.length > 0) {
					    		var length = children.length;
					    		for(var i = 0; i< length; i++){
									var childrenCheckboxNode = children[i] ? children[i].checkboxNode : null;
									if (childrenCheckboxNode && !childrenCheckboxNode.checked) {
										childrenCheckState = false;
										break;
									}
					    		}
					    	}
					    	checkboxNode.set("checked", childrenCheckState);
						}
						// changing the checkbox attr doesn't generate the callback
						// call it explicitly.
 						this.tree._onTreeNodeChecked(parent.item, this.checkboxNode.checked);
					}
					parent = parent.getParent();
				}
								
			}
			this.inherited(arguments);
			//"Select" the tree node.  In some cases, when using the keyboard,
			//the selection highlighting does not get set.
			this.tree.dndController.userSelect(this, dojo.isCopyKey( evt ), evt.shiftKey);

			this._checkBoxClicked = false;
		}
		else 
			this.tree._onEnterKey({"evt":evt});
    },
		
    _onDblClick: function() {
    	// dummy;
    },
	
    _onCheckboxClick: function(e) {
        this.tree._onTreeNodeChecked(this.item, this.checkboxNode.checked);
		this._checkBoxClicked = true;
    },
    
    _setLabelAttr: function(label) {
        this.labelNode.innerHTML = this.labelNode.title = label;
    },
    
    _setStateIconAttr: function(iconClass, updateWidth) {
        if (this.stateIconClass) {
            dojo.removeClass(this.stateIconNode, this.stateIconClass);
        }
        if (iconClass) {
            this.stateIconClass = iconClass;
            dojo.addClass(this.stateIconNode, iconClass);
            dojo.style(this.stateIconNode, "display", "");
        } else {
            this.stateIconClass = null;
            dojo.style(this.stateIconNode, "display", "none");
        }
        // state icon changed, update the label's width, but if an updateWidth param
        // is passed in, honor that. In the start up case, we do width adjustment
        // at last.
        if (updateWidth == undefined || updateWidth) {
            this.updateLabelClipWidth(this.widthHint, false);
        }
    }
});

var ActionableTreeNode = com.ibm.asq.common.web.ui.internal.widgets.navigator.ActionableTreeNode;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.navigator.ActionableTree",
             dijit.Tree,
{
    /*
     * Installs action to the tree nodes by providing an actionRegistry object to the tree.
     */ 
    actionRegistry: null,   
    showExpandAction: false,
    isRemote: false,
    // 'end' to place the action menu on tree nodes to the right most position.
    // 'align' to place action menu close to the tree node's label in alignment w/ other tree nodes
    actionMenuPlacement: 'align',
    
    // set to true to have tree items as check boxes items. In addition, the model
    // can turn on check boxes on tree items individually. In the model interface
    // implement a 'getTreeItemType'. Returns 'checkbox' to show check box in a tree item.
    // For example: in the model class, implements a method:
    //
    // getTreeItemType: function(modelItem) { /* based on the modelItem */ return 'checkbox' }
    //
    checkBoxTreeItem: false,
    
    textDir: BidiUtil.getWidgetBaseTextDir(),
    
    addToTree: function(item){
        var node = this._itemNodeMap[this.model.getIdentity(item)];
        if(node){
            this._expandNode(node);
        }       
    },

    setAriaLabel: function(label){
    	this.setAttribute("aria-label", label);
    },
	_onEnterKey: function(/*Object*/ message)
	{
		//Strangely enough _onEnterKey gets called for the space key as well
		//We only want to handle the Enter key differently.
		if (message.evt.keyCode == dojo.keys.ENTER && this.onTreeEnterKey) {
			this.onTreeEnterKey(message.evt);
		}
		else
			this.inherited(arguments);		
	},
	    
    onClick: function(/* dojo data */ item, /* TreeNode*/ node)
    {
        if (item.artifact && item.supportsRichHover) {
            window.open(item.artifact.enclosure);
        } else {
            this.tree.focusNode(node);
        }
    },
    
    onDblClick: function(/* dojo.data */ item, /*TreeNode*/ node, /*Event*/ evt)
    {
        // process dbl click, if the item has actions and has default action
        // invoke it.
        if (this.actionRegistry) {
            var actionDesc = this.actionRegistry.getContextActions(this.model, item);
            if (actionDesc) {
                var action = this.actionRegistry.getDefaultAction(actionDesc);
                if (action) 
                    action.callback(item.parent, item);
            }
        }
    },
        
    // Change the behavior so nodes are loaded each time their parent is expanded whether they've 
    // been expanded or not before.
    onOpen: function(item, node)
    {
        node.state = "UNCHECKED";
        if (dojo.isFunction(this.onNodeOpen)) {
            this.onNodeOpen(item, node);
        }
    },
    
    onClose: function(item, node)
    {
        if (dojo.isFunction(this.onNodeClose)) {
            this.onNodeClose(item, node);
        }
    },
    
    // returns all of the checked tree nodes in the tree. If the model and/or the
    // tree is set to support check boxes.
    // for example:
    //    var items = tree.get("checkedItems");
    //
    _getCheckedItemsAttr: function() {
        var checkedItems = [];
        var checkedMap = this._checkedItemMap;
        if(checkedMap) {
            for(var id in checkedMap) {
                checkedItems.push(checkedMap[id]);
            }
        }
        return checkedItems;
    },
    
    _onTreeNodeChecked: function(item, checked) {
        if(!this._checkedItemMap) {
            this._checkedItemMap = {};
        }
        var id = this.model.getIdentity(item);
        if(checked) {
            this._checkedItemMap[id] = item;
        }
        else if(this._checkedItemMap[id]) {
            delete this._checkedItemMap[id];
        }
    },
    
    _createTreeNode: function(/*Object*/ args){
        if (this.actionMenuHoverClass) {
            args.actionMenuHoverClass = this.actionMenuHoverClass;
        }
        if(this.checkBoxTreeItem || this.model.getTreeItemType && this.model.getTreeItemType(args.item) == "checkbox") {
            args.checkbox = true;
        }
        // apply the width hint to the tree node. If it's IE. We need
        // a proper width to clip the tree node's label to the container's
        // width in IE.
        if (ActiveClipping && this.sizeHint) {
            args.widthHint = this.sizeHint.w;
        }
		args.isRemote = this.isRemote;
        args.actionMenuPlacement = this.actionMenuPlacement;
        return new ActionableTreeNode(args);
    },
    
    onResized: function(sizeHint) {
        // if this is IE, update each of the tree node to the new width if the tree container
        // size has been changed.
        if(ActiveClipping && this.rootNode) {
            this.rootNode.updateLabelClipWidth(sizeHint.w, /*recurse*/true);
        }
    }    
});


})();
