/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The explorer view consists of two horizontally layout areas. One is an expandable/collapsable
 //    panel that holds the exploring area. The other is a container that can hold any detailed widget
 //    that corresponds to the selected item in the exploring area. The client decides what to 
 //    put into the container.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.navigator.ExplorerView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");

dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.ExpandoPanel");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.StackViewContainer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.TreeView");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.NodeTypes");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

(function() {

var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
var Session = com.ibm.team.repository.web.client.session;
var TreeView = com.ibm.asq.common.web.ui.internal.widgets.navigator.TreeView;
var NodeTypes = com.ibm.asq.common.web.ui.internal.widgets.navigator.NodeTypes;
var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.navigator.ExplorerView",
             [dijit._Widget, dijit._Templated],
{ 
    templateString: '<div class="com-ibm-rqm-explorer-view" dojoAttachPoint="domNode">' +
                    '  <table class="content-table" role="presentation">' +
                    '    <tbody>' +
                    '       <tr>' +
                    '          <td class="explorer-navigator">' +
                    '            <div dojoType="com.ibm.asq.common.web.ui.internal.widgets.navigator.ExpandoPanel" height=${height} expandWidth=${initExplorerWidth}' + 
                    '                 dojoAttachPoint="navNode" dojoAttachEvent="onOpen:_onExpandoOpen, onClose:_onExpandoClose">' +
                    '            </div>' +
                    '          </td>' +
                    '          <td class="explorer-detail">' +
                    '            <table style="width:100%;table-layout:fixed;" role="presentation">' +
                    '              <tbody>' +
                    '                <tr>' +
                    '                  <td>' +
                    '                    <div dojoAttachPoint="loadingProgressNode" class="loading-style" style="display:none">' +
                    '                      <span dojoAttachPoint="loadingLabelNode">${loadingLabel}</span>' +
                    '                    </div>' +
                    '                    <div class="detail-content" dojoAttachPoint="container"></div>' +
                    '                  </td>' +
                    '                </tr>' +
                    '              </tbody>' +
                    '            </table>' +
                    '          </td>' +
                    '       </tr>' +
                    '    </tbody>' +
                    '  </table>' +
                    '</div>',
                    
    widgetsInTemplate: true,
    loadingLabel: MessageUtil.getMessage(MessageBundle.COMMON, "tableViewer_loading"),
    height: 500,
    initExplorerWidth: 240,
    title: null,
    modelProviders: null,
    currentProvider: null,
    _lastModelProvider: null,
    manageProvider: true,
    initOpenFiltersView: true,
    
    postCreate: function(){
        this.stackContainer = new com.ibm.asq.common.web.ui.internal.widgets.navigator.StackViewContainer({
            initOpen: this.initOpenFiltersView,
            manageChildren: this.manageProvider,
		    initOpenIndex: 0  // default to open the first child initially
        });
        this.navNode.set("contentArea", this.stackContainer);
        
        // The tree layout of the providers that is saved and will be used to restore
        // the opened nodes when a provider is selected.  It is in this format:
        //     {
        //         "lastProvider": providerKey,
        //         "lastProvidersInfo": {
        //             <providerKey>: {
        //                 "params": {},            // persisted params from the provider
        //                 "openedNodes": {
        //                     <nodeItemId>: {         // top level node
        //                         <nodeItemId>: {     // children node
        //                         ...
        //                         }
        //                     },
        //                     ...
        //                 }
        //             },
        //             ...
        //         }
        //     }
        //
        this.treeViewLayoutCache = {};
        
        try {
            this.currentUserId = Session.getAuthenticatedContributor().userId;
        } catch (ex) {
            // Session is not initialized with user info if this is remotely
            // invoked through OSLC.
        }
        if (!this.currentUserId) {
            // InitializationData is not loaded if this is used remote in an iFrame of the
            // OSLC dialog.  Need to load it to get the user id.
            var self = this;
            var callback = {
                success: function(initData) {
                    Session.init(initData);
                    self.currentUserId = Session.getAuthenticatedContributor().userId;
                },
                error: function(status){
                }
            };
            var handler = new ServiceResponseHandler(callback, "success", "error");
            RepositoryClient.getInitializationData(handler);
        }
    },
    
    onDetailContainerLoadingChange: function(isLoading){
        // provide our default behavior when the detail container's 
        // loading state is being changed.
        //
        dojo.style(this.container, "display", isLoading ? "none" : "");
        dojo.style(this.loadingProgressNode, "display", isLoading ? "" : "none");
        
    },
    
    reloadContent: function(/* optional */ keepWidget) {
        if (this.currentProvider) {
            var view = this.providerViews[this.currentProvider.key];
            view.loadContent(keepWidget);
            view.isStarted = true;
        }
    },
    
    // Create a view container for the specified provider. Provider can implement
    // the getViewObjectClass method which must return a view of the type StackableView view.
    // If that method doesn't return a view object class, the default TreeView is used.
    // 
    // To have complete control of the views to be created in the explorer's expando panel.
    // override this method.
    //
    createProviderView: function(provider) {
        var viewObjectClass = provider.getViewObjectClass() || TreeView;
        var args = dojo.mixin({
            providerKey: provider.key,
            title: provider.title,
            onLoad: dojo.hitch(this, function(provider, onComplete){
                var model = provider.getModel();
                onComplete(model);
                if (dojo.isFunction(provider.onLoadComplete)) {
                    provider.onLoadComplete();
                }
            }, provider)
        }, provider.getViewObjectCreateArgs());

        // hook in the the default onSelect handler if it's a tree view
        if (viewObjectClass == TreeView) {
            args.onClick = dojo.hitch(this, "onSelect");
            args.onNodeOpen = dojo.hitch(this, "onNodeOpen");
            args.onNodeClose = dojo.hitch(this, "onNodeClose");
            args.onContentLoaded = dojo.hitch(this, "_restoreTreelayout");
        }
        
        var view = new viewObjectClass(args);
        view.onViewStateChanged = dojo.hitch(this, "onViewStateChanged");
        
        var menuProvider = provider.treeMenuProvider ? dojo.hitch(provider, "treeMenuProvider") : null;
        if (viewObjectClass == TreeView && menuProvider) {
            // add the Collapse All action if it is a TreeView
            menuProvider = dojo.hitch(this, function() {
                var modelMenuProvider = provider.treeMenuProvider();
                modelMenuProvider.actions.push({
                    label: MessageUtil.getMessage(MessageBundle.COMMON, "labelCollapseAll"),
                    iconClass: "collapsse-all-tree-node-icon-image",
                    onClick: dojo.hitch(this, "collapseAllNodes")
                });
                return modelMenuProvider;
            });
        }
        view.set("menuProvider", menuProvider);
        view.set("actionRegistry", provider.actionRegistry ? provider.actionRegistry() : null);
        
        this.stackContainer.addChild(view);
        this.providerViews[provider.key] = view;
    },
    
    _orderProviders: function() {
        var providers = this.modelProviders;
        var orderedList = [];
        var count = 0;
        for (var key in providers) {
            var provider = providers[key];
            if(provider.position == 'undefined') {
                provider.position = count;
            }
            orderedList.push(providers[key]);
            count++;
        }        
        orderedList.sort(function(a, b) {
            var p1 = a.position;
            var p2 = b.position;
            return (p1 > p2) ? 1 : -1;
        });
        return orderedList;
    },
    
    _initializeProviders: function() {
        if (this.modelProviders) {
            this.currentProvider = this.currentProvider || null;
            this.providerViews = {};
            var firstSelect = this.currentProvider;
            var stackContainerIndex = null;
            var orderedList = this._orderProviders();
            
            if (!this.currentProvider) {
                // restore the last selected provider if current provider is not specified
                var provider = this.currentProvider || orderedList[0];
                // Get provider info to determine the key to the local storage for the
                // tree layout cache.  It contains which provider is selected last.
                var key = (provider || this.tableViewerId) ? this._getTreeLayoutCacheId(provider) : null;
                // key is deduced from provider or specified tableViewerId
                if (key) {
                    this._initTreeViewLayoutCache(key);
                    var treeCache = this.treeViewLayoutCache[key];
                    var lastProviderKey = treeCache["lastProvider"];
                    if (lastProviderKey) {
                        // restore the last selected provider
                        firstSelect = this.currentProvider = this.modelProviders[lastProviderKey];
                    }
                }
            }

            for (var i = 0; i < orderedList.length; i++){
                var provider = orderedList[i];
                var key = provider.key;
                
                this.createProviderView(provider);
                
                if(firstSelect == null) {
                    firstSelect = provider;  // use the first provider as default if no current is predefined
                    stackContainerIndex = i;
                } else if (stackContainerIndex == null && firstSelect.key == key) {
                    stackContainerIndex = i;
                }
            }
            if(firstSelect && !this.currentProvider) {
                this.set("currentProvider", firstSelect);
            }
            this.isInitialized = true;
            this.stackContainer.initOpenIndex = stackContainerIndex;
            
            // if we are not managing the providers, i.e., allowing only 1 provider to
            // be opened at a time, this means all provider views can be opened and the
            // height of the views could extent beyond the initial height. If 'autoExtent'
            // is turned on, get the extent changed notify from our stack container and
            // change the navigator panel's extent as appropriate.
            if(!this.manageProvider && this.autoExtent) {
                this.stackContainer.set('contentExtentChangedNotify', dojo.hitch(this, function(marginBox) {
                    dojo.style(this.navNode.domNode, 'height', Math.max(this.height, marginBox.h) + 'px');
                }));
            }
        } else {
            console.error("*** No provider specified to be initialized");  //$NON-NLS
        }
    },
    
    _onExpandoOpen: function() {
        // Our providers are lazily initialized. We don't initialize them until
        // our navigation panel is first open. Create the views for them now.
        if (!this.isInitialized) {
            this._initializeProviders();
        }
        
        // tighten up the gap between the explorer panel and the detail
        // when we're open. We hide the sash when open and that give a wider
        // empty gap.
        dojo.toggleClass(this.container, "detail-content-open", true);
        
        if (this.currentProvider && this.manageProvider) {
            var treeView = this.providerViews[this.currentProvider.key];
            // instruct the tree view to load its content. This will
            // call back into the provider's content.
            if (treeView && !treeView.isStarted) {
                // the view could be just added to our management. Let its dom node
                // obtain a size before loading its contents.
                setTimeout(dojo.hitch(this, function(){
                    treeView.loadContent();
                    treeView.isStarted = true;
                    setTimeout(dojo.hitch(this, "onExplorerOpen"), 1);
                }), 1);
            }
            else {
                setTimeout(dojo.hitch(this, "onExplorerOpen"), 1);
            }
        }
    },
    
    _onExpandoClose: function() {
        dojo.toggleClass(this.container, "detail-content-open", false);
        setTimeout(dojo.hitch(this, "onExplorerClose"), 1);
    },
    
    onExplorerOpen: function() {
        if (this.currentProvider && this.manageProvider) {
            this.currentProvider.onExplorerOpen();
        }
    },
    
    onExplorerClose: function() {
        if (this.currentProvider && this.manageProvider) {
            this.currentProvider.onExplorerClose();
        }
    },
    
    onSelect: function(item) {
    	// track the current selected item
        if (this.lastSeletedItem && this.lastSeletedItem.selected) {
            delete this.lastSeletedItem.selected;
        }
        this.lastSeletedItem = item;
        this.lastSeletedItem.selected = true;
        
        if (this.currentProvider && this.manageProvider) {
            this.currentProvider.onSelect(item);
        }
    },
    
    /**
     * Get the unique id for the tree layout cache for this specific explorer view.
     * The format is "project id|user id|table id".  It is the key for the local storage.
     */
    _getTreeLayoutCacheId: function(provider) {
        if (!this.treeLayoutCacheId) {
            // get table id from the table viewer of the provider if it is not specified
            if (!this.tableViewerId && provider) {
                if (provider.tableViewer) {
                    this.tableViewerId = provider.tableViewer.tableId;
                } else if (provider.parentView && provider.parentView.tableViewer) {
                    // get the table viewer through the parent view if the table
                    // viewer of the provider is not yet initialized
                    this.tableViewerId = provider.parentView.tableViewer.tableId;
                }
            }
            var projectAreaId = Utility.getCurrentProjectAreaId();
            if (this.tableViewerId && this.currentUserId) {
                var key = [ projectAreaId, this.currentUserId,  this.tableViewerId];
                this.treeLayoutCacheId = key.join("|");
            }
//            console.log("this.treeLayoutCacheId: " + this.treeLayoutCacheId);
        }
        return this.treeLayoutCacheId;
    },
    
    /**
     * Get the params from the local storage for the provider.
     */
    getProviderCachedParams: function(provider) {
        var key = this._getTreeLayoutCacheId(provider);
        var params = null;
        if (key) {
            this._initTreeViewLayoutCache(key);
            var treeCache = this.treeViewLayoutCache[key];
            var providerKey = provider.key;
            if (treeCache["lastProvidersInfo"] && treeCache["lastProvidersInfo"][providerKey]) {
                params = treeCache["lastProvidersInfo"][providerKey]["params"];
            }
        }
        return params;
    },
    
    /**
     * Persist a param in the local storage for the provider.
     */
    cacheProviderParam: function(provider, paramName, paramValue) {
        var key = this._getTreeLayoutCacheId(provider);
        if (key) {
            this._initTreeViewLayoutCache(key);
            var providerKey = provider.key;
            if (!this.treeViewLayoutCache[key]["lastProvidersInfo"]) {
                this.treeViewLayoutCache[key]["lastProvidersInfo"] = {};
            }
            if (!this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey]) {
                this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey] = {};
            }
            if (!this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey]["params"]) {
                this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey]["params"] = {};
            }
            if (this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey]["params"][paramName] != paramValue) {
                this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey]["params"][paramName] = paramValue;
                this._saveTreeViewLayoutCache(key);
            }
        }
    },
    
    /**
     * Handler invoked when a tree node is opened
     */
    onNodeOpen: function(item, node) {
        if (this.currentProvider && this.manageProvider) {
            // The root node is opened implicitly when the provider is selected
            // to expose the tree.  Skip it as it is not displayed in the tree.
            if (item.type != NodeTypes.ROOT_TYPE) {
                // update the tree layout cache with new opened node
                this._addOpenedNode(this.currentProvider, item);
            }
        }
    },
    
    /**
     * Handler invoked when a tree node is opened
     */
    onNodeClose: function(item, node) {
        if (this.currentProvider && this.manageProvider) {
            // update the tree layout cache to remove the closed node
            this._clearOpenedNode(this.currentProvider, item);
        }
    },
    
    /**
     * Collapse all tree node of the current provider
     */
    collapseAllNodes: function() {
        var treeView = this.get("treeView");
        treeView.tree.collapseAll();
    },
    
    /**
     * Restore the tree layout for the current provider.
     * This is the callback for TreeView that is invoked after the tree content is loaded.
     */
    _restoreTreelayout: function() {
        var key = this._getTreeLayoutCacheId(this.currentProvider);
        if (key) {
            this._initTreeViewLayoutCache(key);
            var treeCache = this.treeViewLayoutCache[key];
            var providerKey = this.currentProvider.key;
            if (treeCache["lastProvidersInfo"] && treeCache["lastProvidersInfo"][providerKey]) {
                // there are opened nodes in the tree layout cache for the provider
                var openNodes = treeCache.lastProvidersInfo[providerKey]["openedNodes"];
                for (var nodeItemId in openNodes) {
                    this._expandTreeLayoutCacheNode(openNodes, nodeItemId);
                }
            }
            
            // the tree layout is restore only when the provider is selected,
            // update the cache if it different
            if (this.treeViewLayoutCache[key]["lastProvider"] != providerKey) {
                this.treeViewLayoutCache[key]["lastProvider"] = providerKey;
                this._saveTreeViewLayoutCache(key);
            }
        }
    },
    
    /**
     * Expand a branch, node and its descendants, in the tree layout cache.
     */
    _expandTreeLayoutCacheNode: function(openNodes, identity) {
        var treeView = this.get("treeView");
        var nodes = treeView.tree.getNodesByItem(identity);
        // just one since identity of a tree node is unique
        var node = nodes.length > 0 ? nodes[0] : null;
        if (node) {
            dojo.when(treeView.tree._expandNode(node), dojo.hitch(this, function() {
                // deferred after node is expanded, expand children nodes in the cache
                var itemNode = openNodes[identity];
                if (itemNode) {
                    for (var childrenIdentity in itemNode) {
                        setTimeout(dojo.hitch(this, "_expandTreeLayoutCacheNode", itemNode, childrenIdentity), 10);
                    }
                }
            }));
        } else {
            // The node is no longer opened, remove from cache.  This can happen
            // if the user changed the setting not to explore the artifact in the
            // Category Explorer with artifact nodes opened.
            delete openNodes[identity];
            this._saveTreeViewLayoutCache(this.treeLayoutCacheId);
//            console.log(identity + " not found in " + treeView.tree);
//            console.dir(treeView.tree._itemNodesMap);
        }
    },
    
    /**
     * Add a opened node of the provider tree to the tree layout cache.
     */
    _addOpenedNode: function(provider, item) {
        var key = this._getTreeLayoutCacheId(provider);
        if (key) {
            this._initTreeViewLayoutCache(key);
            if (!this.treeViewLayoutCache[key]["lastProvidersInfo"]) {
                this.treeViewLayoutCache[key]["lastProvidersInfo"] = {};
            }
            var providerKey = provider.key;
            if (!this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey]) {
                this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey] = {};
            }
            if (!this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey]["openedNodes"]) {
                this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey]["openedNodes"] = {};
            }
            // get the hierarchy of the tree nodes for the item
            //
            var hierarchy = this._getNodeItemHierarchy(item);
            var openedNodes = this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey]["openedNodes"];
            var currentNode = openedNodes;
            // add node to the tree layout cache from the top level node if it is not yet added
            //
            var haveChanges = false;
            for (var i = 0; i < hierarchy.length; i++) {
                if (!currentNode[hierarchy[i]]) {
                    currentNode[hierarchy[i]] = {};
                    haveChanges = true;
                }
                currentNode = currentNode[hierarchy[i]];
            }
            if (haveChanges) {
                this.treeViewLayoutCache[key]["lastProvidersInfo"][providerKey]["openedNodes"] = openedNodes;
                this._saveTreeViewLayoutCache(key);
            }
        }
    },
    
    /**
     * Removed a closed node of the provider tree from the tree layout cache.
     */
    _clearOpenedNode: function(provider, item) {
        var key = this._getTreeLayoutCacheId(provider);
        if (key) {
            this._initTreeViewLayoutCache(key);
            var treeCache = this.treeViewLayoutCache[key];
            var providerKey = provider.key;
            if (treeCache["lastProvidersInfo"] && treeCache["lastProvidersInfo"][providerKey]) {
                var currentNode = treeCache["lastProvidersInfo"][providerKey]["openedNodes"];
                // get the hierarchy of the tree nodes for the item
                //
                var hierarchy = this._getNodeItemHierarchy(item);
                // walk the hierarchy in the tree layout cache from the top level node
                // to the immediate parent
                //
                for (var i = 0; i < hierarchy.length - 1; i++) {
                    if (!currentNode[hierarchy[i]]) {
                        return;
                    }
                    currentNode = currentNode[hierarchy[i]];
                }
                var treeView = this.get("treeView");
                var identity = treeView.model.getIdentity(item);
                if (currentNode[identity]) {
                    // remove the node with the children in the branch
                    delete currentNode[identity];
                    this._saveTreeViewLayoutCache(key);
                }
            }
        }
    },
    
    /**
     * Return the hierarchy identity of an item from the root node.
     */
    _getNodeItemHierarchy: function(item) {
        var treeView = this.get("treeView");
        // Get the identity from the model.  The itemId may not be unique in the tree.
        var identity = treeView.model.getIdentity(item);
        var hierarchy = [ identity ];
        var itemNode = item.parent;
        while (itemNode && itemNode.type != NodeTypes.ROOT_TYPE) {
            hierarchy.push(treeView.model.getIdentity(itemNode));
            itemNode = itemNode.parent;
        }
        hierarchy = hierarchy.reverse();
//        console.log("Hierarchy: " + hierarchy);
        return hierarchy;
    },
    
    /**
     * Update the tree layout cache of the selected provider
     */
    _updateLastProvider: function(provider) {
        var key = this._getTreeLayoutCacheId(provider);
        if (key) {
            this._initTreeViewLayoutCache(key);
            if (this.treeViewLayoutCache[key]["lastProvider"] != provider.key) {
                this.treeViewLayoutCache[key]["lastProvider"] = provider.key;
                this._saveTreeViewLayoutCache(key);
            }
        }
    },
    
    /**
     * Initialize the tree layout cache from the local storage if it is not
     * yet initialized.
     */
    _initTreeViewLayoutCache: function(key) {
        if (!this.treeViewLayoutCache[key]) {
            var treeCache = {};
            if (typeof(localStorage) != "undefined") {
                treeCache = dojo.fromJson(localStorage.getItem(key));
            }
            this.treeViewLayoutCache[key] = treeCache || {};
        }
    },
    
    /**
     * Update the local storage with the tree layout cache.
     */
    _saveTreeViewLayoutCache: function(key) {
        var treeCache = this.treeViewLayoutCache[key];
        if (typeof(localStorage) != "undefined") {
            localStorage.setItem(key, dojo.toJson(treeCache));
        }
//        console.log("Cache for '" + key + "':");
//        console.dir(treeCache);
    },
    
    _getTreeViewAttr: function() {
        if(this.currentProvider && this.providerViews)
            return this.providerViews[this.currentProvider.key];
        return null;    
    },
    
    _setLoadingTreeAttr: function(isLoading) {
        if (this.currentProvider) {
            var treeView = this.providerViews[this.currentProvider.key];
            treeView.set("loading", isLoading);
        }
    },
    
    _setLoadingContainerAttr: function(isLoading) {
        this.onDetailContainerLoadingChange(isLoading);
    },
    
    _setDetailContainerAttr: function(widget) {
        dojox.xml.parser.removeChildren(this.container);
        this.container.appendChild(widget.domNode);  
    },
    
    _setModelProvidersAttr: function(/* providers associative array */ providers) {
        var providersInitialized = this.modelProviders ? true : false;
        this.modelProviders = providers;
        if (providersInitialized) {
            // the providers are already initialized, need to reset the panel selector with new provider options
            this._initializeProviders();
        }
    },
    
    _getModelProvidersAttr: function() {
        return this.modelProviders;
    },
    
    getModelProviderView: function(provider) {
        return (provider && provider.key && this.providerViews) ? this.providerViews[provider.key] : null;
    },

    _setCurrentProviderAttr: function(/* must be one of the providers */ provider){
        var providerKey = provider ? provider.key : null;
        if (providerKey && this.modelProviders && this.modelProviders[providerKey]) {
            this._lastModelProvider = this.currentProvider;

            // switching filter provider, clear the current filters first
            // before showing the new provider.
            var view = this.providerViews ? this.providerViews[providerKey] : null;
            if(view) {
                view.set('state', 'open');
            }
            else if(this._lastModelProvider && this.isInitialized) {
                this._lastModelProvider.onProviderViewClose();                
            }
            this.currentProvider = this.modelProviders[providerKey];
        } else {
            console.error("*** Provider with key " + providerKey + " not found to set to current");  //$NON-NLS
        }
    },
    
    _setViewOpenAttr: function(open) {
        if(open) {
            this.navNode.expand(0);
        }
        else {
            this.navNode.collapse(0);
        }
    },
    
    // handle the view state changed.
    onViewStateChanged: function(view, state) {
        var key = view.providerKey;
        
        if(state == "close") {
            var provider = this.modelProviders[key];
            if(provider) {
                provider.onProviderViewClose();
            }
            this.currentProvider = null;
        }
        else {
            this.set("currentProvider", this.modelProviders[key]);
            this.onExplorerOpen();
            
            // no need to recreate the tree widget if just changing provider
            this.reloadContent(/* keepWidget */ view.isStarted);
            
            // notify the provider change.
            this.onProviderChanged(this.currentProvider);
            
            // update tree layout cache of the last opened provider
            this._updateLastProvider(this.currentProvider);
        }
    },
    
    onProviderChanged: function(newProvider) {
        // external notification of a provider changed. The current
        // provider in charged is sent in this notification. Interested
        // party can override this method.
    },
    
    dummy:null
});

})();
