/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.rational.datacollection.web.ui.internal.editors.ConfigureResourceGroup"); //$NON-NLS-1$

dojo.require("com.ibm.rational.datacollection.web.ui.internal.dialogs.AddResourceGroupDialog"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.ResourceGroupSection"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.BaseResourceGroupSection"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.ResourceHelper");//$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.WorkingCopy"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.AbstractEditor"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.underlay"); //$NON-NLS-1$
dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "ConfigureResourceGroupMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var AddResourceGroupDialog = com.ibm.rational.datacollection.web.ui.internal.dialogs.AddResourceGroupDialog;
var BaseResourceGroupSection = com.ibm.rational.datacollection.web.ui.internal.editors.BaseResourceGroupSection;
var ResourceGroupSection = com.ibm.rational.datacollection.web.ui.internal.editors.ResourceGroupSection;
var DataCollectionClient = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient;
var DataCollectionConstants = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants;
var ResourceHelper = com.ibm.rational.datacollection.web.ui.internal.common.ResourceHelper;
var WorkingCopy = com.ibm.rational.datacollection.web.ui.internal.common.WorkingCopy;
var AbstractEditor = com.ibm.rational.datacollection.web.ui.internal.editors.AbstractEditor;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;

dojo.declare("com.ibm.rational.datacollection.web.ui.internal.editors.ConfigureResourceGroup", [AbstractEditor], { //$NON-NLS-1$

	templatePath: dojo.moduleUrl("com.ibm.rational.datacollection.web", "ui/internal/editors/templates/ConfigureResourceGroup.html"), //$NON-NLS-1$ //$NON-NLS-2$
    baseResourceGroupSections: null,
	workingCopy: null,
	
	// widget initialization
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "ConfigureResourceGroupMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	postCreate: function () {
		this.inherited(arguments);
		
		this.baseResourceGroupSections = [];
		this.refresh();
//		this.connect(this._editLink, "onclick", function(e) { //$NON-NLS-1$
//			this.toggleEditMode(true);
//		});
//		
//		this.connect(this._viewLink, "onclick", function(e) { //$NON-NLS-1$
//			this.toggleEditMode(false);
//		});
//		
		this.connect(this._saveButton, "onclick", function(e) { //$NON-NLS-1$
			jazz.ui.underlay.show();
			this.workingCopy.doSave(dojo.hitch(this, function(error) {
				if (error) {
					this.displayDetailedErrorMessage(this.messages.saveResourceGroupFailed, error);
					jazz.ui.underlay.hide();
				} else {
					jazz.ui.underlay.hide();
					this.refresh();
				}
			}))
		});
		
		this.connect(this._addButton, "onclick", function(e) { //$NON-NLS-1$
			new AddResourceGroupDialog({
				configureResourceGroupPage: this
			});
		});
		
		// Add the page hover description
		this._pageHelpHover = new PageHoverHelp({ contentString: this.messages.pageHoverHelpDescription });
		this.helpHover.appendChild(this._pageHelpHover.domNode);
	},
	
	toggleEditMode: function (edit) {
		dojo.toggleClass(this._editLink, "disabled", edit); //$NON-NLS-1$
		dojo.toggleClass(this._viewLink, "disabled", !edit); //$NON-NLS-1$
		for (var i = 0; i < this.resourceGroupSections.length; i++) {
			var section = this.resourceGroupSections[i];
			section.toggleEditMode(edit);
		}
	},

	_initializeContent: function () {
		this._loadBaseResourceGroup();
	},
	
	_loadBaseResourceGroup:  function () {
		DataCollectionClient.getBaseResourceGroupCollection({
			load: dojo.hitch(this, function(data, ioArgs) {
				this.workingCopy.initializeBaseResourceGroup(data);
				for (var i = 0; i < data.length; i++) {
					var baseResourceGroupSection = new BaseResourceGroupSection({
						"data": this.workingCopy.getBaseResourceGroup(data[i].uri), //$NON-NLS-1$
						"controller": this //$NON-NLS-1$
					});
					this.baseResourceGroupSections.push(baseResourceGroupSection);
					this.contentArea.appendChild(baseResourceGroupSection.domNode);
					if(i !== 0){
						baseResourceGroupSection.collapsePanel();
					}
				}
				this._loadResourceGroup();
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				this.displayDetailedErrorMessage(this.messages.loadBaseResourceGroupFailed, error);
				jazz.ui.underlay.hide();
			})
		});
	},
	
	//Only expand one section at one time.
	layoutSections: function (section){
		
		if(section instanceof BaseResourceGroupSection){
			for(var i=0; i< this.baseResourceGroupSections.length; i++){
				var thisBaseResourceGroup = this.baseResourceGroupSections[i];
				if(thisBaseResourceGroup !== section){
					thisBaseResourceGroup.collapsePanel();
				}
			}
		}else if(section instanceof ResourceGroupSection){
			var baseRscGroup = section.controller;
			
			for(var i=0; i< baseRscGroup.resourceGroupSections.length; i++){
				var thisResourceGroup = baseRscGroup.resourceGroupSections[i];
				if(thisResourceGroup !== section){
					thisResourceGroup.collapsePanel();
				}
			}
		}
	},
	
	_loadResourceGroup:  function () {
		DataCollectionClient.getResourceGroupCollection({
			load: dojo.hitch(this, function(data, ioArgs) {
				var readGroups = dojo.hitch(this, function(callback, currentIndex) {
					if (data.length === currentIndex) {
						callback();
					} else {
						var uri = data[currentIndex].uri;
						DataCollectionClient.getResourceGroup({
							"url": uri, //$NON-NLS-1$
							load: dojo.hitch(this, function(data, ioArgs) {
								var etag = ioArgs.xhr.getResponseHeader(DataCollectionConstants.HEADER_ETAG);
								var item = this.workingCopy.addResourceGroup(data, etag);
								var baseResourceGroupUri = ResourceHelper.getBaseResourceGroupUri(data);
								var section = this.getBaseResourceGroupSection(baseResourceGroupUri);
								
								
								var resourceGroupSection = new ResourceGroupSection({
									controller: section,
									data: item,
									lockId: true,
									identifier: section.data["dcterms:identifier"] //$NON-NLS-1$
								});
								resourceGroupSection.toggleEditMode(true);
								section.resourceGroupSections.push(resourceGroupSection);
								section.contentArea.appendChild(resourceGroupSection.domNode);
								
								//Only expand the first Section of the BaseResourceGroup
								if(section.resourceGroupSections.length > 1){
									resourceGroupSection.collapsePanel();
								}
								readGroups(callback, ++currentIndex);
							}),
							error: dojo.hitch(this, function(error, ioArgs) {
								this.displayDetailedErrorMessage(this.messages.loadResourceGroupFailed, error);
								jazz.ui.underlay.hide();
							})
						})
						
					}
				});
				
				readGroups(dojo.hitch(this, this.onLoadingComplete), 0);
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				this.displayDetailedErrorMessage(this.messages.loadResourceGroupFailed, error);
				jazz.ui.underlay.hide();
			})
		});
	},
	
	onLoadingComplete: function () {
		this.hideMessage();
		jazz.ui.underlay.hide();
		this._addButton.disabled = false;
	},
	
	_getChanges: function () {
		var changes = {
			"changes": [], //$NON-NLS-1$
			"properties": [] //$NON-NLS-1$
		};
		for (var i=0; i < this._services.length; i++) {
			var obj = this._services[i]._getChanges();
			changes["changes"] = changes["changes"].concat(obj["changes"]); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			changes["properties"] = changes["properties"].concat(obj["properties"]); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		return changes;
	},
	
	setChange: function() {
		this._saveButton.disabled = false;
	},
	
	resetSaveButton: function() {
		this._saveButton.disabled = true;
	},
	
	getWorkingCopy: function() {
		return this.workingCopy;
	},
	
	getBaseResourceGroupSection: function(uri) {
		var section = null;
		for (var i = 0; i < this.baseResourceGroupSections.length; i++) {
			var group = this.baseResourceGroupSections[i];
			if (group.data["uri"] === uri) { //$NON-NLS-1$
				section = group;
				break;
			}
		}
		return section;
	},
	
	refresh: function() {
		this.hideMessage();
		this.displayInfoMessage(this.messages.loading);
		this.workingCopy = new WorkingCopy();
		for (var i = 0; i < this.baseResourceGroupSections.length; i++) {
			this.baseResourceGroupSections[i].destroyRecursive();
		}
		
		this.baseResourceGroupSections = [];
		this._saveButton.disabled = true;
		this._addButton.disabled = true;
		jazz.ui.underlay.show();
		this._initializeContent();
	},
	
	uninitialize: function(){
		for (var i = 0; i < this.baseResourceGroupSections.length; i++) {
			this.baseResourceGroupSections[i].destroyRecursive();
		}
		
		this._pageHelpHover.destroyRecursive();
		
		this.inherited(arguments);
	}
});

})();