/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.rational.datacollection.web.ui.internal.editors.DataCollectionJobStatus"); //$NON-NLS-1$

dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.AbstractEditor"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.ValidatingTextBox"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.DeleteStatusesDialog"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojo.date.locale"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "DataCollectionJobStatusMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
	
var DataCollectionClient = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient;
var AbstractEditor = com.ibm.rational.datacollection.web.ui.internal.editors.AbstractEditor;
var ValidatingTextBox = com.ibm.rational.datacollection.web.ui.internal.editors.ValidatingTextBox;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;
var DEFAULT_HISTORY_SIZE = 50;
var LOG_LINK = "<a href=\"datacollection/logger?qualifier=${0}_${1}\">Log</a>";  //$NON-NLS-1$
var DeleteStatusesDialog = com.ibm.rational.datacollection.web.ui.internal.editors.DeleteStatusesDialog;

dojo.declare("com.ibm.rational.datacollection.web.ui.internal.editors.DataCollectionJobStatus", [AbstractEditor], { //$NON-NLS-1$

	templatePath: dojo.moduleUrl("com.ibm.rational.datacollection.web", "ui/internal/editors/templates/DataCollectionJobStatus.html"), //$NON-NLS-1$ //$NON-NLS-2$
	jobTable: null,
	items: null,
	// widget initialization
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "DataCollectionJobStatusMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this._sectionTwistieOpenIconSrc = dojo.moduleUrl("com.ibm.rational.datacollection.web", "graphics/icons/twistie-open.gif"); //$NON-NLS-1$ //$NON-NLS-2$
		this._sectionTwistieCollapseIconSrc = dojo.moduleUrl("com.ibm.rational.datacollection.web", "graphics/icons/twistie-closed.gif"); //$NON-NLS-1$  //$NON-NLS-2$
	},

	postCreate: function () {
		this.inherited(arguments);
		
		this._createTable();
		this._createHistoryInput();
		this.connect(this._runQueryButton, "onclick", function(e){ //$NON-NLS-1$
			this._runQueryButton.disabled = true;
			this.refresh();
		});
		// Add the page hover description
		this._pageHelpHover = new PageHoverHelp({ contentString: this.messages.pageHoverHelpDescription });
		this.helpHover.appendChild(this._pageHelpHover.domNode);
		this.refresh();
	},
	
	_createTable: function() {
		this.runningJobTable = new jazz.ui.StyledTable({
			columns : [ {
				label : "", //$NON-NLS-1$
				sortable : false,
				width : "0px" //$NON-NLS-1$
			}, {
				label : this.messages.jobName,
				sortable : false,
				width : "20%" //$NON-NLS-1$
			}, {
				label : this.messages.startTime,
				sortable : false,
				width : "40%" //$NON-NLS-1$
			}, {
				label : this.messages.runningTime,
				sortable : false,
				width : "40%" //$NON-NLS-1$
			}],
			sortBy : 1,
			sortByAscending : false,
			handleSort : function() {
			},
			trim : jazz.ui.styledTable.TrimStyles.BLUE
		});
		
		this.runningJobTable.domNode.style.width = "100%"; //$NON-NLS-1$
		this.runningJobList.appendChild(this.runningJobTable.domNode);
		
		this.historyJobTable = new jazz.ui.StyledTable({
			columns : [ {
				label : "", //$NON-NLS-1$
				sortable : false,
				width : "0px" //$NON-NLS-1$
			}, {
				label : this.messages.jobName,
				sortable : false,
				width : "20%" //$NON-NLS-1$
			}, {
				label : this.messages.status,
				sortable : false,
				width : "5%" //$NON-NLS-1$
			}, {
				label : this.messages.startTime,
				sortable : false,
				width : "25%" //$NON-NLS-1$
			}, {
				label : this.messages.endTime,
				sortable : false,
				width : "25%" //$NON-NLS-1$
			}, {
				label : this.messages.timeTaken,
				sortable : false,
				width : "25%" //$NON-NLS-1$
			}],
			sortBy : 1,
			sortByAscending : false,
			handleSort : function() {
			},
			trim : jazz.ui.styledTable.TrimStyles.BLUE
		});
		this.historyJobTable.domNode.style.width = "100%"; //$NON-NLS-1$
		this.jobList.appendChild(this.historyJobTable.domNode);
	},
	
	addQueuingJobRow: function(item) {
		var row, collapse, name, startTime, runningTime;
		row = document.createElement("tr"); //$NON-NLS-1$
		collapse = document.createElement("td"); //$NON-NLS-1$
		name = document.createElement("td"); //$NON-NLS-1$
		startTime = document.createElement("td"); //$NON-NLS-1$
		runningTime = document.createElement("td"); //$NON-NLS-1$
		
		name.innerHTML = item.name;
		startTime.innerHTML = this.messages.waitingToStart; 
		runningTime.innerHTML = this._getDurationTimeString(0);
		row.appendChild(collapse);
		row.appendChild(name);
		row.appendChild(startTime);
		row.appendChild(runningTime);
		this.runningJobTable.tbody.appendChild(row);
	},
	
	addRunningJobRow: function(item) {
		var row, resourceRow, resourceTd, collapse, name, status, startTime, endTime, runningTime, processedResources, deliveredRows, failedResources, log;
		row = document.createElement("tr"); //$NON-NLS-1$
		collapse = document.createElement("td"); //$NON-NLS-1$
		name = document.createElement("td"); //$NON-NLS-1$
		startTime = document.createElement("td"); //$NON-NLS-1$
		runningTime = document.createElement("td"); //$NON-NLS-1$
		
		dojo.addClass(collapse, "center-column"); //$NON-NLS-1$
		var collapseAnchor = document.createElement("a"); //$NON-NLS-1$
		var collapseImage = document.createElement("img"); //$NON-NLS-1$
		collapseAnchor.appendChild(collapseImage);
		dojo.attr(collapseAnchor, "href", "#"); //$NON-NLS-1$ //$NON-NLS-2$
		collapseImage.src = this._sectionTwistieCollapseIconSrc;
		collapse.appendChild(collapseAnchor);
		
		name.innerHTML = item.name;
		startTime.innerHTML = this._formatTimeString(item.startTime);
		runningTime.innerHTML = this._getDurationTimeString(item.runningTime);
		
		row.appendChild(collapse);
		row.appendChild(name);
		row.appendChild(startTime);
		row.appendChild(runningTime);
		this.runningJobTable.tbody.appendChild(row);
		
		resourceRow = document.createElement("tr"); //$NON-NLS-1$
		dojo.addClass(resourceRow, "hide"); //$NON-NLS-1$
		resourceTd =  document.createElement("td"); //$NON-NLS-1$
		dojo.attr(resourceTd, "colspan", "4"); //$NON-NLS-1$ //$NON-NLS-2$
		resourceRow.appendChild(resourceTd);
		this.runningJobTable.tbody.appendChild(resourceRow);
		
		var resourceTable = new jazz.ui.StyledTable({
			columns : [ {
				label : this.messages.resourceGroup,
				sortable : false,
				width : "15%" //$NON-NLS-1$
			}, {
				label : this.messages.startTime,
				sortable : false,
				width : "20%" //$NON-NLS-1$
			}, {
				label : this.messages.endTime,
				sortable : false,
				width : "20%" //$NON-NLS-1$
			}, {
				label : this.messages.runningTime,
				sortable : false,
				width : "20%" //$NON-NLS-1$
			}, {
				label : this.messages.processedResources,
				sortable : false,
				width : "5%" //$NON-NLS-1$
			}, {
				label : this.messages.deliveredRows,
				sortable : false,
				width : "5%" //$NON-NLS-1$
			}, {
				label : this.messages.failedResources,
				sortable : false,
				width : "5%" //$NON-NLS-1$
			}, {
				label : this.messages.log,
				sortable : false,
				width : "5%" //$NON-NLS-1$
			}],
			sortBy : 1,
			sortByAscending : false,
			handleSort : function() {
			},
			trim : jazz.ui.styledTable.TrimStyles.BLUE
		});
		resourceTable.domNode.style.width = "100%"; //$NON-NLS-1$
		resourceTd.appendChild(resourceTable.domNode);
		
		this.connect(collapseAnchor, "onclick", function(e) { //$NON-NLS-1$
			dojo.stopEvent(e);
			if (dojo.hasClass(resourceRow, "hide")) { //$NON-NLS-1$
				collapseImage.src = this._sectionTwistieOpenIconSrc;
			} else {
				collapseImage.src = this._sectionTwistieCollapseIconSrc;
			}
			dojo.toggleClass(resourceRow, "hide"); //$NON-NLS-1$
		});
		
		for (var i = 0; i < item.resourceGroups.length; i++) {
			var isPending = (item.resourceGroups[i].startTime === 0);
			row = document.createElement("tr"); //$NON-NLS-1$
			name = document.createElement("td"); //$NON-NLS-1$
			startTime = document.createElement("td"); //$NON-NLS-1$
			endTime = document.createElement("td"); //$NON-NLS-1$
			runningTime = document.createElement("td"); //$NON-NLS-1$
			processedResources = document.createElement("td"); //$NON-NLS-1$
			deliveredRows = document.createElement("td"); //$NON-NLS-1$
			failedResources = document.createElement("td"); //$NON-NLS-1$
			log = document.createElement("td"); //$NON-NLS-1$
			
			name.innerHTML = item.resourceGroups[i].name;
			startTime.innerHTML = isPending ? this.messages.pending : this._formatTimeString(item.resourceGroups[i].startTime);
			if (isPending) {
				endTime.innerHTML = this.messages.emptyColumn;
			} else if (item.resourceGroups[i].endTime === 0) {
				endTime.innerHTML = this.messages.running;
			} else {
				endTime.innerHTML = this._formatTimeString(item.resourceGroups[i].endTime);
			}
			runningTime.innerHTML = isPending ? this.messages.emptyColumn : this._getDurationTimeString(item.resourceGroups[i].runningTime);
			processedResources.innerHTML = item.resourceGroups[i].processedResources;
			deliveredRows.innerHTML = item.resourceGroups[i].deliveredRows;
			failedResources.innerHTML = item.resourceGroups[i].failedResources;
			log.innerHTML = dojo.string.substitute(LOG_LINK, [item.id, item.resourceGroups[i].id]);
			
			row.appendChild(name);
			row.appendChild(startTime);
			row.appendChild(endTime);
			row.appendChild(runningTime);
			row.appendChild(processedResources);
			row.appendChild(deliveredRows);
			row.appendChild(failedResources);
			row.appendChild(log);
			
			resourceTable.tbody.appendChild(row);
		}
	},
	
	addHistoryJobRow: function(item) {
		var row, resourceRow, resourceTd, collapse, name, status, startTime, endTime, runningTime, processedResources, deliveredRows, failedResources, log;
		row = document.createElement("tr"); //$NON-NLS-1$
		collapse = document.createElement("td"); //$NON-NLS-1$
		name = document.createElement("td"); //$NON-NLS-1$
		status = document.createElement("td"); //$NON-NLS-1$
		startTime = document.createElement("td"); //$NON-NLS-1$
		endTime = document.createElement("td"); //$NON-NLS-1$
		runningTime = document.createElement("td"); //$NON-NLS-1$
		
		dojo.addClass(collapse, "center-column"); //$NON-NLS-1$
		var collapseAnchor = document.createElement("a"); //$NON-NLS-1$
		var collapseImage = document.createElement("img"); //$NON-NLS-1$
		collapseAnchor.appendChild(collapseImage);
		dojo.attr(collapseAnchor, "href", "#"); //$NON-NLS-1$ //$NON-NLS-2$
		collapseImage.src = this._sectionTwistieCollapseIconSrc;
		collapse.appendChild(collapseAnchor);

		name.innerHTML = item.name;
		if("Success"==item.status){
			status.innerHTML=this.messages.statusSuccess;
		}else{
			status.innerHTML=this.messages.statusFailed;
		}
		startTime.innerHTML = this._formatTimeString(item.startTime);
		endTime.innerHTML = this._formatTimeString(item.endTime);
		runningTime.innerHTML = this._getDurationTimeString(item.runningTime);
		
		row.appendChild(collapse);
		row.appendChild(name);
		row.appendChild(status);
		row.appendChild(startTime);
		row.appendChild(endTime);
		row.appendChild(runningTime);
		
		this.historyJobTable.tbody.appendChild(row);
		
		resourceRow = document.createElement("tr"); //$NON-NLS-1$
		dojo.addClass(resourceRow, "hide"); //$NON-NLS-1$
		resourceTd =  document.createElement("td"); //$NON-NLS-1$
		dojo.attr(resourceTd, "colspan", "6"); //$NON-NLS-1$ //$NON-NLS-2$
		resourceRow.appendChild(resourceTd);
		this.historyJobTable.tbody.appendChild(resourceRow);
		
		var resourceTable = new jazz.ui.StyledTable({
			columns : [ {
				label : this.messages.resourceGroup,
				sortable : false,
				width : "10%" //$NON-NLS-1$
			}, {
				label : this.messages.status,
				sortable : false,
				width : "5%" //$NON-NLS-1$
			}, {
				label : this.messages.startTime,
				sortable : false,
				width : "20%" //$NON-NLS-1$
			} , {
				label : this.messages.endTime,
				sortable : false,
				width : "20%" //$NON-NLS-1$
			} , {
				label : this.messages.runningTime,
				sortable : false,
				width : "20%" //$NON-NLS-1$
			}, {
				label : this.messages.processedResources,
				sortable : false,
				width : "5%" //$NON-NLS-1$
			}, {
				label : this.messages.deliveredRows,
				sortable : false,
				width : "5%" //$NON-NLS-1$
			}, {
				label : this.messages.failedResources,
				sortable : false,
				width : "5%" //$NON-NLS-1$
			}, {
				label : this.messages.log,
				sortable : false,
				width : "5%" //$NON-NLS-1$
			}],
			sortBy : 1,
			sortByAscending : false,
			handleSort : function() {
			},
			trim : jazz.ui.styledTable.TrimStyles.BLUE
		});
		resourceTable.domNode.style.width = "100%"; //$NON-NLS-1$
		resourceTd.appendChild(resourceTable.domNode);
		
		this.connect(collapseAnchor, "onclick", function(e) { //$NON-NLS-1$
			dojo.stopEvent(e);
			if (dojo.hasClass(resourceRow, "hide")) { //$NON-NLS-1$
				collapseImage.src = this._sectionTwistieOpenIconSrc;
			} else {
				collapseImage.src = this._sectionTwistieCollapseIconSrc;
			}
			dojo.toggleClass(resourceRow, "hide"); //$NON-NLS-1$
		});
		
		for (var i = 0; i < item.resourceGroups.length; i++) {
			row = document.createElement("tr"); //$NON-NLS-1$
			name = document.createElement("td"); //$NON-NLS-1$
			status = document.createElement("td"); //$NON-NLS-1$
			startTime = document.createElement("td"); //$NON-NLS-1$
			endTime = document.createElement("td"); //$NON-NLS-1$
			runningTime = document.createElement("td"); //$NON-NLS-1$
			processedResources = document.createElement("td"); //$NON-NLS-1$
			deliveredRows = document.createElement("td"); //$NON-NLS-1$
			failedResources = document.createElement("td"); //$NON-NLS-1$
			log = document.createElement("td"); //$NON-NLS-1$
			
			name.innerHTML = item.resourceGroups[i].name;
			if("Success"==item.resourceGroups[i].status){
				status.innerHTML=this.messages.statusSuccess;
			}else{
				status.innerHTML=this.messages.statusFailed;
			}
			startTime.innerHTML = this._formatTimeString(item.resourceGroups[i].startTime);
			endTime.innerHTML = this._formatTimeString(item.resourceGroups[i].endTime);
			runningTime.innerHTML = this._getDurationTimeString(item.resourceGroups[i].runningTime);
			processedResources.innerHTML = item.resourceGroups[i].processedResources;
			deliveredRows.innerHTML = item.resourceGroups[i].deliveredRows;
			failedResources.innerHTML = item.resourceGroups[i].failedResources;
			log.innerHTML =  dojo.string.substitute(LOG_LINK, [item.id, item.resourceGroups[i].id]);
			
			row.appendChild(name);
			row.appendChild(status);
			row.appendChild(startTime);
			row.appendChild(endTime);
			row.appendChild(runningTime);
			row.appendChild(processedResources);
			row.appendChild(deliveredRows);
			row.appendChild(failedResources);
			row.appendChild(log);
			
			resourceTable.tbody.appendChild(row);
		}
	},
	
	_isJobRunning: function(job) {
				
		if ( job.status == "Running") {
			return true;
		}
		if ( job.resourceGroups ) {
			for (var i = 0; i < job.resourceGroups.length; i++ ) {
				if ( job.resourceGroups[i].status == "Running") {
					return true;
				}
			}
		}
		return false;
	},
		
	_initializeContent: function () {
		DataCollectionClient.getRunningJobs({
			load: dojo.hitch(this, function(data, ioArgs) {
				if (data.length == 0) {
					this._addEmptyRow(this.runningJobTable, 4);
				} else {
					//add queuing jobs first
					for (var i = 0; i < data.length; i++) {
						if ( this._isJobRunning(data[i]) ) {
							if ( !data[i].startTime ) {
								this.addQueuingJobRow(data[i]);
							} 
						}
					}
					//add running jobs		
					for (var i = 0; i < data.length; i++) {
					   if ( this._isJobRunning(data[i]) ) {
							if ( data[i].startTime ) {
								this.addRunningJobRow(data[i]);
							} 
						}
					}
				}
				
				this.onLoadingComplete();
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				this.displayDetailedErrorMessage(this.messages.getRunningJobsFailed, error);
			})
		});
		
		DataCollectionClient.getHistoryJobs({
			pageSize: this.numberOfJobsInput.inputNode.value,
			load: dojo.hitch(this, function(data, ioArgs) {
				if (data.length == 0) {
					this._addEmptyRow(this.historyJobTable, 6);
				} else {
					for (var i = 0; i < data.length; i++) {
						this.addHistoryJobRow(data[i]);
					}
				}
				this.onLoadingComplete();
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				this.displayDetailedErrorMessage(this.messages.getHistoryJobsFailed, error);
			})
		});
	
	},
	
	_addEmptyRow : function(table, colspan) {
		var emptyRow = document.createElement("tr"); //$NON-NLS-1$
		emptyTd =  document.createElement("td"); //$NON-NLS-1$
		dojo.attr(emptyTd, "colspan", colspan); //$NON-NLS-1$
		emptyRow.appendChild(emptyTd);
		emptyTd.innerHTML = this.messages.noRecord;
		table.tbody.appendChild(emptyRow);
	},
	
	onLoadingComplete: function () {
		this.hideMessage();
	},
	
	_deleteStatus: function(e) {
		e.preventDefault();
		var menu = new dijit.Menu();
		dojo.safeMixin(menu, {
			onOpen : function() {
				this.inherited(arguments); 
				this.focus(); 
			},
			cancel : function cancel() {
				this.onCancel();
			},
			onCancel : function onCancel() {
			},
			onBlur : function onBlur() {
				this.cancel();
			} 
		});
		menu.addChild(new dijit.MenuItem({
       		label: this.messages.deleteAll,
       		onClick: dojo.hitch(this, function(){
       			new DeleteStatusesDialog({
	       			getContent: dojo.hitch(this, function() {
	       				var node = document.createElement("div"); //$NON-NLS-1$
	       				node.appendChild(document.createElement("br")); //$NON-NLS-1$
	       				node.appendChild(document.createTextNode(this.messages.deleteAllMessage));
	       				return node;
	       			}),
	       			onOk: dojo.hitch(this, function() {
						DataCollectionClient.deleteHistoryJobs({
							load: dojo.hitch(this, function(data, ioArgs) {
								this.refresh();
							}),
							error: dojo.hitch(this, function(error, ioArgs) {
								this.displayDetailedErrorMessage(this.messages.deleteJobStatusesFailed, error);
							})
						});
					})
					
				});
       		})
    	}));
    	menu.addChild(new dijit.MenuItem({
       		label: this.messages.deleteBefore,
       		onClick: dojo.hitch(this, function(){
       			var before = new dijit.Calendar({
       				value: new Date(), 
       				isDisabledDate: function(d){
            			var d = new Date(d); d.setHours(0, 0, 0, 0);
            			var today = new Date(); today.setHours(0, 0, 0, 0);
            			return dojo.date.difference(d, today) < 0;
        		}});
       			new DeleteStatusesDialog({
	       			getContent: dojo.hitch(this, function() {
	       				var node = document.createElement("div"); //$NON-NLS-1$
	       				node.appendChild(document.createElement("br")); //$NON-NLS-1$
	       				node.appendChild(document.createTextNode(this.messages.deleteBeforeMessage));
	       				node.appendChild(document.createElement("br")); //$NON-NLS-1$
	       				node.appendChild(document.createElement("br")); //$NON-NLS-1$
	       				node.appendChild(before.domNode);
	       				return node;
	       			}),
	       			onOk: dojo.hitch(this, function() {
	       			    var beforeDate = before.get('value'); 
	       			    if(dojo.i18n.normalizeLocale() == "en-us") {
	       			    	beforeDate = dojo.date.locale.format(beforeDate, {datePattern : "yyyy-MM-dd", selector :'date'}); //$NON-NLS-1$
				  	     }else{
				  	    	beforeDate = dojo.date.locale.format(beforeDate, {selector :'date'}); //$NON-NLS-1$
				  	     }
  					  		DataCollectionClient.deleteHistoryJobs({
						    before : beforeDate,
							load: dojo.hitch(this, function(data, ioArgs) {
								this.refresh();
							}),
							error: dojo.hitch(this, function(error, ioArgs) {
								this.displayDetailedErrorMessage(this.messages.deleteJobStatusesFailed, error);
							})
						});
					})
					
				});
       		})
    	}));
    	menu.addChild(new dijit.MenuItem({
       		label: this.messages.deleteAfter,
       		onClick: dojo.hitch(this, function(){
       			var after = new dijit.Calendar({
       				value: new Date(), 
       				isDisabledDate: function(d){
            			var d = new Date(d); d.setHours(0, 0, 0, 0);
            			var today = new Date(); today.setHours(0, 0, 0, 0);
            			return dojo.date.difference(d, today) < 0;
        		}});
       			new DeleteStatusesDialog({
	       			getContent: dojo.hitch(this, function() {
	       				var node = document.createElement("div"); //$NON-NLS-1$
	       				node.appendChild(document.createElement("br")); //$NON-NLS-1$
	       				node.appendChild(document.createTextNode(this.messages.deleteAfterMessage));
	       				node.appendChild(document.createElement("br")); //$NON-NLS-1$
	       				node.appendChild(document.createElement("br")); //$NON-NLS-1$
	       				node.appendChild(after.domNode);
				   		return node;
	       			}),
	       			onOk: dojo.hitch(this, function() {
	       			    var afterDate = after.get('value'); 
	       			 	if(dojo.i18n.normalizeLocale() == "en-us") {
	       			    	
	       			 		afterDate = dojo.date.locale.format(afterDate, {datePattern : "yyyy-MM-dd", selector :'date'}); //$NON-NLS-1$
				  	    }else{
				  	    	
				  	    	afterDate = dojo.date.locale.format(afterDate, 	{selector :'date'}); //$NON-NLS-1$
				  	    }
  					  
						DataCollectionClient.deleteHistoryJobs({
						    after: afterDate,
							load: dojo.hitch(this, function(data, ioArgs) {
								this.refresh();
							}),
							error: dojo.hitch(this, function(error, ioArgs) {
								this.displayDetailedErrorMessage(this.messages.deleteJobStatusesFailed, error);
							})
						});
					})
					
				});
       		})
    	}));
    	menu.addChild(new dijit.MenuItem({
       		label: this.messages.deleteBetween,
       		onClick: dojo.hitch(this, function(){
       		    var start = new dijit.Calendar({
       				value: new Date(), 
       				isDisabledDate: function(d){
            			var d = new Date(d); d.setHours(0, 0, 0, 0);
            			var today = new Date(); today.setHours(0, 0, 0, 0);
            			return dojo.date.difference(d, today) < 0;
        		}});
				var end = new dijit.Calendar({
       				value: new Date(), 
       				isDisabledDate: function(d){
            			var d = new Date(d); d.setHours(0, 0, 0, 0);
            			var today = new Date(); today.setHours(0, 0, 0, 0);
            			return dojo.date.difference(d, today) < 0;
        		}});
       			new DeleteStatusesDialog({
	       			getContent: dojo.hitch(this, function() {
	       				var node = document.createElement("div"); //$NON-NLS-1$
	       				node.appendChild(document.createElement("br")); //$NON-NLS-1$
	       				node.appendChild(document.createTextNode(this.messages.deleteBetweenMessage));
	       				node.appendChild(document.createElement("br")); //$NON-NLS-1$
	       				node.appendChild(document.createElement("br")); //$NON-NLS-1$
	       				var tableDiv = document.createElement("table"); //$NON-NLS-1$
	       				node.appendChild(tableDiv);
	       				var tr1 = document.createElement("tr"); //$NON-NLS-1$
	       				tableDiv.appendChild(tr1);
	       				var tr2 = document.createElement("tr"); //$NON-NLS-1$
	       				tableDiv.appendChild(tr2);
	       				
	       				var th1 = document.createElement("th"); //$NON-NLS-1$
	       				tr1.appendChild(th1);
	       				var th1a = document.createElement("th"); //$NON-NLS-1$
	       				tr1.appendChild(th1a);
	       				th1a.style.width="20px";
	       				var th2 = document.createElement("th"); //$NON-NLS-1$
	       				tr1.appendChild(th2);
	       				th1.appendChild(document.createTextNode(this.messages.startDate));
	       				th2.appendChild(document.createTextNode(this.messages.endDate));
	       				
	       				var td1 = document.createElement("td"); //$NON-NLS-1$
	       				tr2.appendChild(td1);
	       				var td1a = document.createElement("td"); //$NON-NLS-1$
	       				tr2.appendChild(td1a);
	       				var td2 = document.createElement("td"); //$NON-NLS-1$
	       				tr2.appendChild(td2);
	       				td1.appendChild(start.domNode);
	       				td2.appendChild(end.domNode);
	       				return node;
	       			}),
	       			onOk: dojo.hitch(this, function() {
	       				var beforeDate = end.get('value'); 
	       		   		var afterDate = start.get('value'); 
		       		   	if(dojo.i18n.normalizeLocale() == "en-us") {
	       			    	beforeDate = dojo.date.locale.format(beforeDate, {datePattern : "yyyy-MM-dd", selector :'date'}); //$NON-NLS-1$
	       			    	afterDate = dojo.date.locale.format(afterDate, 	{datePattern : "yyyy-MM-dd", selector :'date'}); //$NON-NLS-1$
				  	    }else{
				  	    	beforeDate = dojo.date.locale.format(beforeDate, {selector :'date'}); //$NON-NLS-1$
				  	    	afterDate = dojo.date.locale.format(afterDate, 	{selector :'date'}); //$NON-NLS-1$
				  	    }
  					  			  	
						DataCollectionClient.deleteHistoryJobs({
						    before: beforeDate,
						    after: afterDate,						     
							load: dojo.hitch(this, function(data, ioArgs) {
								this.refresh();
							}),
							error: dojo.hitch(this, function(error, ioArgs) {
								this.displayDetailedErrorMessage(this.messages.deleteJobStatusesFailed, error);
							})
						});
					})
					
				});
       		})
    	}));
    	dijit.popup.open({
			parent: this,
			popup: menu,
			around: this._deleteStatusLink,
			orient: {'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'},
		  	onExecute: dojo.hitch(this, function(){ 
		  	    dijit.popup.close(menu);
			}),
		 	onCancel: function(){
		 		dijit.popup.close(menu);
			},
			onClose: function(){
			}					    
		});
    	
  	},
	
	refresh: function() {
		this.hideMessage();
		this.displayInfoMessage(this.messages.loading);
		this.runningJobTable.clearBody();
		this.historyJobTable.clearBody();
		this._runQueryButton.disabled = true;
		this._initializeContent();		
	},
	
	_getDurationTimeString: function (durationTime) {
		durationTime = parseInt(durationTime / 1000);
		var hh = parseInt(durationTime / 3600);
		durationTime %= 3600; 
		var mm = parseInt(durationTime / 60);
		var ss = durationTime % 60;
		
		return dojo.string.substitute(this.messages.durationTemplate, [hh, mm, ss]);
	},
	
	_formatTimeString: function (time) {
		// format YYYY-MM-dd hh:mm:ss
			
		
		if(dojo.i18n.normalizeLocale() == "en-us") {
			return dojo.date.locale.format(new Date(time), {datePattern : "yyyy-MM-dd HH:mm:ss", selector :'date'}); //$NON-NLS-1$//$NON-NLS-2$
		}else{
			return dojo.date.locale.format(new Date(time), {formatLength: 'medium'}); //$NON-NLS-1$
		}
	},
	
	_getDoubleDig: function (d) {
	    
	    return d < 10 ? '0' + d : d; 
	},  
	
	_createHistoryInput : function() {
		this.numberOfJobsInput = new ValidatingTextBox();
		this.numberOfJobsInput.inputNode.value = DEFAULT_HISTORY_SIZE;
		this.numberOfJobsInput.setErrorMessage(this.messages.numberOfJobsValidationError);
		var self = this;
		this.numberOfJobsInput.validate = dojo.hitch(this, function(value) {
			var valid = value - 0 == value && value.length > 0
					&& value <= 1000 && value >= 1 && value % 1 == 0;
			if (valid) {
				self._runQueryButton.disabled = false;
			} else {
				self._runQueryButton.disabled = true;
			}
			return valid;
		});
		dojo.addClass(this.numberOfJobsInput.inputNode, "smallTextBox"); //$NON-NLS-1$
		this._numberOfJobsContainer.appendChild(this.numberOfJobsInput.domNode);
	},
	
	uninitialize: function(){
		this.inherited(arguments);
	}
});

})();