/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.rational.datacollection.web.ui.internal.editors.ScheduleEditor"); //$NON-NLS-1$
	
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.AbstractEditor"); //$NON-NLS-1$
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.IntervalScheduleWidget"); //$NON-NLS-1$
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.DayTimesScheduleWidget"); //$NON-NLS-1$
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants"); //$NON-NLS-1$
	dojo.require("dojox.html.entities"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "ScheduleEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var AbstractEditor = com.ibm.rational.datacollection.web.ui.internal.editors.AbstractEditor;
	var DataCollectionConstants = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants;
	var IntervalScheduleWidget = com.ibm.rational.datacollection.web.ui.internal.editors.IntervalScheduleWidget;
	var DayTimesScheduleWidget = com.ibm.rational.datacollection.web.ui.internal.editors.DayTimesScheduleWidget;
		
	dojo.declare("com.ibm.rational.datacollection.web.ui.internal.editors.ScheduleEditor", [AbstractEditor], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.rational.datacollection.web", "ui/internal/editors/templates/ScheduleEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		schedule: null,
	
		constructor: function(schedule) {
			this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "ScheduleEditorMessages");//$NON-NLS-1$ //$NON-NLS-2$
			this.schedule = schedule;
			this._widgets = {
				"http://jazz.net/ns/dcs/schedule#IntervalRecurringSchedule": new IntervalScheduleWidget(), //$NON-NLS-1$
				"http://jazz.net/ns/dcs/schedule#DayTimesRecurringSchedule": new DayTimesScheduleWidget(), //$NON-NLS-1$
			};
		},
		
		postCreate: function() {
			this.inherited(arguments);
			
			if (this.schedule && this.schedule.identifier) {
				for (var i = 0; i < this._selectScheduleTypeCombo.options.length; i++) {
					if (this._selectScheduleTypeCombo.options[i].value === this.schedule.identifier) {
						this._selectScheduleTypeCombo.selectedIndex = i;
						this._widgets[this._selectScheduleTypeCombo.options[i].value].init(this.schedule);
						break;
					}
				}
			}
			var scheduleOption = this._selectScheduleTypeCombo.options[this._selectScheduleTypeCombo.selectedIndex];
			this._title.innerHTML = dojox.html.entities.encode(scheduleOption.text);
			this._contentWidget.appendChild(this._widgets[scheduleOption.value].domNode);
		},
		
		validate: function() {
			var scheduleOption = this._selectScheduleTypeCombo.options[this._selectScheduleTypeCombo.selectedIndex];
			var scheduleWidget = this._widgets[scheduleOption.value];
			if (scheduleWidget) {
				var error = scheduleWidget.validate ? scheduleWidget.validate() : undefined;
				if (error) {
					this.displayErrorMessage(error);
					return error;
				}
			}
		},
		
		getSchedule: function(originalSchedule) {
			var scheduleOption = this._selectScheduleTypeCombo.options[this._selectScheduleTypeCombo.selectedIndex];
			var scheduleWidget = this._widgets[scheduleOption.value];
			if (scheduleWidget) {
				return scheduleWidget.getSchedule(originalSchedule);
			}
		},
		
		_onScheduleTypeChanged: function(e) {
			var scheduleOption = this._selectScheduleTypeCombo.options[this._selectScheduleTypeCombo.selectedIndex];
			var newWidget = this._widgets[scheduleOption.value];
			if (newWidget.domNode !== this._contentWidget.children[0]) {
				if ( this._contentWidget.children[0] ) {
					this._contentWidget.removeChild(this._contentWidget.children[0])
				}
				this._title.innerHTML = dojox.html.entities.encode(scheduleOption.text);
				this._contentWidget.appendChild(newWidget.domNode);
			}
		}
				
	});
	
})();
