/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.rational.datacollection.web.ui.internal.editors.StringParameter"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.AbstractParameter"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "StringParameterMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var AbstractParameter = com.ibm.rational.datacollection.web.ui.internal.editors.AbstractParameter;
dojo.declare("com.ibm.rational.datacollection.web.ui.internal.editors.StringParameter", [AbstractParameter], { //$NON-NLS-1$

	templatePath: dojo.moduleUrl("com.ibm.rational.datacollection.web", "ui/internal/editors/templates/StringParameter.html"), //$NON-NLS-2$ //$NON-NLS-1$
	// widget initialization
	
	constructor: function() {
		this._errIconScr = dojo.moduleUrl("com.ibm.rational.datacollection.web", "graphics/icons/inline-error.gif"); //$NON-NLS-2$ //$NON-NLS-1$
		this._delIconScr = dojo.moduleUrl("com.ibm.rational.datacollection.web", "graphics/icons/remove.gif"); //$NON-NLS-2$ //$NON-NLS-1$
		this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "StringParameterMessages"); //$NON-NLS-2$ //$NON-NLS-1$
		this.types = [];
	},

	postCreate: function () {
		this.connect(this.editName, "onchange", function(e) { //$NON-NLS-1$
			this.name = this.editName.value;
			this.viewName.innerHTML = this.editName.value;
			this.controller.setChange();
			this.onNameChange();
		});
		
		this.connect(this.editValue, "onchange", function(e) { //$NON-NLS-1$
			this.value = this.editValue.value;
			this.viewValue.innerHTML = this.editValue.value;
			this.controller.setChange();
			this.onValueChange();
		});
		
		this.connect(this.editType, "onchange", function(e) { //$NON-NLS-1$
			this.type = this.editType.value;
			this.viewType.innerHTML = this.editType.value;
			this.controller.setChange();
			this.onTypeChange();
		});
		
		this._initializeContent();
	},

	_initializeContent: function () {
		this.viewName.innerHTML = this.name;
		this.editName.value = this.name;
		this.viewValue.innerHTML = this.value;
		this.editValue.value = this.value;
		
		for(var i = 0; i < this.types.length; i++){
			var newOption = new Option(this.types[i].name, this.types[i].value);
			this.editType.options[i] = newOption;
		}
		this.editType.value = this.type;
		
	},
	
	toggleEditMode: function (edit) {
		if (edit) {
			dojo.style(this.editValue, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.viewValue, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.editName, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.viewName, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.editType, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.viewType, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
		} else {
			dojo.style(this.editValue, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.viewValue, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.editName, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.viewName, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.editType, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
			dojo.style(this.viewType, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
		}
	},
	
	_onDeleteClick : function(evt) {
		dojo.stopEvent(evt);
		this.controller.removeItem(this);
	},
	
	_handleKeyUp : function(e) {
		if (e.target && e.target.tagName === "INPUT") { //$NON-NLS-1$
			if (this.onKeyUpTimout) {
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, function() {
				this._checkValidity(e.target);
			});
			var timeout = 500;
	
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	_checkValidity : function(target) {
		var value = target.value;
		if (target === this.editName) {
			if (this.validate(value) === true) {
				this._markValidName();
			} else {
				this._markInvalidName();
			}
		} else if (target === this.editValue) {
			if (this.validate(value) === true) {
				this._markValidValue();
			} else {
				this._markInvalidValue();
			}
		}
	},
	
	validate : function(value) {
		if (value === "") { //$NON-NLS-1$
			return false;
		}
		return true;
	},
	
	isValid : function(){
		return !this._invalidName && !this._invalidValue;
	},
	
	_markValidName : function() {
		while (this._invalidNameMsg.childNodes.length > 0) {
			this._invalidNameMsg.removeChild(this._invalidNameMsg.firstChild);
		}
		this._nameErrIcon.style.display = ""; //$NON-NLS-1$
		this._nameErrIcon.style.visibility = "hidden"; //$NON-NLS-1$
		this._invalidName = false;
	},
	
	_markInvalidName : function() {
		this._invalidName = true;
		while (this._invalidNameMsg.childNodes.length > 0) {
			this._invalidNameMsg.removeChild(this._invalidNameMsg.firstChild);
		}
	
		if (this.messages.nameErrorMessage) {
			this._invalidNameMsg.appendChild(document.createTextNode(this.messages.nameErrorMessage));
		}
		this._nameErrIcon.style.visibility = "visible"; //$NON-NLS-1$
	},
	
	_markValidValue : function() {
		while (this._invalidValueMsg.childNodes.length > 0) {
			this._invalidValueMsg.removeChild(this._invalidValueMsg.firstChild);
		}
		this._valueErrIcon.style.display = ""; //$NON-NLS-1$
		this._valueErrIcon.style.visibility = "hidden"; //$NON-NLS-1$
		this._invalidValue = false;
	},
	
	_markInvalidValue : function() {
		this._invalidValue = true;
		while (this._invalidValueMsg.childNodes.length > 0) {
			this._invalidValueMsg.removeChild(this._invalidValueMsg.firstChild);
		}
	
		if (this.messages.valueErrorMessage) {
			this._invalidValueMsg.appendChild(document.createTextNode(this.messages.valueErrorMessage));
		}
		this._valueErrIcon.style.visibility = "visible"; //$NON-NLS-1$
	},
	
	onTypeChange : function() {
	},
	
	onNameChange : function() {
	},
	
	onValueChange : function() {
	},
	
	uninitialize: function(){
		this.inherited(arguments);
	}
});

})();