/*
 Licensed Materials - Property of IBM - some unique copyrights (i.e., the ability to copy,  modify,
 distribute) may be licensed in certain circumstances (e.g., sample/template code  intended to be used by
 customers to build upon and distribute as their own products).

 (c) Copyright IBM Corporation 2007, 2015. All Rights Reserved.

 U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP
 Schedule Contract with IBM Corp.
*/
(function () {
dojo.provide("com.ibm.rdm.web.artifactgrid.ArtifactDisplay");

dojo.require("com.ibm.rdm.web.project.DashboardConstants");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.model.types.TypesManager");
dojo.require("com.ibm.rdm.web.project.EditManager");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.mail.mailutils");
dojo.require("com.ibm.rdm.web.common.UserHover");
dojo.require("com.ibm.rdm.web.server.client.utils.UserCache");
dojo.require("com.ibm.rdm.web.common.AttributeDisplayHelper");
dojo.require("com.ibm.rdm.web.common.DateTimeHelper");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.editor.InlineGridEditor");
dojo.require("com.ibm.rdm.web.ext.ArtifactExtension");
dojo.require("com.ibm.rdm.web.artifactgrid.ArtifactConstants");
dojo.require("com.ibm.rdm.web.artifactgrid.ContentCellWidget");
dojo.require("com.ibm.rdm.web.common.ArtifactIcons");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("dojox.layout.ResizeHandle");
dojo.require("com.ibm.rdm.web.common.ReuseHover");
dojo.require("com.ibm.rdm.web.artifactgrid.CommentHover");
dojo.require("com.ibm.rdm.web.suspicion.SuspectLinksUtil");
dojo.require("com.ibm.rdm.web.suspicion.SuspicionStatus");
dojo.require("dojo.regexp");
dojo.require("com.ibm.rdm.web.util.bidiutils"); //bidi
//ACGC_Bidi_start date support for non-gregorian calendars
dojo.require("dojox.date.islamic.Date");
dojo.require("dojox.date.islamic.locale");
//ACGC_Bidi_End
dojo.require("com.ibm.rdm.web.editor.EditorConstants");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("com.ibm.rdm.web.locks.LockManager");
dojo.require("com.ibm.rdm.web.viewer.EmbeddedResourceLoader");


dojo.requireLocalization("com.ibm.rdm.web.artifactgrid", "ArtifactDisplay");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");

var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var EditManager = com.ibm.rdm.web.project.EditManager;
var DomUtils = com.ibm.rdm.web.util.domutils;
var MailUtils = com.ibm.rdm.web.mail.mailutils;
var UserCache = com.ibm.rdm.web.server.client.utils.UserCache;
var AttributeDisplayHelper = com.ibm.rdm.web.common.AttributeDisplayHelper;
var DateTimeHelper = com.ibm.rdm.web.common.DateTimeHelper;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var ArtifactExtension = com.ibm.rdm.web.ext.ArtifactExtension;
var ReuseHover = com.ibm.rdm.web.common.ReuseHover;
var CommentHover = com.ibm.rdm.web.artifactgrid.CommentHover;
var BidiUtils = com.ibm.rdm.web.util.bidiutils;  //bidi
var SuspectLinksUtil = com.ibm.rdm.web.suspicion.SuspectLinksUtil;
var LockManager = com.ibm.rdm.web.locks.LockManager;
var TypesManager = com.ibm.rdm.web.model.types.TypesManager;
var EmbeddedResourceLoader = com.ibm.rdm.web.viewer.EmbeddedResourceLoader;
var ArtifactConstants = com.ibm.rdm.web.artifactgrid.ArtifactConstants;


//Variable to hold Rowtype for a row built from resource info records. Loaded in Constructor
var RESOURCE_ROW = ArtifactConstants.RESOURCE_ROW;
var MVA_LIMIT = 9;


/*
 * Provides interaction with the grid to display requirements data
 */
dojo.declare(
		"com.ibm.rdm.web.artifactgrid.ArtifactDisplay", null,
{
	//Constants identifying the column category
	LINK_COL: ArtifactConstants.LINK_COL_TYPE,
	SYSTEM_COL: ArtifactConstants.SYSTEM_COL_TYPE,
	ATTRIBUTE_COL: ArtifactConstants.ATTRIBUTE_COL_TYPE,
	
	//The project URI for the context
	_projectURI: null,
	
		/* Maps the columns we have by columnId, to the type and URI/Key we have in them:
		<string> colId -> {colId: <the colId, which can vary between sessions>,
							type: <link|definition|system>,
							 key: <the uri of the definition or link type, or the col_id of a 'system' column>}. */
	_idToColumnDetailsMapping: null,
	/* Maps the link columns we have by link uri
		<string> linkTypeURI (with _BI, _OBJ, _SUB suffix) ->
						 {colId: <the colId, which can vary between sessions>,
						   type: <link|definition|system>,
							key: <the uri of the definition or link type, or the col_id of a 'system' column>}. */
	_linkUriToColumnDetailMapping: null,
	/* Maps the attribute columns we have by attribute definition uri
		<string> attrDefURI  ->
						 {colId: <the colId, which can vary between sessions>,
						   type: <link|definition|system>,
							key: <the uri of the definition or link type, or the col_id of a 'system' column>}. */
	_attributeUriToColumnDetailMapping: null,
	
	//A callback to allow calls to be deferred until type system is loaded.
	_addTypeSystemDeferredFunction: null,
	_connectHandles: [],
	_toDestroy: [],
	
	// Find/Replace variables
	_ptMarkedPieces: null,
	_ptCurrent: null,

	// hash containing callbacks that editors can make to their containing grid.
	_editorCallbacks: null,
	
	//A mechanism to control individual repaints, reloads etc on a one-off basis
	_loadingContext: null,
	
	_blankGif: dojo.moduleUrl("dojo", "resources/blank.gif").toString(),
	
	constructor: function(context) {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.artifactgrid", "ArtifactDisplay");
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
		this._connectHandles = [];
		this._toDestroy = [];
		this._projectURI = context.projectURI;
		this._baselineURI = context.baselineURI;
		this._grid = context.grid;
		this._addTypeSystemDeferredFunction = context.typeSystemReady;
		this._typeCache = context.typeCache;
		this._dataCache = context.dataCache;
		this._permissionsCache = context.permissionsCache;
		this._viewType = context.viewType;
		this._moduleStructure = context.moduleStructure;
		this._commentingInterface = context.commentingInterface;
		this._isInModuleBaseline = false;
		this._isInReviewMode = context.isInReviewMode;
		this._isInHistoryMode = context.isInHistoryMode;
		this._showBaseProperties = context.showBaseProperties;
		this._loadingContext = {};
		
		//These manage the column mapping - and should be dealt with as a unit, by the mapColumn and related methods.
		this._idToColumnDetailsMapping = {};
		this._linkUriToColumnDetailMapping = {};
		this._attributeUriToColumnDetailMapping = {};
		
		var messages = this._messages;
		
		this._ptMarkedPieces = {};
		this._ptCurrent = {};
		
		this.headerTitles = {
			NAME: messages.name,
			DESCRIPTION: messages.description,
			ALT_SPELLING: messages.alternativeSpellings,
			FORMAT: messages.format,
			USER: messages.lastModifiedBy,
			DATE: messages.lastModifiedDate,
			CREATED_DATE: messages.createdDate,
			CREATOR: messages.creator,
			MANAGEDREQUIREMENT: messages.managedRequirement,
			IMPLEMENTED_BY: messages.implementedBy,
			TRACKED_BY: messages.trackedBy,
			AFFECTED_BY: messages.affectedBy,
			VALIDATED_BY: messages.validatedBy,
			TAGS: messages.tags,
			ID: messages.id,
			OBJECT_TYPE: messages.objectType,
			PRIMARY_TEXT: messages.primaryText,
			MODULE_SECTION: messages.moduleSection,
			MODULE_COMMENTS: messages.moduleComments,
			CONTENT: messages.content,
			SUSPICION: messages.suspicion,
			LOCKED_BY: messages.lockedBy,
			LOCKED_ON: messages.lockedOn
		};
		
		if(com.ibm.rdm.web.MODULE_SUPPORT){	
			this.headerTitles.REUSED_IN_MODULES = messages.moduleReuse;
		}
		// Initialise callbacks that are passed to editors so that they
		// can make requests to the containing grid.
		this._editorCallbacks = {
			// Called when the editor wants to request a save.
			requestSave: dojo.hitch(this._grid, this._grid.onEditorSaveKey),
			// Called when the editor wants to cancel the current changes.
			requestCancel: dojo.hitch(this._grid, this._grid.onEditorCancelKey),
			// Called when the editor is initialised.  Make sure the grid if focused on
			// the proper editor.
			onReady: dojo.hitch(this._grid, this._grid.focus)
		};
		
		// Hook into structure so we can update display when it changes.
		if (this._moduleStructure) {
			dojo.connect(this._moduleStructure, "onChange", this, this._onModuleStructureChange);
		}
	},
	
	//////////////////////////////////////////////////
	/// Edit Interface Methods
	
	setEditInterface: function(editInterface) {
		// summary:
		//		Provide methods for supporting editing. It is assumed that these
		//		methods will curry their context - from here they will be called
		//		with ArtifactDisplay as this.
		this._editInterface = editInterface;
	},
	
	invokeEdit: function() {
		if (this._editInterface) {
			this._editInterface.invokeEdit.apply(this, arguments);
		}
	},

	isTextReallyEditable: function() {
		if (this._editInterface) {
			return this._editInterface.isTextReallyEditable.apply(this, arguments);
		}
	},

	getEditableValue: function(artifactURI, attributeURI, functionToCall, resource,
								attrType, containingNode, columnId, vetoEdit, cellOptions, rowKey, editorCallbacks, isFocusCell) {
		// editorCallbacks:
		//		A hash containing functions that the editor can use as callbacks to the containing grid.  Used
		//		to allow hotkeys that are caught by the editor to provoke actions on the overall grid.
		switch (columnId) {
			case ArtifactConstants.MODULE_CONTENT_COL:
				// create a widget that will be used to display/edit more than one attribute value.
				this._createContentDisplayWidget(resource, artifactURI, cellOptions, functionToCall, rowKey, columnId, editorCallbacks, isFocusCell);
				break;
			default:
				if (this._editInterface) {
					this._editInterface.getEditableValue.apply(this, arguments);
				}
				break;
		}
	},
	
	/////////////////////////////////////////////////////////////////////////////
	
	setModuleBaselineInfo: function(isInModuleBaseline, moduleBaselineURI, moduleURI, baselineDisplayDate) {
		// summary:
		//		Pass through information about the module baseline (if there is any).
		this._isInModuleBaseline = isInModuleBaseline;
		this._moduleBaselineURI = moduleBaselineURI;
		this._moduleURI = moduleURI;
		this._displayDate = baselineDisplayDate;
	},
		
	setShowBaseProperties: function(showBaseProperties) {
		this._showBaseProperties = showBaseProperties;
	},
	
	////////////////////////////////////////////////////////////////////////////

	setupGridModelData: function (caller) {	
		var self = this;
		var notVisibleInModule = (this._viewType !== com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE);

		//Define the columns
		var id_col = {
			//name: com.ibm.rdm.web.project.ResultSetGridView.headerTitles.ID,
			name: this.headerTitles.ID,
			width: ArtifactConstants.ID_COL_WIDTH,
			isManualWidth: true,
			isVisible: true,
			isAlwaysVisible: false,
			deferredContentFunction: function(functionToCall, resource, key, containerNode) {
				var moduleBaseline = null;
				if (self._isInModuleBaseline) {
					moduleBaseline = self._moduleBaselineURI;
				}
				var idNode;
				if ((typeof resource.isStub === "function") && resource.isStub()){
					idNode = {
						div: dojo.create("div", { innerHTML: "&mdash;", "class": "stub-id-col" }) 
						};
				} else {
					idNode = self.createResourceLinkHTML(resource, false, true, resource.getID(), true /*noTabFocus*/, null, moduleBaseline, "id-col");
				}
				functionToCall(idNode.div, idNode.destroy);
			},
			sortFunction: dojo.hitch(caller, caller.sortByID),
			navModeActionFunction: function(rowKey, columnId, cellNode) {
				// follow the link that has previously been placed in the cell.
				self._followCellResourceLink(cellNode);
			},

			custom: {type: "S", desc: this._messages.idDesc, wellKnownAttributeName: RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Identifier}
		};

		var titleAttrDef = RRMTypes.getWellKnownAttributeURI(self._projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Title),
			descAttrDef = RRMTypes.getWellKnownAttributeURI(self._projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Description);

		var name_col = {
			name: this.headerTitles.NAME,
			width: ArtifactConstants.TITLE_COL_WIDTH,
			isVisible: notVisibleInModule,
			isAlwaysVisible: notVisibleInModule,
			isReallyEditable: function(rowTypeId, columnId, key, uri) {
				var resource = self._dataCache.getRecordInfo(uri);
				if (!resource) {
					return false;
				}
				return self._isTitleEditableForColumn(columnId, resource);
			},
			deferredContentFunction: function(functionToCall, resource, key, containerNode, indent, columnId) {

				var resourceUri = resource.getResourceURI();
			
				if (EditManager.isUnderEdit(self._projectURI, resourceUri, titleAttrDef)
					&& self._isTitleEditableForColumn(columnId, resource)) {
					
					var lockingURI = null;
					if (self._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
						//If in a module we need to lock the base artifact
						lockingURI = self._dataCache.getBaseArtifactForResource(resourceUri);
					}
					
					EditManager.getEditor({
						projectURI: self._projectURI,
						artifactURI: resourceUri,
						lockingURI: lockingURI,
						definitionURI: titleAttrDef,
						callback: function (node) {
							functionToCall(node);
							self._grid.runRowClassFunctions([resourceUri]);
						},
						resource: resource,
						containingNode: containerNode
					});
				} else {
					var moduleBaseline = null;
					if (self._isInModuleBaseline) {
						moduleBaseline = self._moduleBaselineURI;
					}
					var nameDiv = self.createResourceLinkHTML(resource, false, false, null/*label*/, true /*noTabFocus*/, null, moduleBaseline, "name");
					functionToCall(nameDiv.div, nameDiv.destroy);
				}
				
			},
			sortFunction: dojo.hitch(caller, caller.sortByName),
			navModeActionFunction: function(rowKey, columnId, cellNode) {
				// follow the link that has previously been placed in the cell.
				self._followCellResourceLink(cellNode);
			},
			focusOnEditorFunction: dojo.hitch(self._editInterface, self._editInterface.focusOnEditor,
											RRMTypes.getWellKnownAttributeURI(self._projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Title)),

			custom: {type: "S", desc: this._messages.nameDesc, wellKnownAttributeName: RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Title}
		};
		
		//Ensure no editing in a baseline
		if (!this._baselineURI) {
			name_col.invokeEditFunction = function(rowTypeId, columnId, key, uri, cellNode, eventNode, refreshRow){
				return self.invokeEdit(rowTypeId, columnId, key, uri, refreshRow);
			};
		}
		
		var module_section_col = {
			name: this.headerTitles.MODULE_SECTION,
			width: 75,
			isManualWidth: true,
			isVisible: false,
			isAlwaysVisible: false,
			deferredContentFunction: function(functionToCall, resource, key, containerNode) {
				
				var sectionNumber = self._getResourceSection(resource);
				var sectionDiv = dojo.create("span", { "class": "sectionNumber", innerHTML: "<span class='number'>"+sectionNumber+"</span>" });
				
				functionToCall(sectionDiv);
			},
			custom: {type: "S", desc: this._messages.moduleSection},
			showFullTitleOnHover: true
		};
		
		
		var description_col = {
			name: this.headerTitles.DESCRIPTION,
			width: 200,
			isVisible: false,
			// InvokeEdit function null in a baseline to ensure that editing can never happen
			invokeEditFunction: this._baselineURI ? null : function(rowTypeId, columnId, key, uri, cellNode, eventNode, refreshRow) {
				return self.invokeEdit(rowTypeId, columnId, key, uri, refreshRow);
			},
			deferredContentFunction: function(functionToCall, resource, key, containerNode) {
				var resourceUri = resource.getResourceURI();

				if (EditManager.isUnderEdit(self._projectURI, resourceUri, descAttrDef)) {
					
					var lockingURI = null;
					if (self._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
						//If in a module we need to lock the base artifact
						lockingURI = self._dataCache.getBaseArtifactForResource(resourceUri);
					}					
					
					EditManager.getEditor({
						projectURI: self._projectURI,
						artifactURI: resourceUri,
						lockingURI: lockingURI,
						definitionURI: descAttrDef,
						callback: function (node) {
							functionToCall(node);
							  self._grid.runRowClassFunctions([resourceUri]);
						},
						resource: resource,
						containingNode: containerNode
					});
				} else {
					functionToCall(resource.getDescription());
				}
				
			},
			deferredPlaceholderFunction: function(rowId, colId, record){ return DomUtils.getHTMLEncodedText(record.getDescription());},
			focusOnEditorFunction: dojo.hitch(self._editInterface, self._editInterface.focusOnEditor,
										RRMTypes.getWellKnownAttributeURI(self._projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Description)),
			recordFunction: function(record){ return DomUtils.getHTMLEncodedText(record.getDescription());},
			custom: {type: "S", desc: self._messages.descriptionDesc, wellKnownAttributeName: RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Description},
			showFullTitleOnHover: true
		};
		

		// Alternative Spellings (for terms) column
		var alternateSpelling_col = {
			name: this.headerTitles.ALT_SPELLING,
			width: 145,
			isVisible: false,
			deferredContentFunction: function(functionToCall, resource, key, containerNode) {
				var alternateSpellingAttrDef = RRMTypes.getWellKnownAttributeURI(self._projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.AlternateSpelling);
				var resourceUri = resource.getResourceURI();

				if (EditManager.isUnderEdit(self._projectURI, resourceUri, alternateSpellingAttrDef)) {

					var lockingURI = null;
					if (self._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
						//If in a module we need to lock the base artifact
						lockingURI = self._dataCache.getBaseArtifactForResource(resourceUri);
					}	

					EditManager.getEditor({
						projectURI: self._projectURI,
						artifactURI: resourceUri,
						lockingURI: lockingURI,
						definitionURI: alternateSpellingAttrDef,
						callback: function (node) {
							functionToCall(node);
							self._grid.runRowClassFunctions([resourceUri]);
						},
						resource: resource,
						containingNode: containerNode
					});
				} else {
					functionToCall(DomUtils.getHTMLEncodedText(resource.getAlternateSpelling()));
				}
				
			},
			deferredPlaceholderFunction: function(rowId, colId, record){ return DomUtils.getHTMLEncodedText(record.getAlternateSpelling()); },
			// Ensure no editing in a baseline
			invokeEditFunction: (
				this._baselineURI ?
				null :
				function(rowTypeId, columnId, key, uri, cellNode, eventNode, refreshRow) {
					return self.invokeEdit(rowTypeId, columnId, key, uri, refreshRow);
				}
			),
			isReallyEditable: dojo.hitch(self, this._isAlternativeSpellingsEditableForRow),
			focusOnEditorFunction: dojo.hitch(
				self._editInterface, self._editInterface.focusOnEditor,
				RRMTypes.getWellKnownAttributeURI(self._projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.AlternateSpelling)
			),
			recordFunction: function(record){ return DomUtils.getHTMLEncodedText(record.getAlternateSpelling()); },
			custom: {type: "S", desc: self._messages.alternativeSpellingDesc, wellKnownAttributeName: RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.AlternateSpelling},
			showFullTitleOnHover: true
		};
		
		// Artifact re-use counter in modules column
		// Hide Modules based on flag Defect 57300 
		if(com.ibm.rdm.web.MODULE_SUPPORT){		 	
			var artifactModuleReuse_col = {
					name: this.headerTitles.REUSED_IN_MODULES,
					width: 18,
					isColumnHeaderHideable: true,
					minimumWidth: 18,
					isManualWidth: true,
					isVisible: false, 
					deferredContentFunction: function(functionToCall, resource, key, containerNode) {
						
					
						if (!resource.getReuseCount) {
							return;
						}
						
						var destroys = [];

						//Create the appropiate icon to display depending on the current count
						var getReuseImageIcon= function(reuseCount) {
							var imageURL;
							if (dojo.isString(reuseCount)) {
								reuseCount = parseInt(reuseCount);
							}
							
							if(reuseCount === 0){
								return null;
							}else if(reuseCount === 1){
								imageURL = dojo.moduleUrl("com.ibm.rdm.web", "artifactgrid/images/reuse-only-1.png").toString();
							}else if( (reuseCount > 1) && (reuseCount <= 5) ){
								imageURL = dojo.moduleUrl("com.ibm.rdm.web", "artifactgrid/images/reuse-medium-2-5.png").toString();
							}else if(reuseCount > 5){
								imageURL = dojo.moduleUrl("com.ibm.rdm.web", "artifactgrid/images/reuse-more-than-5.png").toString();
							}
							if(imageURL){
								return dojo.create("img", {
									src: imageURL
								});
							}else{
								return null;
							}					
						};					
						var createImageHoverPopup= function(imageIconNode, moduleBoundArtifact,reuseCount) {					
							var moduleReusePopup = new ReuseHover({
								"resourceObj": resource,
								"modulesInUse":reuseCount,
								"moduleBoundArtifact": moduleBoundArtifact
							});
							moduleReusePopup.createHoverPopup(imageIconNode);
							self._toDestroy.push(moduleReusePopup);
						};
						
						var moduleResourceURI = (resource.getModuleBoundArtifact())?resource.getModuleBoundArtifact():resource.getResourceURI();
						var reuseResourceCount = resource.getReuseCount();
						if (self._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE){
							//If we are in a module, then substract the current module from the reusability count
							reuseResourceCount = reuseResourceCount - 1; 
						}
						if(reuseResourceCount > 0){
							//Create the hover
							var imageIconNode = getReuseImageIcon(reuseResourceCount);
							createImageHoverPopup(imageIconNode,
												  moduleResourceURI,
												  reuseResourceCount);
							functionToCall(imageIconNode, destroys);
						}	
						else{
							functionToCall("");
						}
						
					},
					sortFunction: dojo.hitch(caller, caller.sortByArtifactModuleReuseCount),
					isDefaultSortOrderDescending: true,
					custom: {type: "S", desc: self._messages.reusedInModulesDesc}
			};
		}
		// Artifact comments in modules column
		var artifactComments_col = {
				name: this.headerTitles.MODULE_COMMENTS,
				width: 18,
				minimumWidth: 18,
				isColumnHeaderHideable: true,
				isManualWidth: true,
				isVisible: false,
				deferredContentFunction: function(functionToCall, resource, key, containerNode) {
					
					//TODO
					if (!resource.getRecentCommentCount) {
						return;
					}

					var destroys = [];
					var moduleCommentsPopup;
					
					var showCommentsFunction = dojo.hitch(this, function() {
						self._commentingInterface.showMemberCommentsCallback(key, resource.getModuleBoundArtifact());
					});
					var addCommentFunction = dojo.hitch(this, function() {
						if (moduleCommentsPopup && moduleCommentsPopup._commentHover._isOpen) {
							moduleCommentsPopup._commentHover.close();
						}
						self._commentingInterface.addMemberCommentCallback(key, resource.getModuleBoundArtifact());
					});
					var getCommentButton = function(recentCount, unresolvedCount, resolvedCount, isAddAllowed) {
						var commentToolbar = new jazz.ui.Toolbar();
						destroys.push(commentToolbar);
						self._toDestroy.push(commentToolbar);
						var imageURL; 
						if (recentCount > 0){
							imageURL = dojo.moduleUrl("com.ibm.rdm.web", "artifactgrid/images/comments-recent.gif").toString();
						} else if (unresolvedCount > 0){
							imageURL = dojo.moduleUrl("com.ibm.rdm.web", "artifactgrid/images/comments-present.gif").toString();
						} else if (resolvedCount > 0) {
							imageURL = dojo.moduleUrl("com.ibm.rdm.web", "artifactgrid/images/comments-resolved.gif").toString();
						} else {
							imageURL = dojo.moduleUrl("com.ibm.rdm.web", "artifactgrid/images/add-comment.gif").toString();
						}
						var button = null;
						if ((unresolvedCount > 0 || resolvedCount > 0)) {
							button = new jazz.ui.toolbar.Button({
								onClick: showCommentsFunction, 
								imgSrc: imageURL
							});
						} else {
							if (!isAddAllowed) {
								button = new jazz.ui.toolbar.Button({
									imgSrc: imageURL
								});
								button.domNode.style.opacity = '0.5'; //mozilla
								button.domNode.style.filter = 'alpha(opacity=50)'; //IE
							} else {
								button = new jazz.ui.toolbar.Button({
									onClick: addCommentFunction, 
									imgSrc: imageURL
								});
							}
							// This button is hidden by default.  Shown by mouseenter and focus (in Module.js)
							dojo.addClass(button.domNode, "addCommentButton");
						}
						self._toDestroy.push(button);
						destroys.push(button);
						commentToolbar.addChild(button);
						return button;
					};
					var createCommentHoverPopup = function(node, resource, isAddAllowed, showBaseProperties, moduleURI) {
						moduleCommentsPopup = new CommentHover({
							resource: resource,
							addCommentCallback: addCommentFunction,
							showCommentsCallback: showCommentsFunction,
							isAddAllowed: isAddAllowed,
							showBaseProperties: showBaseProperties,
							moduleURI: moduleURI
						});

						destroys.push(moduleCommentsPopup);

						moduleCommentsPopup.createHoverPopup(node);
						self._toDestroy.push(moduleCommentsPopup);
						functionToCall(node, destroys);
					};
					
					var projectOperationsDeferred = com.ibm.rdm.web.common.ProjectCache.getProjectPermissions(self._projectURI);
					projectOperationsDeferred.addCallback(this, function(projectOperations){
						var isAddAllowed = projectOperations.item(com.ibm.rdm.web.common.ProjectCache.MODIFY_COMMENT);
						// comments cannot be added in history mode or in review mode for module, only module itself is under review
						// when in a review
						isAddAllowed = isAddAllowed && !self._isInReviewMode && !self._isInHistoryMode;
						var node = getCommentButton(resource.getRecentCommentCount(), resource.getUnresolvedCommentCount(), resource.getResolvedCommentCount(), isAddAllowed).domNode;
						createCommentHoverPopup(node, resource, isAddAllowed, self._showBaseProperties, self._moduleStructure.moduleURI);
					});
				},
				navModeActionFunction: function(rowKey, columnId, cellNode) {
					// perform the same action as pressing the button
					self._clickCellButtonLink(cellNode);
				},
				sortFunction: dojo.hitch(caller, caller.sortByCommentCount),
				isDefaultSortOrderDescending: true,
				custom: {type: "S", desc: self._messages.moduleCommentsDesc}
		};
		
		//Object Type Col
		var obj_type_col = {
			name: this.headerTitles.OBJECT_TYPE,
			width: ArtifactConstants.TYPE_COL_WIDTH,
			deferredContentFunction: function (fnToCall, resource) {
				//Ensure this is only called when the type system has been loaded
				self._addTypeSystemDeferredFunction(function () {
					var key;
					var types = resource.getTypes();
					var typesArray = [];
					for (key in types) {
						typesArray.push(key);
					}

					//Only display the first type.
					var ot = self._typeCache.getProjectObjectType(typesArray[0]);
					if (ot) {
						fnToCall(ot.getTitle());
					} else {
						fnToCall("");
					}
				});
			},
			sortFunction: dojo.hitch(caller, caller.sortByType),
			isVisible: notVisibleInModule,
			custom: {type: "S", desc: self._messages.objectTypeDesc}
		};

		var format_col = {
			width: 70,
			name: this.headerTitles.FORMAT,
			deferredContentFunction: function (fnToCall, resource) {
				//Ensure this is only called when the type system has been loaded
				self._addTypeSystemDeferredFunction(function () {
					var formatTitle = self._typeCache.getProjectFormat(resource.getMimeType());
					if (formatTitle) {
						fnToCall(formatTitle);
					}
					else {
						fnToCall("");
					}
				});
			},
			sortFunction: dojo.hitch(caller, caller.sortByFormat),
			isVisible: false,
			custom: {type: "S", desc: self._messages.formatDesc, wellKnownAttributeName: RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ArtifactFormat}
		};

		var mod_user_col = {
			name: this.headerTitles.USER,
			width: ArtifactConstants.CONTRIBUTOR_COL_WIDTH,
			deferredContentFunction: dojo.hitch(this, this._getDeferredLastModifiedByUserDetails),
			navModeActionFunction: dojo.hitch(this, this.doUserNavModeAction),
			//WI 66171: Removed sortable by user id (as it is not the user name)
			//sortFunction: dojo.hitch(caller, caller.sortByUser),
			isVisible: notVisibleInModule,
			custom: {type: "S", desc: self._messages.lastModifiedByDesc, wellKnownAttributeName: RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Contributor},
			showFullTitleOnHover: true
		};

		var mod_date_col = {
			name: this.headerTitles.DATE,
			width: ArtifactConstants.MODIFIED_ON_COL_WIDTH,
			recordFunction: function(record){ 
				if (record) { 
					var date = record.getRevisionDate();
					if (date) {
						//ACGC_Bidi_start Static date support for gregorian and non-gregorian calendars
						return BidiUtils.applyDateSupport(date, {formatLength:"medium"});
						//ACGC_Bidi_End
					}
					return "";
				}
			},
			sortFunction: dojo.hitch(caller, caller.sortByDate),
			isVisible: notVisibleInModule,
			custom: {type: "S", desc: self._messages.lastModifiedDateDesc, wellKnownAttributeName: RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ModifiedOn}
		};

		var creator_col = {
			name: this.headerTitles.CREATOR,
			width: 105,
			deferredContentFunction: dojo.hitch(this, this._getDeferredCreatorDetails),
			navModeActionFunction: dojo.hitch(this, this.doUserNavModeAction),
			//WI 66171: Removed sortable by user id (as it is not the user name)
			//sortFunction: dojo.hitch(caller, caller.sortByCreator),
			isVisible: false,
			custom: {type: "S", desc: self._messages.creatorDesc, wellKnownAttributeName: RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Creator},
			showFullTitleOnHover: true
		};

		var created_date_col = {
			name: this.headerTitles.CREATED_DATE,
			width: 140,
			recordFunction: function(record){ 
				var createdDate = record.getCreatedDate();
				if (createdDate) {
					//ACGC_Bidi_start Static date support for gregorian and non-gregorian calendars
					return BidiUtils.applyDateSupport(record.getCreatedDate(), {formatLength:"medium"});
					//ACGC_Bidi_End
				}
				return "";
			},
			sortFunction: dojo.hitch(caller, caller.sortByCreatedDate),
			isVisible: false,
			custom: {type: "S", desc: self._messages.createdDateDesc, wellKnownAttributeName: RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Created}
		};

		var imp_by_col = {
			name: this.headerTitles.IMPLEMENTED_BY,
			width: 140,
			deferredContentFunction: dojo.hitch(caller, caller.loadImplementedCalmLinks),
			startIntraCellNavigationFunction: dojo.hitch(caller, caller.startIntraCellNavigationForCalmLinks),
			isVisible: false,
			custom: {type: "L", desc: self._messages.implementedByDesc},
			showFullTitleOnHover: true
		};

		var man_by_col = {
			name: this.headerTitles.TRACKED_BY,
			width: 140,
			deferredContentFunction: dojo.hitch(caller, caller.loadTrackedCalmLinks),
			startIntraCellNavigationFunction: dojo.hitch(caller, caller.startIntraCellNavigationForCalmLinks),
			isVisible: false,
			custom: {type: "L", desc: self._messages.trackedByDesc},
			showFullTitleOnHover: true
		};

		var aff_by_col = {
			name: this.headerTitles.AFFECTED_BY,
			width: 140,
			deferredContentFunction: dojo.hitch(caller, caller.loadAffectedCalmLinks),
			startIntraCellNavigationFunction: dojo.hitch(caller, caller.startIntraCellNavigationForCalmLinks),
			isVisible: false,
			custom: {type: "L", desc: self._messages.affectedByDesc},
			showFullTitleOnHover: true
		};

		var val_by_col = {
			name: this.headerTitles.VALIDATED_BY,
			width: 140,
			deferredContentFunction: dojo.hitch(caller, caller.loadValidatedCalmLinks),
			startIntraCellNavigationFunction: dojo.hitch(caller, caller.startIntraCellNavigationForCalmLinks),
			isVisible: false,
			custom: {type: "L", desc: self._messages.validatedByDesc},
			showFullTitleOnHover: true
		};

		var tags_col = {
			name: this.headerTitles.TAGS,
			width: 60,
			deferredContentFunction: dojo.hitch(this, this.showTags),
			isVisible: false,
			custom: {type: "S", desc: self._messages.tagsDesc},
			showFullTitleOnHover: true
		};
		
		var suspicion_col = {
			name: this.headerTitles.SUSPICION,
			width: 18,
			minimumWidth: 18,
			isManualWidth: true,
			deferredContentFunction: dojo.hitch(this, this.showSuspicion),
			isVisible: false,
			sortFunction: dojo.hitch(caller, caller.sortBySuspicion),
			custom: {type: "S", desc: self._messages.suspicionDesc}
		};
		
		var locked_by_col = {
				
			name: this.headerTitles.LOCKED_BY,
			width: 60,
			deferredContentFunction: dojo.hitch(this, this._getDeferredLockedByDetails),
			isVisible: false,
			custom: {
				type: "S",
				desc: self._messages["lockedByDesc"]
			},
			showFullTitleOnHover: true

		};

		var locked_on_col = {
				
			name: this.headerTitles.LOCKED_ON,
			width: 60,
			isVisible: false,
			custom: {
				type: "S",
				desc: self._messages["lockedOnDesc"]
			},
			recordFunction: dojo.hitch(this, function(record) { 
				var lockInfo, date;
				if (record) { 
					if (this._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
						lockInfo = LockManager.getLockInfo(record.getBoundArtifact());
					} else {
						lockInfo = LockManager.getLockInfo(record.getResourceURI());
					}
					if (lockInfo) {
						date = dojo.date.stamp.fromISOString(lockInfo.utcDate);
						if (date) {
							date = new Date(date);
							//ACGC_Bidi_start Static date support for gregorian and non-gregorian calendars
							return BidiUtils.applyDateSupport(date, {formatLength:"medium"});
							//ACGC_Bidi_End
						}
					}
				}
				
				return "";
			}),
			showFullTitleOnHover: true
		};

		//Define the row type
		// some row class functions:
		var rowClassFunctions = [];
		var rcfUnderEdit = dojo.hitch(this, function (key) {
			if (EditManager.isArtifactUnderEdit(this._projectURI, key)) {
				return ["underEdit"];
			}
		});
		rowClassFunctions.push(rcfUnderEdit);
		
		//A function to get the right Locking rowclass
		//one of expLockedByMe, permLockedByMe, expLockByOther, permLockByOther
		var that = this;
		var decodeLockInfo = function (info, key, inBaseline) {
		
			if (inBaseline) {
				return;
			}
			
			if (info) {
			   // This seems a bit hacky. 
			   
			   setTimeout(function() {
					dojo.query("[objectId='"+key+"'] td._selector").forEach(function(node){ 
						com.ibm.rdm.web.server.client.utils.UserCache.getUser(info.userURI).addBoth(function(response) {
							if (response){
								var msg;
								 if (info.byMe) {
									msg = dojo.string.substitute(that._commonMessages["lockedByCurrentUserSince"], [com.ibm.rdm.web.common.DateTimeHelper.getDateTimeDisplayStringFromString(info.utcDate)]);
								 } else {
									msg = dojo.string.substitute(that._commonMessages["lockedByOtherUserSince"], [jazz.util.html.escape(response.name), com.ibm.rdm.web.common.DateTimeHelper.getDateTimeDisplayStringFromString(info.utcDate)]);
								 }
								 
								 
								
								dojo.attr(node, "title", msg);
							}
						});  
					});
				});
			}
			
			if (info && info.byMe) {
				return ["permLockedByMe"];			
			} else if (info && info.byOther) {
				return ["permLockByOther"];
			}
		};
		
		var rcfLocked;
	
		if (!this._isInHistoryMode) {
			if (this._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
				rcfLocked = dojo.hitch(this, function (key) {
					var uri = this._dataCache.getBaseArtifactForResource(key);
					var info = LockManager.getLockInfo(uri);
					return decodeLockInfo(info, key, this._baselineURI);
				});
			} else {
				rcfLocked = dojo.hitch(this, function (key) {
					var info = LockManager.getLockInfo(key);
					return decodeLockInfo(info, key, this._baselineURI);
				});
			}
			rowClassFunctions.push(rcfLocked);
		}
		
		var rcfDepthAndHeading = dojo.hitch(this, function (key) {
			// Applies module-specific classes to the row
			var record = self._dataCache.getRecordInfo(key),
				classes = [];
			
			var depth = this._getResourceDepth(record);
			var depthClass = "depth" + ( (depth < 10) ? depth : "10plus" );
			classes.push(depthClass);
			
			// Indent the row if current view of the module is appropriate.
			var sortBy = this._moduleStructure && this._moduleStructure.getPrimarySortBy();
			if (sortBy &&
				sortBy.order === 'a' &&
				sortBy.def === RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BookOrder &&
				!this._moduleStructure.hasViewFiltersApplied()) {
					classes.push("indent-by-depth");
			}

			
			if (record.isHeading && record.isHeading()) {
				classes.push("heading");
			} else {
				var entry = this._moduleStructure && this._moduleStructure.getArtifactByURI(record.resourceURI);
				if (entry && entry.isHeading) {
					classes.push("heading");
				}
			}
			return classes;
		});
		
		if (this._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
			rowClassFunctions.push(rcfDepthAndHeading);
		}
		
		
		caller.grid.addRowType(RESOURCE_ROW, {
			isSelectable: function () {
				return true;
			},
			hasSelectionBox: true,
			hasMenu: true,
			isPrimary: true,
			allowsMultiSelect: true,
			rowDataFunction: function(uri) {return self._dataCache.getRecordInfo(uri);},
			versionFunction: function(uri) {return self._dataCache.getResourceEtag(uri);},
			menuLayout: caller.MENU_LAYOUT, operations: caller.MENU_OPERATIONS,
			hasExpando: true, hideExpando: true,
			expandoStateFunction: function(rowTypeId, fnToCall, resource, key, indent) {
				return caller._relationManager.baseRowExpandoState(rowTypeId, fnToCall, resource.getResourceURI(), key, indent);
			},
			expandoApplyFunction: function(rowTypeId, key) {
				return caller._relationManager.baseRowExpandoCalled(rowTypeId, key);
			},
			contentSelectableFunction: dojo.hitch(this, function(key) {
				return EditManager.isArtifactUnderEdit(this._projectURI, key);
			}),
			rowClassFunctions: rowClassFunctions,
			rowLabelFunction: dojo.hitch(this, function(resource) {
				return dojo.string.substitute(this._messages.resourceRowAriaLabel, [resource.getID()]);
			}),
			rowDescriptionFunction: dojo.hitch(this, function(resource) {
				return dojo.string.substitute(this._messages.resourceRowAriaDescription, [resource.getID()]);
			}),
			menuCellLabelFunction: dojo.hitch(this, function(resource) {
				return dojo.string.substitute(this._messages.menuCellAriaLabel, [resource.getID()]);
			}),
			showFullTitleOnHover: true
			
			
		});
		
				
		// Add the Primary Text Column.
		var primaryTextUri = RRMTypes.getPrimaryTextURI(self._projectURI);		
		var definePrimaryTextColumn = function(isContentCol, title, isVisible) {
			var col_id = "primary_attr_col",
				columnDefinition = {
				name: title,
				width: 300,
				isVisible: isVisible,
				// SW: poor class name - but gives the primary text the same treatment
				// it'll get in the Editor.
				cellClasses: ["rdm-text-artifact-widget"],
				isReallyEditable: function(rowTypeId, columnId, key, uri) {
					var resource = self._dataCache.getRecordInfo(uri);
					return self._isPrimaryTextEditableForColumn(columnId, resource) && self.isTextReallyEditable(rowTypeId, columnId, key, uri);
				},
				
				isUnderEditFunction: function(rowTypeId, columnId, key, uri){
					// Indicates whether the column is currently under edit
					var resource = self._dataCache.getRecordInfo(uri);
					return self._isPrimaryTextEditableForColumn(columnId, resource) && EditManager.isUnderEdit(self._projectURI, uri, primaryTextUri);
				},
				
				deferredPlaceholderFunction: function (rowTypeId, columnId, contentRow) {
					// SW: this is somewhat pointless, but here
					// we stick the primary text (which we already have courtesy of the resource)
					// into a "placeholder" only for it to be replaced with the same text
					// immediatley afterwards.
					// TODO - there seems no point in this being "deferredContent"
					
					var wrapper = dojo.create("div");
					var div = dojo.create("div", {}, wrapper);
					if (self._viewType !== com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
						dojo.style(div, {
							"maxHeight": com.ibm.rdm.web.editor.InlineGridEditor.MAX_HEIGHT + "px",
							"overflow": "auto"
						});
					}
					div.innerHTML = contentRow.getPrimaryText();
					return wrapper.innerHTML;
				},
				deferredContentFunction: function (functionToCall, resource, key, containerNode, indent, columnId, isFocusColumn) {
					var cellEditable = self._isPrimaryTextEditableForColumn(columnId, resource);
					var primaryAttrType = self._typeCache.getPrimaryTextAttrDef().getAttributeType();
					self.getEditableValue(resource.getResourceURI(), primaryTextUri, functionToCall, resource, primaryAttrType, containerNode, columnId, !cellEditable, null, key, self._editorCallbacks, isFocusColumn);
				},
				
				focusOnEditorFunction: dojo.hitch(self._editInterface, self._editInterface.focusOnEditor, primaryTextUri),
				
				findMarkAllNext: function (key, searchTerm, markClassName) {
					self.findMarkAllNext(key, col_id, searchTerm, markClassName);
				},
				
				findMarkAllPrev: function (key, searchTerm, markClassName) {
					self.findMarkAllPrev(key, col_id, searchTerm, markClassName);
				},
				
				nextFind: function (key, selectClassName, replaceClassName, done, noMore) {
					self.nextFind(key, col_id, selectClassName, replaceClassName, done, noMore);
				},
				
				prevFind: function (key, selectClassName, replaceClassName, done, noMore) {
					self.prevFind(key, col_id, selectClassName, replaceClassName, done, noMore);
				},
				
				replaceFind: function (key, replaceTerm, replaceClassName, findClassNames, onDone) {
					self.replaceFind(key, col_id, replaceTerm, replaceClassName, findClassNames, onDone);
				},
				
				clearFind: function (key) {
					self.clearFind(key, col_id);
				},				
				
				getFindReplaceOccurrences: function (key) {
					return self.getFindReplaceOccurrences(key, col_id);
				},		
						
				getCurrentFindNode: function (key) {
					return self.getCurrentFindNode(key, col_id);
				},				
				custom: {type: "S"},
				showFullTitleOnHover: true
			};
			return columnDefinition;
		};
		
		// The plain primary text
		var primaryAttrColId = "primary_attr_col";
		var primary_attr_col = definePrimaryTextColumn(false, this.headerTitles.PRIMARY_TEXT, false);
		
		//Ensure no editing in a baseline
		if (!this._baselineURI) {
			primary_attr_col.invokeEditFunction = function(rowTypeId, columnId, key, uri, cellNode, eventNode, refreshRow) {
				return self.invokeEdit(rowTypeId, columnId, key, uri, refreshRow);
			};
		}		

		var defineContentColumn = function(title, isVisible) {
			var col_id = ArtifactConstants.MODULE_CONTENT_COL,
				columnDefinition = {
				name: title,
				width: ArtifactConstants.MODULE_CONTENT_COL_WIDTH,
				isVisible: isVisible,
				// SW: poor class name - but gives the primary text the same treatment
				// it'll get in the Editor.
				cellClasses: ["rdm-text-artifact-widget"],
				innerCellClasses: "indenter",
				//Ensure that invokeEditFunction is null if in a baseline or review.
				invokeEditFunction: (self._baselineURI || self._isInReviewMode) ? null : function(rowTypeId, columnId, key, uri, cellNode, eventNode, refreshRow) {
					var cellWidget = self.findContentCellWidget(cellNode);
					return self.invokeEdit(rowTypeId, columnId, key, uri, refreshRow, /*disableDashboard*/false, cellWidget, eventNode);
				},
				isReallyEditable: function(rowTypeId, columnId, key, uri) {
					if (self._baselineURI) {
						//In a baseline, so not editable.
						return false;
					}
					var resource = self._dataCache.getRecordInfo(uri);
					//Check the permissions - return false if we can't edit.
					var map = self._permissionsCache.getModifyArtifactPermission(resource);
					if (!map || !map.isModifyContentAllowed()) {
						return false;
					}
					
					if (resource.getWrappedResourceURI && resource.getWrappedResourceURI()) {
						// never allow editing of wrapped resource in grid.
						return false;
					}
					
					if (ArtifactExtension.isExtendedType(resource.getMimeType(), self._projectURI)) {
						// it's an RRCx artifact - never allow editing of the main content in the grid.
						return false;
					}
										
					return true;
				},

				deferredContentFunction: function (functionToCall, resource, key, containerNode, indent, columnId) {
					
					var titleEditable = self._isTitleEditableForColumn(columnId, resource);
					var primaryTextEditable = self._isPrimaryTextEditableForColumn(columnId, resource);
					var cellOptions = {
						titleEditable: titleEditable,
						primaryTextEditable: primaryTextEditable
					};
					var cellEditable = titleEditable || primaryTextEditable;
					var primaryAttrType = self._typeCache.getPrimaryTextAttrDef().getAttributeType();
					self.getEditableValue(resource.getResourceURI(), primaryTextUri, functionToCall, resource, primaryAttrType,
											containerNode, columnId, !cellEditable, cellOptions, key, self._editorCallbacks);
				},
				
				focusOnEditorFunction: function(resource, rowTypeId, columnId, key, cellNode, event) {
					var cellWidget = self.findContentCellWidget(cellNode);
					var focused = cellWidget.focus();
					return focused;
				},
				
				startIntraCellNavigationFunction:  function(endIntraCellNavigationFunction, cellNode) {
					var cellWidget = self.findContentCellWidget(cellNode);
					cellWidget.startIntraCellNavigation(endIntraCellNavigationFunction);
				},
				
				findMarkAllNext: function (key, searchTerm, markClassName) {
					self.findMarkAllNext(key, col_id, searchTerm, markClassName);
				},
				
				findMarkAllPrev: function (key, searchTerm, markClassName) {
					self.findMarkAllPrev(key, col_id, searchTerm, markClassName);
				},
				
				nextFind: function (key, selectClassName, replaceClassName, done, noMore) {
					self.nextFind(key, col_id, selectClassName, replaceClassName, done, noMore);
				},
				
				prevFind: function (key, selectClassName, replaceClassName, done, noMore) {
					self.prevFind(key, col_id, selectClassName, replaceClassName, done, noMore);
				},
				
				replaceFind: function (key, replaceTerm, replaceClassName, findClassNames, onDone) {
					self.replaceFind(key, col_id, replaceTerm, replaceClassName, findClassNames, onDone);
				},
				
				clearFind: function (key) {
					self.clearFind(key, col_id);
				},				
				
				getFindReplaceOccurrences: function (key) {
					return self.getFindReplaceOccurrences(key, col_id);
				},		
						
				getCurrentFindNode: function (key) {
					return self.getCurrentFindNode(key, col_id);
				},		
								
				custom: {type: "S", desc: self._messages.contentDesc},
				showFullTitleOnHover: true
			};
			return columnDefinition;
		};

		//Add the necessary columns

		this._addSystemColumn(RESOURCE_ROW, "id-col", id_col);
		
		if (!notVisibleInModule) {
			this._addSystemColumn(RESOURCE_ROW, "module-section-col", module_section_col);
		}
		
		this._addSystemColumn(RESOURCE_ROW, "name", name_col);
		this._grid.addColumn(RESOURCE_ROW, primaryAttrColId, primary_attr_col);
		this.mapColumn(primaryAttrColId, this.ATTRIBUTE_COL, primaryTextUri);
		
		if (!notVisibleInModule) {
			var module_content_col = defineContentColumn(this.headerTitles.CONTENT, // title
														!notVisibleInModule);
			this._addSystemColumn(RESOURCE_ROW, ArtifactConstants.MODULE_CONTENT_COL, module_content_col);
		}

		this._addSystemColumn(RESOURCE_ROW, "description", description_col);

		if (this._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
			
			this._addSystemColumn(RESOURCE_ROW, ArtifactConstants.COMMENTS_COL, artifactComments_col);
		}
		
		if(com.ibm.rdm.web.MODULE_SUPPORT){			
			this._addSystemColumn(RESOURCE_ROW, ArtifactConstants.REUSEARTIFACTS_COL, artifactModuleReuse_col);
		}

		this._addSystemColumn(RESOURCE_ROW, "obj_type_col", obj_type_col);

		this._addSystemColumn(RESOURCE_ROW, "format", format_col);

		this._addSystemColumn(RESOURCE_ROW, "mod-user", mod_user_col);

		this._addSystemColumn(RESOURCE_ROW, "mod-date", mod_date_col);

		this._addSystemColumn(RESOURCE_ROW, "creator", creator_col);

		this._addSystemColumn(RESOURCE_ROW, "created_date_col", created_date_col);

		this._addSystemColumn(RESOURCE_ROW, "imp-by", imp_by_col);

		this._addSystemColumn(RESOURCE_ROW, "track-by", man_by_col);

		this._addSystemColumn(RESOURCE_ROW, "aff-by", aff_by_col);

		this._addSystemColumn(RESOURCE_ROW, "val-by", val_by_col);

		this._addSystemColumn(RESOURCE_ROW, "tags", tags_col);
		
		this._addSystemColumn(RESOURCE_ROW, ArtifactConstants.SUSPICION_COL, suspicion_col);
		
		this._addSystemColumn(RESOURCE_ROW, "lockedBy", locked_by_col);
		
		this._addSystemColumn(RESOURCE_ROW, "lockedOn", locked_on_col);

		this._addSystemColumn(RESOURCE_ROW, "alternateSpelling", alternateSpelling_col);
		
		this._grid.setReadOnly(this._isInHistoryMode);
	},
	
	////////////////////////////////////////////////////////////////////////////////////////////////
	// Next Load Context Methods
	
	addNextLoadContextValue: function(key, value) {
		//Summary: Adds a value to the 'Next Load Context'.
		//key: the key for the value to be set
		//value: the value to be set.
		//This allows the control of asynchronous activity, for example when a link is deleted, 
		//telling the redraw to use the cache and not go to the server. Usually such a
		//context value will be cleared when applied.
		//Callers need to be careful that their value will actually be used (and cleared)
		//on the 'next' call, otherwise they could remain and cause problems later.
		
		this._loadingContext[key] = value;
	},
	
	getAndClearNextLoadContextValue: function(key) {
		//Summary: Returns the 'Next Load Context' for some value, and clears it existing value.
		var result = this._loadingContext[key];
		if (result) {
			delete this._loadingContext[key];
		}
		return result;
	},
	
	/////////////////////////////////////////////////////////////////////////////////////////////////
	
	_isPrimaryTextEditableForColumn: function(columnId, resource) {
		// summary:
		//		Indicates whether or not the primary text attribute is editable in
		//		the given column.  The primary text is editable in the leftmost column
		//		in which it can appear.
		var colContainsPT, curColId, firstPTCol,
			cols = this._grid.getUserCols(RESOURCE_ROW),
			numCols = cols.length;

		if (this._baselineURI) { return false; }
		
		//Check the permissions - return false if we can't edit.
		var map = this._permissionsCache.getModifyArtifactPermission(resource);
		if (!map || !map.isModifyContentAllowed()) { return false; }
		
		for (var i = 0; i < numCols; i++) {
			curColId = cols[i].colId;
			
			colContainsPT = (curColId === "primary_attr_col") ||
							(curColId === ArtifactConstants.MODULE_CONTENT_COL);
			if (colContainsPT) {
				firstPTCol = curColId;
				break;
			}
		}
		return (!!firstPTCol && firstPTCol === columnId);
	},
	
	_isTitleEditableForColumn: function(columnId, resource) {
		// summary:
		//		Indicates whether or not the title attribute is editable in
		//		the given column.  The title is editable in the leftmost column
		//		in which it can appear.
		// columnId:
		//		Identifier for the column to check
		// resource:
		//		The resource for which the info is needed.  If provided, may be
		//		used to determine if the resource is a heading.  If not provided,
		//		it is assumed that the resource is a heading.  This covers the
		var cols = this._grid.getUserCols(RESOURCE_ROW),
			numCols = cols.length;

		//If in baseline
		if (this._baselineURI) { return false; }
		
		//Check the permissions - return false if we can't edit.
		var map = this._permissionsCache.getModifyArtifactPermission(resource);
		if (!map || !map.isModifyTitleAllowed()) { return false; }
		
		for (var i=0; i<numCols; i++){
			if (cols[i].colId === "name"){
				return (columnId === "name");
			}
		}
		return false;
	},
	
	_isAlternativeSpellingsEditableForRow: function(rowTypeId, columnId, key, uri) {
		// summary:
		//		Indicates whether or not the Alternative Spellings attribute is editable in
		//		the given row.
		// rowTypeId:
		//		The row type.
		// columnId:
		//		Identifier for the alternative spellings column.
		// key:
		//		Identifier for the row.
		// uri:
		//		Identifier for the resource to be displayed in the row.
		// returns:
		//		An immediate true or false value, or a promise of a true or
		//		false value, indicating whether the alternative spellings
		//		column is editable for this row.
		var resource = this._dataCache.getRecordInfo(uri);
		if (!resource || this._baselineURI) {
			return false;
		}
		var typeURI = resource.getType();
		var getTypeDeferred = TypesManager.getObjectType({
			projectURI: resource.getProjectURI(),
			typeURI: resource.getType()
		});
		var promise = getTypeDeferred.then(function(objectType){
			// Change value passed to future callbacks to a boolean.
			return objectType.isGlossaryTerm();
		});
		return promise;
	},
	
	_followCellResourceLink: function(cellNode) {
		// summary:
		//		Navigation mode action for cells that contain a link.
		//		Finds the link in the cell and navigates to it.
		var linkNode = dojo.query("a.jazz-ui-ResourceLink", cellNode)[0];
		if (linkNode) {
			var href = dojo.attr(linkNode, "href");
			if (href) {
				window.location.href = href;
			}
		}
	},
	
	_clickCellButtonLink: function(cellNode) {
		// summary:
		//		Navigation mode action for cells that contain a button.
		//		Finds the button in the cell, and invokes a click on it.
		// cellNode:
		//		The td node for the cell containing a user action.
		var buttonNode = dojo.query("a.button", cellNode)[0];
		if (buttonNode) {
			DomUtils.triggerDomEvent(buttonNode, "onclick");
		}
	},
	
	doUserNavModeAction: function(key, columnId, cellNode) {
		// summary:
		//		Navigation mode action for cells that contain a
		//		user name.  Finds the username in the cell and
		//		invokes the email operation.
		// cellNode:
		//		The td node for the cell containing a user action.
		var linkNode = dojo.query("a", cellNode)[0];
		if (linkNode) {
			DomUtils.triggerDomEvent(linkNode, "onclick");
		}
	},
	
	destroy: function(){
		dojo.forEach(this._connectHandles, function(con){
			dojo.disconnect(con);
			delete con;
		});
		dojo.forEach(this._toDestroy, function(widget) {
			widget.destroy();
		});
		this._toDestroy = [];
	},
	

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//// Column Mapping functions

	_addSystemColumn: function(row, systemId, colDetails) {
		// summary:
		//		Adds a system column to the grid, and to the column mapping.		
		this._grid.addColumn(row, systemId, colDetails);
		this.mapColumn(systemId, this.SYSTEM_COL, systemId);
		var self = this;
		if (colDetails.custom && colDetails.custom.wellKnownAttributeName) {
			if (RRMTypes.isWellKnownStringTypeAttribute(this._projectURI, colDetails.custom.wellKnownAttributeName) ||
				(colDetails.custom.wellKnownAttributeName === RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Identifier)) {
				// allow find/replace on well-known string type attributes
				var rowType = this._grid.gridmodel.getPrimaryRowTypeId();
				var column = this._grid.gridmodel._getColumn(rowType, systemId);
				column.findMarkAllNext = function (key, searchTerm, markClassName) {
					self.findMarkAllNext(key, systemId, searchTerm, markClassName);
				};
				column.findMarkAllPrev = function (key, searchTerm, markClassName) {
					self.findMarkAllPrev(key, systemId, searchTerm, markClassName);
				};
				column.nextFind = function (key, selectClassName, replaceClassName, done, noMore) {
					self.nextFind(key, systemId, selectClassName, replaceClassName, done, noMore);
				};
				column.prevFind = function (key, selectClassName, replaceClassName, done, noMore) {
					self.prevFind(key, systemId, selectClassName, replaceClassName, done, noMore);
				};
				column.replaceFind = function (key, replaceTerm, replaceClassName, findClassNames, onDone) {
					self.replaceFind(key, systemId, replaceTerm, replaceClassName, findClassNames, onDone);
				};
				column.getFindReplaceOccurrences = function (key) {
					return self.getFindReplaceOccurrences(key, systemId);
				};
				column.getCurrentFindNode = function (key) {
					return self.getCurrentFindNode(key, systemId);
				};				
				column.clearFind = function (key) {
					self.clearFind(key, systemId);
				};			
			}
		}
	},
	
	mapColumn: function(colId, type, key, /*String?*/linkTypeURI) {
		// summary:
		//		Maintains the mapping info between column id and the type and
		//		details of the column displayed.
		// colId:
		//		The column Id - this is unique across all resource row types.
		// type:
		//		The internal type of this col, one of LINK_COL, ATTRIBUTE_COL or SYSTEM_COL
		// key:
		//		The key within the type, eg for ATTRIBUTE_COL, the URI of the Attribute Definition.
		var details = {colId: colId, type: type, key: key};
		if(linkTypeURI) {
			details["linkTypeURI"] = linkTypeURI;
		}

		this._idToColumnDetailsMapping[colId] = details;
		if (type === this.LINK_COL) {
			this._linkUriToColumnDetailMapping[key] = details;
		} else if (type === this.ATTRIBUTE_COL) {
			this._attributeUriToColumnDetailMapping[key] = details;
		}
	},
	
	removeColumn: function(colId) {
		var details = this._idToColumnDetailsMapping[colId];
		if (!details) {
			return;
		}
		
		delete this._idToColumnDetailsMapping[colId];
		if (details.type === this.LINK_COL) {
			delete this._linkUriToColumnDetailMapping[details.key];
		} else if (details.type === this.ATTRIBUTE_COL) {
			delete this._attributeUriToColumnDetailMapping[details.key];
		}		
	},

	getDetailsByColId: function(colId) {
		// summary:
		//		Returns the column details for the given colId.
		//		Or null if not found.
		var results = this._idToColumnDetailsMapping[colId];
		return results;
	},

	getDetailsByLinkKey: function(linkTypeKey) {
		// summary:
		//		Returns the column details for the given linkTypeKey
		//		(uri + _BI/_OBJ/_SUB suffix). Or null if not found.
		return this._linkUriToColumnDetailMapping[linkTypeKey];
	},
	
	getLinkColumnMap: function() {
		// summary:
		//		Returns the link column mapping. Or null if not found.
		return this._linkUriToColumnDetailMapping;
	},
	
	getAllLinkColDetailsByKey: function() {
		// summary:
		//		Returns all link column details by linkTypeKey
		//		(uri + _BI/_OBJ/_SUB suffix). Or null if none found.
		var details = {};
		var any = false;
		for (var linkTypeKey in this._linkUriToColumnDetailMapping) {
			var detail = this._linkUriToColumnDetailMapping[linkTypeKey];
			details[detail.key] = this._linkUriToColumnDetailMapping[linkTypeKey];
			any = true;
		}
		if (!any) {
			return null;
		}
		return details;
	},

	getDetailsByAttribute: function(attributeUri) {
		// summary:
		//		Returns the column details for the given attributeUri.
		//		Or null if not found.
		return this._attributeUriToColumnDetailMapping[attributeUri];
	},

	getDetailsBySystemKey: function(systemKey) {
		// summary:
		//		Returns the column details for the given system key.
		//		Or null if not found.
		return this._idToColumnDetailsMapping[systemKey];
	},
	

	
	/////////////////////////////////////////////////
	//// Display the values
	
	getDisplayValue: function(attrType, attributeURI, functionToCall, resource, columnId, textOnly) {
		//Set the value functions
		var self = this;
		var node, textContent,
			isExtendedType = false; // are we computing the display value for an RRCx type.
		
		if (attrType.isEnum()) {
			// Enumerations
			var enumValueFn = function() {
				var result = self._getEnumValue(attributeURI, resource);
				functionToCall(result);
			};
			this._addTypeSystemDeferredFunction(enumValueFn);	
		} else if (attributeURI === RRMTypes.getPrimaryTextURI(self._projectURI)) {
			// Primary Text
			node = document.createElement('div');
			textContent = resource.getPrimaryText();
			node.innerHTML = textContent;
			
			BidiUtils.applyTextDirSetting(node, textContent, true);//bidi
			// keep a list of things that need to be destroyed when the grid removes this cell
			var toDestroy = [];
			com.ibm.rdm.web.artifactgrid.ArtifactDisplay.loadEmbeddedResources({
				isExtendedType: false,
				baselineURI: this._baselineURI,
				textNode: node,
				displayDate: this._displayDate,
				grid: this._grid,
				resourceURI: resource.resourceURI,
				moduleBaselineURI: this._moduleBaselineURI,
				addDestroy: function (destroyMeOnRemoval) {
					// summary: 
					//		This function is called when we create something as part
					//		of the embed loading we need to destroy e.g. ResourceLinks
					toDestroy.push(destroyMeOnRemoval);
				}
			});
		
			functionToCall(node, {
				// we need a widget type interface here...
				destroyRecursive: function () {
					dojo.forEach(toDestroy, function (destroyMe) {
					   destroyMe.destroyRecursive(); 
					});
				}
			});	
		} else {
			var valueType = attrType.getValueType();
			
			if (valueType === RRMTypes.ATTRIBUTE_TYPES.UserType) {
				// Users
				self._getDeferredUserValue(attributeURI, functionToCall, resource, textOnly);
			} else if (valueType === RRMTypes.ATTRIBUTE_TYPES.DateType) {
				// Dates
				functionToCall(self._getDateValue(attributeURI, resource));
			} else if (valueType === RRMTypes.ATTRIBUTE_TYPES.DateTimeType) {
				// DateTimes
				functionToCall(self._getDateTimeValue(attributeURI, resource));
			} else if (valueType === RRMTypes.ATTRIBUTE_TYPES.TimeType) {
				// Times
				functionToCall(self._getTimeValue(attributeURI, resource));
			} else if (valueType === RRMTypes.ATTRIBUTE_TYPES.DurationType) {
				// Durations
				functionToCall(self._getDurationValue(attributeURI, resource));
			} else if (valueType === RRMTypes.ATTRIBUTE_TYPES.IntegerType || valueType === RRMTypes.ATTRIBUTE_TYPES.FloatType) {
				// Numbers
				functionToCall(self._getNumericValue(attributeURI, resource));
			} else if (valueType === RRMTypes.ATTRIBUTE_TYPES.BooleanType) {
				// Booleans
				functionToCall(self._getBooleanValue(attributeURI, resource));
			} else {
				//All other values
				functionToCall(self._getPlainValue(attributeURI, resource));
			}
		}
	},
	
	_getResourceDepth: function(resource){
		// summary:
		//		Gets the depth of a resource, either from the
		//		resource itself or from the module structure.
		// returns:
		//		the depth.  Shoudl only ever be called in a module context,
		//		but jsut in case, 1 is returned if the depth cannot be
		//		determined.
		
		var uri = resource.getResourceURI(),
			structureInfo = this._moduleStructure.getArtifactByURI(uri);
		if (structureInfo) {
			return structureInfo.depth;
		}
		
		return 1;
	},
	
	_getResourceSection: function(resource) {
		// summary:
		//		Gets the section number for a resource, either from the resource itself,
		//		or from the module structure.
		// returns:
		//		The section number string, or an empty string if the section number cannot
		//		be determined.
		
		if (this._moduleStructure) {
			// full resources do not have section number, so get it from structure.
			var uri = resource.getResourceURI();
			var structureInfo = this._moduleStructure.getArtifactByURI(uri);
			if (structureInfo) {
				return structureInfo.sectionNumber;
			}
		} else if (resource.getModuleSection) {
				return resource.getModuleSection();
		} else {
				return "";
		}
	},
	
	////////////////////////////////////////////////////////////////////
	//// Custom data types
	
	////////////////////////////////////////////////////////////////////////////////////////////
	//// Display methods for the various types
	
	_getDeferredLastModifiedByUserDetails: function(functionToCall, resource) {
		this._addDeferredUserDetails(functionToCall, resource, resource.getLastModifiedBy());
	},

	_getDeferredCreatorDetails: function(functionToCall, resource) {
		this._addDeferredUserDetails(functionToCall, resource, resource.getCreatedBy());
	},
	
	_getDeferredLockedByDetails: function(functionToCall, resource) {
		var lockInfo;
		if (this._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
			lockInfo = LockManager.getLockInfo(resource.getBoundArtifact());
		} else {
			lockInfo = LockManager.getLockInfo(resource.getResourceURI());
		}
		if (lockInfo) {
			this._addDeferredUserDetails(functionToCall, resource, lockInfo.userURI);
		}
	},

	_addDeferredUserDetails: function(functionToCall, resource, user, textOnly) {
		// summary:
		//		This function is used to provide the user name in a asynchronous manner.
		//		I don't believe this will work well however - with quick, rapid calls
		//		this will result in a lot of calls going to the server.
		// user:
		//		may be an array of users
		var self = this;
				
		if (!user) {
			functionToCall("");
			return;
		}
		
		var onProjectMemberCompleteDiv = function(users) {

			var mainDiv= document.createElement("div");
			var link = resource.getResourceURI();
			var onDialogClose = dojo.hitch(this, function(){
				this._grid.retakeFocus();
			});
			var mailParams = {
				subject: resource.getDisplayTitle(),
				bodyHtml: DomUtils.makeAnchorString(link, resource.getDisplayTitle()),
				projectUri: self.projectURI,
				widget: true,
				onDialogClose: onDialogClose
			};
			var destroy = [];

			for(var userUri in users) {
				var userResult = users[userUri];
				var div= document.createElement("div");
				mainDiv.appendChild(div);
				
				var nodeMailTo = MailUtils.createMailTo(userResult, self.projectURI, mailParams);
				// prevent tabbing to the field itself - tab goes to the cell.
				dojo.attr(nodeMailTo.domNode, "tabIndex", "-1");

				this._connectHandles.push(nodeMailTo.con);
				destroy.push({
					destroyRecursive: dojo.hitch(this, function(connection){
						dojo.disconnect(connection);
					}, nodeMailTo.con)
				});
				
				dojo.addClass(nodeMailTo.domNode, "active-text");
				div.appendChild(nodeMailTo.domNode);

				var userHover = new com.ibm.rdm.web.common.UserHover({projectURI : self.projectURI}, null);
				userHover.createHoverPopup(div, userResult);
				destroy.push(userHover);
			}
			
			//Pass the div back to the grid for display (and the hovers for destroying)
			functionToCall(mainDiv, destroy);

			return users;
		};
		
		var onProjectMemberCompleteTextOnly = function(users) {

			var userNames = [];
			for(var userUri in users) {
				userNames.push(users[userUri].name);
			}
			
			functionToCall(userNames.join(", "), []);

			return users;
		};
		
		//User may be an array. Ensure it always is
		if (!dojo.isArray(user)) {
			user = [user];
		}
		
		if (user.length === 0) {
			functionToCall("");
			return;
		}

		if (user.length > 0) {
			if(textOnly) {
				UserCache.getUsers(user)
					.addCallback(this, onProjectMemberCompleteTextOnly);
			}
			else {
				UserCache.getUsers(user)
					.addCallback(this, onProjectMemberCompleteDiv);
			}
		} else {
			//No users - no result
			functionToCall("");
		}
	},
	
	///// User
	_getDeferredUserValue: function (attributeURI, functionToCall, resource, textOnly) {
		var value = resource.getAttributeValue(attributeURI);
		if (value) {
			this._addDeferredUserDetails(functionToCall, resource, value, textOnly);
		} else {
			functionToCall("");
		}
	},
	
		///////////////////////////////////////////////////////////////////////////////////
	//// Attribute Display Functions

	///// Enumeration
	_getEnumValue: function (attributeURI, resource) {
		var i;
		var value = resource.getAttributeValue(attributeURI);
		var attrDef = this._typeCache.getProjectAttributeDefinition(attributeURI);
		var attrType = attrDef.getAttributeType();				
		if (value) {
			
			var attributeValue = function(val){
				var enumEntry = attrType.getEnumEntry(val);
				if (enumEntry) {
					return enumEntry.enumTitle;
				}
			};
						
			if(attrDef.isMultiValued() && value instanceof Array) {
				//sort by order of enums
				value.sort(function(enumValue1, enumValue2){
					return attrType.getEnumEntry(enumValue1).enumOrder - attrType.getEnumEntry(enumValue2).enumOrder;
				});		
				if (value.length > MVA_LIMIT) {
					var init = [];
					var expanded = [];
					for (i = 0; i<value.length; i++) {
						if (i < MVA_LIMIT) {
							init.push(attributeValue(value[i]));
						} else {
							expanded.push(attributeValue(value[i]));
						}
					}
					var initStr = init.join(', ');
					var expandedStr = ", " + expanded.join(', ');
					
					var expandMsg =  dojo.string.substitute(this._messages.enumExpand, [value.length]);
					
					var div = dojo.create('div', {className: "enumToggle"});
					var initSpan = dojo.create('span', {innerHTML: initStr}, div);
					var expandBtn = dojo.create('span', {
						className: "contracted enumBtn", 
						classToggle: "enumExpanded", 
						innerHTML: expandMsg
					}, div);
					
					var restSpan = dojo.create('span', {
						className: "expanded", 
						innerHTML: expandedStr
					}, div);
					
					var contractBtn = dojo.create('span', {
						className: "expanded enumBtn", 
						classToggle: "enumExpanded", 
						innerHTML: this._messages.enumContract
					}, div);
					
					value = div;
				} else {
					//Just display all values
					var titles = [];
					for (i = 0; i<value.length; i++){
						titles.push(attributeValue(value[i]));
					}
					value = titles.join(", ");
				}
			} else {
				value = attributeValue(value);
			}			
		}
		return value;
	},

	///// DateTime
	_getDateTimeValue: function (attributeURI, resource) {
		var value = DateTimeHelper.getDateTimeDisplayStringFromString(resource.getAttributeValue(attributeURI));
		return value;
	},

	///// Time
	_getTimeValue: function (attributeURI, resource) {
		var value = DateTimeHelper.getTimeDisplayStringFromTime(resource.getAttributeValue(attributeURI));
		return value;
	},

	///// Date
	_getDateValue: function (attributeURI, resource) {
		var value = DateTimeHelper.getDateDisplayStringFromString(resource.getAttributeValue(attributeURI));
		return value;
	},

	///// Duration
	_getDurationValue: function (attributeURI, resource) {
		var value = DateTimeHelper.getDurationDisplayString(resource.getAttributeValue(attributeURI));
		return value;
	},
	
	///// Boolean
	_getBooleanValue: function (attributeURI, resource) {
		var value = AttributeDisplayHelper.getBooleanDisplayString(resource.getAttributeValue(attributeURI));
		return value;
	},
 
	///// Float and Integer
	_getNumericValue: function (attributeURI, resource) {
		//We don't have to escape these, as no-one can put 'nasty' values in there!
		var res = resource.getAttributeValue(attributeURI);
		var value = "";
		if (res) {
			value = dojo.number.format(res);
		}
		return value;
	},
	
	///// Other Values
	_getPlainValue: function (attributeURI, resource) {
		var value = resource.getAttributeValue(attributeURI);
		return value;
	},

	
	createResourceLinkHTML: function(resource, includeId, noIcon, label, /*?Boolean/String*/ noTabFocus, /*Boolean?*/ isBaseArtifactLink, /* URI */moduleBaseline, columnType) {
		// summary:
		//		Creates a resource link to open a viewer for the resource given.
		// includeId:
		//		If true, the id and a colon are added to the name.
		// noIcon:
		//		If true, no icon is displayed.
		// label:
		//		If provided, this label is used instead of the name and id from
		//		the resource.
		// noTabFocus:
		//		If provided and true, the resultant link is not given focus using the
		//		tab key.  If set to a string value that value is set as the tabIndex
		//		(so may be set true or to something like "-2").
		// moduleBaseline: 
		//		Provided if the created link is to another artifact within the module baseline.
		//		For module baselines, no revision is used.
		// returns:
		//		An object with:
		//		  .div - the node for the resource link.
		//		  .destroy - the object that destroy should be called on when this row is no longer necessary.
		var contentType = null;
		if (resource.getWrappedResourceURI && resource.getWrappedResourceURI()){
			contentType = resource.getWrappedResourceMimeType();
		}
		else{
			contentType = resource.getMimeType();
		}

		// Add the resource image.
		var div = dojo.create('div', {"class": "icon-resourcelink"});
		var hasImage = false;
		if (!noIcon && contentType) {
			var imageNode = com.ibm.rdm.web.common.ArtifactIcons.getResourceSmallImage(resource);
			dojo.addClass(imageNode, "content-type-entry-image");
			if (!isBaseArtifactLink) {
				div.appendChild(imageNode);
			} else {
				var baseArtifactWrapper = dojo.create("div", {
					"class": "baseArtifactImgWrapper"
				}, div);
				
				var baseArtifactIcon = document.createElement('img');
				baseArtifactIcon.src = this._blankGif;
				baseArtifactIcon.className = "baseartifact-icon";
				
				baseArtifactWrapper.appendChild(imageNode);
				baseArtifactWrapper.appendChild(baseArtifactIcon);
			}
			hasImage = true;
		}

		// Add the resource link.
		var resourceURI = resource.getResourceURI();
		var baselineURI = null;
		if (resource.getSnapshotURI && resource.getSnapshotURI()) {
			baselineURI = resource.getSnapshotURI();
		}
		if (moduleBaseline) {
			baselineURI = moduleBaseline;
		}
		var hoverURL = resourceURI;
		
		if (!label && label !== "") {
			label = resource.getTitle();
			if (!label) {
				label = "";
			}
		}
		
		if (includeId) {
			label = resource.getID()+":"+label;
		}
		
		var moduleInfoArgs = null;
		if (resource.getModuleTitle && resource.getModuleTitle()) {
			moduleInfoArgs = {uri:resource.getModuleHostURI(), title:resource.getModuleTitle()};
		}
		if (this._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
			switch (columnType) {
				case ArtifactConstants.TITLE_COL:
				case ArtifactConstants.ID_COL:
				case ArtifactConstants.MODULE_CONTENT_COL:
					//open core artifact instead of modulefact if navigating from module view
				//	hoverURL = resource.getModuleBoundArtifact();
					moduleInfoArgs = null;
					//moduleInfoArgs = {uri:this._moduleStructure.moduleURI};
					break;
			}
			
		}
		else if (moduleInfoArgs) {
			switch (columnType) {
				case ArtifactConstants.ID_COL:
					moduleInfoArgs.title = null;
					break;
			}
			
		}

		//In the case where the resource is not part of the snapshot for collection, don't make the link
		var link;
		var destroy;
		link = ResourceLinksHandler.createLink({
			uri: hoverURL,
			label: label, 
			baselineURI: baselineURI,
			revisionURI: (baselineURI && !moduleBaseline)?resource.getRevision():null,
			usedInModuleArgs: moduleInfoArgs
		});
		var id = dojo.attr(link, "id");
		destroy = dijit.byId(id);
		if (noTabFocus) {
			if (!dojo.isString(noTabFocus)) {
				noTabFocus = "-1";
			}
			dojo.attr(link, "tabIndex", noTabFocus);
		}

		// Add the link in its own wrapper div, so we can make text wrap nicely.
		var linkWrapperDiv = dojo.create('div', null, div);
		if (hasImage) {
			if(isBaseArtifactLink){
				dojo.addClass(linkWrapperDiv, "with-image-and-base-artifact"); 
			}else{
				dojo.addClass(linkWrapperDiv, "with-image");
			}		
		}
		
		BidiUtils.applyTextDirSetting(linkWrapperDiv, label, true);//bidi
		
		linkWrapperDiv.appendChild(link);
		return {div: div, destroy: destroy};
	},
	
	showTags: function(functionToCall, resource) {
		// summary:
		//		Callback given to the grid for tag column.  Populates a cell that displays tags.
		
		var getBoundArtifactModuleForResource= function(resource){				
			var mapOfModuleBinding= [];
			//HACK! Handle possible multiple bound artifacts returned by the API. Defect 54280
			var moduleBoundArtifactResults = resource.getModuleBoundArtifact();
			var resourceURI = resource.getResourceURI();
			var moduleBoundArtifact;
			if(dojo.isArray(moduleBoundArtifactResults)){
				for(var i=0; i < moduleBoundArtifactResults.length; i++){
					moduleBoundArtifact = moduleBoundArtifactResults[i];
					mapOfModuleBinding[resourceURI]=moduleBoundArtifact;
				}		
			}else{
				moduleBoundArtifact = moduleBoundArtifactResults; //Single result returned by the server
				mapOfModuleBinding[resourceURI]=moduleBoundArtifact;
			}			
			return mapOfModuleBinding[resourceURI];
		};
		
		var boundArtifactModule= null; 
		if(typeof resource.getModuleBoundArtifact === "function"){
			boundArtifactModule = getBoundArtifactModuleForResource(resource);
		}
		
		var populateTagCell = function(resourceTags){
			// summary:
			//		Helper function that does the actual display - formats cell and
			//		invokes callback function on the grid to display it.
			var private_tags = [];
			var public_tags = [];
			var private_tags_base_artifact = [];
			var public_tags_base_artifact= [];
			for (var tagResourceURI in resourceTags) {
				var tag = resourceTags[tagResourceURI];
				if (tag.scope === "public") {
					if (boundArtifactModule && tag.subject){
						if(tag.subject === boundArtifactModule){
							//The current is a tag for the base artifact
							public_tags_base_artifact.push(BidiUtils.setTextDirUCC(tag.title, true)); //bidi
						}
						else{
							public_tags.push(BidiUtils.setTextDirUCC(tag.title, true)); //bidi
						}
					}else{
						//Assume it is a tag for the current resource since no
						//base artifact was found
						public_tags.push(BidiUtils.setTextDirUCC(tag.title, true)); //bidi
					}				
				} else {					
					if (boundArtifactModule && tag.subject){
						if(tag.subject === boundArtifactModule){
							//The current is a tag for the base artifact
							private_tags_base_artifact.push(BidiUtils.setTextDirUCC(tag.title, true)); //bidi
						}
						else{
							private_tags.push(tag.title);
						}
					}else{
						//Assume it is a tag for the current resource since no
						//base artifact was found
						private_tags.push(BidiUtils.setTextDirUCC(tag.title, true)); //bidi
					}		
				}
			}
			
			var publicTagAdded, privateTagAdded = false, iconImg, tagTitle, i;
			var div = document.createElement("div");
			
			var publicDiv;
			if (public_tags.length > 0) {	
				publicTagAdded =true;
				publicDiv = dojo.create("div", {"class": "public-tags"}, div);
				//Process public tags
				var publicTagsString = public_tags.join(", ");
				var publicSpan = DomUtils.createSpannedTextNode("", publicTagsString);
				publicDiv.appendChild(publicSpan);
			}
			if(public_tags_base_artifact.length > 0){			
				if(publicTagAdded){
					//Add a comma to the results
					div.appendChild(document.createTextNode(", "));
				}else{
					publicDiv = document.createElement("div");
					div.appendChild(publicDiv);
					dojo.addClass(publicDiv, "public-tags");
				}
				//Traverse any public tags for the base artifact 
				for(i=0; i < public_tags_base_artifact.length; i++){	
					tagTitle = public_tags_base_artifact[i];
					var publicSpanBaseArtifact = document.createElement("span");
					dojo.addClass(publicSpanBaseArtifact, "base-artifact");
					publicSpanBaseArtifact.appendChild(document.createTextNode(tagTitle));
					div.appendChild(publicSpanBaseArtifact);
					
					if(i < public_tags_base_artifact.length -1 ){
						//Add a comma to the results
						div.appendChild(document.createTextNode(", "));
					}
				}
			}
			var privateDiv;
			if (private_tags.length > 0) {
				privateTagAdded =true;				
				privateDiv = dojo.create("div", {"class": "private-tags"}, div);
				var privateTagsString = private_tags.join(", ");
				var privateSpan = DomUtils.createSpannedTextNode("", privateTagsString);
				privateDiv.appendChild(privateSpan);
			}
			if(private_tags_base_artifact.length > 0){			
				if(privateTagAdded){				
					//Add a comma to the results
					privateDiv.appendChild(document.createTextNode(", "));
				}else{
					privateDiv = document.createElement("div");
					div.appendChild(privateDiv);
					dojo.addClass(privateDiv, "private-tags");
				}
				//Traverse any public tags for the base artifact 
				for(i=0; i < private_tags_base_artifact.length; i++){	
					var privateSpanBaseArtifact = document.createElement("span");
					tagTitle = private_tags_base_artifact[i];
					dojo.addClass(privateSpanBaseArtifact, "base-artifact");
					privateSpanBaseArtifact.appendChild(document.createTextNode(tagTitle));
					privateDiv.appendChild(privateSpanBaseArtifact);
					
					if(i < private_tags_base_artifact.length -1 ){
						//Add a comma to the results
						div.appendChild(document.createTextNode(", "));
					}
				}
			}
			//TODO: Handle disposal of images!
			functionToCall(div);
		};
		
		
		var _showTags = dojo.hitch(this, function(){
			// If the resource already has tags available, fetched by the view service, display them immediately,
			// otherwise fetch and display them.
			if (resource.tags) {
				populateTagCell(resource.tags);
			} else {
				var getTagsDeferred = ProjectCache.getResourceTags(resource.getResourceURI());
				getTagsDeferred.addCallback(this, function(resourceTags) {
					populateTagCell(resourceTags);
				});
			}
		});
		
		
		if (resource.__createSimilarApplyTagsDeferred) {
			// We are doing a create similar.  Only show tags once this deferred has resolved,
			// indicating that the tags have been created on the server.
			resource.__createSimilarApplyTagsDeferred.addCallback(this, _showTags);
		}
		else {
			_showTags();
		}
	},
	
	showSuspicion: function(functionToCall, resource, key, containerNode) {
		var suspectEntryUris = resource.suspectEntries;
		if (suspectEntryUris && suspectEntryUris.length > 0) {
			var suspicionStatusWidget = new com.ibm.rdm.web.suspicion.SuspicionStatus({
				source: SuspectLinksUtil.EVENT_SOURCE.grid,
				projectURI: resource.projectURI,
				suspectEntryURIs: suspectEntryUris,
				targetResourceURI: resource.resourceURI,
				otherTargetResourceURI: (
					this._showBaseProperties && this._moduleStructure ?
					resource.getModuleBoundArtifact() :
					null
				)
			});
			this._toDestroy.push(suspicionStatusWidget);
			functionToCall(suspicionStatusWidget.domNode, suspicionStatusWidget);
		}
		else {
			functionToCall("");
		}
	},
	
	_onModuleStructureChange: function(){
		// summary:
		//		handler for when structure changes, which may result in a need to
		//		make updates to the display of some artifacts.
		this._updateHeadingAndSectionNumbers();
	},
	
	_updateHeadingAndSectionNumbers: function () {

		var gridNode = this._grid.domNode;
		dojo.query('[objectid]', gridNode).forEach(dojo.hitch(this, function(rowNode){
			var resource = this._moduleStructure.getArtifactByURI(dojo.attr(rowNode, "objectId")); 
			if (resource) {
				this._updateExpansionStateForRow(rowNode, resource);
			}
			
			this._updateHeadingAndSectionNumbersForRow(rowNode, resource);
		}));
	},
	
	_updateExpansionStateForRow: function(rowNode, resource) {
		// summary:
		//		Adds the correct class names to the row depending on whether
		//		the resource it represesnts in the module has children and
		//		whether it is collapsed or expanded.
		var contentCellNode = dojo.query(".rdm-artifactgrid-contentcellwidget", rowNode)[0],
			hasChildren = resource.children.length > 0;
		
		if (contentCellNode) {
			if (resource.collapsed) {
				dojo.addClass(contentCellNode, "collapsed");
			}
			else {
				dojo.removeClass(contentCellNode, "collapsed");
			}
			
			if (hasChildren) {
				dojo.addClass(contentCellNode, "expandable");
			}
			else {
				dojo.removeClass(contentCellNode, "expandable");
			}
		}
		
	},
	
	_updateHeadingAndSectionNumbersForRow: function(rowNode, resource){
		// summary:
		//		Updates the styling and content of headers and section numbers.  Relies
		//		on some classes and content that have previously been put in the DOM
		//		for heading content and to identify section numbers.
		
		// Ideally this code would be somewhere in ArtifactDisplay
		
		if (!rowNode) {
			return;
		}
		var structureEntry = resource;
		if (!structureEntry) {
			return;
		}
		
		this._updateDepth(structureEntry, rowNode);
		this._updateHeadingStyling(structureEntry, rowNode);
		this._updateSectionNumberText(structureEntry, rowNode);
	},

	_updateSectionNumberText: function(structureEntry, node) {		
		// update section number text for a node.  The node may be a whole row, or a single cell within
		// a row.  When acting on a row, it catches section numbers in multiple columns in the row
		// (e.g. in main column headings and in section number column).
		var sectionNumber = structureEntry.sectionNumber;
		dojo.query(".sectionNumber>span.number", node).forEach(function(sectionNumberElement){
			if (sectionNumberElement.innerHTML !== sectionNumber) {
				sectionNumberElement.innerHTML = sectionNumber;
			}
		}, this);
	},
	
	_updateDepth: function(structureEntry, rowNode) {
		// summary:
		//		Update the depth tag so that the row is
		//		styled appropriate to the level (e.g. indented)
		//		We add depth classes in the range depth1 -> depth10plus
		//		Any existing depth class is removed, in case the node has changed depth.
		var depth = structureEntry.depth;
		var depthClass = "depth" + ( (depth < 10) ? depth : "10plus" );
		dojo.removeClass(rowNode, "depth1 depth2 depth3 depth4 depth5 depth6 depth7 depth8 depth9 depth10plus");
		dojo.addClass(rowNode, depthClass);
	},
	
	_updateHeadingStyling: function(structureEntry, rowNode) {
		// summary:
		//		If the given row contains a heading, updates the heading
		//		styling.
		if (!dojo.hasClass(rowNode, "heading") && !structureEntry.isHeading) {
			// nothing to do
			return;
		}

		if (structureEntry.isHeading) {
			dojo.addClass(rowNode, "heading");
		} else {
			dojo.removeClass(rowNode, "heading");
		}


		var widget = this.findContentCellWidget(rowNode);
		if (widget){
			widget.updateHeadingStyling(structureEntry.isHeading, structureEntry.depth, structureEntry.sectionNumber);
		}
	},
	
	findContentCellWidget: function(node){
		return com.ibm.rdm.web.artifactgrid.ContentCellWidget.findInNode(node);
	},
	
	findMarkAllNext: function (key, col_id, searchTerm, markClassName) {

		this._ptMarkedPieces[col_id] = [];
		this._ptCurrent[col_id] = -1;
	
		var cellHtml = this._grid.getCellHtml(key, col_id);
		if (cellHtml) {
			this._searchDom(cellHtml, searchTerm, markClassName, this._ptMarkedPieces[col_id]);
		}
	},
	
	findMarkAllPrev: function (key, col_id, searchTerm, markClassName) {

		this._ptMarkedPieces[col_id] = [];
	
		var cellHtml = this._grid.getCellHtml(key, col_id);
		if (cellHtml) {
			this._searchDom(cellHtml, searchTerm, markClassName, this._ptMarkedPieces[col_id]);
		}
		this._ptCurrent[col_id] = this._ptMarkedPieces[col_id].length;
	},
	
	nextFind: function (key, col_id, selectClassName, replaceClassName, done, noMore) {
		var current;
		var currMatch = this._ptCurrent[col_id];
		var noOfMatches = this._ptMarkedPieces[col_id].length;
		//Remove current highlight
		if (currMatch >= 0  && currMatch < noOfMatches) {
			current = this._ptMarkedPieces[col_id][currMatch];
			dojo.removeClass(current, selectClassName);
		}
		//Move forward
		currMatch++;
		if (currMatch > noOfMatches) {
			currMatch = noOfMatches;
		}
		if (currMatch < noOfMatches) {
			current = this._ptMarkedPieces[col_id][currMatch];
			if (dojo.hasClass(current, replaceClassName)) {
				this._ptCurrent[col_id] = currMatch;
				this.nextFind(key, col_id, selectClassName, replaceClassName, done, noMore);
			} else {
				dojo.addClass(current, selectClassName);
				dijit.scrollIntoView(current);
				this._ptCurrent[col_id] = currMatch;
				done();
			}
		} else {
			this._ptCurrent[col_id] = currMatch;
			noMore();
		}
	},
	
	prevFind: function (key, col_id, selectClassName, replaceClassName, done, noMore) {
		var current;
		var currMatch = this._ptCurrent[col_id];
		//Remove current highlight
		if (currMatch >= 0 && currMatch < this._ptMarkedPieces[col_id].length) {
			current = this._ptMarkedPieces[col_id][currMatch];
			dojo.removeClass(current, selectClassName);
		}
		//Move backward
		currMatch--;
		if (currMatch < -1) {
			currMatch = -1;
		}
		if (currMatch >= 0) {
			current = this._ptMarkedPieces[col_id][currMatch];
			if (dojo.hasClass(current, replaceClassName)) {
				this._ptCurrent[col_id] = currMatch;
				this.prevFind(key, col_id, selectClassName, replaceClassName, done, noMore);
			} else {
				dojo.addClass(current, selectClassName);
				dijit.scrollIntoView(current);
				this._ptCurrent[col_id] = currMatch;
				done();
			}
		} else {
			this._ptCurrent[col_id] = currMatch;
			noMore();
		}
	},
	
	replaceFind: function (key, col_id, replaceTerm, replaceClassName, findClassNames, onDone) {
		var currMatch = this._ptCurrent[col_id];
		var current = this._ptMarkedPieces[col_id][currMatch];
		current.childNodes[0].nodeValue = replaceTerm;
		for (var i=0; i<findClassNames.length; i++) {
			dojo.removeClass(current, findClassNames[i]);
			dojo.removeClass(current, findClassNames[i]);
		}
		dojo.addClass(current, replaceClassName);
		onDone();
	},
	
	getFindReplaceOccurrences: function (key, col_id) {
		return this._ptMarkedPieces[col_id];
	},
	
	getCurrentFindNode: function (key, col_id) {
		var currMatch = this._ptCurrent[col_id];
		var current = this._ptMarkedPieces[col_id][currMatch];
		return current.childNodes[0];
	},
	
	clearFind: function (key, col_id) {
		//Remove all markup
		this._clearUpDom(this._ptMarkedPieces[col_id]);
		
		delete this._ptMarkedPieces[col_id];
		delete this._ptCurrent[col_id];
	},
	
	
	_searchDom: function(cellHtml, searchTerm, markupClass, matchArray) {
		// summary:
		//		Searches a dom for a given term (case insensitive only). Entries are found 
		// cellHtml:
		//		The dom to be searched.
		// searchTerm:
		//		A string that is being sought. It will be compared in a case insensitive manner.
		// markupClass:
		//		The class name that matches will be put into a span and marked with.
		// matchArray:
		//		A collection parameter, gathering all the 'span' elements that contain the matches.
		var i, isIE = !!dojo.isIE;
		searchTerm = searchTerm.toUpperCase();
		
		if (cellHtml.nodeType === Node.ELEMENT_NODE) {
			var children = cellHtml.childNodes;
			for (i=0; i<children.length; i++) {
				var child = children[i];
				if (dojo.hasClass(child, com.ibm.rdm.web.editor.EditorConstants.RICH_TEXT_EMBED_CONTENT_CLASS)) {
					continue;
				} else if (dojo.hasClass(child, ArtifactConstants.CONTENT_WRAPPED_RESOURCE_CLASS)) {
					continue;
				} else if (!(child.nodeName === "SPAN" && dojo.hasClass(child, markupClass))) {
					this._searchDom(child, searchTerm, markupClass, matchArray);
				}
			}
		} else if (cellHtml.nodeType === Node.TEXT_NODE) { 
			var string = cellHtml.nodeValue.toUpperCase();
			//console.log("Trying to match ", searchTerm, "to", string);
			var match = string.indexOf(searchTerm);
			// whole word test
			//if (match !== -1) {
			//	var regex = new RegExp( '\\b' + searchTerm + '\\b' );
			//	var matched = string.search(regex);
			//	if (match !== matched) {
			//		match = -1;
			//	}
			//}
			if (match !== -1) {
				var span = document.createElement('span');

				// split text into 3 parts: before, found and after
				var mid = cellHtml.splitText(match);
				mid.splitText(searchTerm.length);

				// then assign mid part to SPAN
				mid.parentNode.insertBefore(span, mid);
				mid.parentNode.removeChild(mid);
				span.appendChild(mid);
				dojo.addClass(span, markupClass);
				matchArray.push(span);
			}
		}
	},
	
	_clearUpDom: function(matchArray) {
		var i, node, prevNode, prevText, nextNode, nextText, fullText;
		
		var isIE = !!dojo.isIE;
		if (isIE) {
			if (matchArray) {
				for (i =0; i< matchArray.length; i++) {
					node = matchArray[i];
					prevNode = node.previousSibling;
					prevText = prevNode ? (prevNode.innerText || prevNode.data) : "";
					nextNode = node.nextSibling;
					if (nextNode && nextNode.data) {
						nextText = nextNode.data;
					} else {
						nextNode = null;
						nextText = "";
					}
					
					fullText = prevText + node.innerText + nextText;
					
					if (prevNode) {
						if (prevNode.innerText) {
							prevNode.innerText = fullText;
						} else {
							prevNode.data = fullText;
						}
					}
					
					if (node.parentNode) {
						node.parentNode.removeChild(node);
					}
					if (nextNode && nextNode.parentNode) {
						nextNode.parentNode.removeChild(nextNode);
					}
				}
			}
		} else {
			if (matchArray) {
				for (i =0; i< matchArray.length; i++) {
					node = matchArray[i];
					prevNode = node.previousSibling;
					
					prevText = prevNode ? prevNode.textContent : "";
					nextNode = node.nextSibling;
					nextText = nextNode ? nextNode.textContent : "";
					
					fullText = prevText + node.textContent + nextText;
					
					if (prevNode) {
						prevNode.textContent = fullText;
					}
					
					if (node.parentNode) {
						node.parentNode.removeChild(node);
					}
					if (nextNode && nextNode.parentNode) {
						nextNode.parentNode.removeChild(nextNode);
					}
				}
			}
		}
	},
	
	_createContentDisplayWidget: function(resource, artifactURI, cellOptions, onRenderAvailable, rowKey, columnId, editorCallbacks) {
		// summary:
		//		Creates a widget for rendering more than one attribute.
		// resource:
		//		resource or partial resource to be displayed
		// artifactURI:
		//		identifier for the resource
		// cellOptions:
		//		Options for tweaking details of the cell display.
		//		e.g. {	titleEditable: true,			//false to prevent title from being edited.
		//				primaryTextEditable: true,		//false to prevent primaryText from being edited.
		//			 }
		//		true to display both title and primary text for non-heading artifacts.
		// onRenderAvailable:
		//		Callback to be called with the created domNode when the content has
		//		been rendered sufficiently for display.
		// rowKey:
		//		The key identifying the row that will host the widget.
		// columnId:
		//		Identifier for the column that will host the widget.
		// editorCallbacks:
		//		A hash containing functions that the editor can use as callbacks to the containing grid.  Used
		//		to allow hotkeys that are caught by the editor to provoke actions on the overall grid.
		
		var structureInfo = this._moduleStructure.getArtifactByURI(artifactURI);

		// The following 'if' is here to address Defect 83704.  We account for
		// the fact that the user has created a "stub row" in which team area
		// permissions checking has found that the none of the currently
		// displayed attributes are editable.  In this case the normal flow
		// of making use of the EditManager to do our "saving" is bypassed.
		// Instead, after creating the stub we immediately save the row
		// (in _saveUnmodifiableStubRow from ModuleArtifactCommands) and here
		// we account for the fact that module structure doesn't contain a
		// mapping for this row keyed by 'artifactURI' (as the row was never
		// moved to edit mode).  Instead the module structure has an entry
		// for this stub based on the URI found in the resource's
		// 'stubIdOfUserUnmodifiableRow' property.
		if (!structureInfo && resource.stubIdOfUserUnmodifiableRow) {
			structureInfo = this._moduleStructure.getArtifactByURI(resource.stubIdOfUserUnmodifiableRow);
		}

		var lockingURI = this._dataCache.getBaseArtifactForResource(artifactURI);
		
		var cellWidget = new com.ibm.rdm.web.artifactgrid.ContentCellWidget({
			projectURI: this._projectURI,
			baselineURI: this._baselineURI,
			displayDate: this._displayDate,
			resource: resource,
			artifactURI: artifactURI,
			lockingURI: lockingURI,
			isHeading: structureInfo.isHeading,
			sectionNumber: structureInfo.sectionNumber,
			depth: structureInfo.depth,
			titleEditable: cellOptions.titleEditable,
			primaryTextEditable: cellOptions.primaryTextEditable,
			grid: this._grid,
			expandable: structureInfo.children.length > 0,
			rowKey: rowKey,
			columnId: columnId,
			editorCallbacks: editorCallbacks,
			collapsed: structureInfo.collapsed,
			artifactDisplay: this,
			moduleBaselineURI: this._moduleBaselineURI,
			onToggleChildren: dojo.hitch(this, function () {
				this._moduleStructure.collapseOrExpandSection(artifactURI);
			})
		});
		onRenderAvailable(cellWidget.domNode, cellWidget);
					
		// 67289: Scale image artifacts in modules to the width of its containing ContentCellWidget.
		if (this._moduleStructure
			&& cellWidget.resource.getWrappedResourceURI
			&& cellWidget.resource.getWrappedResourceURI()
		) {
			var img = dojo.query(".primaryTextWrapper img", cellWidget.domNode)[0];
			if (img) {
				img.style.maxWidth = "100%";
			}
		}
	}
	
});


com.ibm.rdm.web.artifactgrid.ArtifactDisplay.loadEmbeddedResources = function (args) {
		// if this is a module (do we need to bother with that check ?) run an
		// embedded resource loader over the cell to expand out embedded artifacts
		// this will also expand out any RRCx artifacts directly in the content column
		
		var loader = new EmbeddedResourceLoader(args.textNode, {
			projectSnapshotURI: args.baselineURI,
			displayDate: args.displayDate,
			moduleBaselineURI: args.moduleBaselineURI,
			onCreate:  function (decorator) {
				if (args.isExtendedType) {
					var img = dojo.query("img", decorator.getContentNode())[0];
					if (args.grid.pageModel) {
						args.grid.pageModel.addImage(args.resourceURI, img);
					}
					
					// 67289: Scale RRCx artifacts in modules to the width of of its containing ContentCellWidget.
					if (args.grid.module) {
						var imgContainer = decorator.getContentNode().children[0].children[0];
						if (imgContainer) {
			
							// Strip off the dimensions on the container that you get from the converter.
							//   Ugly, I know.. But without some converter changes, we are stuck having to do this.
							imgContainer.style.width = "";
							imgContainer.style.height = "";
						}
			
						// Set the max-width on the image itself
						var img = dojo.query("img", imgContainer)[0];
						if (img) {
							img.style.maxWidth = "100%";
						}
					}
				}         
				// measure grid heights
				var queryAndHookupResourceLinks = function (root) {
					dojo.query("a", root).forEach(function (node) {
						if (! dojo.hasClass(node, "jazz-ui-ResourceLink") && !(dojo.hasClass(node, "button"))) {
							var link,   
								idAttr = dojo.attr(node, "id");
								
							if (idAttr) {
								// prevent widget ID name clashes.
								dojo.removeAttr(node, "id");
							}   
							link = new jazz.ui.ResourceLink({
								retainLinkText: true,
								lazyFetch: true
							}, node);
								
							// TODO: add connections to an [] to destroy when this widget is destroyed
							dojo.connect(node, "onclick", function (evt) {
								dojo.stopEvent(evt);
								com.ibm.rdm.web.viewer.viewerutils.openResourceUsingAppropriateViewer(
									dojo.attr(node, "href"));
							});
							
							args.addDestroy(link);
						}
					});
				};
				
				queryAndHookupResourceLinks(decorator.getRootNode());
				
				if (decorator._embeddedWrapper) {	//add css class to show the embedded artifact in normal mode (todo: it should be done only on the initial creation, otherwise it should maintain last state)
					dojo.addClass(decorator._embeddedWrapper, "showContent");
				}
				
				if (typeof decorator.contentChanged === "function") {
					dojo.connect(decorator, "contentChanged",  function(){
						queryAndHookupResourceLinks(decorator.getRootNode());
						args.grid.measureRows();
					});
				}
				
				// 51744 - measure the grid if the user clicks in the toolbar, not ideal but should work for now. It's difficult to more cleanly
				// attach to more "semantic" events, since there is some complexity around getting these functions to work in a CK Editor.
				dojo.query(".minimiseButton,.maximiseButton,.showContentButton", decorator.getRootNode()).connect("onclick", function () {
					  args.grid.measureRows();
				});
				
				args.grid.measureRows();
			},
			onImageLoad: dojo.hitch(this, function () {
				// measure grid heights
				args.grid.measureRows();
			}),
			remove: false
		});
		
		loader.loadEmbeddedResources();
	};

}) ();
