/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2015. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.rdm.web.artifactgrid.ArtifactLinks");

dojo.require("com.ibm.rdm.web.artifactgrid.ArtifactConstants");
dojo.require("com.ibm.rdm.web.artifactgrid.LinkNavigationWidget");
dojo.require("com.ibm.rdm.web.project.DashboardConstants");
dojo.require("com.ibm.rdm.web.calm.CalmConstants");
dojo.require("com.ibm.rdm.web.calm.CalmLinkRowWidget");
dojo.require("com.ibm.rdm.web.calm.CalmLinkUtils");
dojo.require("com.ibm.rdm.web.calm.CalmRegistry");
dojo.require("com.ibm.rdm.web.calm.DerivedLinkUtils");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.MessageForm");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.ScrollableMenu");
dojo.require("com.ibm.rdm.web.editor.InlineGridEditor");
dojo.require("com.ibm.rdm.web.links.Link");
dojo.require("com.ibm.rdm.web.links.LinkDialog");
dojo.require("com.ibm.rdm.web.links.LinkTypeUtils");
dojo.require("com.ibm.rdm.web.model.resources.ResourceManager");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.project.EditManager");
dojo.require("com.ibm.rdm.web.server.cache.ProcessOperationsCache");
dojo.require("com.ibm.rdm.web.server.client.utils.LinkUtils");
dojo.require("com.ibm.rdm.web.suspicion.SuspectLinksUtil");
dojo.require("com.ibm.rdm.web.suspicion.SuspicionStatus");
dojo.require("com.ibm.rdm.web.common.StatusDisplay");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.model.types.TypesManager");
dojo.require("dijit.MenuItem");
dojo.require("jazz.ui.Menu");
dojo.require("jazz.ui.toolbar.MenuButton");
dojo.require("com.ibm.rdm.web.extensions.LinkedArtifactFetcher");

dojo.requireLocalization("com.ibm.rdm.web.artifactgrid", "ArtifactLinks");
dojo.requireLocalization("com.ibm.rdm.web.artifactgrid", "ArtifactDisplay");
dojo.requireLocalization("com.ibm.rdm.web.links", "LinksListWidget");

var ArtifactConstants = com.ibm.rdm.web.artifactgrid.ArtifactConstants;
var CalmRegistry = com.ibm.rdm.web.calm.CalmRegistry;
var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
var ProcessOperationsCache = com.ibm.rdm.web.server.cache.ProcessOperationsCache;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var DerivedLinkUtils = com.ibm.rdm.web.calm.DerivedLinkUtils;
var LinkTypeUtils = com.ibm.rdm.web.links.LinkTypeUtils;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var QueryUtils = com.ibm.rdm.web.util.queryutils;
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;
var CalmLinkUtils = com.ibm.rdm.web.calm.CalmLinkUtils;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var LinkUtils = com.ibm.rdm.web.server.client.utils.LinkUtils;
var linkutils = com.ibm.rdm.web.links.linkutils;
var ScrollableMenu = com.ibm.rdm.web.common.ScrollableMenu;
var SuspectLinksUtil = com.ibm.rdm.web.suspicion.SuspectLinksUtil;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var StatusDisplay = com.ibm.rdm.web.common.StatusDisplay;
var TypesManager = com.ibm.rdm.web.model.types.TypesManager;

//Rowtype for a row built from resource info records.
var RESOURCE_ROW = com.ibm.rdm.web.artifactgrid.ArtifactConstants.RESOURCE_ROW;

//Variable to allow unique key generation
var uniqueId = 1;


var MAXURLSizeInColumn = 25;

// tabIndex to put on nodes that are targets for internal navigation.
var INTERNAL_NAVIGATION_TABINDEX = "-2";

var DEFAULT_TRACE_FORMAT = {
	showIcon: true,
	showTitle: true,
	showAttributeName: true
};

var OBJ_TO_SUB_LINKS_EXCLUDE_FROM_VIEW = [
	"http://open-services.net/ns/rm#elaboratedBy",
	"http://open-services.net/ns/rm#specifiedBy",
	"http://open-services.net/ns/rm#elaborates",
	"http://open-services.net/ns/rm#specifies"
];

/*
 * Provides link loading and querying behaviour for the grid.
 */
dojo.declare(
		"com.ibm.rdm.web.artifactgrid.ArtifactLinks", null,
{
	//The project URI for the context
	_projectURI: null,
	
	//The callback to allow enabling controls disabled for the duration of the edit.
	_enableControlsCallback: null,
	
	//Messages for localisation
	_messages: null,
	
	//Messages for localization, specifally for deleting link messages
	_linkMessages: null,
	
	//RequirementType Cache
	_typeCache: null,
	
	//ArtifactDisplay
	_display: null,
	
	//Link to the underlying context
	context: null,
	
	//The underlying grid
	grid: null,
	
	//containers to allow proper destruction
	_editImageHandles: null,
	_toDestroy: null,
	
	//The operations allowed.
	_projectOperations: null,
	
	// Linking preferences for the project
	_linkTypePreferences: null,
	
	// Attributes to display in traceability columns
	_traceabilityAttributesByColumn: null,
	
	// Format to apply to attributes displayed in traceability columns
	_traceabilityAttributeFormatByColumn: null,
	
	_clmFilters: null,
	
	_blankGif: dojo.moduleUrl("dojo", "resources/blank.gif").toString(),
	
	SYSTEM_LINKS_EXCLUDED_FROM_VIEW_URIS: null,

	constructor: function(context) {
		
		// This is necessary because of a defect found
		// post-string-freeze in 4.0.1 RC1. This will safely
		// extend the messages dictionary with the
		// ArtifactDisplay strings, and as long as we keep
		// the ArtifactLinks getLocalization call at the
		// end, collisions will be resolved in favor of the 
		// correct strings for this class. --Christian
		this._messages = dojo.mixin(
			dojo.i18n.getLocalization("com.ibm.rdm.web.artifactgrid",
					"ArtifactDisplay"), 
			// THIS ONE MUST ALWAYS BE LAST!
			dojo.i18n.getLocalization("com.ibm.rdm.web.artifactgrid",
					"ArtifactLinks"));
		
		this._linkMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.links", "LinksListWidget");

		this.context = context;
		this._projectURI = context.projectURI;
		this._baselineURI = context.baselineURI;
		this._typeCache = context.typeCache;
		this._dataCache = context.dataCache;
		this._isInHistoryMode = context.isInHistoryMode;
		this._isInReviewMode = context.isInReviewMode;
		this._display = context.display;
		this._commands = context.commands;
		this._grid = context.grid;
		this._onGetLinkedRowsForLinktypeForColumn = context.onGetLinkedRowsForLinktypeForColumn;
		this._getLinkedRowsForLinktypeForArtifact = context.getLinkedRowsForLinktypeForArtifact;
		this._traceabilityAttributesByColumn = {};
		this._traceabilityAttributeFormatByColumn = {};
		this._editImageHandles = [];
		this._toDestroy = [];
		this.grid = context.grid;
		
		this.SYSTEM_LINKS_EXCLUDED_FROM_VIEW_URIS = [];
	},
	
	setProjectOperations: function(projectOperations) {
		this._projectOperations = projectOperations;
	},
	
	setCLMFilters: function(clmFilters) {
		this._clmFilters = clmFilters;
	},
	
	destroy: function() {
		dojo.forEach(this._editImageHandles, function(entry, i){
			dojo.disconnect(entry);
		});
		this._editImageHandles = [];
		dojo.forEach(this._toDestroy, function(entry, i){
			entry.destroyRecursive();
		});
		this._toDestroy = [];
	},
////////////////////////////////////////////////////////////////////////////////////////
////  Link Creation methods

	initialiseLinkCreation: function(menuLayout, menuOperations, caller) {
		//Summary: called when the linktypes are available. Must create
		//	the operations and the menu_layout for creating them in the grid, and add columns to the grid for display as columns.

		var self = this;
		var layoutOperations = [];
		var relatedLinkColumns = [];
		var clmLinks = [];
		var newOp = null;
		
		this.destroy();
		
		// Retrieve the link type preferences for the project for using when showing the Add Link menu.
		var preferencesDeferred = LinkTypeUtils.getLinkingPreferences().getLinkTypePreferencesForProject(this._projectURI);
		preferencesDeferred.addCallback(this, function(prefs) {
			this._linkTypePreferences = prefs;
			return prefs;
		});
		
		dojo.forEach(this._typeCache.getLinkTypeKeys(), function (linkTypeUri) {
			var linkType = self.getApplicableLinkType(linkTypeUri);
			//Only if we have the link
			if (linkType) {
				//Check it is in our project, and not a calm link before allowing anything
				if (linkType.getProjectURI() === self._projectURI && 
					!LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
					self._addLinkRelatedColumns(relatedLinkColumns, linkType);
				} else {
					if (!LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
						ErrorHandler.warn("Unknown LinkType when initialising!", linkTypeUri, linkType);
					}
				}
			} else {
				ErrorHandler.warn("Unknown LinkType when initialising!", linkTypeUri, linkType);
			}
		});
		
		// Create the Add Link menu operations
		var creationLinkTypeRolesDeferred = LinkTypeUtils.getAvailableCreationLinkTypesRoles(this._projectURI);
		creationLinkTypeRolesDeferred.addCallback(this, function(linkTypeRoleTuples) {
			for (var i=0; i<linkTypeRoleTuples.length; i++) {
				var roleTuple = linkTypeRoleTuples[i];
				for (var j=0; j<roleTuple.length; j++) {
					var linkTypeRole = roleTuple[j];
					var linkType = linkTypeRole.getLinkType();
					var sameAs = linkType.getSameAs();
					var separatorAbove = (j === 0);
					if (LinkTypeUtils.isCalmLink(sameAs) || LinkTypeUtils.isDMLinkType(sameAs) || LinkTypeUtils.isRMtoRMLinkType(sameAs)) {
						var oslcLinkTypeValue = LinkTypeUtils.getOslcLinkTypeValueForLinkType(linkType);
						// create CLM operation
  				        this._createCalmLinkOperation(oslcLinkTypeValue, linkTypeRole, layoutOperations, separatorAbove, caller);
					} else {
						// create non-CLM operation
						newOp = self._getLinkOperation(linkTypeRole);
						menuOperations[newOp.getId()] = newOp;
						layoutOperations.push({id: newOp.getId(), separatorAbove: separatorAbove});
					}
				}
			}
			// Create the More... option.  When the menu is invoked we'll test to see if the option
			// should be visible
			newOp = self._getMoreLinksOperation();
			menuOperations[newOp.getId()] = newOp;
			layoutOperations.push({id: newOp.getId(), separatorAbove: true});
			return linkTypeRoleTuples;
		});

		//Now build the new layout, and amend the row type
		dojo.forEach(menuLayout, function(group) {
			if (group.groupId === "CreateLinks") {
				group.operations = layoutOperations;
			}
		});
		
		//Now add all the linktype rows that can be added and displayed in the grid - and remember their ids
		var existingDetailsMap = self._display.getAllLinkColDetailsByKey();
							
		if (existingDetailsMap) {
			//We need to 'merge' the new with the existing, adding and removing as we go
			self._mergeLinkColumns(existingDetailsMap, relatedLinkColumns);
		} else {
			//No existing - just add
			uniqueId = 1;
			dojo.forEach(relatedLinkColumns, function(relatedLinkDetails) {
				var skipLinkColumn = self._skipLinkType(relatedLinkDetails);
				if (!skipLinkColumn) {
					var linkColId = "lk" + (uniqueId++);
					self._grid.addColumn(RESOURCE_ROW, linkColId, relatedLinkDetails.col);
					self._display.mapColumn(linkColId, self._display.LINK_COL, relatedLinkDetails.key, relatedLinkDetails.linkTypeURI);
				}
			});
		}
		linkutils.setLinkColumnMapping(self._display.getLinkColumnMap());

		this._grid.amendRowType(RESOURCE_ROW, {menuLayout: menuLayout, operations: menuOperations});
	},
	
	_skipLinkType: function(linkDetails) {
		// Summary: Filters out the "object to subject" direction of the OSLC Elaboration and Specification link types
		// 			since they are unusually represented by two link types.  See work item 276369 for background info.
		var skip = false;
		var isSubject = (linkDetails.key.indexOf(com.ibm.rdm.web.extensions.LinkDirection.OUT,
			linkDetails.key.length - com.ibm.rdm.web.extensions.LinkDirection.OUT.length) !== -1);
		if (!isSubject) {
			var result = this._typeCache.getLinkType(linkDetails.linkTypeURI);
			var sameAsLink = result.getSameAs();
			if (dojo.indexOf(OBJ_TO_SUB_LINKS_EXCLUDE_FROM_VIEW, sameAsLink) != -1) {
				skip = true;
			}
		}
		
		return skip;
	},
	
	_mergeLinkColumns: function(existingDetailsMap, relatedLinkColumns) {

		dojo.forEach(relatedLinkColumns, function(relatedLinkDetails) {
			if (existingDetailsMap[relatedLinkDetails.key]) {
				//Existing - remove from map and do nothing
				delete existingDetailsMap[relatedLinkDetails.key];
			} else {
				// New - add column details except for the case of Specifies/Specified By and Elaborates/Elaborated By.
				// These link types require special handling bc they are represented by two link types unlike the other link types.
				// If these are encountered, skip. See work item 276369 for background info.
				var skipLinkColumn = this._skipLinkType(relatedLinkDetails);
				
				if (!skipLinkColumn) {
					var linkColId = "lk" + (uniqueId++);
					this._grid.addColumn(RESOURCE_ROW, linkColId, relatedLinkDetails.col);
					this._display.mapColumn(linkColId, this._display.LINK_COL, relatedLinkDetails.key, relatedLinkDetails.linkTypeURI);
				}
			}
		}, this);							
			
		for (var key in existingDetailsMap) {
			//Any here have been deleted
			var details = existingDetailsMap[key];
			this._grid.removeColumn(RESOURCE_ROW, details.colId);
			this._display.removeColumn(details.colId);
		}						
	},	
	
	getAndResetInflightLinkedRows: function() {
		var rows = this._inflight_getLinkedRows;
		this._inflight_getLinkedRows = null;
		return rows;
	},
	
	_addLinkRelatedColumns: function(cols, linkType) {
		//Summary: adds the columns for the given linktype (either one or two, depending on whether the link is bidirectional or not.

		var link_col;
		var linkTypeKey;
		if (linkType.isBidirectional()) {
			link_col = {};
			linkTypeKey = linkType.getResourceURI()+ArtifactConstants.BIDIRECTIONAL_LINK;
			var name = linkType.getSubjectToObjectLabel() || linkType.getName();
			link_col.name = dojo.string.substitute(this._messages.linkPrefix, [name]); 
			link_col.width = 200;
			link_col.isVisible = false;
			link_col.showFullTitleOnHover = true;
			link_col.deferredContentFunction = dojo.hitch(this, this.loadAllLinksOfTypeForCol, linkTypeKey);
			//Ensures the column has custom data of type = "L".
			link_col.custom = {};
			link_col.custom.type="L";
			link_col.custom.desc=dojo.string.substitute(this._messages.linkPrefixDesc, [name]); 
			link_col.startIntraCellNavigationFunction = dojo.hitch(this, this.startIntraCellNavigation);

			cols.push({col: link_col, key: linkTypeKey, linkTypeURI: linkType.getResourceURI()});
		} else {
			link_col = {};
			linkTypeKey = linkType.getResourceURI()+ArtifactConstants.SUBJECT_LINK;
			link_col.name = dojo.string.substitute(this._messages.linkPrefix, [linkType.getSubjectToObjectLabel()]); 
			link_col.width = 200;
			link_col.isVisible = false;
			link_col.showFullTitleOnHover = true;
			link_col.deferredContentFunction = dojo.hitch(this, this.loadAllLinksOfTypeForCol, linkTypeKey);
			//Ensures the column has custom data of type = "L".
			link_col.custom = {};
			link_col.custom.type="L";
			link_col.custom.desc=dojo.string.substitute(this._messages.linkPrefixDesc, [linkType.getSubjectToObjectLabel()]);
			link_col.startIntraCellNavigationFunction = dojo.hitch(this, this.startIntraCellNavigation);

			cols.push({col: link_col, key: linkTypeKey, linkTypeURI: linkType.getResourceURI()});
			//63499: Remove the 'Derived From' link types from Tree View and column lists
			var isDMLinkType = LinkTypeUtils.isDMLinkType(linkType.getSameAs());
			if(isDMLinkType){
				return;
			}
		
			link_col = {};
			linkTypeKey = linkType.getResourceURI()+ArtifactConstants.OBJECT_LINK;
			link_col.name = dojo.string.substitute(this._messages.linkPrefix, [linkType.getObjectToSubjectLabel()]); 
			link_col.width = 200;
			link_col.isVisible = false;
			link_col.showFullTitleOnHover = true;
			link_col.deferredContentFunction = dojo.hitch(this, this.loadAllLinksOfTypeForCol, linkTypeKey);
			//Ensures the column has custom data of type = "L".
			link_col.custom = {};
			link_col.custom.type="L"; 
			link_col.custom.desc=dojo.string.substitute(this._messages.linkPrefixDesc, [linkType.getObjectToSubjectLabel()]); 
			link_col.startIntraCellNavigationFunction = dojo.hitch(this, this.startIntraCellNavigation);

			cols.push({col: link_col, key: linkTypeKey, linkTypeURI: linkType.getResourceURI()});
		}
	},

	getApplicableLinkType: function(linkTypeUri) {
		//Summary: Gets the applicable linktype for this Uri.
		//			Deals with the mapping of foreign linktypes and sameAs relations.
		var result = this._typeCache.getLinkType(linkTypeUri);
		if (!result) {
			var newUri = this._typeCache.getForeignToMain(linkTypeUri);
			result = this._typeCache.getLinkType(newUri);
		}
		return result;
	},
	
	_getLinkOperation: function(linkTypeRole) {
		var self = this;

		var label = linkTypeRole.getLabel();

		var labelFunction = function (count)
			{return self._getLinkCreationText(label, count);};

		var enabledFunction = dojo.hitch(self, self._isLinkCreationEnabledDummy);
		var executeFunction = function(rowIds, count) {return self._executeLinkCreation(linkTypeRole.getLinkType(), linkTypeRole.isSubjectRole(), rowIds);};
		var enabledDeferredFunction = function(rowIds, count) {return self._isLinkCreationEnabledDeferred(linkTypeRole.getLinkType(), rowIds, count);}; 
		var isLinkVisibleFunction = function(rowIds, count) {return self._isLinkVisibleFunction(linkTypeRole.getLinkType(), linkTypeRole.isSubjectRole(), rowIds, count);}; 

		var id = ".LO"+label.replace(/[ ,]/gi, "-");

		var newOp = new com.ibm.rdm.web.grid.Operation(id, "", labelFunction, enabledFunction, executeFunction, isLinkVisibleFunction, enabledDeferredFunction);
		return newOp;
	},
	
	_getMoreLinksOperation: function() {
		// summary:
		//		Return an Operation for showing the links dialog via a More... option

		var self = this;

		var label = this._messages.more;

		var labelFunction = function (count)
			{return self._getLinkCreationText(label, count);};

		var enabledFunction = dojo.hitch(self, function() {return true;});
		var executeFunction = function(rowIds, count) {return self._executeLinkCreation(null, null, rowIds);};
		var isMoreVisibleFunction = function(rowIds, count) {return self._isMoreVisibleFunction(rowIds, count);}; 

		var id = ".LO"+label.replace(/[ ,]/gi, "-");

		var newOp = new com.ibm.rdm.web.grid.Operation(id, "", labelFunction, enabledFunction, executeFunction, isMoreVisibleFunction);
		return newOp;
	},
	
	
	loadAllLinksOfTypeForCol: function(linkTypeUri, functionToCall, resource) {
		var self = this;
		
		var resourceUri = resource.resourceURI;

		//Now we have the links, get the resources
		self.getLinkedRowsForLinktypeForColumn(resourceUri, linkTypeUri, function () {self._displayAllLinkedResources(resourceUri, linkTypeUri, functionToCall);});
	},
	
	getLinktypesForArtifact: function(resourceUri, fnToCall) {
		//Summary: this function loads the actual traceability link resources for a resource, not the resources they are linked to!
		//			It only does this once we are 
		//console.log("RSGV getLinktypesForArtifact", resourceUri);
		var resource = this._dataCache.getRecordInfo(resourceUri);
		var self = this;
		var linkTypes;
		if (this._dataCache.hasLinkedRows(resourceUri)) {
			fnToCall(this._dataCache.getLinkTypesForRows(resourceUri));			
		} else if (this._dataCache.hasLinkIndicatedRows(resourceUri)) {
			//These rows have been marked as having links, even if we don't know what they are!
			//We may not have the linktypes used available, so just make sure
			this._ensureLinkTypesAreAvailable(resourceUri).addCallback(function () {
				var linkTypes = [];
				
				self._addExtractedLinkTypes(linkTypes, self._dataCache.getLinkIndicatedRows(resourceUri).objectLinks, ArtifactConstants.OBJECT_LINK);
				self._addExtractedLinkTypes(linkTypes, self._dataCache.getLinkIndicatedRows(resourceUri).subjectLinks, ArtifactConstants.SUBJECT_LINK);

				fnToCall(linkTypes);
			});
		} else if (this._dataCache.getLinks(resourceUri)) {
			//If we have already loaded the links, just process the list we have
			linkTypes = this._extractLinkTypes(this._dataCache.getLinks(resourceUri), resourceUri);
			fnToCall(linkTypes);
		} else {
			fnToCall([]);
		}
	},
	
	_handleKeyPress: function (event, args) {
		//If user hits enter or space, click image
		if(event.keyCode === dojo.keys.ENTER || event.charCode === dojo.keys.SPACE || event.which === dojo.keys.SPACE){
			args.node = args.node || event.target;
			//Needed for IE8 (kept trying to close the EditMenu right away)
			dojo.stopEvent(event);
			this._handleClick(args);
		}
	},
	
	_handleClick: function (args) {
		//Summary:
		//		Creates Menu after a user has clicked the edit image in a link column in the grid.
		//Arguments:
		//		node: The node that the menu will replace (e.g. editDiv)
		//		resourceUri: Used to complete Create and Remove options- the URI of the source resource (e.g. "https://kgithen:9444/rdm/resources/_3ObAIMorEeGL4-WH_u2u4A")
		//		linkDetails: Provides existing links, used to remove links.  An array of tuples that varies with the type attribute:
		//			//if type === "INTERNAL"
		//				linkUri: Link Resource URI (e.g. "https://kgithen:9444/rdm/links/_jFTBpsoeEeGL4-WH_u2u4A")
		//				type: "INTERNAL"
		//				uri: The Resource URI of the target resource of the link (e.g. "https://kgithen:9444/rdm/resources/_S74uGMoeEeGL4-WH_u2u4A")
		//			//if type === "EXTERNAL"
		//				link: Link object of type com.ibm.rdm.web.links.Link
		//				otherEnd: External URI (e.g. "https://www.google.com")
		//				type: "EXTERNAL"
		//			//if type === "XPROJECT"
		//				linkUri: Link Resource URI (e.g. "https://kgithen:9444/rdm/links/_JJuqoMuWEeGe_5lJfRXVJg")
		//				object: Additional information about the target resource of the link
		//				type: "XPROJECT"
		//		linkTypeKey: Used to determine link direction.  Link Type URI + _<direction> (direction = OBJ || SUB || BI)
		//		permissions: Used to determine whether or not to create menus.  If user does not have permission, option is not shown.  Hash object:
		//			canCreate: True if the user has permission to create this link type in this project
		//			canRemoveArray: An array of booleans that has the same indices of linkDetails.
		//							true if user has permission to remove link with same index in LinkDetails
		//			canRemoveSome: True if the user has permission to remove at least one link in linkDetails
		//		calmLinkRowWidgets: Used to list CALM link title (e.g. "58: Requests sent in form of hard copy mail") instead of URI. 
		//							An array of objects of type "com.ibm.rdm.web.calm.CalmLinkRowWidget".
		//		resourceLinks: Used to list RM-RM link title (e.g. "58: Requests sent in form of hard copy mail") instead of URI. 
		//						An array of ResourceLink objects.
		var menu = new jazz.ui.Menu();
		var editMenu = new jazz.ui.toolbar.MenuButton(
				{
					"class" : "com-ibm-rdm-web-artifactgrid-ArtifactLinks-LinkColImage",
					imgClass : "editDescriptionCommand",
					title : this._messages.editLinks,
					menuProvider : function() {
						return menu;
					},
					_onResize: function() {
						dijit.popup.close(menu);
					}
				});
		this._toDestroy.push(editMenu);

		var node = args.node;
		var resourceUri = args.resourceUri;
		var linkDetails = args.linkDetails;
		var linkTypeKey = args.linkTypeKey;
		var permissions = args.permissions;
		/*Optional args*/
		var calmLinkRowWidgets = args.calmLinkRowWidgets;
		var resourceLinks = args.resourceLinks;
		
		var linkTypeInfo = this._getLinkDirectionInfo(linkTypeKey);
		var linkType = linkTypeInfo.linkType;
		var isSubject = linkTypeInfo.isSubject;
		var direction = linkTypeInfo.direction;
		
		var isDMLinkType = LinkTypeUtils.isDMLinkType(linkType.getSameAs());
		var isCalmLink = LinkTypeUtils.isCalmLink(linkType.getSameAs());
		var isRMtoRmLinkType = LinkTypeUtils.isRMtoRMLinkType(linkType.getSameAs());
		
		//Only enable create if User has permission
		if(permissions.canCreate){
			var addMenuItem = new dijit.MenuItem({
				label: this._messages.addMenuItem, 
				iconClass: "com-ibm-rdm-web-artifactgrid-ArtifactLinks-AddLinkMenuItem",
				onClick: dojo.hitch(this, function(){
					this.showCreateLinkDialog(linkType, isSubject, [resourceUri]);
				})
			});
			this._toDestroy.push(addMenuItem);
			menu.addChild(addMenuItem);
		}

		//Only enabled remove if Links of this LinkType exist for this artifact and have permissions to remove at least one link
		if(linkDetails.length > 0 && permissions.canRemoveSome){
			var removeSubMenu = new ScrollableMenu();	
			var i;
			for(i = 0; i < linkDetails.length; i++){
				if(permissions.canRemoveArray[i]){
					var link = linkDetails[i];
					var label = "";
					var resource = null;
					var subject, object;
					if (link.type && link.type === "EXTERNAL") {
						link = link.link;
						
						// 75315: Labels in the dropdown should match what we display in the grid unless there is no title in the grid, in which case we display a default string.
						label = link.title && link.title !== "" ? link.title : this._messages.untitledLink;
						if((direction === ArtifactConstants.SUBJECT_LINK) || link.linkDetails.isDiscoveredLink){
							subject = resourceUri;
							object = link.uri;
						} else {
							subject = link.uri;
							object = resourceUri;
						}
						
						if((isRMtoRmLinkType || isDMLinkType) && resourceLinks[i]){
							label = resourceLinks[i].text;
						} else if(isCalmLink && calmLinkRowWidgets[i]){
							label = calmLinkRowWidgets[i].calmLink.getTitle();
							subject = calmLinkRowWidgets[i].calmLink.getSubject();
							object = calmLinkRowWidgets[i].calmLink.getObject();
						}
						//Resource is null for an external link
					}
					else{
						var artifactURI = link.uri;
						//Cross Project links are structured slightly differently
						if(link.type === "XPROJECT"){
							artifactURI = link.object.resourceURI;
						}
						resource = this._dataCache.getLinkedResource(artifactURI);
						label = resource.getID() + " " + resource.getTitle();
						var modTitle = (resource.getModuleTitle && resource.getModuleTitle());
						
						if(modTitle){
							label +=  " (" + modTitle + ")";
						}
						
						if(isSubject){
							object = resource.getResourceURI();
							subject = resourceUri;
						} else {
							object = resourceUri;
							subject = resource.getResourceURI();
						}
					}
					
					var removeArgs = {
						link : link,
						label : label,
						linkType : linkType,
						subject : subject,
						object : object,
						direction : direction
					};
					
					var removeMenuItem = new dijit.MenuItem({
						label: label,
						onClick: dojo.hitch(this, function(removeArgs){
							//Used to remove Link- link specified by linkDetails (varies depending on type of link)
							var link = removeArgs.link;
							//Used to display confirmation message- Displayed label of Target Resource
							var label = removeArgs.label;
							//Used to remove Link- Link Type object
							var linkType = removeArgs.linkType;
							//Used to Refresh Grid
							var subject = removeArgs.subject;
							var object = removeArgs.object;
							var direction = removeArgs.direction;
							
							var linkObject = new com.ibm.rdm.web.links.Link({
								resourceURI: link.linkUri,
								linkType: linkType.getResourceURI(),
								subject: subject,
								object: object,
								isDiscoveredLink: link.linkDetails && link.linkDetails.isDiscoveredLink
							});
							
							var remove = dojo.hitch(this, function(){
								this.removeLink(linkObject, linkType, direction);
							});
							
							var title = this._messages.confirmRemoveTitle;
							
							var message = dojo.string.substitute(this._messages.confirmRemoveQuestion, [label.italics()]);	
		
							var msgForm = new com.ibm.rdm.web.common.MessageForm({
								confirmContent: message,
								dialogTitle: title,
								dialogStyle: com.ibm.rdm.web.common.MessageForm.DIALOG_TYPES.YesNo,
								dialogWidth: "400px",
								onOk: remove
							});
							msgForm.show();
											
							this._toDestroy.push(msgForm);
							editMenu._currentMenu = menu;
							dijit.popup.close(menu);

						}, removeArgs)
					});
					dojo.setAttr(removeMenuItem.domNode, "title", link.uri); // Add a tooltip to the MenuItem, so we can see the URI.
					this._toDestroy.push(removeMenuItem);
					removeSubMenu.addChild(removeMenuItem);
				}
			}
			this._toDestroy.push(removeSubMenu);
			
			var removeMenu = new dijit.PopupMenuItem({
				label: this._messages.removeMenuItem,
				iconClass: "com-ibm-rdm-web-artifactgrid-ArtifactLinks-RemoveLinkSubMenu",
				popup: removeSubMenu
			});
			this._toDestroy.push(removeMenu);
			menu.addChild(removeMenu);
		}
		this._toDestroy.push(menu);
		
		var handle = this._editImageHandles[resourceUri + linkTypeKey + "click"]; 
		dojo.disconnect(handle);
		handle = this._editImageHandles[resourceUri + linkTypeKey + "press"];
		dojo.disconnect(handle);
			
		dojo.place(editMenu.domNode, node, "replace");
		
		//editMenu will not open without a timeout of zero
		setTimeout(dojo.hitch(this, function() {
			editMenu.open();
			// now, set focus on the first child, for consistency with the row's main edit menu,
			// but more importantly, to fix defect 93625 (a Chrome-specific issue)
			menu.focus();
		}), 0);
		
		dojo.connect(editMenu.domNode, "onclick", dojo.hitch(this, function() {
			// future clicks in this row will reuse this menu, instead of recreating it, so we need to
			// connect the focus event to those clicks as well. See the above setTimeout call for more information
			menu.focus();
		}));
	},
	
	_getLinkDirectionInfo: function(linkTypeKey){
		//Summary: 
		//		Given a linkTypeUri returns information about the direction of the link
		//Returns:
		//		linkType- linktype object
		//		isSubject- true if artifact is hte subject of this linktype
		//		direction- _SUB/_OBJ/_BI depending on the direction of the linktype
		var linkTypeURI = linkTypeKey;
		
		//Determine the direction of the link.
		var lastIndex = linkTypeKey.lastIndexOf("_");
		var isSubject;
		var suffix;
		if(lastIndex > 1){
			suffix = linkTypeKey.substr(lastIndex, linkTypeKey.length);
			if(suffix === ArtifactConstants.OBJECT_LINK){
				isSubject = false;
			}
			else{
				isSubject = true;
			}
			linkTypeURI = linkTypeKey.substr(0, lastIndex);
		}
		var linkType = this.getApplicableLinkType(linkTypeURI);
		
		return {linkType: linkType, isSubject: isSubject, direction: suffix};
	},
	
	_getPermissions: function(resourceUri, linkDetails, linkTypeKey){
		//Summary:
		//	Returns a Deferred hash object with attributes:
		//		canCreate: true if user has permission to create specified linkType
		//		canRemoveSome: true if user has permission to remove at least one of Links of specified LinkType
		//		canRemoveArray: an [] with similar indices to linkDetails, values are boolean, true if user has permission to remove specified Link
		//		resourceUri
		//		linkDetails
		//		linkTypeKey
		var deferred = new dojo.Deferred();
		
		//Used to ensure all created deferreds callback before this deferred calls back.
		var numOfCallbacks = 1 + linkDetails.length;
		
		var canCreate = false;
		var canRemoveSome = false;
		var canRemoveArray = [];
		
		var linkTypeInfo = this._getLinkDirectionInfo(linkTypeKey);
		var linkType = linkTypeInfo.linkType;
		var isSubject = linkTypeInfo.isSubject;
		
		var isDMLinkType = LinkTypeUtils.isDMLinkType(linkType.getSameAs());
		var isCalmLink = LinkTypeUtils.isCalmLink(linkType.getSameAs());
		var isRMtoRmLinkType = LinkTypeUtils.isRMtoRMLinkType(linkType.getSameAs());
		
		var resource = this._dataCache.getRecordInfo(resourceUri);
		
		var checkCallbackStatus = function(){
			numOfCallbacks--;
			if(numOfCallbacks === 0){
				var permissions = {
					canCreate: canCreate, 
					canRemoveSome: canRemoveSome, 
					canRemoveArray: canRemoveArray
				};
					
				var toReturnHash = {
					permissions: permissions, 
					resourceUri: resourceUri,
					linkDetails: linkDetails,
					linkTypeKey: linkTypeKey
				};
				deferred.callback(toReturnHash);
			}
		};
		
		//Check Create Permissions
		var canCreateDeferred = this._checkPermissionForResource(isCalmLink, isDMLinkType,isRMtoRmLinkType, null, linkType, isSubject, resource, "create");
		canCreateDeferred.addCallback(this, function(isAllowed) {
			canCreate = isAllowed;
			checkCallbackStatus();
			return isAllowed;
		});
		canCreateDeferred.addErrback(this, function(e){
			canCreate = false;
			checkCallbackStatus();
			return e;
		});
		
		//Check Remove Permissions for each Link
		if(linkDetails.length > 0){
			var i;
			for(i = 0; i < linkDetails.length; i++){
				var link = linkDetails[i];
				var otherResource = null;
				var isProjectLink = isCalmLink || isDMLinkType || isRMtoRmLinkType;
				if (!isProjectLink && (link.type && link.type === "EXTERNAL")) {
					//Do not need permission to Remove an External Link
					canRemoveArray[i] = true;
					canRemoveSome = true;
					checkCallbackStatus();
				}
				else{
					var otherArtifactURI = link.uri;
					//Cross Project links are structured slightly differently
					if(link.type === "XPROJECT"){
						otherArtifactURI = link.object.resourceURI;
					}
					otherResource = this._dataCache.getLinkedResource(otherArtifactURI);
					
					var subject, object;
					if(isSubject){
						object = otherResource;
						subject = resource;
					} else {
						object = resource;
						subject = otherResource;
					}
					
					var linkObject = new com.ibm.rdm.web.links.Link({
						object: object,
						resourceURI: link.linkUri,
						subject: subject
					});
					var index = i;
					var canRemoveDeferred = this._checkPermissionForResource(isCalmLink, isDMLinkType, isRMtoRmLinkType, linkObject, linkType, isSubject, resource, "remove");
					canRemoveDeferred.addCallback(this, function(index, isAllowed) {
						canRemoveArray[index] = isAllowed;
						if(canRemoveArray[index]){
							canRemoveSome = true;
						}
						checkCallbackStatus();
						return isAllowed;
					}, index);
					canRemoveDeferred.addErrback(this, function(index, e){
						canRemoveArray[index] = false;
						checkCallbackStatus();
						return e;
					}, index);
				}
			}
		}
		return deferred;
	},
	
	_checkPermissionForResource: function(isCalmLink, isDMLinkType, isRMtoRmLinkType, linkObject, linkType, isSubject, resource, permissionType){
		//Summary:
		//	Checks to see if the current user has permission to do whatever action is specified by permissionType
		//	current permissionType options "create" or "remove"
		//Returns:
		//	Deferred boolean, true if user has permission to do specified action (by permissionType) on specifid LinkObject
		var deferred = new dojo.Deferred();
		var processAreaURI = null;
		if(resource){
			processAreaURI = resource.getProcessAreaURI();
		}
		if(processAreaURI === null ){
			processAreaURI = this._projectURI;
		}
		
		if (permissionType === "create"){
			if(isCalmLink || isDMLinkType || isRMtoRmLinkType){
				//Cannot add Referenced By links
				if(isRMtoRmLinkType && !isSubject){
					deferred.callback(false);
				}else {
					//Check Association First, if no association, cannot create link type
					var hasAssociationDef = CalmRegistry.getProjectAreaLinks(this._projectURI);
					hasAssociationDef.then(dojo.hitch(this, function(results){
						var hasAssociation = false;
						dojo.forEach(results, function(r){
							if(com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkTypeToCalmLinkTypeURI[r.linkType] === linkType.getSameAs()){
								hasAssociation = true;
							}
						});
						if(hasAssociation){
							// Do two permissions calls, because CLM links are still using the old-style permissions
							var projectOperationsDeferred = com.ibm.rdm.web.common.ProjectCache.getProjectPermissions(this._projectURI);
							projectOperationsDeferred.addCallback(this, function(projectOperations){
								var isCLMLinkAllowed = projectOperations.item(com.ibm.rdm.web.common.ProjectCache.MODIFY_CLM_LINK);
								deferred.callback(isCLMLinkAllowed);
								return projectOperations;
							});	
						} else {
							deferred.callback(false);
						}
						return results;
					}));
				}
			} else {
				var canCreate = ProcessOperationsCache.getPermissibleLinkTypesForCreation(this._projectURI, processAreaURI);
				canCreate.addCallback(this, function(linkTypePermissions) {
					deferred.callback(linkTypePermissions[linkType.getActionId()]);
					return linkTypePermissions;
				});
				canCreate.addErrback(this, function(e){
					deferred.callback(false);
					return e;
				});
			}
		} else if(permissionType === "remove"){
			if(isCalmLink || isDMLinkType || isRMtoRmLinkType){
				// Do two permissions calls, because CLM links are still using the old-style permissions
				var projectOperationsDeferred = com.ibm.rdm.web.common.ProjectCache.getProjectPermissions(this._projectURI);
				projectOperationsDeferred.addCallback(this, function(projectOperations){
					var isCLMLinkAllowed = projectOperations.item(com.ibm.rdm.web.common.ProjectCache.MODIFY_CLM_LINK);
					deferred.callback(isCLMLinkAllowed);
					return projectOperations;
				});
				projectOperationsDeferred.addErrback(this, function(e){
					deferred.callback(false);
					return e;
				});
			} else {
				// Check this side of the Remove Link
				var canRemove = ProcessOperationsCache.isDeleteLinkAllowed(this._projectURI, linkType.getActionId(), processAreaURI);
				canRemove.addCallback(this, function(isAllowed) {
					if(!isAllowed){
						deferred.callback(isAllowed);
					} else {
						// Check Other side of the Remove Link
						var canRemoveOther = this._checkPermissionForReferencedResource(linkObject, linkType, isSubject, "remove");
						canRemoveOther.addCallback(this, function(isOtherAllowed) {
							deferred.callback(isOtherAllowed);
							return isOtherAllowed;
						});
						canRemoveOther.addErrback(this, function(e){
							deferred.callback(false);
							return e;
						});
					}
					return isAllowed;
				});
				canRemove.addErrback(this, function(e){
					deferred.callback(false);
					return e;
				});
			}
		}
		return deferred;
	},
	
	_checkPermissionForReferencedResource: function (link, linkType, isSubject, permissionType) {
		// summary:
		//		Method to test for whether or not to a menu action is permissible for the resource referenced in 
		//		the given link.  
		// returns:
		//		Deferred, true if user has permission to do specified action (by permissionType) on specifid LinkObject
		var deferred = new dojo.Deferred();
		var object = link.getObject();
		var subject = link.getSubject();			
		var toCheck = object.getResourceURI();
		// Get the referenced resource URI
		if (!isSubject) {
			toCheck = subject.getResourceURI();
		} 

		// we must do a describe to get the processAreaURI for the referenced resource and then do the
			// permission check
		var descriptionDeferred = com.ibm.rdm.web.model.resources.ResourceManager.getResourceDescribe({
			resourceURI: toCheck
		});
		descriptionDeferred.addCallback(this, function(descriptionResponse){
			if (descriptionResponse.projectURI !== this._projectURI) {
				if (!linkType.isSystemDefined()) {
					// permissions can only be enforced when cross-project linking if the link type is system defined.
					deferred.callback(true);
				}
			}
			var processAreaURI = descriptionResponse.processArea;
			var permissionDeferred = null;
			if (permissionType === "edit") {
				permissionDeferred = ProcessOperationsCache.isModifyLinkTypeAllowed(this._projectURI, linkType.getActionId(), processAreaURI);
			} else {
				permissionDeferred = ProcessOperationsCache.isDeleteLinkAllowed(this._projectURI, linkType.getActionId(), processAreaURI);
			}		
			permissionDeferred.addCallback(this, function(isAllowed) {
				deferred.callback(isAllowed);
				return isAllowed;
			});
			permissionDeferred.addErrback(this, function(e){
				deferred.callback(false);
				return e;
			});
			return descriptionResponse;
		});
		descriptionDeferred.addErrback(this, function(e){
			deferred.callback(false);
			return e;
		});
		return deferred;
	},
	
	removeLink: function(link, linkType, direction){
		//Summary:
		//		Removes specified link of specified linktype
		var isDMLinkType = LinkTypeUtils.isDMLinkType(linkType.getSameAs());
		var isRMtoRmLinkType = LinkTypeUtils.isRMtoRMLinkType(linkType.getSameAs());
		var isCalmLink = LinkTypeUtils.isCalmLink(linkType.getSameAs());
		
		//We want deleting a link not to refresh from the server
		this._display.addNextLoadContextValue("LINK_REFRESH","NON-SERVER");
		
		if (isCalmLink) {
			StatusDisplay.showStatusMessage(this._linkMessages.deletingLinks);
			CalmLinkUtils.deleteLink(this._projectURI, link).then(dojo.hitch(this, function(){
				StatusDisplay.hideStatusMessage();
				//At this point send a notification to refresh the page
				ChangeNotifier.notifyListeners({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.LINK_DELETED,
					link: link,
					direction: direction,
					resource: link.getResourceURI(),
					crossWindow: true
				});								
			}));
		} else if ((isDMLinkType || isRMtoRmLinkType) && link._resourceURI === ""){
			//If Discovered Derived or RM2RM Link
			StatusDisplay.showStatusMessage(this._linkMessages.deletingLinks);
			//resourceURI cannot be null
			link._resourceURI = dojox.uuid.generateRandomUuid();
			if (isRMtoRmLinkType) {
				// Reverse subject and object values
				var tmpURI = link._subject;
				link._subject = link._object;
				link._object = tmpURI;
			}
			
			DerivedLinkUtils.deleteLink(this._projectURI, link).then(dojo.hitch(this, function(){
				StatusDisplay.hideStatusMessage();
				//At this point send a notification to refresh the page
				ChangeNotifier.notifyListeners({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.LINK_DELETED,
					link: link,
					direction: direction,
					resource: link.getResourceURI(),
					crossWindow: true
				});								
			}));
		}
		else {
			StatusDisplay.showStatusMessage(this._linkMessages.deletingLinks);
			LinkUtils.deleteLink({link: link, direction: direction}).then(dojo.hitch(this, function(){
				StatusDisplay.hideStatusMessage();
			}));
		}
	},

	_createEditDiv: function(args){
		//summary:
		//		Creates an clickable image in an anchor tag if the user has permission to create and/or remove at least one link of specified link type
		//	args:
		//		permissions = hash of canCreate, canRemoveSome and canRemoveArray
		//		resourceURI
		//		linkDetails
		//		linkTypeKey
		//		div Node where Menu will be placed
		//		/*optional*/ calmLinkRowWidgets Used for CALM links to display actual artifact name rather than external uri
		//		/*optional*/ resourceLinks Used for RM-RM links to display actual artifact name rather than external uri
		var permissions = args.permissions;
		var resourceUri = args.resourceUri;
		var linkDetails = args.linkDetails;
		var linkTypeKey = args.linkTypeKey;
		var div = args.div;
		var calmLinkRowWidgets = args.calmLinkRowWidgets;
		var resourceLinks = args.resourceLinks;
		
		var editDiv = null;
		
		//If the user has at permission to create or remove at least one link and this is not a baseline, history mode or review mode
		if((permissions.canCreate || permissions.canRemoveSome) && !(this._baselineURI || this._isInHistoryMode || this._isInReviewMode)){
			//Add gesture to add/remove links
			editDiv = dojo.create("a", {
				"aria-label": this._messages.editLinks,
				"class": "editButton rdm-button-link com-ibm-rdm-web-artifactgrid-ArtifactLinks-LinkColImage",
				role: "button",
				tabindex:"-1",
				title: this._messages.editLinks
			});
			
			var editLinkChild = dojo.create("span", {
				role: "presentation",
				className: "editDescriptionCommand"
			}, editDiv);
			
			var editClickConnection = dojo.connect(editDiv, "onclick", dojo.hitch(this, function(){
				this._handleClick({
					node: editDiv, 
					resourceUri: resourceUri, 
					linkDetails: linkDetails, 
					linkTypeKey: linkTypeKey, 
					permissions: permissions, 
					calmLinkRowWidgets: calmLinkRowWidgets, 
					resourceLinks: resourceLinks
				});
			}));
			var editPressConnection = dojo.connect(editDiv, "onkeydown", dojo.hitch(this, function(event){
				this._handleKeyPress(event, {
					node: editDiv, 
					resourceUri: resourceUri, 
					linkDetails: linkDetails, 
					linkTypeKey: linkTypeKey, 
					permissions: permissions, 
					calmLinkRowWidgets: calmLinkRowWidgets, 
					resourceLinks: resourceLinks
				});
			}));
			//Allows connect handles to be disconnected properly
			this._editImageHandles[resourceUri + linkTypeKey + "click"] = editClickConnection;
			this._editImageHandles[resourceUri + linkTypeKey + "press"] = editClickConnection;
			
			div.appendChild(editDiv);
		}
	},
	
	_displayAllLinkedResources: function(resourceUri, linkTypeKey, functionToCall) {
		var i, toDestroy = [];
		var div = document.createElement("div"),
			moduleArtifactGroup = dojo.create("div", {
				"class": "moduleArtifactLinks"
			}),
			baseArtifactGroup = dojo.create("div", {
				"class": "baseArtifactLinks"
			});
	
		// Suspicion status
		var linkTable      = dojo.create("div");
		var suspicionWidth = 17;
		
		//Summary: displays all the given resources in div		
		var hasLinkedColsForType = this._dataCache.hasLinkedColsForType(resourceUri, linkTypeKey);
		var intrinsiLinksColsForType =null;
		if(this.context.viewer._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE && this.context.viewer._showBaseProperties){
			intrinsiLinksColsForType = this._dataCache.getIntrinsicLinkedColsForType(resourceUri, linkTypeKey);
		}
		
		if (hasLinkedColsForType || intrinsiLinksColsForType) {
			var linkDetails = this._dataCache.getLinkedColsForType(resourceUri, linkTypeKey);
			if(intrinsiLinksColsForType){
				//Handle scenario in which linkDetails is not defined
				if(!linkDetails){
					linkDetails =[];
				}
				for(i=0; i < intrinsiLinksColsForType.length; i++){
					intrinsiLinksColsForType[i].isBaseArtifactLink =true;
					if (dojo.indexOf(linkDetails, intrinsiLinksColsForType[i]) === -1) {
						linkDetails.push(intrinsiLinksColsForType[i]);
					}
				}				
			}
			var suspect = false;
			var truncatedLink = this._messages.truncatedLink;
			
			var linkTypeInfo = this._getLinkDirectionInfo(linkTypeKey);
			var linkType = linkTypeInfo.linkType;
			var isSubject = linkTypeInfo.isSubject;
			
			var isDMLinkType = LinkTypeUtils.isDMLinkType(linkType.getSameAs());
			var isCalmLink = LinkTypeUtils.isCalmLink(linkType.getSameAs());
			var isRMtoRmLinkType = LinkTypeUtils.isRMtoRMLinkType(linkType.getSameAs());
			
			var editDiv = this._getPermissions(resourceUri, linkDetails, linkTypeKey);
			var resourceLinks = [];
			
			// RM-RM links are external links that need to be converted to titles, need to send ResourceLinks too
			// RM-DM links need linkDetails to fetch title, later
			var isResourceLinksNeeded = isRMtoRmLinkType || isDMLinkType;
			if(!isResourceLinksNeeded || linkDetails.length === 0){
			editDiv.addCallback(this, function(div, args) {
					args.div = div;
					this._createEditDiv(args);
			}, div);
			}

			if (linkDetails) {
				for (i = 0; i < linkDetails.length; i++) {
					var resource;
					var nameDiv;
					var linkDetail = linkDetails[i];

					if (linkDetail.type && linkDetail.type === "EXTERNAL") {
						//Only calculate an adjusted label for external links
						var adjustedLinkLabel;
						if (linkDetail && linkDetail.link && linkDetail.link.getValue && linkDetail.link.getValue()){
							adjustedLinkLabel = linkDetail.link.getValue();
						} else if (linkDetail && linkDetail.link && linkDetail.link.getTitle && linkDetail.link.getTitle()) {
							adjustedLinkLabel = linkDetail.link.getTitle();
						} else {
							var linkUrl = linkDetail.otherEnd;
							adjustedLinkLabel = linkDetail.link.title;
							if (adjustedLinkLabel && adjustedLinkLabel.length > MAXURLSizeInColumn){
								adjustedLinkLabel = dojo.string.substitute(truncatedLink, [adjustedLinkLabel.substring(0, MAXURLSizeInColumn)]);
							}
						}

						//Create hyperlink image
						var divForExternal = dojo.create("div");
						var baseArtifactIcon = null;
						if (linkDetail.isBaseArtifactLink) {
							dojo.addClass(divForExternal, "baseArtifactImgWrapper");
							baseArtifactIcon = document.createElement('img');
							baseArtifactIcon.src = this._blankGif;
							baseArtifactIcon.className = "baseartifact-icon";
							baseArtifactGroup.appendChild(divForExternal);
						} else {
							dojo.addClass(divForExternal, "discoveredImgWrapper");
							moduleArtifactGroup.appendChild(divForExternal);
						}
						
						// Note that adjustedLinkLabel may be undefined here meaning that no label will be displayed
						//    for the link in the grid. According to UX, this is the desired behavior when we do not
						//    provide a link title (75315, 75464).
						//    However, we will provide the link URI as the tooltip and alt text on the link icon in the grid.
						var iconText = adjustedLinkLabel ? adjustedLinkLabel : linkDetail.link.uri;
						var iconImage = dojo.create("img", {
							"src": dojo.moduleUrl("com.ibm.rdm.web", "common/images/hyperlink.gif").toString(),
							"alt": iconText,
							"title": iconText,
							"class": "content-type-entry-image"
						}, divForExternal);
						
						if(baseArtifactIcon) {
							dojo.place(baseArtifactIcon, iconImage, "after");
						}
						//Decorate link if it is a discovered link within a baseline
						if(this._baselineURI && ((linkDetail.isDiscoveredLink) || 
							(linkDetail.link && linkDetail.link.linkDetails	&& linkDetail.link.linkDetails.isDiscoveredLink))) {
							// visually indicate discovered links within a baseline with an overlay icon.
							// Use the same style for displaying content link overlays
							var overlayDiscoveredImg = document.createElement("img");
							overlayDiscoveredImg.src = this._blankGif;
							overlayDiscoveredImg.className = "content-overlay-image discovered-overlay-image";
							dojo.place(overlayDiscoveredImg, iconImage, "after");
						}
						
						// 75464: If this is an external link, It may not have a label. If that is the case, show the first n chars.
						//   The number of chars shown is determined by the width of the cell in the grid.
						var shownLabel = com.ibm.rdm.web.util.domutils.getHTMLEncodedText(adjustedLinkLabel);
						if (!shownLabel || shownLabel === "") {
							shownLabel = linkDetail.link.uri;
						}
						var linkAnchor = ResourceLinksHandler.createLink({
							uri: linkDetail.otherEnd,
							label: shownLabel,
							iconNode: iconImage,
							lazyFetch: false
						});
						resourceLinks.push(linkAnchor);
						dojo.attr(linkAnchor, "tabIndex", INTERNAL_NAVIGATION_TABINDEX);
						dojo.style(linkAnchor, "verticalAlign", "top");
						
						var linkWrapperDiv = document.createElement('div');
						linkWrapperDiv.className = "with-image";
						linkWrapperDiv.appendChild(linkAnchor);
						
						divForExternal.appendChild(linkWrapperDiv);
						
					}
					else {
						var attrFormat =
							this.getTraceabilityAttributeColumnFormat(linkTypeKey);
						var label = attrFormat.showTitle ? null : resource.getID();
						if (linkDetail.type && linkDetail.type === "XPROJECT") { //Check if the link is pointing to a resource in an external project
							resource = this._dataCache.getLinkedResource(linkDetail.object.getResourceURI());
							if (linkDetail.err || !resource) {
							//Link is to a is an error, eg it is to a deleted resource - do and show nothing
							} else {
								nameDiv = this._display.createResourceLinkHTML(
									resource, attrFormat.showTitle,
									!attrFormat.showIcon, label,
									INTERNAL_NAVIGATION_TABINDEX/*noTabFocus*/,
									linkDetail.isBaseArtifactLink);
								resourceLinks.push(nameDiv.destroy);
								if (linkDetail.isBaseArtifactLink) {
									baseArtifactGroup.appendChild(nameDiv.div);
								} else {
									moduleArtifactGroup.appendChild(nameDiv.div);
								}
								
								toDestroy.push(nameDiv.destroy);
								if(this._traceabilityAttributesByColumn[linkTypeKey]) {
									this._displayCrossProjectTraceabilityAttributes(
										linkDetail.isBaseArtifactLink ? baseArtifactGroup : moduleArtifactGroup,
										linkTypeKey, resource);
								}
							}		
						}
						else {
							//Deal with INTERNAL links
							resource = this._dataCache.getLinkedResource(linkDetail.uri);
							if (linkDetail.err || !resource) {
							//Link is to a is an error, eg it is to a deleted resource - do and show nothing
							} else {
								
								nameDiv = this._display.createResourceLinkHTML(
									resource, attrFormat.showTitle,
									!attrFormat.showIcon, label,
									INTERNAL_NAVIGATION_TABINDEX/*noTabFocus*/,
									linkDetail.isBaseArtifactLink);
	
								if (linkDetail.isBaseArtifactLink) {
									baseArtifactGroup.appendChild(nameDiv.div);
								} else {
									moduleArtifactGroup.appendChild(nameDiv.div);
								}
								
								toDestroy.push(nameDiv.destroy);
								if(this._traceabilityAttributesByColumn[linkTypeKey]) {
									this._displayTraceabilityAttributes(
										linkDetail.isBaseArtifactLink ? baseArtifactGroup : moduleArtifactGroup,
										linkTypeKey, resource);
								}
							}
						}
					}

					resource = this._dataCache.getRecordInfo(resourceUri);
					var suspectEntryUris = resource && resource.suspectEntries;
					
					// WI 71123: Fix the suspicion icon spacing for WebKit/Blink tables when using
					//           the rowspan attribute, which causes the suspicion icons to be misaligned
					//           with the links they're supposed to mark
					//
					//           To do this, each row is a table itself, and thus we force the icons to 
					//           line up with their respective links.
					if (suspectEntryUris && suspectEntryUris.length > 0) {
						// Create inner tables for each row, and place the suspect icon in the first
						// cell of that row
						var suspicionCell;
						var linkCell = dojo.create("td");
						var nextTable = dojo.create("table", null, linkTable);
						var nextRow = dojo.create("tr", null, nextTable);
						
						// Create the suspicion cell and force its width, in order to keep the columns lined up
						suspicionCell = dojo.create("td", null, nextRow);
						dojo.style(suspicionCell, "width", suspicionWidth + "px");
						
						var suspicionStatusWidget = new com.ibm.rdm.web.suspicion.SuspicionStatus({
							source: SuspectLinksUtil.EVENT_SOURCE.gridLinksColumn,
							projectURI: this._projectURI,
							suspectEntryURIs: suspectEntryUris,
							sourceResourceURI: linkDetail.uri,
							targetResourceURI: resource.resourceURI
						});
						
						dojo.place(suspicionStatusWidget.domNode, suspicionCell);
						if (nameDiv && nameDiv.div) {
							dojo.style(nameDiv.div, "left", "3px");
						}
						dojo.style(suspicionStatusWidget.domNode, "visibility", "hidden");
						toDestroy.push(suspicionStatusWidget);
						suspect = true;
						
						// Place the link content in the second cell of this new row
						dojo.place(nameDiv.div, linkCell);
						dojo.place(linkCell, nextRow);
					}
				}
				if(isResourceLinksNeeded && resourceLinks.length > 0){
					editDiv.addCallback(this, function(args) {
						args.div = div;
						args.resourceLinks = resourceLinks;
						this._createEditDiv(args);
					});
				}
			}
		}else {
			var editDiv = this._getPermissions(resourceUri, [], linkTypeKey);
			editDiv.addCallback(dojo.hitch(this, function(args){
				args.div = div;
				this._createEditDiv(args);
			}));
		}
		
		if (moduleArtifactGroup.firstChild) {
			if (baseArtifactGroup.firstChild) {
				var inThisModuleGroupHeader = dojo.create("span", {
					"class": "linkGroupMsg",
					innerHTML: this._messages["thisModuleOnly"]
				});
				
				dojo.place(inThisModuleGroupHeader, moduleArtifactGroup, "first");
			}
			
			div.appendChild(moduleArtifactGroup);
		}
		
		if (baseArtifactGroup.firstChild) {
			if (moduleArtifactGroup.firstChild) {
				var onBaseArtifactGroupHeader = dojo.create("span", {
					"class": "linkGroupMsg",
					innerHTML: this._messages["shared"]
				});
				
				dojo.place(onBaseArtifactGroupHeader, baseArtifactGroup, "first");
			}
			
			div.appendChild(baseArtifactGroup);
		}
		
		this.grid.measureRows();
		
		var self = this;
		var toShow;
		
		// Show the table we've built only if there are suspicion icons involved.
		// Otherwise show the neatly wrapped div of links.
		if (suspect) {
			toShow = linkTable;
		}
		else {
			dojo.destroy(linkTable);
			toShow = div;
		}
		
		functionToCall(toShow, {destroyRecursive: function() {self._callDestroy(toDestroy);}});
		
	},	
	
	_callDestroy: function(toDestroy) {
		var i, widget;
		if (toDestroy) {
			for (i = 0; i< toDestroy.length; i++) {
				widget = dijit.byId(toDestroy[i]);
				widget && widget.destroyRecursive();
			}
		}
	},
	
	_ensureLinkTypesAreAvailable: function(resourceUri) {
		
		var i, linkType;		
		var newDeferred = new dojo.Deferred();
		var linkTypesToLoad = [];
		var linkTypeUris = this._dataCache.getLinkIndicatedRows(resourceUri).objectLinks;
		if (linkTypeUris.length > 0) {
			for (i = 0; i < linkTypeUris.length; i++) {
				linkType = this.getApplicableLinkType(linkTypeUris[i]);
				if (!linkType) {
					if (dojo.indexOf(this.SYSTEM_LINKS_EXCLUDED_FROM_VIEW_URIS, linkTypeUris[i]) === -1) {
						//These are links we don't have the type for, but should
						linkTypesToLoad.push(linkTypeUris[i]);
					}
				}
			}
		}

		linkTypeUris = this._dataCache.getLinkIndicatedRows(resourceUri).subjectLinks;		
		if (linkTypeUris.length > 0) {
			for (i = 0; i < linkTypeUris.length; i++) {
				linkType = this.getApplicableLinkType(linkTypeUris[i]);
				if (!linkType) {
					if (dojo.indexOf(this.SYSTEM_LINKS_EXCLUDED_FROM_VIEW_URIS, linkTypeUris[i]) === -1) {
						//These are links we don't have the type for, but should
						linkTypesToLoad.push(linkTypeUris[i]);
					}
				}
			}
		}
		
		var self = this;
		
		if (linkTypesToLoad.length > 0) {
			//We have linktypes we need to load
			LinkTypeUtils.getLinkTypes(linkTypesToLoad).addCallback(function(linkTypes) {
				//Add the new linktypes
				self._typeCache.addNewLinkTypes(linkTypes);
				newDeferred.callback();
			});
		} else {
			//No link types to load
			newDeferred.callback();
		}

		return newDeferred;
	},


	
	_addExtractedLinkTypes: function (existingLinkTypes, linkTypes, suffix) {
		//Summary: This function extracts the linktypes from a list of linktypes.
		if (linkTypes.length > 0) {
			var i;
			for (i = 0; i < linkTypes.length; i++) {
				var suffixToUse = suffix;
				var linkTypeUri = linkTypes[i];
				var linkType = this.getApplicableLinkType(linkTypeUri);
				if (linkType) {
					if (linkType.isBidirectional()) {
						suffixToUse = ArtifactConstants.BIDIRECTIONAL_LINK;
					}
					var linkTypeKey = linkType.getResourceURI()+suffixToUse;

					//Now check its one we should be displaying
					if (dojo.indexOf(existingLinkTypes, linkTypeKey) === -1) {
							existingLinkTypes.push(linkTypeKey);
					}
				} else {
					//Ignore the ones we know we're ignoring
					if (dojo.indexOf(this.SYSTEM_LINKS_EXCLUDED_FROM_VIEW_URIS, linkTypeUri) === -1) {
						//Cross project ones may not be there! Definitely worth an error log.
						//Although it could be a link type that has recently been created (obscure, but could happen!).
						ErrorHandler.error("Unknown Linktype", linkTypeUri);
					}
				}
			}
		}
	},
	
	_extractLinkTypes: function (links, resourceUri) {
		//summary: This function extracts the right linktypes from a list of links.
		var i;
		var foundLinkTypes = [];
		var crossProject = [];
		if (links.length > 0) {
			for (i = 0; i < links.length; i++) {
				var link = links[i];
				var suffix = ArtifactConstants.SUBJECT_LINK;
				if (link.getObject() === resourceUri) {
					suffix = ArtifactConstants.OBJECT_LINK;
				}
				var linkTypeUri = link.getLinkType();
				var linkType = this.getApplicableLinkType(linkTypeUri);
				if (linkType) {
					if (linkType.isBidirectional()) {
						suffix = ArtifactConstants.BIDIRECTIONAL_LINK;
					}
					var linkTypeKey = linkType.getResourceURI()+suffix;

					//Now check its one we should be displaying
					if (dojo.indexOf(foundLinkTypes, linkTypeKey) === -1) {
						foundLinkTypes.push(linkTypeKey);
					}
				} else {
					//Ignore the ones we know we're ignoring
					if (dojo.indexOf(this.SYSTEM_LINKS_EXCLUDED_FROM_VIEW_URIS, linkTypeUri) === -1) {
						//This must be a cross project one!
						//crossProject.push(linkTypeUri);
						//We are currently loading ALL linktypes - so cross project ones should be there! Definitely worth an error log.
						ErrorHandler.error("Unknown Linktype", linkTypeUri);
					}
				}
			}
		}
		return foundLinkTypes;
	},
	
	getLinkedRowsForLinktypeForColumn: function(resourceUri, desiredLinkType, fnToCall) {
		//Summary: Sees if the desired row information is stored locally, and if not, goes and gets it.
		//         Because this is for a column, we don't need to follow the rules when getting the info.

		//Ignore cache because before it was not taking multiple links into account.  For example, if an artifact has one "Link To" link, and a user
		//	adds another, it would not update cache because this._dataCache.hasLinkedColsForType(resourceUri, desiredLinkType) evaluates to true
		var newFn = dojo.hitch(this, this._cacheLinkedRowsForCol, resourceUri, desiredLinkType, fnToCall);
			
		//We are going to collect all the entries up, and ensure that they are called in bulk		
		if (this._inflight_getLinkedRows) {
			this._inflight_getLinkedRows.push({uri: resourceUri, linkType: desiredLinkType, fn: newFn});
		} else {
			this._inflight_getLinkedRows = [ {uri: resourceUri, linkType: desiredLinkType, fn: newFn}];
			//And set up the deferred call
			setTimeout(dojo.hitch(this, this._onGetLinkedRowsForLinktypeForColumn), 0);
		}
	},
	
	_cacheLinkedRowsForCol: function(resourceUri, desiredLinkType, fnToCall, results) {
		//Summary: Used to hold the links retrieved for columns.
	
		this._dataCache.setLinkedColsForType(resourceUri, desiredLinkType, results);
		
		fnToCall(results);
	},

	getLinkedRowsForLinktypeForRow: function(resourceUri, desiredLinkType, fnToCall) {
		//Summary: Sees if the desired row information is stored locally, and if not, goes and gets it.
		
		if (this._dataCache.hasLinkedRowsForType(resourceUri, desiredLinkType)) {
			//If we have already loaded the links, just return the list we have
			fnToCall(this._dataCache.getLinkedRowsForType(resourceUri, desiredLinkType));
		} else {
			this._getLinkedRowsForLinktypeForArtifact(resourceUri, desiredLinkType, fnToCall);
		}
	},
	
	startIntraCellNavigation: function(endIntraCellNavigationFunction, cellDomNode) {
		// summary:
		//		Starts keyboard navigation within the cell.
		// endIntraCellNavigationFunction:
		//		Function to be called when navigation is to end and be passed back to the parent grid.
		// cellDomNode:
		//		The cell content that is to be wrapped by a new navigation widget.		
		var navigationWidget = new com.ibm.rdm.web.artifactgrid.LinkNavigationWidget({
			onEndNavigation: endIntraCellNavigationFunction,
			focusNodeQuery: "[tabindex="+INTERNAL_NAVIGATION_TABINDEX+"]"
		}, cellDomNode);
		if (navigationWidget.noNavigationRequired()) {
			// the widget has no work to do - get rid of it immediately to avoid having it
			// catch keypresses.
			navigationWidget.destroyRecursive(/*preserveDOM*/true);
		}
	},
	
	//Link Menu Support Functions
	//////////////////////////////////////////////
	/// Link Menu functions

	_getLinkCreationText: function(linkLabel, count) {
		if (count === 1) {
			return dojo.string.substitute(this._messages.newlinkOne, [linkLabel, count]);
		} else {
			return dojo.string.substitute(this._messages.newlinkN, [linkLabel, count]);
		}
	},

	_isLinkCreationEnabledDummy: function(rowIds, count) {
		// summary:
		//		Method to test for whether or not to enable the create link menu option.  Return false so we will execute 
		//		the _isLinkCreationEnabledDeferred method to check permissions.
		// returns:
		//		false
		return false;
	},
	
	_isLinkCreationEnabledDeferred: function(linkType, rowIds, count) {
		// summary:
		//		Method to test for whether or not to enable the create link menu option for the specified linkType.  Returns a deferred since 
		//		enablement depends on permissions check.  
		// returns:
		//		Deferred
		var deferred = new dojo.Deferred();
		if (!this._isLinkCreationEnabled(rowIds, count)) {	
			deferred.callback(false);
		} else if (count > 1) {
			deferred.callback(true);
		} else {
			var resource = null, key;
			for (key in rowIds) {
				resource = this._dataCache.getRecordInfo(key);
			}
			var permissionsDeferred = ProcessOperationsCache.getPermissibleLinkTypesForCreation(resource.projectURI, resource.getProcessAreaURI());
			permissionsDeferred.addCallback(this, function(linkTypePermissions) {
				deferred.callback(linkTypePermissions[linkType.getActionId()]);
				return linkTypePermissions;
			});
		}
		return deferred;
	},

	_isLinkCreationEnabled: function(rowIds, count) {
		//Subject: used in link creation operations
		var createRecord = null, key;
		for (key in rowIds) {
			createRecord = this._dataCache.getRecordInfo(key);
		}
		var projectOperations = this._projectOperations;

		//since C/ALM rows do not contain the same methods, disable action.
		if (this._commands.containsCalmRow(rowIds)) {
			return false;
		}
		return (projectOperations.item(ProjectCache.MODIFY_RESOURCE) && !this._baselineURI);
	},

	executeLinkCreation: function(linkType, isSubject, rowIds) {
		var resourceURIs = [], key;
		for (key in rowIds) {
			resourceURIs.push(key);
		}
		if (resourceURIs.length > 0) {
			this.showCreateLinkDialog(linkType, isSubject, resourceURIs);
		}
	},
	
	_getLinkTypePreferencesForArtifactTypes: function(artifactTypeURIs) {
		// summary:
		//		Return the link type preferences for the given artifact types.  This method will return the union
		//		of the link type preferences specified for the artifact types.  If preferences are not specified for
		//		any of the artifact types, an empty list will be returned, indicating that all link types should be
		//		available.
		// returns: Array of LinkTypePreference
		
		var allPrefsMap = {};
		for (var i=0; i<artifactTypeURIs.length; i++) {
			var typeURI = artifactTypeURIs[i];
			var preferences = this._linkTypePreferences[typeURI];
			if (!preferences) {
				preferences = [];
			}
			if (preferences.length === 0) {
				// preferences are not defined, so all link types are active
				allPrefsMap = {};
				break;
			}
			for (var j=0; j<preferences.length; j++) {	
				var preference = preferences[j];
				allPrefsMap[preference.getLinkTypeURI() + preference.isSubjectPreference()] = preference;
			}
		}
		var allPrefs = [];
		for (var key in allPrefsMap) {
			var preference = allPrefsMap[key];
			allPrefs.push(preference);
		}
		return allPrefs;
	},
	
	_isMoreVisibleFunction: function(rowIds, count) {
		// summary:
		//		Method to test for whether or not to show the More... link menu option.  Show the menu option
		//		if preferred link types are being applied to the current selection.
		// returns:
		//		Boolean
		
		var key = null;
		var artifactTypes = [];
		for (key in rowIds) {
			var resource = this._dataCache.getRecordInfo(key);
			artifactTypes.push(resource.getType());
		}
		var prefs = this._getLinkTypePreferencesForArtifactTypes(artifactTypes);
		if (prefs.length === 0) {
			return false;
		}
		return true;
	},

	_isLinkItemPreferred: function(linkTypeURI, isSubject, rowIds, count) {
		// summary:
		//		Return whether on not the given linkTypeURI is preferred for the current selection in the grid.
		// returns:
		//		Boolean

		var key = null;
		var artifactTypes = [];
		for (key in rowIds) {
			var resource = this._dataCache.getRecordInfo(key);
			artifactTypes.push(resource.getType());
		}
		var preferences = this._getLinkTypePreferencesForArtifactTypes(artifactTypes);
		if (preferences.length === 0) {
			return true;
		}
		for (var i=0; i<preferences.length; i++) {
			var preference = preferences[i];
			if (preference.getLinkTypeURI() === linkTypeURI) {
				if (isSubject) {
					if (preference.isSubjectPreference()) {
						return true;
					}
				} else  {
					if (preference.isObjectPreference()) {
						return true;
					}
				}
			}
		}
		return false;
	},

	_isLinkVisibleFunction: function(linkType, isSubject, rowIds, count) {
		// summary:
		//		Method to test for whether or not to show the particular create link menu option.  Show the menu option
		//		if it is a preferred link type for the selected artifacts or if link type preferences are not defined for the artifacts.
		// returns:
		//		Boolean
		
		return this._isLinkItemPreferred(linkType.getResourceURI(), isSubject, rowIds, count);
	},

	//CALM Link Menu Functions
	_isCLMLinkCreationEnabled: function(rowIds, count) {
		if (count > 1) {
			return false;
		}
		
		var projectOperations = this._projectOperations;	

		//since C/ALM rows do no contain the same methods, disable action.
		if (this._commands.containsCalmRow(rowIds)) {
			return false;
		}
		return projectOperations.item(ProjectCache.MODIFY_CLM_LINK);
	},
	
	_isTrackedByCreationEnabled: function (rowIds, count) {
		var enabled = this._isCLMLinkCreationEnabled(rowIds, count);
		if (enabled) {
			var createRecord, key = null;
			for (key in rowIds) {
				createRecord = this._dataCache.getRecordInfo(key);
				var isCollection = (createRecord.getMimeType() === MimeTypes.getArtifactFormat(MimeTypes.COLLECTION_NAME, this._projectURI)
						|| createRecord.getMimeType() === MimeTypes.getArtifactFormat(MimeTypes.MODULE_NAME, this._projectURI));
				if (isCollection) {
					enabled = false;
					break;
				}
			}
		}
		return enabled;
	},
	
	_createCalmLinkOperation: function(oslcLinkTypeValue, linkTypeRole, layoutOperations, separateAbove, caller) {
		var self = this;
		var stub = {
			_resourceUri : oslcLinkTypeValue,
			_linkTypeURI: linkTypeRole.getLinkType().getResourceURI(),
			getResourceURI: function() {
				return this._resourceUri;
			},
			getLinkTypeURI: function() {
				return this._linkTypeURI;
			}
		};
		var labelFunction = function (count, rowIds, key) {
			return self._getLinkCreationText(linkTypeRole.getLabel(), count);
		};
		var enablementFunction = oslcLinkTypeValue === com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType["trackedBy"] 
			? dojo.hitch(this, this._isTrackedByCreationEnabled)
			: dojo.hitch(this, this._isCLMLinkCreationEnabled);
		var executeFunction = function(rowIds, count) {return self._executeLinkCreation(stub, true, rowIds);};
		var isLinkVisibleFunction = function(rowIds, count) {return self._isCLMLinkVisibleFunction(stub, linkTypeRole.isSubjectRole(), rowIds, count);}; 
		var label = linkTypeRole.getLabel();
		var id = ".LO"+label.replace(/[ ,]/gi, "-");
		var op = new com.ibm.rdm.web.grid.Operation(id, "", labelFunction, enablementFunction, executeFunction, isLinkVisibleFunction);
		caller.MENU_OPERATIONS[op.getId()] = op;
		layoutOperations.push({id: op.getId(), separatorAbove: separateAbove});
	},
	
	_isCLMLinkVisibleFunction: function(linkType, isSubject, rowIds, count) {
		// summary:
		//		Method to test for whether or not to show the create link menu option.  Uses link type preferences to determine
		//		visibility.
		// returns:
		//		Boolean
		
		return this._isLinkItemPreferred(linkType.getLinkTypeURI(), isSubject, rowIds, count);
		
	},

	loadCalmLinks: function(functionToCall, resource, linkType) {
		var statusMsgHandle = StatusDisplay.showStatusMessage(this._linkMessages.loadingLinks, true);
		var linkTypeDeferred = LinkTypeUtils.getLinkTypeForProjectAndSameAs(this._projectURI, linkType);
		var i;
		linkTypeDeferred.addCallback(this, function(linkType) {
			if (linkType) {
				var resourceUri = resource.resourceURI;
				var linkTypeUri = linkType.getResourceURI() + ArtifactConstants.SUBJECT_LINK;
				var thisFunc = dojo.hitch(this, function() {
					var hasLinkedColsForType = this._dataCache.hasLinkedColsForType(resourceUri, linkTypeUri);
					var intrinsiLinksColsForType = null;
					if (this.context.viewer._viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE && this.context.viewer._showBaseProperties) {
						intrinsiLinksColsForType = this._dataCache.getIntrinsicLinkedColsForType(resourceUri, linkTypeUri);
					}
					if (hasLinkedColsForType || intrinsiLinksColsForType) {
						var linkDetails = this._dataCache.getLinkedColsForType(resourceUri, linkTypeUri);
						
						var div = document.createElement("div");
						var editDiv = this._getPermissions(resourceUri, linkDetails, linkTypeUri);
						
						if(intrinsiLinksColsForType){
							//Handle scenario in which linkDetails is not defined
							if (!linkDetails){
								linkDetails =[];
							}
							for (i = 0; i < intrinsiLinksColsForType.length; i++){
								intrinsiLinksColsForType[i].isBaseArtifactLink = true;
								linkDetails.push(intrinsiLinksColsForType[i]);
							}				
						}
						var convertedLinks = [];
						for (i = 0; i < linkDetails.length; i++) {
							var link = linkDetails[i];
							convertedLinks.push(new com.ibm.rdm.web.links.Link({
								subject: resourceUri,
								object: link.link.uri,
								linkType: link.link.linkType,
								title: link.link.title
							}));
						}
						functionToCall(this._displayCALMLinks.call(this, convertedLinks, editDiv, div));
					} else {
						functionToCall("");
					}
				});
				this.getLinkedRowsForLinktypeForColumn(resourceUri, linkTypeUri, thisFunc);
			}
			StatusDisplay.hideStatusMessage(statusMsgHandle);
		});
	},
	
	_displayCALMLinks: function(links, editDiv, div) {
		var table = document.createElement("table");
		dojo.addClass(table, "no-table-style");
		var tbody = document.createElement("tbody");
		tbody.appendChild(div);
		table.appendChild(tbody);
		var i;
		var rows = [];
		for (i = 0; i < links.length; i++) {
			var resourceURI = links[i].getSubject();
			var targetURI = links[i].getObject();
			var resource = this._dataCache.getRecordInfo(resourceURI);
			var filterIcon = null;
			if (this._clmFilters && this._clmFilters.length > 0 && resource.clmFilterLinks) {
				var foundFilter = null;
				for (var j = 0; j < this._clmFilters.length; j++) {
					var clmFilter = this._clmFilters[j];
					var linksForFilter = resource.clmFilterLinks[clmFilter.getLinkType()];
					if (linksForFilter && dojo.indexOf(linksForFilter, targetURI) >= 0) {
						foundFilter = clmFilter;
						break;
					}
				}
				if (foundFilter && foundFilter.getImage()) {
					filterIcon = dojo.create("img", {"src" : foundFilter.getImage()});
				}
			}

			var row = new com.ibm.rdm.web.calm.CalmLinkRowWidget({
				calmLink: links[i],
				hideCalmIcon: filterIcon !== null,
				tabIndex: INTERNAL_NAVIGATION_TABINDEX
			});
			
			rows.push(row);
			
			if (filterIcon) {
				dojo.addClass(row.domNode, "calm-filter-item");
				row._img.appendChild(filterIcon);
			}
			tbody.appendChild(row.domNode);
			
			var suspectEntryUris = resource && resource.suspectEntries;
			if (suspectEntryUris && suspectEntryUris.length > 0) {
				var suspicionStatusWidget = new com.ibm.rdm.web.suspicion.SuspicionStatus({
					source: SuspectLinksUtil.EVENT_SOURCE.gridLinksColumn,
					projectURI: this._projectURI,
					suspectEntryURIs: suspectEntryUris,
					sourceResourceURI: targetURI,
					targetResourceURI: resource.resourceURI
				});
				dojo.style(suspicionStatusWidget.domNode, "visibility", "hidden");
				dojo.place(suspicionStatusWidget.domNode, row._suspicion, "only");
				dojo.removeClass(row._suspicion, "suspicion");
			}
		}
		// Create an outer div, becuase we do not want to put editDiv inside a table as it will never appear on IE8.
		var topDiv = document.createElement("div");
		var otherArgs = {div: topDiv, rows: rows};
		editDiv.addCallback(this, function(otherArgs, args) {
			args.div = otherArgs.div;
			args.calmLinkRowWidgets = otherArgs.rows;
			this._createEditDiv(args);
		}, otherArgs);
		
		if(div.firstChild){
			//Replaces the div with the first child of the div.
			dojo.place(div.firstChild, div, "replace");
		}
		topDiv.appendChild(table);
		return topDiv;
	},

	/////////////////////////////////////////////////////////////
	// Functions for the Link Group entry in the menu
	
	isLinkGroupEnabled: function(rowIds, count) {
		//Subject: used in link creation operations
		var createRecord = null, key;
		for (key in rowIds) {
			createRecord = this._dataCache.getRecordInfo(key);
			break;
		}

		var projectOperations = this._projectOperations;		

		//since C/ALM rows do no contain the same methods, disable action.
		if (this._commands.containsCalmRow(rowIds)) {
			return false;
		}
		if (projectOperations && projectOperations.item(ProjectCache.MODIFY_RESOURCE) && !this._baselineURI && !this._isInHistoryMode && !this._isInReviewMode) {
			return true;
		}
		return ((count === 1) && projectOperations && projectOperations.item(ProjectCache.MODIFY_CLM_LINK) && !this._baselineURI && !this._isInHistoryMode && !this._isInReviewMode);
	},

	getLinkGroupText: function(count) {
		if (count === 1) {
			return dojo.string.substitute(this._messages.createLinksGroup1, [count]);
		} else {
			return dojo.string.substitute(this._messages.createLinksGroupN, [count]);
		}
	},

	////////////////////////////////////////////
	/// Link Creation function

	_executeLinkCreation: function(linkType, isSubject, rowIds) {
		var key, resourceURIs = [];
		for (key in rowIds) {
			resourceURIs.push(key);
		}
		if (resourceURIs.length > 0) {
			this.showCreateLinkDialog(linkType, isSubject, resourceURIs);
		}
	},
	
	showCreateLinkDialog: function(linkType, isSubject, resourceURIs, processAreaURI, artifactTypeURI) {
		//Summary: This shows a dialog to allow the creation of M-M links.
		//  linkType: the link type (object) for the to-be-created link.
		//  isSubject: true if the link to be created is the subject side of the relation.
		//  resourceURI: the URI of the object to create the links 'from'.
		
		if (!resourceURIs || resourceURIs.length < 1) { return; }
		processAreaURI = null;
		artifactTypeURI = null;
		// don't uncomment this until have the drop-down link types submenu working properly

		if (resourceURIs.length === 1) {
			// if one link is selected, pass along the processAreaURI for permissions testing
			// if multiple links are selected, it will be difficult to collectively enforce the permissions, so
			// do not pass along the processArea URI in this case
			var resource = this._dataCache.getRecordInfo(resourceURIs[0]);
			processAreaURI = resource.getProcessAreaURI();
			artifactTypeURI = resource.getType();
		}
		
		var descriptionDeferred = com.ibm.rdm.web.model.resources.ResourceManager.getResourceDescribe({
			resourceURI: resourceURIs[0]
		});
		descriptionDeferred.addBoth(this, function(descriptionResponse){
			var normalizedContentType = QueryUtils.getNormalizedContentType(descriptionResponse.contentType);
			var isCollection = (normalizedContentType === MimeTypes.getArtifactFormat(MimeTypes.COLLECTION_NAME, this._projectURI)
					|| normalizedContentType === MimeTypes.getArtifactFormat(MimeTypes.MODULE_NAME, this._projectURI));
			var self = this;
			var filteredArtifacts = [];
			var filteredLinks = [];
			var resourceInfos = [];
			var resourceName = "", x, dialog;
			
			for (x = 0; x <resourceURIs.length; x++) {
				filteredArtifacts.push(resourceURIs[x]);
				var resource = this._dataCache.getRecordInfo(resourceURIs[x]);				
				var resourceInfo = resource;				

				if (this.context.viewer && this.context.viewer.moduleStructure) {									
					com.ibm.rdm.web.model.resources.ResourceManager.get({
						baselineURI: this.context.viewer.moduleStructure.baselineURI,
						resourceURI: this.context.viewer.moduleStructure.moduleURI,
						skipCache: false
					}).then(function (moduleResource) {						
						resourceInfo.usedInModuleArgs = {uri:moduleResource.resourceURI, title: moduleResource.getTitle()};
					});					
				}
				resourceInfos.push(resourceInfo);
			}
			
			var linkTypeURI = linkType ? linkType.getResourceURI() : null;
			
			//Map CALM links to the expected linktype
			if (linkType && linkType.getSameAs) {
				switch (linkType.getSameAs()) {
					case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["implementedBy"]:
						linkTypeURI = com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType["implementedBy"];
						break;
					case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["validatedBy"]:
						linkTypeURI = com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType["validatedBy"];
						break;
					case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["trackedBy"]:
						linkTypeURI = com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType["trackedBy"];
						break;
					case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["references"]:
						linkTypeURI = com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType["relatedTo"];
						break;
					case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["elaboratedBy"]:
						linkTypeURI = com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType["elaboratedBy"];
						break;
					case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["refine"]:
						linkTypeURI = com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType["refine"];
						break;
					case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["trace"]:
						linkTypeURI = com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType["trace"];
						break;
					case com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["satisfy_dm"]:
						linkTypeURI = com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType["satisfy_dm"];
						break;						
				} 
			}

			dialog = new com.ibm.rdm.web.links.LinkDialog({
				projectURI: self._projectURI,
				resourceURIs: resourceURIs,
				processAreaURI: processAreaURI,
				artifactTypeURI: artifactTypeURI,
				resourceName: resourceName,
				resourceInfos: resourceInfos,
				linkTypeURI: linkTypeURI,
				resourceIsObject: !isSubject,
				isCollection: isCollection,
				filterArtifacts: filteredArtifacts
			});

			dialog.show(
				// Note: We're passing the onDialogClose param here ...
				dojo.hitch(this, function(){
					this.grid.retakeFocus();
				})
			);
		});
	},

	////////////////////////////////////////////
	/// Attributes in Trace Columns functions

	setTraceabilityAttributesForColumn: function(linkURI, traceabilityAttrs) {
		this._traceabilityAttributesByColumn[linkURI] =
			dojo.clone(traceabilityAttrs);
	},
	
	getTraceabilityAttributesForColumn: function(linkURI) {
		return this._traceabilityAttributesByColumn[linkURI];
	},
	
	clearTraceabilityAttributesForColumn: function(linkURI) {
		if(this._traceabilityAttributesByColumn[linkURI]) {
			this._traceabilityAttributesByColumn[linkURI] = null;
			delete this._traceabilityAttributesByColumn[linkURI];
		}
	},
	
	setTraceabilityAttributeColumnFormat: function(linkURI, traceAttrFormat) {
		this._traceabilityAttributeFormatByColumn[linkURI] =
			dojo.clone(traceAttrFormat);
	},
	
	clearTraceabilityAttributeColumnFormat: function(linkURI) {
		if(this._traceabilityAttributeFormatByColumn[linkURI]) {
			this._traceabilityAttributeFormatByColumn[linkURI] = null;
			delete this._traceabilityAttributeFormatByColumn[linkURI];
		}
	},
	
	getTraceabilityAttributeColumnFormat: function(linkURI) {
		if(this._traceabilityAttributeFormatByColumn[linkURI]) {
			return this._traceabilityAttributeFormatByColumn[linkURI];
		}
		return DEFAULT_TRACE_FORMAT;
	},
	
	_displayTraceabilityAttributes: function(div, linkKey, resource) {
		// Add attributes to the div as well.
		var traceAttrs = this._traceabilityAttributesByColumn[linkKey];
		var attrFormat = this.getTraceabilityAttributeColumnFormat(linkKey);
		var singleValueSpan = null;

		var traceAttributesDiv = dojo.create("div", {
			className: "trace-attributes " +
				(attrFormat.showIcon ? "icon" : "no-icon")
		}, div);
		
		dojo.forEach(traceAttrs, function(traceAttr) {
			if(resource.hasAttribute(traceAttr.uri)) {
				var attributeDefinition = this._typeCache
						.getProjectAttributeDefinition(traceAttr.uri);
				var attrDiv = dojo.create("div", {
					className: "attribute-set"
				}, traceAttributesDiv);
				if(attributeDefinition) {
					var attrType = attributeDefinition.getAttributeType();
					var attrCallback = dojo.hitch(this, function(attributeValue){
						singleValueSpan = this._placeTraceAttribute(
							attrDiv, singleValueSpan, attrFormat,
							attributeDefinition, attributeValue
						);
					});
					this._display.getDisplayValue(attrType, traceAttr.uri,
						attrCallback, resource, null, true);
				}
				else if(traceAttr.uri === RRMTypes.WELL_KNOWN_OBJECT_TYPE) {
					var objectTypeStr = this._messages.objectType;
					var typeValue = resource.getType();
					
					TypesManager.getArtifactTypes(this._projectURI, [typeValue])
					.addCallback(this, function(typeObjects) {
						// We requested only one
						var typeObject = typeObjects[0];
						
						// The following two objects are hacks to emulate
						// the attribute type and definition objects which
						// are not provided by the types system for the
						// artifact type.
						var attrType = {
							getValueType: function() {
								return RRMTypes.ATTRIBUTE_TYPES.StringType;
							},
							getValue: function() {
								return typeObject.getTitle();
							},
							isEnum: function() {
								return false;
							}
						};
						var attrDef = {
							getTitle: function() {
								return objectTypeStr;
							},
							getSameAs: function() {
								return null;
							}
						};
						var attrCallback = dojo.hitch(
							this,
							function(targetDiv, attrDef, attrValue) {
								// Ignore the attribute value, since it's
								// just going to be a URL.
								singleValueSpan = this._placeTraceAttribute(
									targetDiv, singleValueSpan, attrFormat,
									attrDef, typeObject.getTitle()
								);
							},
							attrDiv, attrDef);
						this._display.getDisplayValue(attrType, traceAttr.uri,
							attrCallback, resource, null, true);
					});
				}
			}
		}, this);
	},
	
	_placeTraceAttribute: function(targetDiv, singleValueSpan, attrFormat, attrDef, attrValue) {
		if(attrFormat.showAttributeName) {
			// Only show the name label when specified in the format
			var label = dojo.string.substitute(
				this._messages.traceAttrLabel,
				[this._getAttributeLabel(attrDef)]);
			dojo.create("span", {
				className: "name",
				innerHTML: label
			}, targetDiv);
		}

		// TODO: This should be something like attrFormat.useCSV
		if(attrFormat.showAttributeName) {
			if (typeof(attrValue) === 'string') {
				dojo.create("span", {
					className: "value",
					innerHTML: attrValue
				}, targetDiv);
			}
			else {
				targetDiv.appendChild(attrValue);
			}
		}
		else {
			if(!singleValueSpan) {
				singleValueSpan = dojo.create("span", {
					className: "value",
					innerHTML: attrValue
				}, targetDiv);
			}
			else {
				singleValueSpan.innerHTML += ", " + attrValue;
			}
		}
		return singleValueSpan;
	},
	
	_getAttributeLabel: function(attrDef) {
		var sameAs = attrDef.getSameAs();
		var sameAsTypes = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES;

		if(sameAs) {
			switch(sameAs) {
			case sameAsTypes.Creator:
				return this._messages.creator;
			case sameAsTypes.Created:
				return this._messages.createdDate;
			case sameAsTypes.Contributor:
				return this._messages.lastModifiedBy;
			case sameAsTypes.ModifiedOn:
				return this._messages.lastModifiedDate;
			}
		}

		return attrDef.getTitle();
	},
	
	_displayCrossProjectTraceabilityAttributes: function(div, linkKey, resource) {
		// Note: If cross-project attributes are ever supported,
		// this is where you will want to do it. Check out the function
		// _displayTraceabilityAttributes above as an example.
		var traceAttrs = this._traceabilityAttributesByColumn[linkKey];
		var attrFormat = this.getTraceabilityAttributeColumnFormat(linkKey);
		var traceAttributesDiv = dojo.create("div", {
			className: "trace-attributes " +
				(attrFormat.showIcon ? "icon" : "no-icon")
		}, div);
		dojo.create("em", {
			className: "name",
			innerHTML: this._messages.traceAttrNotAvailable
		}, traceAttributesDiv);
	}
});
}) ();
