/*
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
 * ModuleArtifactCommands.js
 * (c) Copyright IBM Corporation 2011, 2015.
 * U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {

dojo.provide("com.ibm.rdm.web.artifactgrid.ModuleArtifactCommands");

dojo.require("dojo.DeferredList");

dojo.require("com.ibm.rdm.web.artifactgrid.ArtifactCommands");
dojo.require("com.ibm.rdm.web.artifactgrid.GridUtils");
dojo.require("com.ibm.rdm.web.artifactgrid.headingTypes");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.StatusDisplay");
dojo.require("com.ibm.rdm.web.grid.Module");
dojo.require("com.ibm.rdm.web.grid.Operation");
dojo.require("com.ibm.rdm.web.locks.LockManager");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.model.types.TypesManager");
dojo.require("com.ibm.rdm.web.modules.ArtifactCopierClient");
dojo.require("com.ibm.rdm.web.modules.PasteSpecialDialog");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.picker.ArtifactPickerForm");
dojo.require("com.ibm.rdm.web.picker.ArtifactTypePickerForm");
dojo.require("com.ibm.rdm.web.project.UploadArtifactForm");
dojo.require("com.ibm.rdm.web.query.ViewServiceClient");
dojo.require("com.ibm.rdm.web.server.cache.ProcessOperationsCache");
dojo.require("com.ibm.rdm.web.server.client.utils.ResourceUtil");
dojo.require("com.ibm.rdm.web.tags.TagUtil");
dojo.require("com.ibm.rdm.web.util.clipboard");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.project.EditManager");


var TagUtil = com.ibm.rdm.web.tags.TagUtil;
var ResourceUtil = com.ibm.rdm.web.server.client.utils.ResourceUtil;
var StatusDisplay = com.ibm.rdm.web.common.StatusDisplay;
var MessageHandler = com.ibm.rdm.web.common.MessageHandler;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var Operation = com.ibm.rdm.web.grid.Operation;
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;
var ProcessOperationsCache = com.ibm.rdm.web.server.cache.ProcessOperationsCache;
var TypesManager = com.ibm.rdm.web.model.types.TypesManager;
var ArtifactCopierClient = com.ibm.rdm.web.modules.ArtifactCopierClient;
var PasteSpecialDialog = com.ibm.rdm.web.modules.PasteSpecialDialog;
var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
var LockManager = com.ibm.rdm.web.locks.LockManager;
var GridUtils = com.ibm.rdm.web.artifactgrid.GridUtils;
var clipboard = com.ibm.rdm.web.util.clipboard;
var headingTypes = com.ibm.rdm.web.artifactgrid.headingTypes;
var EditManager = com.ibm.rdm.web.project.EditManager;

var CUT_ACTION = clipboard.Actions.CUT,
	COPY_ACTION = clipboard.Actions.COPY;


dojo.declare(
	"com.ibm.rdm.web.artifactgrid.ModuleArtifactCommands",
	com.ibm.rdm.web.artifactgrid.ArtifactCommands,
{
	// _permissionsCache: Object
	//		An instance of ArtifactPermissionsCache.
	_permissionsCache: null,

	// summary:
	//		Intended to provide a set of commands specific to a module.

	constructor: function(params) {
		// Note: the superclass uses private field names for things like '_dataCache'
		// and '_typeCache' ... so in that spirit of things we're using
		// '_permissionsCache' here.
		this._permissionsCache = params.permissionsCache;

		this.moduleStructure = params.moduleStructure;
		this.onTypeSystemReady = params.onTypeSystemReady;
		
		this._escapeConnection = dojo.connect(this.grid.module, "onEscKey", this, "clearClipboard");
	},
	
	onDestroy: function() {
		if (this._onExecConn) {
			dojo.disconnect(this._onExecConn);
			delete this._onExecConn;
		}
		if (this._escapeConnection) {
			dojo.disconnect(this._onExecConn);
			delete this._onExecConn;
		}
	},
	
	setCommentingInterface: function(commentingInterface) {
		this._commentingInterface = commentingInterface;
	},
	
	// TODO: Of course the base class should provide a generic list of commands
	// and we should add in the ones we want,
	getOperations: function(caller) {
	
		var self = this;
		var menuOperations = {
			".delete": //Params: _id, _iconClass, _displayText, _isEnabledMethod, _executeMethod
				new Operation(".delete", "removeCommand",
					function(numSelected) {return self.removeFromModuleText(numSelected); },
					function(rowIds, count) {
						// Note: We need to hear back from deferreds (in
						// getRemoveArtifactEnablementDeferred) so for now return false
						// (knowing that we'll conditionally enable this menu item when
						// we hear back from our deferreds).
						return false;
					},
					function(rowIds) {return self.executeDelete(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getRemoveArtifactEnablementDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".delete");})),
			".open": //Params: _id, _iconClass, _displayText, _isEnabledMethod, _executeMethod
				new Operation(".open", "openCommand",
					function(numSelected) {return self._messages.openArtifact; },
					function(rowIds, count) {return self.isOpenEnabled(rowIds, count);},
					function(rowIds) {self.executeOpen(rowIds);},
					function(rowIds, count) {
						return count === 1 && self._defaultIsVisible(rowIds, count);
					}),
			".openInNewTab":
				new Operation(".openInNewTab", "openInNewTabCommand",
					function(numSelected) {return self.openArtifactInNewTabText(numSelected);},
					function(rowIds, count) {return self.isOpenInNewTabOrWindowEnabled(count);},
					function(rowIds) {self.executeOpenInNewTab(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					null,
					dojo.hitch(this, function(count){
						return this._buildPreexistingBrowserBasedKeyCombo(".openInNewTab", count);
					})),
			".openInNewWindow":
				new Operation(".openInNewWindow", "openInNewWindowCommand",
					function(numSelected) {return self.openArtifactInNewWindowText(numSelected);},
					function(rowIds, count) {return self.isOpenInNewTabOrWindowEnabled(count);},
					function(rowIds) {self.executeOpenInNewWindow(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					null,
					dojo.hitch(this, function(count){
						return this._buildPreexistingBrowserBasedKeyCombo(".openInNewWindow", count);
					})),
			".openInNewSideBySideWindows":
				new Operation(".openInNewSideBySideWindows", "openInNewSideBySideWindowsCommand",
					function(numSelected) {return self._messages.openArtifactsSideBySide;},
					function(rowIds, count) {return count === 2;},
					function(rowIds) {self.executeOpenInNewSideBySideWindows(rowIds);},
					function(rowIds, count) {
						return count === 2 && self._defaultIsVisible(rowIds, count);
					}),
			".selectTags":
				new Operation(".selectTags", "selectTagsCommand",
					function(numSelected) {return self.selectTagsText(numSelected); },
					function(rowIds, count) {return self.isSelectTagsEnabled(rowIds, count);},
					function(rowIds) {self.executeSelectTags(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getSelectTagsDeferred(rowIds, count);}),
			".showUrl":
				new Operation(".showUrl", "showUrlCommand",
					function(numSelected) {return self.showArtifactUrlText(numSelected);},
					function(rowIds, count) {return true;},
					function(rowIds) {self.executeShowUrl(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);}
				),
			".lock":
				new Operation(".lock", "lockCommand",
					function(numSelected) {return self.lockText(numSelected); },
					function(rowIds, count) {return self.isLockEnabled(rowIds, count);},
					function(rowIds) {self.executeLock(rowIds);},
					function(rowIds, count) {return self.isLockVisible(rowIds, count);},
					// Use the EditRow enablement to work out if we can lock (only allowed if we can edit).
					// Note we are checking perms on the module artifact, not the base artifact we use for other locking issues.
					function(rowIds, count) {return self.getLockRowEnablementDeferred(rowIds, count);}),		
			".unlock":
				new Operation(".unlock", "unlockCommand",
					function(numSelected) {return self.unlockText(numSelected); },
					function(rowIds, count) {return self.isUnlockEnabled(rowIds, count);},
					function(rowIds) {self.executeUnlock(rowIds);},
					function(rowIds, count) {return self.isUnlockVisible(rowIds, count);},
					function(rowIds, count) {return self.getReleaseLockEnablementDeferred(rowIds, count);}),
			".copy":
				new Operation(".copy", "copyCommand",
					function(numSelected) {return self.copyText(numSelected); },
					function(rowIds, count) {return self._isCopyEnabled(rowIds, count);},
					function(rowIds) {self.executeCopyStructure(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					null,
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".copy");})),
			".export":
				new Operation(".export", "exportCommand",
					function(numSelected) {return self.exportText(numSelected); },
					function(rowIds, count) {return true;},
					function(rowIds) {self.executeExport(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);}),
			".loading":
				new Operation(".loading", "loadingDisplay",
					function() {return self._messages.loadingLinkTypes;},
					function() {return false;},
					function() {}),
			".loadingPreferredTypes":
				new Operation(".loadingPreferredTypes", "loadingDisplay",
					function() {return self._messages.loadingPreferred;},
					function() {return false;},
					function() {}),
			".createReport": //Params: _id, _iconClass, _displayText, _isEnabledMethod, _executeMethod
				new Operation(".createReport", "createReportCommand",
					function(numSelected) {return self.reportGenerateText(numSelected); },
					function(rowIds, count) {return self.isReportGenerate(rowIds, count);},
					function(rowIds) {self.doGenerateReport(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);}),
			".parseRequirements": //Params: _id, _iconClass, _displayText, _isEnabledMethod, _executeMethod
				new Operation(".parseRequirements", "parseRequirementsCommand",
					function(numSelected) {return self.parseRequirementText(numSelected); },
					function(rowIds, count) {return self.isParseEnabled(rowIds, count);},
					function(rowIds) {self.doParseRequirements(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);}),
			".comment":
				new Operation(".comment", "comment",
					function(numSelected) {return self._messages.openComments; },
					function(rowIds, count) {return self._isShowCommentsEnabled(rowIds, count);},
					function(rowIds) {self._executeOpenComments(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					null,
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".comment");})),
			".addComment":
				new Operation(".addComment", "addCommentCommand",
					function(numSelected) {return self._messages.addComment; },
					function(rowIds, count) {return self._isAddCommentEnabled(rowIds, count);},
					function(rowIds) {return self._executeAddComment(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getAddCommentDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".addComment");})),
			".addBelow":
				new Operation(".addBelowModule", "insert-row-below",
					function(numSelected) {return self._messages.belowElipses;},
					function(rowIds, count) {return self.isAddArtifactsOrCreateEnabled(rowIds, count);},
					function(rowIds) {
						return self.executeAddExisting(rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW
							);
						},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getAddArtifactsOrCreateEnabledDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".addBelow");})),
			".addBefore":
				new Operation(".addBeforeModule", "insert-row-before",
					function(numSelected) {return self._messages.beforeElipses;},
					function(rowIds, count) {return self.isAddArtifactsOrCreateEnabled(rowIds, count);},
					function(rowIds) {
						return self.executeAddExisting(rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE
							);
						},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getAddArtifactsOrCreateEnabledDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".addBefore");})),
			".addAfter":
				new Operation(".addAfterModule", "insert-row-after",
					function(numSelected) {return self._messages.afterElipses;},
					function(rowIds, count) {return self.isAddArtifactsOrCreateEnabled(rowIds, count);},
					function(rowIds) {
						return self.executeAddExisting(rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER
							);
						},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getAddArtifactsOrCreateEnabledDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".addAfter");})),
			".createSimilarBelow":
				new Operation(".createSimilarBelow", "insert-row-below",
					function(numSelected, rowIds, key) {
						return self.createSimilarWhereText(
								numSelected,
								rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW
							);
						},
					function(rowIds, count) {return self.isAddArtifactsOrCreateEnabled(rowIds, count);},
					function(rowIds) {
						return self.executeCreateSimilar(
								rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW
							);
						},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getAddArtifactsOrCreateEnabledDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".createSimilarBelow");})),
			".createSimilarBefore":
				new Operation(".createSimilarBefore", "insert-row-before",
					function(numSelected, rowIds, key) {
						return self.createSimilarWhereText(
								numSelected,
								rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE
							);
						},
					function(rowIds, count) {return self.isAddArtifactsOrCreateEnabled(rowIds, count);},
					function(rowIds) {
						return self.executeCreateSimilar(
								rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE
							);
						},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getAddArtifactsOrCreateEnabledDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".createSimilarBefore");})),
			".createSimilarAfter":
				new Operation(".createSimilarAfter", "insert-row-after",
					function(numSelected, rowIds, key) {
						return self.createSimilarWhereText(
								numSelected,
								rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER
							);
						},
					function(rowIds, count) {return self.isAddArtifactsOrCreateEnabled(rowIds, count);},
					function(rowIds) {
						return self.executeCreateSimilar(
								rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER
							);
						},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getAddArtifactsOrCreateEnabledDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".createSimilarAfter");})),
			".createAfter":
				new Operation(".createAfter", "insert-other-artifact-type-row-below",
					dojo.hitch(this, function(numSelected, rowIds, key) {
						return this._messages.insertOtherArtifactTypeAfter;
					}),
					dojo.hitch(this, function(rowIds, count) {return this.isAddArtifactsOrCreateEnabled(rowIds, count);}),
					dojo.hitch(this, function(rowIds) {
						return this.executeCreate(
							rowIds,
							com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER
						);
					}),
					dojo.hitch(this, function(rowIds, count) {return this._defaultIsVisible(rowIds, count);}),
					dojo.hitch(this, function(rowIds, count) {return this.getAddArtifactsOrCreateEnabledDeferred(rowIds, count);}),
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".createAfter");})),
			".uploadBelow":
				new Operation(".uploadBelow", "insert-row-below",
					dojo.hitch(this, function(numSelected, rowIds, key) {
						return this.createWhereText(
								numSelected,
								rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW
							);
						}),
					dojo.hitch(this, function(rowIds, count) {return this.isAddArtifactsOrCreateEnabled(rowIds, count);}),
					dojo.hitch(this, function(rowIds) {
						return this.executeUploadNew(
							rowIds,
							com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW
						);
					}),
					dojo.hitch(this, function(rowIds, count) {return this._defaultIsVisible(rowIds, count);}),
					dojo.hitch(this, function(rowIds, count) {return this.getAddArtifactsOrCreateEnabledDeferred(rowIds, count);}),
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".uploadBelow");})),
			".uploadBefore":
				new Operation(".uploadBefore", "insert-row-before",
					dojo.hitch(this, function(numSelected, rowIds, key) {
						return this.createWhereText(
								numSelected,
								rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE
							);
						}),
					dojo.hitch(this, function(rowIds, count) {return this.isAddArtifactsOrCreateEnabled(rowIds, count);}),
					dojo.hitch(this, function(rowIds) {
						return this.executeUploadNew(
							rowIds,
							com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE
						);
					}),
					dojo.hitch(this, function(rowIds, count) {return this._defaultIsVisible(rowIds, count);}),
					dojo.hitch(this, function(rowIds, count) {return this.getAddArtifactsOrCreateEnabledDeferred(rowIds, count);}),
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".uploadBefore");})),
			".uploadAfter":
				new Operation(".uploadAfter", "insert-row-after",
					dojo.hitch(this, function(numSelected, rowIds, key) {
						return this.createWhereText(
								numSelected,
								rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER
							);
						}),
					dojo.hitch(this, function(rowIds, count) {return this.isAddArtifactsOrCreateEnabled(rowIds, count);}),
					dojo.hitch(this, function(rowIds) {
						return this.executeUploadNew(
							rowIds,
							com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER
						);
					}),
					dojo.hitch(this, function(rowIds, count) {return this._defaultIsVisible(rowIds, count);}),
					dojo.hitch(this, function(rowIds, count) {return this.getAddArtifactsOrCreateEnabledDeferred(rowIds, count);}),
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".uploadAfter");})),
			".cut":
				new Operation(".cut", "cutArtifactCommand",
					function(numSelected, rowIds, key) {return self._cutText(numSelected);},
					function(rowIds, count) {
						// Note: We need to hear back from deferreds (in getCutEnablementDeferred)
						// so for now return false (knowing that we'll conditionally enable
						// this menu item when we hear back from our deferreds).
						return false;
					},
					function(rowIds) {
						self.executeModuleCut(rowIds);
						},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getCutEnablementDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".cut");})),
			".cancelCut":
				new Operation(".cancelCut", "cancelEdit",
					function(numSelected) {return self._messages.cancelCut; },
					function(rowIds, count) {return (self._isUnderCut())? true : false;},
					function(rowIds) {
						self.clearClipboard();
						},
					function(rowIds, count) {return (self._isUnderCut() ? true : false) && self._defaultIsVisible(rowIds, count);},
					null,
					dojo.hitch(this, function(){return "Esc";})),
			".pasteBelow":
				new Operation(".pasteBelow", "insert-row-below",
					function(numSelected, rowIds, key) {return self._pasteText(numSelected,
							com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW);},
					function(rowIds, count) {
						// Note: We need to hear back from deferreds (in getPasteEnablementDeferred)
						// so for now return false (knowing that we'll conditionally enable
						// this menu item when we hear back from our deferreds).
						return false;
					},
					function(rowIds) {
						return self.executeModulePaste(rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW
							);
						},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getPasteEnablementDeferred(rowIds, count);}),
			".pasteBefore":
				new Operation(".pasteBefore", "insert-row-before",
					function(numSelected, rowIds, key) {return self._pasteText(numSelected,
							com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE);},
					function(rowIds, count) {
						// Note: We need to hear back from deferreds (in getPasteEnablementDeferred)
						// so for now return false (knowing that we'll conditionally enable
						// this menu item when we hear back from our deferreds).
						return false;
					},
					function(rowIds) {
						return self.executeModulePaste(rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE
							);
						},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getPasteEnablementDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".pasteBefore");})),
			".pasteAfter":
				new Operation(".pasteAfter", "insert-row-after",
					function(numSelected, rowIds, key) {return self._pasteText(numSelected,
							com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER);},
					function(rowIds, count) {
						// Note: We need to hear back from deferreds (in getPasteEnablementDeferred)
						// so for now return false (knowing that we'll conditionally enable
						// this menu item when we hear back from our deferreds).
						return false;
					},
					function(rowIds) {
						return self.executeModulePaste(rowIds,
								com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER
							);
						},
					function(rowIds, count) {return (self._defaultIsVisible(rowIds, count));},
					function(rowIds, count) {return self.getPasteEnablementDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".pasteAfter");})),
			".pasteAsLink":
				new Operation(".pasteAsLink", "pasteAsLinkCommand",
					function(numSelected) {return self.pasteText(numSelected); },
					function(rowIds, count) {return self.isPasteAsLinkEnabled(rowIds, count);},
					function(rowIds) {self.executePasteAsLink(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					null,
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".pasteAsLink");})),
			".pasteSpecial":
				new Operation(".pasteSpecial", "",
					function(numSelected, rowIds, key) {return self._messages.pasteSpecial;},
					function(rowIds, count) {
						// Note: We need to hear back from deferreds (in getPasteSpecialEnablementDeferred)
						// so for now return false (knowing that we'll conditionally enable
						// this menu item when we hear back from our deferreds).
						return false;
					},
					function(rowIds) {
						self.executeModulePasteSpecial(rowIds);
						},
					function(rowIds, count) {return (self._defaultIsVisible(rowIds, count));},
					function(rowIds, count) {return self.getPasteSpecialEnablementDeferred(rowIds, count);}),
			".indent":
				new Operation(".indent", "indentArtifactCommand",
					function(numSelected, rowIds, key) {return self._indentText(numSelected);},
					function(rowIds, count) {
						// Note: We need to hear back from deferreds (in getIndentEnablementDeferred)
						// so for now return false (knowing that we'll conditionally enable
						// this menu item when we hear back from our deferreds).
						return false;
					},
					function(rowIds) {
						self.executeModuleIndent(rowIds);
					},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getIndentEnablementDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".indent");})),
			".outdent":
				new Operation(".outdent", "outdentArtifactCommand",
					function(numSelected, rowIds, key) {return self._outdentText(numSelected);},
					function(rowIds, count) {
						// Note: We need to hear back from deferreds (in getOutdentEnablementDeferred)
						// so for now return false (knowing that we'll conditionally enable
						// this menu item when we hear back from our deferreds).
						return false;
					},
					function(rowIds) {
						self.executeModuleOutdent(rowIds);
					},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getOutdentEnablementDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".outdent");})),
			".openDiagramView":
				new Operation(".openDiagramView", "openDiagramViewCommand",
					function(numSelected) {return self.openLinksExplorerText(numSelected); },
					function(rowIds, count) {return true;},
					function(rowIds) {self.openLinksExplorer(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);})
		};
		
		if (this._editInterface) {
			menuOperations[".editAttribs"] =
				new Operation(".editAttribs", "editAttribsCommand",
					function(numSelected) {return self._editInterface.editAttribsText(numSelected);},
					function(rowIds, count) {
						// Note: We need to hear back from deferreds (in
						// getEditAttributesEnablementDeferred) so for now return
						// false (knowing that we'll conditionally enable this
						// menu item when we hear back from our deferreds).
						return false;
					},
					function(rowIds) {
						var rowIdsArray = self._getUniqueKeysFromRowIds(rowIds);
						var resourceInfos = self._getRecordInfos(rowIdsArray);
						self._editInterface.executeEditAttribs(resourceInfos);
					},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getEditAttributesEnablementDeferred(rowIds, count);});
			menuOperations[".editRow"] =
				new Operation(".editRow", "editRowCommand",
					function(numSelected) {return self._messages.editArtifact; },
					function(rowIds, count) {
						// Note: We need to hear back from deferreds (in
						// getEditRowEnablementDeferred) so for now return
						// false (knowing that we'll conditionally enable this
						// menu item when we hear back from our deferreds).
						return false;
					},
					function(rowIds) {self._editInterface.editRow(rowIds);},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getEditRowEnablementDeferred(rowIds, count);});
			menuOperations[".toggleHeading"] =
				new Operation(".toggleHeading", "toggleHeadingCommand",
					function(numSelected, rowIds, key) {return self.getToggleHeadingText(numSelected, rowIds, key); },
					function(rowIds, count, key) {return self.isToggleHeadingEnabled(count);},
					function(rowIds, key) {
						self.executeOpenToggleHeadingAndType(rowIds, key);
					},
					function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
					function(rowIds, count) {return self.getHeadingEnablementDeferred(rowIds, count);},
					dojo.hitch(this, function(){return this._getKeyComboForOperation(".toggleHeading");}));
		}
		
		menuOperations[".openDiagramView"] = //Params: _id, _iconClass, _displayText, _isEnabledMethod, _executeMethod
			new Operation(".openDiagramView", "openDiagramViewCommand",
				function(numSelected) {return self.openLinksExplorerText(numSelected); },
				function(rowIds, count) {return true;},
				function(rowIds) {self.openLinksExplorer(rowIds);},
				function(rowIds, count) {return self._defaultIsVisible(rowIds, count);});
		
		var supportsEdit = !!this._editInterface,
			supportsLinks = !!this._linksInterface;

		//This defines how the menu is laid out - ie the grouping etc.
		var menuLayout = [];
		
		var createSimilarGroup = {
			groupId: "CreateSimilar",
			groupName: dojo.hitch(this, function(count, rowIds) {return this.createSimilarText(1, rowIds); }),
			isShown: true,
			isEnabled: dojo.hitch(this, function(rowIds, count) {return (count===1 && !(this._baselineURI || this._isInHistoryMode));}),
			isVisible: dojo.hitch(this, function(rowIds, count) {return this._defaultIsVisible(rowIds, count);}),
			separatorAbove: true,
			iconClass: "createArtifactCommand",
			operations: [
				{id: ".createSimilarAfter"},
				{id: ".createSimilarBefore"},
				{id: ".createSimilarBelow"},
				{id: ".createAfter", separatorAbove: true}
			]
		};
		menuLayout.push(createSimilarGroup);

		var addItemGroup = {
			//Intially the linking group is loading.
			groupId: "AddItem",
			groupName: dojo.hitch(this, function(count) {return this._messages.addArtifact;}),
			isShown: true,
			isEnabled: dojo.hitch(this, function(rowIds, count) {return (count===1 && !(this._baselineURI || this._isInHistoryMode));}),
			isVisible: dojo.hitch(this, function(rowIds, count) {return this._defaultIsVisible(rowIds, count);}),
			iconClass: "addArtifactCommand",
			operations: [
				{id: ".addAfter"},
				{id: ".addBefore"},
				{id: ".addBelow"}
			]
		};
		menuLayout.push(addItemGroup);
		
		var uploadGroup = {
			groupId: "UploadItem",
			groupName: dojo.hitch(this, function(count, rowIds) {return this._messages.uploadArtifact; }),
			isShown: true,
			isEnabled: dojo.hitch(this, function(rowIds, count) {return (count===1 && !(this._baselineURI || this._isInHistoryMode));}),
			isVisible: dojo.hitch(this, function(rowIds, count) {return this._defaultIsVisible(rowIds, count);}),
			iconClass: "uploadArtifactCommand",
			operations: [
				{id: ".uploadAfter"},
				{id: ".uploadBefore"},
				{id: ".uploadBelow"}
			]
		};
		menuLayout.push(uploadGroup);
		
		var editGroupOne = {
			groupId: "EditGroupOne",
			groupName: "Edit Group One",
			isShown: false
		};
		var editGroupOneOperations = [];
		var headingGroup = {
			isShown: true,
			separatorAbove: true,
			groupId: "HeadingGroup",
			iconClass: "uploadArtifactCommand",
			groupName: dojo.hitch(this, function(count, rowIds) {return this._messages.headingOrContent;}),
			isEnabled: dojo.hitch(this, function(rowIds, count) {return this.isToggleHeadingEnabled();}),
			isVisible: dojo.hitch(this, function(rowIds, count) {return this._defaultIsVisible(rowIds, count);})
		};
		var headingGroupOperations = [];
		var editGroupTwo = {
			groupId: "EditGroupTwo",
			groupName: "Edit Group Two",
			isShown: false
		};
		var editGroupTwoOperations = [];
		var openGroup = {
			groupId: "OpenGroup",
			iconClass: "openCommand",
			separatorAbove: true,
			groupName: dojo.hitch(this, function(count) {
				return (count > 1) ? this._messages.openArtifacts : this._messages.openArtifact;
			}),
			isShown: true,
			isEnabled: dojo.hitch(this, function(rowIds, count) { return true; }),
			isVisible: dojo.hitch(this, function(rowIds, count) { return this._defaultIsVisible(rowIds, count); }),
			operations: [
				{id: ".open"},
				{id: ".openInNewSideBySideWindows"},
				{id: ".openInNewTab"},
				{id: ".openInNewWindow"}
			]
		};
		menuLayout.push(openGroup);
		if (supportsEdit && !this._baselineURI) {
			editGroupOneOperations.push({id: ".editAttribs"});
			editGroupOneOperations.push({id: ".editRow"});
			headingGroupOperations.push({id: ".loadingPreferredTypes"});
			this.onTypeSystemReady(dojo.hitch(this, function() {
				var amendGrid = dojo.hitch(this, function(type) {
					headingGroupOperations = this._getHeadingGroupOperations(menuOperations, type);
					headingGroup.operations = headingGroupOperations;
					this.grid.amendRowType(com.ibm.rdm.web.artifactgrid.ArtifactConstants.RESOURCE_ROW,
							{menuLayout: menuLayout, operations: menuOperations});
				});
				ChangeNotifier.addListener({
					eventTopics: [com.ibm.rdm.web.notifications.EventTopics.OBJECT_TYPE_CHANGED],
					callbackFunction: dojo.hitch(this, function(eventParams) {
						var moduleTypeURI =
							this._dashboardInterface.getModuleArtifactTypeURI();
						var typeURI = eventParams.notifierParams.resourceURI;
						if (typeURI === moduleTypeURI) {
							TypesManager.getObjectType({
								projectURI: this._projectURI,
								projectSnapshotURI: this._baselineURI,
								typeURI: typeURI,
								skipCache: true
							}).then(
								dojo.hitch(this, function(type) {
									amendGrid(type);
								})
							);
						}
					}),
					callbackScope: this,
					neverDelayCallback: true
				});
				amendGrid();
			}));
		}
		editGroupTwoOperations.push( {id: ".outdent"}, {id: ".indent"} );
		if (supportsEdit && !this._baselineURI) {
			editGroupTwoOperations.push( {id: ".lock"}, {id: ".unlock"} );
		}
		var clipboardGroupOne = {
			groupId: "ClipboardGroupOne",
			groupName: "Clipboard Group One",
			isShown: false,
			separatorAbove: true,
			operations: [
				{id: ".cut"},
				{id: ".copy"},
				{id: ".cancelCut"}
			]
		};
		var pasteGroup = {
			groupId: "PasteItem",
			groupName: function(count, rowIds) { return self._pasteText(count); },
			isShown: true,
			isEnabled: function(rowIds, count) { return self.getPasteEnablementDeferred(rowIds, count); },
			isVisible: function(rowIds, count) { return self._defaultIsVisible(rowIds, count); },
			iconClass: "pasteArtifactCommand",
			operations: [
				{id: ".pasteAfter"},
				{id: ".pasteBefore"},
				{id: ".pasteBelow"}
			]
		};
		var clipboardGroupTwo = {
			groupId: "ClipboardGroupTwo",
			groupName: "Clipboard Group Two",
			isShown: false,
			operations: [
				{id: ".pasteAsLink"},
				{id: ".pasteSpecial"},
				{id: ".delete"}
			]
		};
		
		editGroupOne.operations = editGroupOneOperations;
		headingGroup.operations = headingGroupOperations;
		editGroupTwo.operations = editGroupTwoOperations;
		menuLayout.push(editGroupOne, headingGroup, editGroupTwo, clipboardGroupOne, pasteGroup, clipboardGroupTwo);
		
		var commentGroup = {
			//Initially the linking group is loading.
			groupId: "Comment",
			groupName: dojo.hitch(this, function(count) {return this._messages.comments;}),
			isShown: true,
			isEnabled: dojo.hitch(this, function(rowIds, count) {return (count===1);}),
			isVisible: dojo.hitch(this, function(rowIds, count) {return this._defaultIsVisible(rowIds, count);}),
			separatorAbove: true,
			iconClass: "commentCommand",
			operations: [
				{id: ".comment"},
				{id: ".addComment"}
			]
		};
		menuLayout.push(commentGroup);
		
		var otherActionsGroup = {
			//Intially the linking group is loading.
			groupId: "OtherActionsGroup",
			groupName: dojo.hitch(this, function(count) {return this._messages.otherActions;}),
			isShown: true,
			isEnabled: dojo.hitch(this, function(rowIds, count) {return true;}),
			isVisible: dojo.hitch(this, function(rowIds, count) {return this._defaultIsVisible(rowIds, count);}),
			operations: [
				{id: ".selectTags"},
				{id: ".export"},
				{id: ".openDiagramView"}
			]
		};
		menuLayout.push(otherActionsGroup);
		
		if (supportsLinks) {
			var group2 = {
				//Intially the linking group is loading.
				groupId: "CreateLinks",
				groupName: function(count) {return self._linksInterface.getLinkGroupText(count);},
				isShown: true,
				isEnabled: function(rowIds, count) {return self._linksInterface.isLinkGroupEnabled(rowIds, count);},
				isVisible: function(rowIds, count) {return self._defaultIsVisible(rowIds, count);},
				separatorAbove: true,
				iconClass: "addLinksCommand",
				operations: [ {id: ".loading"} ]
			};
			menuLayout.push(group2);
		}

		var group3 = {
			groupId: "ShowURLGroup",
			isShown: false,
			isEnabled: true,
			operations: [{id: ".showUrl"}]
		};

		menuLayout.push(group3);

		return {operations: menuOperations, layout: menuLayout};
	},
	
	_getHeadingGroupOperations: function(menuOperations, moduleType) {
		var headingOperations = [];
		if (!moduleType) {
			var moduleTypeURI =
				this._dashboardInterface.getModuleArtifactTypeURI();
			moduleType =
				this._typeCache.getProjectObjectType(moduleTypeURI);
		}
		var headingTypes =
			this._typeCache.getPreferredHeadingObjectTypesForType(moduleType);
		var nonHeadingTypes =
			this._typeCache.getPreferredNonHeadingObjectTypesForType(moduleType);
		var prunedListsObject =
				com.ibm.rdm.web.artifactgrid.GridUtils
					.getPreferredObjectTypesToShow(headingTypes, nonHeadingTypes);
		var numHeadingTypes = prunedListsObject.headingObjectTypesToShow.length;
		var numNonHeadingTypes = prunedListsObject.nonHeadingObjectTypesToShow.length;
		
		var createOperation = function(type, isHeading, addSeparator) {
			var operation = type ?
					this._getHeadingOperation(type, isHeading) :
					this._getUnavailableHeadingOperation(isHeading);
			menuOperations[operation.getId()] = operation;
			headingOperations.push({id: operation.getId(), separatorAbove: addSeparator});
		};
		
		if (numHeadingTypes === 0) {
			createOperation.call(this, null, true, false);
		} else {
			dojo.forEach(prunedListsObject.headingObjectTypesToShow, function(type) {
				createOperation.call(this, type, true, false);
			}, this);
		}
		
		if (numNonHeadingTypes === 0) {
			createOperation.call(this, null, false, true);
		} else {
			dojo.forEach(prunedListsObject.nonHeadingObjectTypesToShow, function(type, index) {
				createOperation.call(this, type, false, index === 0);
			}, this);
		}
		
		var moreOptionsOperation = this._getMoreHeadingOptionsOperation();
		menuOperations[moreOptionsOperation.getId()] = moreOptionsOperation;
		headingOperations.push({id: moreOptionsOperation.getId(), separatorAbove: true});

		return headingOperations;
	},
	
	_getHeadingOperation: function(type, isHeading) {
		var iconClass = isHeading ? "row-type-icon numbered-row" : "row-type-icon not-numbered-row";
		var id = type.getResourceURI();
		var labelFunction = dojo.hitch(this, function(count) {
			var baseMessage = isHeading ? this._messages.numberedRow : this._messages.normalRow;
			return dojo.string.substitute(baseMessage, [type.getTitle()]);
		});
		var enabledFunction = function(rowIds, count, key) {
			return true;
		};
		var executeFunction = dojo.hitch(this, function(rowIds, key) {
			return this._executeToggleHeadingAndType(type, isHeading, rowIds, key);
		});
		var visibilityFunction = dojo.hitch(this, function(rowIds, count, key) {
			return this._toggleHeadingAndTypeVisibility(type, isHeading, rowIds, count, key);
		});
		var enablementDeferredFunction = dojo.hitch(this, function(rowIds, count, key) {
			return this._toggleHeadingAndTypeEnablementDeferred(type, isHeading, rowIds, count, key);
		});
		
		var operation = new Operation(id, iconClass, labelFunction,
				enabledFunction, executeFunction, visibilityFunction, enablementDeferredFunction);
		return operation;
	},
	
	_getUnavailableHeadingOperation: function(isHeading) {
		var id = ".unavailable" + (isHeading ? "Heading" : "NonHeading");
		var operation = new Operation(id, "",
				dojo.hitch(this, function() {
					var baseMessage = isHeading ? this._messages.numberedRow : this._messages.normalRow,
						noneAvailable = com.ibm.rdm.web.util.domutils.getXMLEncodedText(
							dojo.string.substitute(
								this._messages.angleBracketsMessage, [this._messages.noneAvailable]
							)
						);
					return dojo.string.substitute(baseMessage, [noneAvailable]);
				}),
				function() {return false;},
				function() {}
		);
		return operation;
	},
	
	_getNumberOfRowIds: function(rowIds) {
		var count = 0,
			uri;
		for (uri in rowIds) {
			count++;
		}
		return count;
	},
	
	_getMoreHeadingOptionsOperation: function() {
		var operation = new Operation(".preferredMoreOptions", "",
				dojo.hitch(this, function() {
					return this._messages.moreOptions;
				}),
				dojo.hitch(this, function() {
					// We return false and instead rely on this Operation's
					// getEnablementDeferred to set the enablement appropriately.
					return false;
				}),
				dojo.hitch(this, function(rowIds, key) {
					var count = this._getNumberOfRowIds(rowIds),
						objectType = this._getTypeForResourceURI(key),
						resource = this._dataCache.getResourceByUri(key),
						preselectHeadingType = (
							this.moduleStructure.isHeading(key) ?
							headingTypes.Heading :
							headingTypes.NonHeading
						),
						isModifyArtifactTypeAllowed = (
							count === 1 ?
							this._permissionsCache.getModifyArtifactPermission(
								resource
							).isModifyArtifactTypeAllowed() :
							true
						);

					var form = new com.ibm.rdm.web.picker.ArtifactTypePickerForm({
						destroyContents: true,
						projectURI: this._projectURI,
						folderURI: this._dashboardInterface.getAssetFolderURI(),
						viewType: com.ibm.rdm.web.picker.ArtifactTypePickerForm.VIEW_TYPE.ModuleGridEditMenu,
						title: this._messages.pickerDialogTitle,
						description: this._messages.pickerDialogDescription,
						artifactTypeUriToPreselect: objectType.getResourceURI(),
						onlyShowArtifactTypesOfModuleFormat: false,
						allowMultipleSelections: true,
						headingTypeToPreselect: preselectHeadingType,
						isModifyArtifactTypeAllowed: isModifyArtifactTypeAllowed,
						permissionType: ProcessOperationsCache.PERMISSION_TYPES.Modify,
						tooltipToUseForDisabledListItems: (
							isModifyArtifactTypeAllowed ?
							null :
							dojo.string.substitute(
								this._messages.cannotChangeArtifactTypeTooltip,
								[objectType.getTitle()]
							)
						),
						allowOverridingHeadingType: true,
						numberOfTargetArtifacts: count,
						targetArtifactHeadingType: preselectHeadingType,
						onOk: dojo.hitch(this, function() {
							var typeAndHeadingInfo = form.getSelectedArtifactTypeAndHeadingType();
							var type = typeAndHeadingInfo.objectType || objectType;
							var isHeading = typeAndHeadingInfo.headingType === headingTypes.Heading;
							this._executeToggleHeadingAndType(
								type, isHeading, rowIds, key, objectType, preselectHeadingType
							);
						})
					});
					form.show(dojo.hitch(this, this._onLaunchedDialogHasClosed));
				}),
				function() { return true; },
				dojo.hitch(this, function(){
					var deferred = new dojo.Deferred();
					if (this._isModifyArtifactsAllowedWithoutConsideringTeamOperations()) {
						this.getIsModifyModuleContentAllowed().then(
							function(isModifyModuleContentAllowed){
								deferred.resolve(isModifyModuleContentAllowed);
							}
						);
					} else {
						deferred.resolve(false);
					}
					return deferred;
				}),
				dojo.hitch(this, function(){return this._getKeyComboForOperation(".toggleHeading");})
		);
		return operation;
	},
	
	_getModifyTypeAllowedResourceURIs: function(resourceInfos) {
		var modifyTypeAllowedResourceURIs = [];
		
		dojo.forEach(
			resourceInfos,
			function(resourceInfo) {
				var modifyArtifactPermissions = this._permissionsCache.getModifyArtifactPermission(resourceInfo),
					isModifyTypeAllowed;
				if (modifyArtifactPermissions) {
					isModifyTypeAllowed = modifyArtifactPermissions.isModifyArtifactTypeAllowed();
				} else {
					// No permissions - allow it as a fallback.
					isModifyTypeAllowed = true;
				}
				if (isModifyTypeAllowed) {
					modifyTypeAllowedResourceURIs.push(resourceInfo.getResourceURI());
				}
			},
			this
		);

		return modifyTypeAllowedResourceURIs;
	},

	_executeToggleHeadingAndType: function(
		type, isHeading, rowIds, key,
		originalTargetRowArtifactType, originalTargetRowHeadingType)
	{
		var count = this._getNumberOfRowIds(rowIds),
			resourceInfos =
				this._getRecordInfos(this._getUniqueKeysFromRowIds(rowIds)),
			modifyTypeAllowedResourceURIs =
				this._getModifyTypeAllowedResourceURIs(resourceInfos),
			anyDeniedPermission =
				modifyTypeAllowedResourceURIs.length !== count;

		// As an optimization, if there is only a single row selected and
		// we've been asked to change its row style, but not its artifact
		// type, we'll make a call to only change the headingness (not the
		// artifact type as well).
		//
		// In addition, this approach ALSO allows the user to change the
		// headingness for a row even when "Modify this artifact's type"
		// isn't allowed (as it's only the module structure that gets locked
		// in this case [not ALSO the underlying core artifact]).
		//
		// Please note that UI no longer enables the user select >1 row and
		// ONLY toggle headingness.

		if (count === 1
			&& originalTargetRowArtifactType
			&& originalTargetRowArtifactType.getResourceURI() === type.getResourceURI()
			) {
			this._processToggleHeading(isHeading, rowIds, key);
		}
		else if (modifyTypeAllowedResourceURIs.length === 0) {
			// The user can't change the artifact type for any of the
			// rows ... tell the user and bow out ...
			var msgForm = new com.ibm.rdm.web.common.MessageForm({
				confirmMessage: this._messages.cannotChangeTypes,
				dialogTitle: this._messages.cannotChangeTypesDialogTitle,
				dialogStyle: com.ibm.rdm.web.common.MessageForm.DIALOG_TYPES.Ok
			});	
			msgForm.show();
		}
		else {
			this._processToggleHeadingAndType(
				type, isHeading, rowIds, key,
				modifyTypeAllowedResourceURIs, anyDeniedPermission
			);
		}
	},

	_processToggleHeadingAndType: function(
		type, isHeading, rowIds, key,
		modifyTypeAllowedResourceURIs, anyDeniedPermission)
	{
		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI();
		var moduleURI = this.moduleStructure.moduleURI;		
		var coreURI = this._dataCache.getBaseArtifactForResource(key);
		var currentFocusRowKey = this.grid.getFocusRowKey();
		var resourceURIs = this._getResourceURIsFromRowIds(rowIds);
		
		var initialLockObtainedFn;
		var targetURI;
		var singleRowOp = false;
		
		var lockFailedFn = function() {};
		
		var modulelockObtainedFn = dojo.hitch(this, function() {
			this._editInterface.saveStarted(resourceURIs);
			
			StatusDisplay.showStatusMessage(this._messages.updatingArtifact);
			
			this._processToggleHeading(isHeading, rowIds, key, true).then(
				dojo.hitch(this, function() {
					// this will lock the resources. They will be unlocked by this._changeType
					var getResourcesDeferred = this._editInterface.getResources(
						modifyTypeAllowedResourceURIs, false, true, null, anyDeniedPermission
					);
					getResourcesDeferred.then(
						dojo.hitch(this, function(mapping) {
							this._changeType(mapping.resources, mapping.lockURIMap, type).then(
								dojo.hitch(this, function() {
									StatusDisplay.hideStatusMessage();
									LockManager.releaseLockedResource(moduleURI);
									if (singleRowOp) {
										LockManager.releaseLockedResource(coreURI);
									}
									this._editInterface.saveFinished(resourceURIs);
									if (currentFocusRowKey) {
										this.grid.moveFocusToCellOnRow(currentFocusRowKey);
									}
								})
							);
						}),
						dojo.hitch(this, function(error) {
							StatusDisplay.hideStatusMessage();
						})
					);
				})
			);
		});
		
		var artifactLockObtainedFn = dojo.hitch(this, function() {
			this._implicitLockFunction(modulelockObtainedFn, lockFailedFn, moduleURI, this._projectURI, moduleProcessAreaURI);
		});
		
		if (resourceURIs.length == 1) {
			initialLockObtainedFn = artifactLockObtainedFn;
			targetURI = coreURI;
			singleRowOp = true;
		} else {
			initialLockObtainedFn = modulelockObtainedFn;
			targetURI = moduleURI;
		}
		
		this._implicitLockFunction(initialLockObtainedFn, lockFailedFn, targetURI, this._projectURI, moduleProcessAreaURI);
	},
	
	_changeType: function(resources, lockURIMap, type) {
		var allDone = new dojo.Deferred();
		var changeTypeDeferreds = [];
		dojo.forEach(resources, function(resource) {
			var currentTypeURI = resource.getType();
			var newTypeURI = type.getResourceURI();
			if (currentTypeURI !== newTypeURI) {
				changeTypeDeferreds.push(resource.replaceType(resource.getType(), type.getResourceURI()));
			}
		});
		if (changeTypeDeferreds.length === 0) {
			// we're not changing any type, but we need to unlock the resources
			// that were locked by the this._editInterface.getResources() call in _processToggleHeadingAndType()
			var lockedResourcesUris = [];
			dojo.forEach(resources, function(resource) {
				var resourceURI = resource.getResourceURI();
				var lockingURI = lockURIMap[resourceURI] || resourceURI;
				lockedResourcesUris.push(lockingURI);
			});
			var unlockDeferred = LockManager.releaseLockedResources(lockedResourcesUris);
			unlockDeferred.addBoth(this, function() {
				var refresh = [];
				dojo.forEach(lockedResourcesUris, function(key) {
					refresh.push({
						key: key,
						rowtype: com.ibm.rdm.web.artifactgrid.ArtifactConstants.RESOURCE_ROW
					});
				});
				this.grid.refreshRows(refresh);
			});
			unlockDeferred.then(dojo.hitch(this, function() {
				allDone.callback();
			}));
			return allDone;
		}
		var changeTypeDeferredList = new dojo.DeferredList(changeTypeDeferreds);
		changeTypeDeferredList.then(
			dojo.hitch(this, function() {
				var saveDeferredsArray = [];
				var saveInitiatedDeferredArray = [];
				com.ibm.rdm.web.model.resources.ResourceManager.startSaveBatching();
				dojo.forEach(resources, function(resource) {
					// Add resource to permissions cache so that permissions for the new type can
					// be fetched if necessary
					this._permissionsCache.addResource(resource);
					var resourceURI = resource.getResourceURI();
					var lockingURI = lockURIMap[resourceURI] || resourceURI;
					
					var saveInitiatedDeferred = new dojo.Deferred();
					saveInitiatedDeferredArray.push(saveInitiatedDeferred);
					
					saveDeferredsArray.push(resource.save({
						refreshAfterSave: true, 
						releaseLock: true, 
						saveInitiatedDeferred: saveInitiatedDeferred, 
						lockingURI: lockingURI,
						skipLinkCheck: true
					}));
				}, this);
				
				var saveInitiatedDeferredList = new dojo.DeferredList(saveInitiatedDeferredArray);
				saveInitiatedDeferredList.then(dojo.hitch(this, function() {
					com.ibm.rdm.web.model.resources.ResourceManager.completeSaveBatching().then(dojo.hitch(this, function() {
						allDone.callback();
					}));
				}));
				
				var deferredsList = new dojo.DeferredList(saveDeferredsArray);
				deferredsList.addBoth(this, function(response) {
					com.ibm.rdm.web.common.UpdateContext.cachedArtifactHasBeenSaved();
					ChangeNotifier.notifyListeners({
						eventTopic: com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_SAVED,
						projectURI: this._projectURI,
						resources: resources
					});
					
				});
			})
		);
		return allDone;
	},
	
	_toggleHeadingAndTypeVisibility: function(type, isHeading, rowIds, key) {
		return true;
	},

	_toggleHeadingAndTypeEnablementDeferred: function(type, isHeading, rowIds, count, key) {
		var deferred = new dojo.Deferred();
		if (count === 1) {
			var resource = this._dataCache.getResourceByUri(key);
			if (this._isModifyArtifactsAllowedWithoutConsideringTeamOperations()) {
				this.getIsModifyModuleContentAllowed().then(
					dojo.hitch(this, function(isModifyModuleContentAllowed){
						var modifyArtifactPermissions =
							this._permissionsCache.getModifyArtifactPermission(resource);
						// Note: We can only change the row's artifact type or headingness
						// if it's the case that we can both modify the module's content
						// AND modify the row's artifact type.
						deferred.resolve(
							isModifyModuleContentAllowed
							&& modifyArtifactPermissions.isModifyArtifactTypeAllowed()
						);
					})
				);
			} else {
				deferred.resolve(false);
			}
		} else {
			deferred.resolve(true);
		}
		return deferred;
	},
	
	_getTypeForResourceURI: function(resourceURI) {
		var resource = this._dataCache.getResourceByUri(resourceURI);
		var typeURI = resource.getType();
		return this._typeCache.getProjectObjectType(typeURI);
	},
	
	_getResourceURIsFromRowIds: function(rowIds) {
		var resourceURIs = [];
		for (var resourceURI in rowIds) {
			resourceURIs.push(resourceURI);
		}
		return resourceURIs;
	},
	
	executeUploadNew: function(resources, where){
		// summary:
		//		Upload a new artifact and add it into the module structure with
		//		the specified placement.
		// resources:
		//		the resource for the new artifact to be placed in relation to
		// where:
		//		the Below/Before/After placement strategy for the uploaded artifact
		
		var resourceURI,
		selectedFolder,
		uploadDeferred = new dojo.Deferred();
			
		for (var thisResource in resources){
			resourceURI = thisResource;
			break;
		}
		
		// Specify actions to to take on success or failure of acquiring lock
		var lockFailedFn = function() {};
		var modURI = this.moduleStructure.moduleURI;
		
		var lockObtainedFn = dojo.hitch(this, function() {
			//This function will run when the lock is obtained
			
			//Determine current folder for the module
			var assetFolderLoc = this._dashboardInterface.getAssetFolderURI();
			selectedFolder = (assetFolderLoc)?assetFolderLoc:null;
			
			var uploadArtifactForm = new com.ibm.rdm.web.project.UploadArtifactForm({
				artifactFormat: MimeTypes.getArtifactFormat(MimeTypes.WRAPPER_RESOURCE_NAME, this._projectURI),
				projectURI: this._projectURI,
				resourceURI: this.resourceURI,
				folderURI: selectedFolder,
				hideOpen: true,
				quickUpload: false,
				typeURI: ResourceUtil.getLastSelectedObjectType(this._projectURI).typeURI
			});
			
			// Initiate save of the reference artifact if it is under edit  No need to wait for the save to complete before
			// showing the dialog.  The save can just proceed in the background.
			if (resourceURI && this._editInterface.isArtifactUnderEdit(resourceURI)) {
				// returns a deferred, but we don't need to wait for it.
				this._editInterface.saveAllEdits();
			}
			
			var releaseLock = function() {
				LockManager.releaseLockedResource(modURI);
			};
			// Once the artifacts have been uploaded, add them into the module where specified			
			var con = dojo.connect(uploadArtifactForm, "onCreate", this, function (artifact, openArtifactInEditMode){
				uploadDeferred.callback(true);
				dojo.disconnect(con);
				this.addExistingArtifacts(resourceURI, artifact, where).then(releaseLock, releaseLock);
				this.grid.moveFocusToCellOnRow(resourceURI);
			});
			
			uploadArtifactForm.show(dojo.hitch(this, this._onLaunchedDialogHasClosed));
		});
		
		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI();

		// Attempt to acquire up to date lock before operation
		this._implicitLockFunction(lockObtainedFn, lockFailedFn, modURI, this._projectURI, moduleProcessAreaURI);

		// Return final deferred
		return uploadDeferred;
	},
	
	executeAddExisting: function(resources, where, /*Array?*/existingResources){
		var existingArtifacts = [],
			resourceURI,
			addDeferred = new dojo.Deferred(),
			artifactPickerForm,
			targetRowURI = null;
			
		for (var thisResource in resources){
			resourceURI = thisResource;
			break;
		}
			
		var addArtifacts = dojo.hitch(this, function(selectedArtifacts, event) {
			var numAdded = selectedArtifacts.length,
				i;
			var executeAfterRowsAddedFunctions = dojo.hitch(this, function(){
				artifactPickerForm.hideBusy();
				artifactPickerForm.displayOkMessage(this._messages["addedArtifacts"]);
				if (event){
					artifactPickerForm.hide();
				}
				addDeferred.callback(resources);
			});
			var getNthNextSiblingRowURI = dojo.hitch(this, function(targetRowURI, howManySiblingsDown){
				for (i = 0; i < howManySiblingsDown; i++) {
					var rowObject = this.moduleStructure.getNextSibling(targetRowURI);
					if (rowObject) {
						targetRowURI = rowObject.uri;
					}
				}
				return targetRowURI;
			});
			artifactPickerForm.showBusy();
			artifactPickerForm.displayInfoMessage(this._messages["addingToModule"]);
			this.addExistingArtifacts(targetRowURI || resourceURI, selectedArtifacts, where).then(dojo.hitch(this, function(resources){
				switch(where) {
				case com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE:
				case com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST:
					executeAfterRowsAddedFunctions();
					break;
				case com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER:
					// Note: We take care here to make sure that we're always "adding after" with regard
					// to the last added row (i.e. _not_ with regard to the original row used to launch
					// the add existing artifact dialog [as the user may be using "Add" to continually
					// add rows via this dialog, and the user should see that these rows are always
					// placed with regard to the last row added via this dialog]).
					this._invokeOnReconcile(dojo.hitch(this, function(){
						targetRowURI = targetRowURI || resourceURI;
						targetRowURI = getNthNextSiblingRowURI(targetRowURI, numAdded);
						executeAfterRowsAddedFunctions();
					}));
					break;
				case com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW:
					// Note: We take care here to make sure that we're always "adding below (as a child)"
					// with regard to the last added row (i.e. _not_ with regard to the original row
					// used to launch the add existing artifact dialog [as the user may be using "Add"
					// to continually add rows via this dialog, and the user should see that these
					// rows are always placed with regard to the last row added via this dialog]).
					// After placing the first child, we change 'where' to AFTER so that additional
					// clicks on "Add" for the dialog will result in placing the new child after
					// the last added row.
					this._invokeOnReconcile(dojo.hitch(this, function(){
						targetRowURI = targetRowURI || resourceURI;
						var rowObject;
						if (targetRowURI === resourceURI) {
							rowObject = this.moduleStructure.getArtifactByURI(resourceURI);
							if (rowObject) {
								var firstChildURI = rowObject.children[0].uri;
								if (firstChildURI) {
									targetRowURI = firstChildURI;
									if (numAdded > 1) {
										// The user has just clicked for the first time on the "Add" button
										// in the dialog AND there were multiple artifacts selected.  Set
										// targetRowURI to the last added row.
										targetRowURI = getNthNextSiblingRowURI(targetRowURI, numAdded-1);
									}
								}
							} else {
								// Handle the case of the "Add" or "Add & Close" being called for an
								// empty module.
								rowObject = this.moduleStructure.getLast();
								targetRowURI = rowObject.uri;
							}
							where = com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER;
						}
						executeAfterRowsAddedFunctions();
					}));
					break;
				}
				return resources;
			}), dojo.hitch(this, function(error){
				artifactPickerForm.displayMessage(7338);
				console.error(error);
				return error;
			}));
		});
		
		var lockFailedFn = function() {};
		var modURI = this.moduleStructure.moduleURI;
		
		var lockObtainedFn = dojo.hitch(this, function() {
			
			var releaseLock = function() {
				LockManager.releaseLockedResource(modURI);
			};
			//This function will run when the lock is obtained
			if(existingResources) {
				existingResources = dojo.map(existingResources, function(item){
					return {"resourceURI" : item};
				});
				this.addExistingArtifacts(resourceURI, existingResources, where).then(releaseLock, releaseLock);
				return;
			}

			var selectionOptions = {
				excludeArtifactFormat:["Collection", "Module"],
				mimeTypes: MimeTypes.WRAPPERS
			};
			targetRowURI = null;
			artifactPickerForm = new com.ibm.rdm.web.picker.ArtifactPickerForm({
				autoDisableAddedEntries: false,
				projectURI: this._projectURI,
				resourceURI: this.resourceURI,
				style: com.ibm.rdm.web.picker.ArtifactPickerForm.formStyle.ADD_CLOSE,
				filterArtifacts: existingArtifacts,
				showModuleRowStyleIcons: true,
				disablePickerModuleReuseMode: true,
				description: " ",
				title: this._messages.addToModule,
				onAddArtifacts: addArtifacts,
				onAddCallbackScope: this,
				surrenderClose: true,
				onCancel: function() {
					LockManager.releaseLockedResource(modURI);
				},
				onDialogClose: function() {
					LockManager.releaseLockedResource(modURI);
				},
				selectionOptions: selectionOptions
			});
			artifactPickerForm.show(dojo.hitch(this, this._onLaunchedDialogHasClosed));
			
			// Initiate save of the reference artifact if it is under edit  No need to wait for the save to complete before
			// showing the dialog.  The save can just proceed in the background.
			if (resourceURI && this._editInterface.isArtifactUnderEdit(resourceURI)) {
				// returns a deferred, but we don't need to wait for it.
				this._editInterface.saveAllEdits();
			}		
		});

		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI();
		
		this._implicitLockFunction(lockObtainedFn, lockFailedFn, modURI, this._projectURI, moduleProcessAreaURI);
		
		return addDeferred;
	},

	createStubArtifactOfTypeAndHeadingness: function(
		objectType, isHeading, where, /*String?*/relativeSiblingURI)
	{
		// summary:
		//		Create a "stub" row in which the artifact type is determined by
		//		'objectType', headingness is determined by 'isHeading' and
		//		placement is determined by 'where' (with regard to the row
		//		whose artifact ID is 'relativeSiblingURI').
		// where: String
		//		One of the values found in com.ibm.rdm.web.model.resources.modules.ModuleStructure.
		// relativeSiblingURI: String?
		//		This parameter should only be passed when 'where' is NOT set to
		//		ModuleStructure.LAST.  If it IS passed it represents the relative
		//		position of the new stub with regard to the existing row that's
		//		represents an artifact with this artifact URI.

		var mapArtifactUriToResource = {},
			resource = (
				relativeSiblingURI ?
				this._dataCache.getResourceByUri(relativeSiblingURI) :
				null
			);

		if (resource) {
			mapArtifactUriToResource[relativeSiblingURI] = resource;
		}

		return this._createNewOrSimilarArtifacts(
			mapArtifactUriToResource,
			where,
			1,
			null,		   // statusMessageHandle
			true,		   // isCreateNew
			objectType,	   // createBasedOnObjectType,
			isHeading
		);
	},

	createFirstArtifactInModule: function() {
		// summary:
		//		Create the first artifact for a module that currently contains no
		//		artifacts.
		// description:
		//		Normally calling this function will result in opening a new "stub"
		//		row in which the artifact type is set to the first preferred
		//		artifact type listed in the associated module's "Preferred Module
		//		Content" field.  However, it _may_ be the case that no preferred
		//		artifact types have yet been defined ... in which case we instead
		//		open the Create Artifact dialog the have the user fill in details
		//		for the artifact that is to be created.  In either case, the newly
		//		created "first row" will be put in edit mode.

		TypesManager.getObjectTypesForProject({
			projectURI: this._projectURI
		}).then(
			dojo.hitch(this, function(objectTypes){
				var mapArtifactTypeUriToObjectType = {},
					moduleArtifactTypeURI = this._dashboardInterface.getModuleArtifactTypeURI(),
					where = com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST,
					moduleObjectType,
					preferredArtifactTypeUris,
					firstPreferredObjectType,
					i;
				var openCreateArtifactDialog = dojo.hitch(this, function(){
					this._dashboardInterface.createArtifact({
						artifactFormat: RRMTypes.getArtifactFormats(this._projectURI).Text,
						isMoreOption: true,
						where: where,
						onDialogClose: dojo.hitch(this, this._onLaunchedDialogHasClosed)
					});
				});
				for (i = 0; i < objectTypes.length; i++) {
					mapArtifactTypeUriToObjectType[objectTypes[i].getResourceURI()] =
						objectTypes[i];
				}
				moduleObjectType = mapArtifactTypeUriToObjectType[moduleArtifactTypeURI];
				preferredArtifactTypeUris = moduleObjectType.getPreferredArtifactTypes();
				if (preferredArtifactTypeUris.length > 0) {
					GridUtils.getFirstSelectablePreferredObjectTypeForFirstArtifactInModule(
						preferredArtifactTypeUris, objectTypes, mapArtifactTypeUriToObjectType,
						this._dashboardInterface.getAssetFolderURI(), this._projectURI
					).then(
						dojo.hitch(this, function(firstSelectableObjectType){
							if (firstSelectableObjectType) {
								// There exists at least one selectable preferred artifact type
								// for this module (i.e. at one for which the user is allowed
								// to create the associated artifact type within the context folder
								// based on team area permissions checking).
								// Open a new stub row and make sure that the row's artifact type
								// matches this firstSelectableObjectType and that the row's
								// headingness matches this firstSelectableObjectType's default
								// headingness.
								this.createStubArtifactOfTypeAndHeadingness(
									firstSelectableObjectType,
									firstSelectableObjectType.isModuleHeading(),
									where
								);
							} else {
								openCreateArtifactDialog();
							}
						})
					);
				} else {
					// No preferred types have yet been defined for this module.
					// In this case we just open the Create Artifact dialog.
					openCreateArtifactDialog();
				}
			})
		);
	},

	simulateTabIntoGrid: function() {
		// summary:
		//		For details, please see the documentation for a method by this
		//		same name in the Module class.

		this.grid.simulateTabIntoGrid();
	},

	_implicitLockFunction: function(lockObtainedFn, lockFailedFn, resourceURI, projectURI, resourceProcessAreaURI) {

		var processOperationsCache = com.ibm.rdm.web.server.cache.ProcessOperationsCache;
		LockManager.obtainImplicitLock(LockManager.MSG_LOCK, lockObtainedFn, lockFailedFn, resourceURI, projectURI, resourceProcessAreaURI, processOperationsCache);
	},
		
	addExistingArtifacts: function(targetURI, selectedArtifacts, where){
		var continueWithSave = dojo.hitch(this, function() {
			var viewParams = this._dashboardInterface.getCurrentViewServiceRequestParams();
			var moduleStructurePosition = where;
			var siblingAndPosition;
			var addDeferred = new dojo.Deferred();
			
			var artifactUris = [];
			var i;
			for (i = 0; i < selectedArtifacts.length; i++) {
				artifactUris.push(selectedArtifacts[i].resourceURI);
			}

			var parentURI = this.moduleStructure.findProperParentURI(targetURI, where);
			var targetEntry = this.moduleStructure.getArtifactByURI(targetURI);
			
			// 76179: If this an empty module, there may not be a target entry for the case when where === "below".
			//  Add the artifact(s) to the end of the module in this case.
			if (!targetEntry && where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW) {
				where = com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST;
			}
			var relativeSiblingURI;
					
			siblingAndPosition = this._getRelativeSiblingAndPosition(targetEntry, targetURI, where);
			where = siblingAndPosition.where;
			relativeSiblingURI = siblingAndPosition.relativeSiblingURI;


			if (!targetURI){
				targetURI = this.moduleStructure.compositeRootURI;
			}

			StatusDisplay.showStatusMessage(this._messages.addingArtifact);
			//Do necessary posts to create re-use artifacts
			ResourceUtil.createReusedArtifacts({
				// we post to the parent with relative sibling URI
				targetURI: parentURI,
				resourceURIs: artifactUris,
				projectURI: this._projectURI,
				relativeSiblingURI: relativeSiblingURI,
				where: where
			}).addBoth(dojo.hitch(this, function(newArtifactURIs){
				
				var viewDeferred = com.ibm.rdm.web.query.ViewServiceClient.getViewContents({
					artifactUris: newArtifactURIs,
					// ask the view service if the newly created artifact is a heading, if the artifact type has the ModuleHeading role.
					requestedAttributeUris: viewParams.requestedAttributeUris.concat([RRMTypes.WELL_KNOWN_IS_HEADING]),
					projectURI: this._projectURI,
					moduleURI: this.moduleStructure.moduleURI
				});
				
				viewDeferred.then(dojo.hitch(this, function (response) {
					var result = com.ibm.rdm.web.query.ViewServiceClient.parseViewContentsResponse(response, this._projectURI, {}, false),
						resources = com.ibm.rdm.web.query.ViewServiceClient.getResourcesOrderedByOriginalRequestOrdering(
							result.resources, newArtifactURIs
						);
					this._dataCache.addResources(resources);
					var args = {
						targetURI: targetURI,
						resources: resources,
						resourceFitsView: true, /* This will tell any listener (e.g. the grid) that it doesn't need to bother */
						where: moduleStructurePosition,			/* loading the resource again, it can use the resources: argument */
						deferred: addDeferred,
						skipOperationQueuing: true
					};
				
					this.moduleStructure.placeChildren(args).then(dojo.hitch(this, function () {
						// remove status message
						StatusDisplay.hideStatusMessage();
						addDeferred.callback(resources);
						this._invokeOnReconcile(dojo.hitch(this, function(){
							for (i = 0; i < resources.length; i++) {
								var rowURI = resources[i].resourceURI;
								this.grid.scrollIntoView(rowURI, dojo.hitch(this, function(){
									this._highlightNewRow(rowURI);
								}));
							}
						}));
					}));
					
					return response;
				}));
					
			}));
			return addDeferred;
		});
		
		var latestDeferred = new dojo.Deferred();
		this.moduleStructure.checkStructureCurrentByParentAndUpdateETag(targetURI, where, null).then(dojo.hitch(this, function(shouldContinue){
			if (shouldContinue){
				continueWithSave().then(dojo.hitch(this, function(saveResp){
					latestDeferred.callback(saveResp);
				}));
			}
			else{
				StatusDisplay.hideStatusMessage();
				latestDeferred.callback();
			}
		}));
		return latestDeferred;
		
	},
	
	_highlightNewRow: function (uri) {
		var row = this.grid.getRowByKey(uri);
		if (row && row.node) {
			dojo.addClass(row.node, "justPasted");
			setTimeout(function(){
					dojo.removeClass(row.node, "justPasted");
			}, com.ibm.rdm.web.artifactgrid.ArtifactConstants.MODULE_HIGHLIGHT_TIMEOUT);
		}
	},
	
	_waitForReconcileAndHighlight: function (uri) {
		this._invokeOnReconcile(dojo.hitch(this, function () {
			this.grid.scrollIntoView(uri, dojo.hitch(this, function () {
				this._highlightNewRow(uri);
			}));
		}));
	},
	
	_waitForReconcileAndPutInEditMode: function(uri) {
		this._invokeOnReconcile(dojo.hitch(this, function () {
			this.grid.scrollIntoView(uri, dojo.hitch(this, function () {
				var rowIds = {};
				rowIds[uri] = com.ibm.rdm.web.artifactgrid.ArtifactConstants.RESOURCE_ROW;
				this._editInterface.editRow(rowIds, true, true);
			}));
		}));
	},
	
	_invokeOnReconcile: function (/*Function*/callback) {
		// summary:
		//		Creates a one time listener which will listen for
		//		and remove itself after the next grid reconcile event
		//		useful for commands that need to wait until the
		//		grid has been updated before doing something.
		var reconciledListener = ChangeNotifier.addListener({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.MODULE_GRID_RECONCILED,
			callbackFunction: function() {
				ChangeNotifier.removeListener(reconciledListener);
				callback();
			},
			callbackScope: this
		});
		
	},
	
	_insertRowsIntoModuleGrid: function(args) {
		//!!!!!!! DO NOT CHANGE - AB
		var inserArtifactsInGridDeferred = this._insertArtifacts(args);
		inserArtifactsInGridDeferred.addCallback(this, function(){
			args.deferred.callback(args);
			this.grid.scrollIntoView(args.resources[0].getResourceURI());
		});
		return inserArtifactsInGridDeferred;
	},
		
//		args{
//			parentURI: targetURI,
//			resources: [req],
//			where: where
//		}
	_insertArtifacts: function(args){
		// returns a deferred that resolves when the rows have been fully drawn on the grid.
		var insertDeferreds = [];
		
		dojo.forEach(args.resources, dojo.hitch(this, function(thisResource){
			this._dataCache.addResource(thisResource);
			if (this._dataCache.getArrayListOfResources().length === 1) {
			//Must be the one we added - so redraw to get rid of message
				this.grid.hardClear();
			}
			
			var before = this.moduleStructure.getPrecedingArtifact(args.targetURI, args.where);
			
			this.grid.addToNewArtifacts(thisResource.getResourceURI());
			var deferred = this.grid.insertRow(before && before.uri, thisResource.getResourceURI(), com.ibm.rdm.web.artifactgrid.ArtifactConstants.RESOURCE_ROW);
			insertDeferreds.push(deferred);
			StatusDisplay.hideStatusMessage();
			// update the structure
			// TODO: this would need to actually make an artifact entry in the required
			// shape for the structure, this requires depth, sectionNumber, BookOrder in addition
			// to the URI.
		}));

		return new dojo.DeferredList(insertDeferreds);
	},

	executeCreate: function(rowIds, where){
		var resourceURI = null,
			overallDeferred = new dojo.Deferred(),
			modURI = this.moduleStructure.moduleURI;

		for (resourceURI in rowIds) {
			break;
		}

		var lockObtainedFn = dojo.hitch(this, function() {
			
			var parentURI = this.moduleStructure.findProperParentURI(resourceURI, where);
			var	targetEntry = this.moduleStructure.getArtifactByURI(parentURI);
			var	moduleStructurePosition = where;
					
			var	siblingAndPosition = this._getRelativeSiblingAndPosition(targetEntry, resourceURI, where);
			var relativeSiblingURI = siblingAndPosition.relativeSiblingURI;
			
			where = siblingAndPosition.where;
			
			this._dashboardInterface.createArtifact({
				moduleContextURI: parentURI,
				where: where,
				relativeSiblingURI: relativeSiblingURI,
				onDialogClose: dojo.hitch(this, this._onLaunchedDialogHasClosed)
			});
		});
		
		var lockFailedFn = function () {};
		
		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI();
		
		this._implicitLockFunction(lockObtainedFn, lockFailedFn, modURI, this._projectURI, moduleProcessAreaURI);
		
		return overallDeferred;

	},

	_createNewOrSimilarArtifacts: function(
		resources, where, numResources, /*Int?*/statusMessageHandle,
		/*Boolean?*/isCreateNew, /*ObjectType?*/createBasedOnObjectType, /*Boolean*/doCreateAsHeading)
	{
		// summary:
		//		Create a new "stub" row in which the underlying artifact is either
		//		similar to an existing artifact or is based on a particular
		//		artifact type and "headingness".
		// description:
		//		In practice, when used for "new similar" only the first
		//		four parameters should be passed.
		// isCreateNew: Boolean
		//		If passed, create a new artifact using the artifact type found in
		//		'createBasedOnObjectType' and row style based on 'doCreateAsHeading'.

		var createSimilarArtifactsImpl = dojo.hitch(this, function() {
			var deferredsArray = [];
			var createSingleResource = dojo.hitch(this, function(/*String?*/resourceURI) {
				// Note: If 'isCreateNew' is true, resourceURI will be passed as null.
				var resource = resourceURI ? resources[resourceURI] : null,
					typeURI = isCreateNew ? createBasedOnObjectType.getResourceURI() : resource.getType(),
					structureEntry = (
						resource ?
						this.moduleStructure.getArtifactByURI(resource.resourceURI) : // isHeading not requested in page fetch, get from structure.
						null
					);
				
				var contentType = isCreateNew ? createBasedOnObjectType.getDefaultFormat() : resource.getMimeType(),
					projectURI = this._projectURI,
					description = "",
					relativeSiblingURI,
					targetURI,
					targetEntry,
					siblingAndPosition,
					moduleStructurePosition; 
					
				targetURI = (
					resource ?
					this.moduleStructure.findProperParentURI(resource.resourceURI, where) :
					this.moduleStructure.compositeRootURI
				);
				targetEntry = this.moduleStructure.getArtifactByURI(targetURI);
				moduleStructurePosition = where;

				if (resource) {
					siblingAndPosition = this._getRelativeSiblingAndPosition(targetEntry, resource.resourceURI, where);
					where = siblingAndPosition.where;
					relativeSiblingURI = siblingAndPosition.relativeSiblingURI;
				} else {
					relativeSiblingURI = null;
				}
										
				var createResourceArgs = {
						format: contentType,
						projectURI: projectURI,
						resourceUriScope: this.resourceURI,
						isHeading: isCreateNew ? doCreateAsHeading : (structureEntry && structureEntry.isHeading),
						typeURI: typeURI,
						title: "",
						description: description,
						enableEditInGrid: true,
						moduleContextURI: targetURI,
						relativeSiblingURI: relativeSiblingURI,
						where: where,
						createStub: (numResources === 1),
						folderURI: this._dashboardInterface.getAssetFolderURI()
					};
				
				var onCreateArtifactFormCancelClicked = dojo.hitch(this, function(){
					LockManager.releaseLockedResource(this.moduleStructure.moduleURI);
					StatusDisplay.hideStatusMessage(statusMessageHandle);
				});
				var createResourceDeferred = (
					numResources > 1 ?
					ResourceUtil.createResource(createResourceArgs) :
					this._createResourceAfterPerformingPermissionCheck(
						createResourceArgs,
						onCreateArtifactFormCancelClicked
					)
				);
				// Copy tags from reference resource to created resource
				if (!isCreateNew) {
					createResourceDeferred.then(
						dojo.hitch(this, function(tags, createdResource) {
							createdResource.similarResource = resource;
							var tagDeferred = null;
							if (resource.__tags.length > 0) {
								tagDeferred = TagUtil.setTagsForArtifact([], tags, createdResource.getResourceURI(), this._projectURI, false);
								createdResource.__createSimilarApplyTagsDeferred = tagDeferred;
								tagDeferred.addCallback(function(){return createdResource;});
							}
							else{
								tagDeferred = new dojo.Deferred();
								tagDeferred.callback();
							}
							//Set appropriate tags information and parent
							createdResource.__tags = tags;
							createdResource.parentResourceURI = resource.parentResourceURI;
							//fallback on module asset folder
							if (!createdResource.parentResourceURI || createdResource.parentResourceURI.length === 0){
								createdResource.parentResourceURI = this._dashboardInterface.getAssetFolderURI();
							}
							return tagDeferred;
						}, resource.__tags)
					);
				}
				// Insert the new resource relative to the reference resource - both in the internal
				// structure and on the server.
				var insertAndPlaceDeferred = new dojo.Deferred();
				createResourceDeferred.then(
					dojo.hitch(this, function (req) {
						var args = {
							targetURI: resource ? resource.resourceURI : null,
							resources: [req],
							where: moduleStructurePosition,
							//callback: dojo.hitch(this, function () {} ),//this._insertRowsIntoModuleGrid),
							deferred: insertAndPlaceDeferred,
							skipOperationQueuing: true
						};
		
		
						this._waitForReconcileAndHighlight(req.resourceURI);
						this._invokeOnReconcile(dojo.hitch(this, function () {
							var editArgs = {};
							editArgs[req.resourceURI] = "";
							this._editInterface.editRow(editArgs, true, true);
							// If the team area permssions state that the user can't edit any
							// of this row's currently displayed attributes, we immediately
							// save the row (instead of putting it in edit mode).  [Note: the
							// following 'isAnyUnderEdit' call enables us to know if we were
							// successfull in putting the row into "edit mode" ... i.e. if
							// no cell editors have been opened we know there are team area
							// permssions issues.]
							if (!EditManager.isAnyUnderEdit(this._projectURI)) {
								this._saveUnmodifiableStubRow(req);
							}
						}));

						this.moduleStructure.placeChildren(args).then(dojo.hitch(this, function () { 
							// remove status message
							StatusDisplay.hideStatusMessage(statusMessageHandle);
							insertAndPlaceDeferred.callback(req);
						}));
						return insertAndPlaceDeferred;
					})
				);
				return insertAndPlaceDeferred;
			});
			for (var resourceURI in resources) {
				deferredsArray.push(createSingleResource(resourceURI));
			}
			if (deferredsArray.length === 0) {
				// Note: If 'where' is set to:
				//    com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST
				// the 'resources' param will contain no mappings, but we still need
				// to create a stub for this case ...
				deferredsArray.push(createSingleResource());
			}
			var deferredsList = new dojo.DeferredList(deferredsArray);
			deferredsList.addBoth(this, function(response){
				LockManager.releaseLockedResource(this.moduleStructure.moduleURI);
				StatusDisplay.hideStatusMessage();
			});
			
			// For the deferred that we are going to return, make the parameter to its callbacks be
			// an array of newly created artifact URIs.
			deferredsList.addCallback(this, function(deferredListResponseArray){
				var createdArtifactURIs = [];
				var length = deferredListResponseArray.length;
				for (var i = 0 ; i < length; i++) {
					var entry = deferredListResponseArray[i];
					if (entry[0]) {
						// success
						createdArtifactURIs.push(entry[1].getResourceURI());
					} // else error - ignore?
				}
				return createdArtifactURIs;
			});
			return deferredsList;
		});
		
		var saveEditsAndCreate = dojo.hitch(this, function(){
			// Saves any outstanding edits if necessary, then invokes the create operation.
			for (var resourceURI in resources) {
				if (this._editInterface.isArtifactUnderEdit(resourceURI)) {
					// At least one resource we are copying is under edit.
					var createDeferred = createSimilarArtifactsImpl();
					var saveAllDeferred = this._editInterface.saveAllEdits();
					return saveAllDeferred;
				}
			}
			// None of the resources we are interested in is currently under edit
			return createSimilarArtifactsImpl();
		});
		
		var latestDeferred = new dojo.Deferred();
		var lockObtainedFn = dojo.hitch(this, function() {
			saveEditsAndCreate().addCallback(latestDeferred);
		});
		
		var lockFailedFn = dojo.hitch(this, function() {
			// Remove status message once lock failure is resolved
			StatusDisplay.hideStatusMessage();
		});
		
		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI();

		this._implicitLockFunction(lockObtainedFn, lockFailedFn, this.moduleStructure.moduleURI, this._projectURI, moduleProcessAreaURI);
				
		return latestDeferred;
	},

	_saveUnmodifiableStubRow: function(resource) {
		// summary:
		//		This function is only to be called when we create a stub row and then
		//		notice that the user doesn't have appropriate team area permissions
		//		to modify any of the row's displayed attributes.
		// description:
		//		As a fix for Defect 83704, this function is used to immediately
		//		save a newly created stub row when we notice that the row can't be
		//		put in edit mode.  Please see Comments 3 and 4 in that defect for
		//		implementation details.

		var stubId = resource.stubId;
		resource.save({
			refreshAfterSave: false
		}).then(
			dojo.hitch(this, function(resource){
				var resourceURI = resource.getResourceURI();

				// Note: In a similar fashion to ResourceUtil's createRequirementStub
				// adding a 'stubId' property, we're adding a 'stubIdOfUserUnmodifiableRow'
				// property for downstream code that reacts to the ARTIFACT_SAVED
				// event.
				resource.stubIdOfUserUnmodifiableRow = stubId;

				// Make sure that after saving the "unmodifiable stub row" we set focus
				// to this newly saved row.
				var listener = ChangeNotifier.addListener({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_SAVED,
					callbackFunction: function() {
						ChangeNotifier.removeListener(listener);
						this.grid.moveFocusToCellOnRow(resourceURI);
					},
					callbackScope: this
				});

				// Send out an ARTIFACT_SAVED event notification.
				ChangeNotifier.notifyListeners({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_SAVED,
					resourceURI: resourceURI,
					projectURI: resource.projectURI,
					resources: [resource]
				});
			})
		);
	},

	_createSimilarArtifacts: function(resources, where, numResources, statusMessageHandle)
	{
		// summary:
		//		This is an override of a superclass method.

		return this._createNewOrSimilarArtifacts(resources, where, numResources, statusMessageHandle);
	},
		
	_cutText: function(num){
		if (num > 1){
			return dojo.string.substitute(this._messages.cutArtifacts, [num]);
		}
		return this._messages.cutArtifact;
	},
	
	_indentText: function(num){
		return num === 1 ? this._messages["indentArtifact"] : dojo.string.substitute(this._messages["indentArtifacts"], [num]);
	},
	
	_outdentText: function(num){
		return num === 1 ? this._messages["outdentArtifact"] : dojo.string.substitute(this._messages["outdentArtifacts"], [num]);
	},
	
	_pasteText: function(num, where){
		if(where && this._messages[where]) { return this._messages[where]; }
		
		var count = 1,
			clipboardData = clipboard.get();
		if(clipboardData && clipboardData.count) {
			count = clipboardData.count;
		}
		return count === 1 ? this._messages["pasteArtifact"] : dojo.string.substitute(this._messages["pasteArtifacts"], [count]);		
	},

	_isShowCommentsEnabled: function(rowIds, count){
		// summary:
		//		Determine if the commenting menu option should be enabled.
		// rowIds: int
		//		 The identifier of each of the rows that are selected.
		// count: int
		//		The count of the rows that have been selected.
		return count === 1;
	},

	_isAddCommentEnabled: function(rowIds, count){
		// summary:
		//		Determine if the add comment menu option should be enabled. It should only be opened if
		//		there is one row selected. It is not available in a baseline.
		// rowIds: int
		//		The identifier of each of the rows that are selected.
		// count: int
		//		The count of the rows that have been selected.
		return false;
	},
	
	getAddCommentDeferred: function(rowIds, count){
		var deferred = new dojo.Deferred();
		var projectDeferred = ProjectCache.getProjectPermissions(this._projectURI);
		
		projectDeferred.then(dojo.hitch(this, function() {
			var projectOperations = this._projectOperations;
			if (this._baselineURI || this._isInHistoryMode || !projectOperations.item(ProjectCache.MODIFY_COMMENT)) {
				deferred.callback(false);
			}
			else {
				deferred.callback(count === 1);
			}
		}));
		
		return deferred;
	},
	
	_isIndentEnabledWithoutConsideringTeamOperations: function(rowIds, count){
		//Only allow indent/outdent on a single artifact.
		//Do not allow indentation when there is no discernible parent
		var thisResource,
			canIndent = true,
			before,
			resource,
			resourceURIs = [];
			
		if (!this.structuralOperationsAllowed()){
			canIndent = false;
			return canIndent;
		}
		
		if (this.moduleStructure.getArtifacts().length < 2){
			canIndent = false;
			return canIndent;
		}
		
		for (thisResource in rowIds){
			resourceURIs.push(thisResource);
		}
		resourceURIs = this.moduleStructure.stripToLowestCommonAncestors(resourceURIs);
		
		for (thisResource in resourceURIs){
			resource = this.moduleStructure.getArtifactByURI(resourceURIs[thisResource]);
			before = this.moduleStructure.getParentForIndent(resourceURIs[thisResource], rowIds);
			if (!before){
				canIndent = false;
				break;
			}
		}
		
		return canIndent && !(this._isInReviewMode || this._isInHistoryMode);
	},

	getIndentEnablementDeferred: function(rowIds, count) {
		// summary:
		//		Returns a deferred which will be called back with a single
		//		param representing whether the "Demote Artifact" menu item
		//		should be enabled.
		// returns:
		//		A Deferred.
		var deferred = new dojo.Deferred();
		if (this._isIndentEnabledWithoutConsideringTeamOperations(rowIds, count)) {
			this.getIsModifyModuleContentAllowed().then(
				dojo.hitch(this, function(isModifyModuleContentAllowed) {
					deferred.callback(isModifyModuleContentAllowed);
				})
			);
		} else {
			deferred.callback(false);
		}
		return deferred;
	},
	
	_isOutdentEnabledWithoutConsideringTeamOperations: function(rowIds, count){
		//Do not allow outdentation of a top-level artifact
		var thisResource,
			canOutdent = true,
			before,
			resource,
			resourceURIs = [];
			
		if (!this.structuralOperationsAllowed()){
			canOutdent = false;
			return canOutdent;
		}
		
		if (this.moduleStructure.getArtifacts().length < 2){
			canOutdent = false;
			return canOutdent;
		}
		
		for (thisResource in rowIds){
			resourceURIs.push(thisResource);
		}
		resourceURIs = this.moduleStructure.stripToLowestCommonAncestors(resourceURIs);
		
		for (thisResource in resourceURIs){
			resource = this.moduleStructure.getArtifactByURI(resourceURIs[thisResource]);
			if (!resource || resource.depth < 2){
				canOutdent = false;
				break;
			}
			before = this.moduleStructure.getParentForOutdent(resourceURIs[thisResource], rowIds);
			if (!before){
				canOutdent = false;
				break;
			}
		}
		
		return canOutdent;
	},

	getOutdentEnablementDeferred: function(rowIds, count) {
		// summary:
		//		Returns a deferred which will be called back with a single
		//		param representing whether the "Promote Artifact" menu item
		//		should be enabled.
		// returns:
		//		A Deferred.
		var deferred = new dojo.Deferred();
		if (this._isOutdentEnabledWithoutConsideringTeamOperations(rowIds, count)) {
			this.getIsModifyModuleContentAllowed().then(
				dojo.hitch(this, function(isModifyModuleContentAllowed) {
					deferred.callback(isModifyModuleContentAllowed);
				})
			);
		} else {
			deferred.callback(false);
		}
		return deferred;
	},
	
	
	_gatherConsecutiveSiblings: function(operationResourceURIs, initialResourceIndex) {
		// summary:
		//		Given a resource, finds any consecutive siblings that are to be included in the
		//		same operation.  This allows indent operations to operate on several
		//		rows in a single operation, so saving round-trips.
		// operationResourceURIs:
		//		An array of URIs of the resources to be operated on.  Must be in book order.
		// initialResourceIndex:
		//		The index into operationResourceURIs of the starting resource for an operation.
		// returns:
		//		An array containing at least one entry (that being operationResourceURIs[initialResourceIndex])
		//		and possibly other entries that are found to be consecutive siblings of that resource.
		var initialResourceURI = operationResourceURIs[initialResourceIndex],
			currentResourceModuleIndex,
			consecutiveResourceURIs = [initialResourceURI],		// Always add the first one
			consecutiveIndex = initialResourceIndex,
			nextResourceURI,
			nextResource,
			nextResourceModuleIndex,
			resource = this.moduleStructure.getArtifactByURI(initialResourceURI);
		currentResourceModuleIndex = this.moduleStructure.indexOf(resource);
		while (++consecutiveIndex < operationResourceURIs.length) {
			nextResourceURI = operationResourceURIs[consecutiveIndex];
			nextResource = this.moduleStructure.getArtifactByURI(nextResourceURI);
			if (nextResource.depth !== resource.depth) {
				// starting point is different depth - can't be same parent.
				break;
			}
			nextResourceModuleIndex = this.moduleStructure.indexOf(nextResource);
			if (nextResourceModuleIndex !== (currentResourceModuleIndex+1)) {
				// resource is not consecutive in the module
				break;
			}
			// We've found another row that is also being indented, and is already
			// a sibling of the previous one, so include it in the operation.
			consecutiveResourceURIs.push(nextResourceURI);
			currentResourceModuleIndex = nextResourceModuleIndex;
		}
		return consecutiveResourceURIs;
	},
	
	_gatherConsecutiveSiblingsReverse: function(operationResourceURIs, initialResourceIndex) {
		// summary:
		//		Given a resource, finds any consecutive siblings that are to be included in the
		//		same operation.  This allows outdent operations to operate on several
		//		rows in a single operation, so saving round-trips.
		// operationResourceURIs:
		//		An array of URIs of the resources to be operated on.  Must be in REVERSE book order.
		// initialResourceIndex:
		//		The index into operationResourceURIs of the starting resource for an operation.
		// returns:
		//		An array containing at least one entry (that being operationResourceURIs[initialResourceIndex])
		//		and possibly other entries that are found to be consecutive siblings of that resource.  Enties
		//		are in book order (not reverse book order).
		var initialResourceURI = operationResourceURIs[initialResourceIndex],
			currentResourceModuleIndex,
			consecutiveResourceURIs = [],		// Always add the first one
			consecutiveIndex = initialResourceIndex,
			previousResourceURI,
			previousResource,
			previousResourceModuleIndex,
			resource = this.moduleStructure.getArtifactByURI(initialResourceURI);
		currentResourceModuleIndex = this.moduleStructure.indexOf(resource);
		while (++consecutiveIndex < operationResourceURIs.length) {
			previousResourceURI = operationResourceURIs[consecutiveIndex];
			previousResource = this.moduleStructure.getArtifactByURI(previousResourceURI);
			if (previousResource.depth !== resource.depth) {
				// starting point is different depth - can't be same parent.
				break;
			}
			previousResourceModuleIndex = this.moduleStructure.indexOf(previousResource);
			if (previousResourceModuleIndex !== (currentResourceModuleIndex-1)) {
				// resource is not consecutive in the module
				break;
			}
			// We've found another row that is also being indented, and is already
			// a sibling of the next one, so include it in the operation.
			consecutiveResourceURIs.push(previousResourceURI);
			currentResourceModuleIndex = previousResourceModuleIndex;
		}
		// Reverse the array to make entries in the right order
		consecutiveResourceURIs.reverse();
		consecutiveResourceURIs.push(initialResourceURI);
		return consecutiveResourceURIs;
	},
	
	executeModuleIndent: function(rowIds){
		StatusDisplay.showStatusMessage(this._messages.demotingArtifact);
		var overallDeferred,
			parent,
			resource,
			where,
			thisResource,
			resourceURIs = [],
			idx = 0,
			doIndent,
			startNextIndent,
			savingParentURI;
			
		for (thisResource in rowIds){
			resourceURIs.push(thisResource);
		}
		resourceURIs = this.moduleStructure.stripToLowestCommonAncestors(resourceURIs);
		this.moduleStructure.arraySortOnBookorder(resourceURIs);
		
		if(this._editInterface){
			this._editInterface.saveStarted(resourceURIs);
		}
		
		overallDeferred = new dojo.Deferred();
		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI();
		var lockFailedFn = dojo.hitch(this, function() {
			StatusDisplay.hideStatusMessage();
			this._editInterface.saveFinished(resourceURIs);
		});
		var modURI = this.moduleStructure.moduleURI,
			currentFocusRowKey = this.grid.getFocusRowKey();
		
		var lockObtainedFn = dojo.hitch(this, function() {
		
			doIndent = dojo.hitch(this, function(){
				
				// Get the resource to be indented, and its new parent
				var currentResourceURI = resourceURIs[idx],
					resourceURIsToIndent;
				resource = this.moduleStructure.getArtifactByURI(currentResourceURI);
				parent = this.moduleStructure.getParentForIndent(currentResourceURI, rowIds);
				// If there are consecutive resources in the selection and they
				// are at the same depth, do the indent as a single operation.  This
				// does not necessarily find all rows that are to move to the same parent,
				// but it covers the common case where a block of consecutive rows are to
				// be indented.  More complex scenarios are dealt with through further
				// iterations through this function.
				resourceURIsToIndent = this._gatherConsecutiveSiblings(resourceURIs, idx);
									
				if (parent.depth > resource.depth){
					where = com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER;
					if (parent.parent){
						savingParentURI = parent.parent.uri;
					}
					else{
						savingParentURI = null;
					}
				}
				else{
					where = com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW;
					savingParentURI = parent.uri;
				}
				
				if (savingParentURI){
					this._editInterface.saveStarted([savingParentURI]);
				}
				
				this.moduleStructure.placeChildren({
					targetURI: parent.uri,
					resourceURIs: resourceURIsToIndent,
					where: where,
					isMove: true,
					forceStructureVersionLoad: (idx > 0)
				}).then(dojo.hitch(this, function(){
					if (savingParentURI){
						this._editInterface.saveFinished([savingParentURI]);
					}
					this._editInterface.saveFinished(resourceURIsToIndent);
					// start the next indent or finish.
					startNextIndent(resourceURIsToIndent.length);
				}), dojo.hitch(this, function(error){
					if (savingParentURI){
						this._editInterface.saveFinished([savingParentURI]);
					}
					this._editInterface.saveFinished(resourceURIs);
					overallDeferred.errback(error);
				}));
			});
			
			startNextIndent = dojo.hitch(this, function(indentedRowCount){
				idx += indentedRowCount;
				if (idx < resourceURIs.length){
					doIndent();
				}
				else{
					overallDeferred.callback();
				}
			});
			
			doIndent();
			
			overallDeferred.addBoth(this, function(response){
				StatusDisplay.hideStatusMessage();
				LockManager.releaseLockedResource(modURI);
				if (currentFocusRowKey) {
					this.grid.moveFocusToCellOnRow(currentFocusRowKey);
				}
				return response;
			});
		});

		this._implicitLockFunction(lockObtainedFn, lockFailedFn, modURI, this._projectURI, moduleProcessAreaURI);
		
		return overallDeferred;
	},
	
	executeModuleOutdent: function(rowIds){
		StatusDisplay.showStatusMessage(this._messages.promotingArtifact);
		var overallDeferred,
			parent,
			resource,
			thisResource,
			resourceURIs = [],
			idx = 0,
			doOutdent,
			processOutdent,
			savingParentURI,
			reparentDeferred;
			
		for (thisResource in rowIds){
			resourceURIs.push(thisResource);
		}
		resourceURIs = this.moduleStructure.stripToLowestCommonAncestors(resourceURIs);
		this.moduleStructure.arraySortOnReverseBookorder(resourceURIs);
		
		if(this._editInterface){
			this._editInterface.saveStarted(resourceURIs);
		}
		
		overallDeferred = new dojo.Deferred();
		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI();
		var lockFailedFn = dojo.hitch(this, function() {
			StatusDisplay.hideStatusMessage();
			this._editInterface.saveFinished(resourceURIs);
		});
		var modURI = this.moduleStructure.moduleURI,
			currentFocusRowKey = this.grid.getFocusRowKey();
		
		var lockObtainedFn = dojo.hitch(this, function() {
			var startNextOutdent; // function declared below - to keep jslint happy.
			doOutdent = dojo.hitch(this, function(){
				// Get the resource to be outdented, and its new parent
				var currentResourceURI = resourceURIs[idx],
					resourceURIsToOutdent;
				resource = this.moduleStructure.getArtifactByURI(currentResourceURI);
				parent = this.moduleStructure.getParentForOutdent(currentResourceURI, rowIds);
				// If there are consecutive resources in the selection and they
				// are at the same depth, do the outdent as a single operation.  This
				// does not necessarily find all rows that are to move to the same parent,
				// but it covers the common case where a block of consecutive rows are to
				// be outdented.  More complex scenarios are dealt with through further
				// iterations through this function.
				resourceURIsToOutdent = this._gatherConsecutiveSiblingsReverse(resourceURIs, idx);

				if(parent.parent){
					savingParentURI = parent.parent.uri;
					this._editInterface.saveStarted([savingParentURI]);
				}
				else{
					savingParentURI = null;
				}
				
				var lastOutdentURI = resourceURIsToOutdent[resourceURIsToOutdent.length-1];
				
				var children = parent.children;
				var reparentDeferred = true;	// may or may not be set to a deferred. (for used in dojo.when)
				if (parent.children[children.length - 1].uri !== lastOutdentURI){
					// There are siblings after the last one being outdented.  They must be
					// moved to be children of the last outdented row.  Build a list of their URIs.
					var childIndex = 0,
						childURIs = [];
					while (childIndex < children.length) {
						if (children[childIndex].uri === lastOutdentURI) {
							break;
						}
						childIndex++;
					}
					while (++childIndex < children.length) {
						childURIs.push(children[childIndex].uri);
					}
					
					// Move these children to under the last row being outdented.
					if (childURIs.length > 0){
						reparentDeferred = this.moduleStructure.placeChildren({
							targetURI: lastOutdentURI,
							resourceURIs: childURIs,
							where: com.ibm.rdm.web.model.resources.modules.ModuleStructure.LAST,
							isMove: true,
							forceStructureVersionLoad: true
						});
					}
				}
				
				dojo.when(reparentDeferred, dojo.hitch(this, function(){
					this.moduleStructure.placeChildren({
						targetURI: parent.uri,
						resourceURIs: resourceURIsToOutdent,
						where: com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER,
						isMove: true,
						forceStructureVersionLoad: true
					}).then(dojo.hitch(this, function(){
						if (savingParentURI){
							this._editInterface.saveFinished([savingParentURI]);
						}
						this._editInterface.saveFinished(resourceURIsToOutdent);
						startNextOutdent(resourceURIsToOutdent.length);
					}), dojo.hitch(this, function(error){
						if (savingParentURI){
							this._editInterface.saveFinished([savingParentURI]);
						}
						this._editInterface.saveFinished(resourceURIs);
						overallDeferred.errback(error);
					}));
				}, dojo.hitch(this, function(error){
					if (savingParentURI){
						this._editInterface.saveFinished([savingParentURI]);
					}
					this._editInterface.saveFinished(resourceURIs);
					overallDeferred.errback(error);
				})));
			});
			
			startNextOutdent = dojo.hitch(this, function(outdentedRowCount){
				idx += outdentedRowCount;
				if (idx < resourceURIs.length){
					doOutdent();
				}
				else{
					overallDeferred.callback();
				}
			});
			
			doOutdent();
			
			overallDeferred.addBoth(this, function(response){
				StatusDisplay.hideStatusMessage();
				LockManager.releaseLockedResource(modURI);
				if (currentFocusRowKey) {
					this.grid.moveFocusToCellOnRow(currentFocusRowKey);
				}
				return response;
			});
		});
		
		this._implicitLockFunction(lockObtainedFn, lockFailedFn, modURI, this._projectURI, moduleProcessAreaURI);
		
		return overallDeferred;
	},

	_executeAddComment: function(rowIds) {
		// should be exactly one row selected
		// returns:
		//		A deferred whose callback is called when the comment dialog has been closed.
		//Ensure this row is not deselected
		this.setNonClearingCommand();
		
		var resourceURI;
		for (resourceURI in rowIds){break;}
		var resource = this._dataCache.getRecordInfo(resourceURI);
		var baseURI = resource.getModuleBoundArtifact();
		var commentCallback = this._commentingInterface.addMemberCommentCallback(resourceURI, baseURI);

		commentCallback.promise.always(
			dojo.hitch(this, this._onLaunchedDialogHasClosed)
		);

		return commentCallback;
	},
	
	_executeOpenComments: function(rowIds){
		//Ensure this row is not deselected
		this.setNonClearingCommand();
		
		var resourceURI;
		for (resourceURI in rowIds){break;}
		var resource = this._dataCache.getRecordInfo(resourceURI);
		var baseURI = resource.getModuleBoundArtifact();
		this._commentingInterface.showMemberCommentsCallback(resourceURI, baseURI);
		this.grid.retakeFocus();
	},
		
	executeModuleCut: function(rowIds, applyCutStyle){
		// default to true
		applyCutStyle = (typeof applyCutStyle === "undefined") ? true : applyCutStyle;
		//Remove existing cut/copy styling
		this.clearClipboard();
		var forCut = [],
			totalCut = {},	// keeps track of overlapping artifacts
			cutCount = 0;
		
		for (var thisRowId in rowIds) {
			if (applyCutStyle) {
				totalCut[thisRowId] = thisRowId;
				//Set new cut styling
				var entry = this.moduleStructure.getArtifactByURI(thisRowId),
					children = this.moduleStructure.getDescendants(entry);
				
				var thisRow = this.grid.getRowByKey(thisRowId);
				if (thisRow) {
					dojo.addClass(thisRow.node, com.ibm.rdm.web.grid.Module.ClassNames.ROW_CUT);
				}
				
				if (children) {
					dojo.forEach(children, dojo.hitch(this, function (child) {
						totalCut[child.uri] = child.uri;
						var row = this.grid.getRowByKey(child.uri);
						if (row) {
							dojo.addClass(row.node, com.ibm.rdm.web.grid.Module.ClassNames.ROW_CUT);
						}
					}));
				}
			}
			forCut.push(thisRowId);
		}
		// count the entries
		for(var uri in totalCut) {
			cutCount++;
		}
		
		var cutData = {
			action: CUT_ACTION,
			uris: forCut,
			projectURI: this._projectURI,
			moduleURI: this.moduleStructure.moduleURI,
			count: cutCount
		};
		clipboard.put(cutData);
		
		// make sure focus stays within the module, so subsequent keyboard operations work
		if (applyCutStyle) {
			this.grid.focus();
		}
	},
	
	clearClipboard: function(){
		dojo.query("."+ com.ibm.rdm.web.grid.Module.ClassNames.ROW_CUT, this.grid.domNode).removeClass(com.ibm.rdm.web.grid.Module.ClassNames.ROW_CUT);
		dojo.query("."+ com.ibm.rdm.web.grid.Module.ClassNames.ROW_COPY, this.grid.domNode).removeClass(com.ibm.rdm.web.grid.Module.ClassNames.ROW_COPY);
		
		// Wipe the clipboard
		clipboard.clear();
	},
	
	
	_executeModuleCopyPaste: function (targetURI, pasteSpecialInfo, where, pasteData) {
		// summary:
		//		Execute a paste following a copy,
		//		invokes the ArtifactCopierClient to do the
		//		dirty work.
		// args {
		//		targetURI,
		//		pasteInfo (if from a paste sepcial) optional
		//		where - optional target - if both this arg and pasteInfo are present
		//			    then the target specified here will win.
		//		
		// Secure clipboard contains cut data, give that priority
		var data = pasteData || clipboard.get(),
			sourceURIs = data.uris,
			pasteInfo = pasteSpecialInfo,
			targetEntry,
			relativeSiblingURI,
			operationTarget = targetURI,
			copyComplete = new dojo.Deferred();
			
		if (!sourceURIs) {
			console.error("Paste should not have been enabled...");
		}

		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI();
		var lockFailedFn = function() {
			copyComplete.errback();
		};
		var moduleStructure = this.moduleStructure;
		var modURI = moduleStructure.moduleURI;
		
		var lockObtainedFn = dojo.hitch(this, function() {
			if(pasteInfo && !pasteInfo.sourceModule) {
				// This paste did not originate from a module. Execute Add Existing
				var position,
					rowIds = {};
				rowIds[targetURI] = targetURI;
				// We obtained this position from the paste special dialog.
				// This dialog uses the full position url, and we only need the postion ie: "after", "before", etc
				switch(pasteInfo.position){
					case RRMTypes.MODULE_RELATIVE_TARGET_POSITION.BEFORE:
						position = com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE;
					break;
					case RRMTypes.MODULE_RELATIVE_TARGET_POSITION.BELOW: 
						position = com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW;
					break;
					default: 
						position = com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER;
					break;
				}
				return this.executeAddExisting(rowIds, position, sourceURIs);
			}
			StatusDisplay.showStatusMessage(this._messages.copyingArtifacts);
			
			pasteInfo = pasteInfo || {
				copyPolicy: RRMTypes.MODULE_COPY_POLICIES.DUPLICATE_ALL,
				copyLinks: true,
				position: RRMTypes.MODULE_RELATIVE_TARGET_POSITION.AFTER
			};

			// Note by Sean on 02/16/2015: Steve added the following sourceURIs.sort call (via
			// Defect 62324), but to address Defect 94843 I'm conditionalizing it to NOT do anything
			// if the "modulefacts to copy" originated in a different module (if we just let the
			// sort proceed without this check, all the entryA/entryB values would be undefined).
			if (moduleStructure.getArtifactByURI(sourceURIs[0])) {
				// make sure the selections are sent in Book order
				// TODO: this may not be required.
				sourceURIs.sort(function (a, b) {
					var entryA = moduleStructure.getArtifactByURI(a),
						entryB = moduleStructure.getArtifactByURI(b);
					return moduleStructure.indexOf(entryA) - moduleStructure.indexOf(entryB);
				});
			}
			
			// remove children where the an ancestor is also selected
			sourceURIs = moduleStructure.stripToLowestCommonAncestors(sourceURIs);
			// find the target,  
			targetURI = moduleStructure.findProperParentURI(targetURI, where);
			targetEntry = moduleStructure.getArtifactByURI(targetURI);
			var siblingAndPosition = this._getRelativeSiblingAndPosition(targetEntry, operationTarget, where);
			relativeSiblingURI = siblingAndPosition.relativeSiblingURI;
			// this is because we interpret below as meaning first child, we don't use
			// the relativesibling style API internally, although maybe it would be 
			// easier to read of we did.
			var moduleStructurePosition = where;
			where = siblingAndPosition.where;
			// now map onto copier service style names, this should perhaps be
			// hidden in there.
			if (where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER) {
				pasteInfo.position =  RRMTypes.MODULE_RELATIVE_TARGET_POSITION.AFTER;
			} else if (where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE) {	
				pasteInfo.position =  RRMTypes.MODULE_RELATIVE_TARGET_POSITION.BEFORE;
			} else if (where === null || where === undefined) {	
				pasteInfo.position =  null;
			}
			

			ArtifactCopierClient.requestCopy({
				sourceURIs: sourceURIs,
				targetURI: targetURI,
				copyPolicy: pasteInfo.copyPolicy,
				relativeSiblingURI: relativeSiblingURI,
				targetPosition: pasteInfo.position,
				copyLinks: pasteInfo.copyLinks,
				copyAndLink: pasteInfo.copyAndLink,
				copyAndLinkType: pasteInfo.copyAndLinkType,
				// TODO: we should use a deferred here probably,
				onComplete: dojo.hitch(this, function (uris, createdLinks) {
					var reconciledListener;
						
					StatusDisplay.hideStatusMessage();
					LockManager.releaseLockedResource(modURI);
				
					reconciledListener = ChangeNotifier.addListener({
						eventTopic: com.ibm.rdm.web.notifications.EventTopics.MODULE_GRID_RECONCILED,
						callbackFunction: function () {
							var idx, row = {}, nodeList = new dojo.NodeList();
							this.grid.scrollIntoView(uris[0]);
							this.grid.moveFocusToCellOnRow(uris[0]);
							
							for (idx=0; idx<uris.length && row; idx++) {
								row = this.grid.getRowByKey(uris[idx]);
								if (row) {
									nodeList.push(row.node);
								}
							}
							
							nodeList.addClass("justPasted");
							setTimeout(function () {
								nodeList.removeClass("justPasted");
							}, 1000);
							
							ChangeNotifier.removeListener(reconciledListener);
						},
						callbackScope: this
					});
					
					if(createdLinks) {
						ChangeNotifier.notifyListeners({
							eventTopic: com.ibm.rdm.web.notifications.EventTopics.LINK_CREATED,
							links	  : createdLinks,
							projectURI: com.ibm.rdm.web.CURRENT_PROJECT,
							crossWindow: true
						});
					}
					
					this.moduleStructure.updateETagOfResources([this.moduleStructure.compositeRootURI, targetURI]);
					
					copyComplete.callback();
					
					// this triggers the grid to request the missing artifacts, when that is done
					// we need to find those artifacts and scroll them into view, highlight them etc.
					this.moduleStructure.requestAndPlaceURIs(operationTarget, uris, moduleStructurePosition);
				}),
				onError: function (response) {
					LockManager.releaseLockedResource(modURI);
					copyComplete.errback();
					StatusDisplay.hideStatusMessage();
					MessageHandler.showError(7741);
				}
				// Don't need this at the moment, but cad add it if we ever have anything useful for it to do.
				//onProgressUpdate: function (status) {
				//}
			});
		});
		
		var hasLatestDef = moduleStructure.checkStructureCurrentByParentAndUpdateETag(targetURI, where, copyComplete);
		hasLatestDef.then(dojo.hitch(this, function(shouldContinue){
			if (shouldContinue) {
				this._implicitLockFunction(lockObtainedFn, lockFailedFn, modURI, this._projectURI, moduleProcessAreaURI);
			}
			else{
				copyComplete.errback();
			}
		}), dojo.hitch(this, function(error){
			//errored 
			copyComplete.errback();
		}));
		
		return copyComplete;
	},
	
	executeModulePasteSpecial: function (rowIds, position, pasteData) {
	
		var dialog,
			targetURI,
			handlePaste,
			hidePosition = (pasteData ? true : false),
			isCut = (pasteData ? false : this._isUnderCut());
			
		for (var thisResource in rowIds) {
			targetURI = thisResource;
			break;
		}

		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI();
		var lockFailedFn = function() {};
		var modURI = this.moduleStructure.moduleURI;
		var data = pasteData || clipboard.get();
		var lockObtainedFn = dojo.hitch(this, function() {		
			handlePaste	= dojo.hitch(this, function () {
				var pasteInfo = dialog.getPasteInfo(),
					where;
				if (pasteInfo.position) {
					if (pasteInfo.position === RRMTypes.MODULE_RELATIVE_TARGET_POSITION.AFTER) {
						where = com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER;
					} else if (pasteInfo.position === RRMTypes.MODULE_RELATIVE_TARGET_POSITION.BEFORE) {
						where = com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE;
					} else {
						where = com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW;
					}
				} else {
					where = position;
				}
				
				if (isCut) {
					this.executeModulePaste(rowIds, where);
				} else {
					this._executeModuleCopyPaste(targetURI, pasteInfo, where, data);
				}
			});
			// pop up the dialog....
			dialog = new PasteSpecialDialog({
				onOk: handlePaste,
				isCut: isCut,
				position: position,
				projectURI: this._projectURI,
				targetURI: targetURI,
				sourceURIs: data.uris,
				hidePosition: hidePosition,
				sourceModURI: data.moduleURI
			});
			dialog.show(dojo.hitch(this, this._onLaunchedDialogHasClosed));
		});
		
		this._implicitLockFunction(lockObtainedFn, lockFailedFn, modURI, this._projectURI, moduleProcessAreaURI);
	},
	
	executeModulePaste: function(rowIds, where, pasteData, waitForIndexing){
		var data = pasteData || clipboard.get(),
			resourceURIs = data.uris,
			sourceModURI = data.moduleURI,
			targetURI;
		for (var thisResource in rowIds) {
			targetURI = thisResource;
			break;
		}
		
		var outerDeferred = new dojo.Deferred();

		var lockFailedFn = function() {};
		var modURI = this.moduleStructure.moduleURI;
		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI();
		
		var lockObtainedFn = dojo.hitch(this, function() {
			if (this._isUnderCut()) {
				var deferred,
					resourcesPending;

				// Ensure that pasted artifacts are in the original order, not the
				// selected order.
				this.moduleStructure.arraySortOnBookorder(resourceURIs);
				
				resourcesPending = dojo.clone(resourceURIs);
				
				if (where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW) {
					resourcesPending.push(targetURI);
				}
				
				if (this._editInterface) {
					this._editInterface.saveStarted(resourcesPending);
				}
				deferred = this.moduleStructure.placeChildren({
					targetURI: targetURI,
					resourceURIs: resourceURIs,
					where: where,
					isMove: true, 
					waitForIndexing: waitForIndexing
				});
				deferred.then(dojo.hitch(this, function(response){
					//clear clipboard
					this.clearClipboard();
					// Highlight the pasted rows
					dojo.forEach(resourceURIs, function(resourceURI){
						this._highlightNewRow(resourceURI);
					}, this);
				}));
				deferred.addBoth(this, function(){
					this.grid.retakeFocus();
					if (this._editInterface) {
						this._editInterface.saveFinished(resourcesPending);
					}
				});
				deferred.addCallback(this, function(param){
					// return a deferred, so further callbacks wait for locks to be released.
					var releaseLockDeferred = LockManager.releaseLockedResource(modURI);
					return releaseLockDeferred;
				});
				deferred.addCallback(this, function(param){
					outerDeferred.callback(resourceURIs);
					return param;
				});
				deferred.addErrback(this, function(param){
					LockManager.releaseLockedResource(modURI);
					outerDeferred.errback(param);
				});
			} else {
				// a copy paste has been performed...
				if(!sourceModURI) {
					// This copy didn't originate from a module. Execute add existing
					var rowIds = {};
					rowIds[targetURI] = targetURI;
					return this.executeAddExisting(rowIds, where, resourceURIs);
				}
				// This copy did originate from a module. Execute paste
				return this._executeModuleCopyPaste(targetURI, null, where, pasteData);
			}
		});

		this._implicitLockFunction(lockObtainedFn, lockFailedFn, modURI, this._projectURI, moduleProcessAreaURI);

		return outerDeferred;
	},
	
	executeDelete: function(rowIds){
		var continueWithDelete = dojo.hitch(this, this.inherited, arguments);
		
		// Ensure the rows we have are the roots of non-overlapping trees.  If we leave
		// descendants in this list the delete of some atrifacts fails because they may
		// have already been deleted when the parent was deleted.
		rowIds = this._removeDescendants(rowIds);
		return continueWithDelete();
		
	},
	
	executePostLinkEvent: function(linkTypeURI, createdLink, flipLinks) {	
		if(this._isUnderCut()) {
			this.clearClipboard();
		}
		this.inherited(arguments);
	},
	
	_removeDescendants: function(rowIds) {
		// summary:
		//		Generates a list of rowIds that excludes any that are descendants of others in the
		//		list.  So the resultant list will consist only of root nodes of sub-trees that have
		//		no overlap.
		// rowIds:
		//		hash of rowIds to row types.
		// returns:
		//		hash similar to rowIds, but without entries that are desccendants of any other entries.
		//		
		var filteredResources = {};
		for (var resourceId in rowIds) {
			var ancestors = this.moduleStructure.getAncestors(resourceId);
			var includeResource = true;
			for (var i = 0 ; i < ancestors.length; i++) {
				if (rowIds[ancestors[i]]) {
					includeResource = false;
					break;
				}
			}
			if (includeResource) {
				filteredResources[resourceId] = rowIds[resourceId];
			}
		}
		return filteredResources;
	},
	
	createSimilarWhereText: function(num, rowIds, where){
		// summary:
		//		Text that should be used for copy menu option
		// num: int
		//		The number of rows that are selected in the grid.
		var artifactURI,
			message;
		for (var key in rowIds) {
			artifactURI = key;
		}
		var resource = this._dataCache.getRecordInfo(artifactURI);
		var objectTypes = resource.getTypes();
		var objectType;
		for (var type in objectTypes) {
			objectType = this._typeCache.getProjectObjectType(type);
			break;
		}
		
		switch(where){
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER:
				message = this._messages.after;
			break;
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE:
				message = this._messages.before;
			break;
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW:
			default:
				message = this._messages.below;
			break;
		}
		
		return message;
	},

	createWhereText: function(num, rowIds, where){
		// summary:
		//		Text that should be used for create artifact submenu
		// num: int
		//		The number of rows that are selected in the grid.
		// rowIds: object
		//		uris of selected rows (ids)
		// where: string
		//		where to place the new artifact. after, before, or below
		var message;
		
		switch(where){
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER:
				message = this._messages.afterElipses;
			break;
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE:
				message = this._messages.beforeElipses;
			break;
			case com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW:
			default:
				message = this._messages.belowElipses;
			break;
		}
		
		return message;
	},
	
	removeFromModuleText: function(num) {
		// This calculates the display text for the command.
		if (num === 1) {
			return this._messages.removeOne;
		}
		return dojo.string.substitute(this._messages.removeN, [num]);
	},
	
	isMoveEnabled: function(rowIds, count) {
		// summary:
		//		Determine if the Move menu option should be enabled. It should only be enabled if
		//		the current user has the correct permission to move the selected resources.
		// rowIds: int
		//		The identifier of each of the rows that are selected.
		// count: int
		//		The count of the rows that have been selected.
		if (count > 0 && !(this._baselineURI || this._isInHistoryMode || this._isInReviewMode)) {
			//This function decides if move is allowed.
			//since C/ALM rows do no contain the same methods, disable action.
			if (this.containsCalmRow(rowIds)) {
				return false;
			}

			var projectOperations = this._projectOperations;
			var isMoveAllowed = projectOperations.item(ProjectCache.MODIFY_RESOURCE);

			return isMoveAllowed && this.structuralOperationsAllowed();
		}
		return false;
	},

	getCutEnablementDeferred: function(rowIds, count) {
		// summary:
		//		Returns a deferred which will be called back with a single
		//		param representing whether the "Cut" menu item should be
		//		enabled.
		// returns:
		//		A Deferred.
		var deferred = new dojo.Deferred();
		var projectDeferred = ProjectCache.getProjectPermissions(this._projectURI);
		
		projectDeferred.then(dojo.hitch(this, function() {
			if (this.isMoveEnabled(rowIds, count)) {
				this.getIsModifyModuleContentAllowed().then(
					dojo.hitch(this, function(isModifyModuleContentAllowed) {
						deferred.callback(isModifyModuleContentAllowed);
					})
				);
			} else {
				deferred.callback(false);
			}
		}));
		
		return deferred;
	},

	_isPasteEnabledWithoutConsideringTeamOperations: function(rowIds, count) {
		if (!this.structuralOperationsAllowed()) {
			return false;
		}
		if (!this._sameOriginClipboard()) {
			return false;
		}	
		if (!this.isMoveEnabled(rowIds, count)) {
			return false;
		}
		
		var artifactURI,
			key;
		for (key in rowIds) {
			artifactURI = key;
			break;
		}
		
		var uriArray = clipboard.get().uris;
		
		if(this._isUnderCut()) {
			//Make sure we don't try to cut & paste an artifact below it's own children.
			if (this.moduleStructure.isAnyAnAncestor(artifactURI, uriArray)){
				return false;
			}
			
			//Make sure we aren't cut & pasting any artifact on itself
			if (dojo.indexOf(uriArray, artifactURI) !== -1){
				return false;
			}
		}
		
		// We have made it this far... 
		return true;
	},
	
	getPasteEnablementDeferred: function(rowIds, count) {
		// summary:
		//		Returns a deferred which will be called back with a single
		//		param representing whether the "Paste Artifact After" menu
		//		item should be enabled.
		// returns:
		//		A Deferred.
		var deferred = new dojo.Deferred();
		var projectDeferred = ProjectCache.getProjectPermissions(this._projectURI);
		
		projectDeferred.then(dojo.hitch(this, function() {
			if (this._isPasteEnabledWithoutConsideringTeamOperations(rowIds, count)) {
				this.getIsModifyModuleContentAllowed().then(
					dojo.hitch(this, function(isModifyModuleContentAllowed) {
						deferred.callback(isModifyModuleContentAllowed);
					})
				);
			} else {
				deferred.callback(false);
			}
		}));
		
		return deferred;
	},

	isToggleHeadingEnabled: function(count) {
		return this.structuralOperationsAllowed();
	},
	
		/////////////// Toggle heading command //////////////////////////
	
	getToggleHeadingText: function(numSelected, rowIds, key) {
		// summary:
		//		Returns the text to display on the menu for the toggle header operation.
		// key:
		//		Identifier for the artifact to which the menu relates.
		var resource = this.moduleStructure.getArtifactByURI(key);
		var isHeading = resource.isHeading;
		if (numSelected > 1) {
			return isHeading ? dojo.string.substitute(this._messages.changeFromHeadingMultiple, [numSelected]) 
							 : dojo.string.substitute(this._messages.changeToHeadingMultiple, [numSelected]);
		}
		return isHeading ? this._messages.changeFromHeading : this._messages.changeToHeading;
	},

	_processToggleHeading: function(isHeading, rowIds, key, callerAlreadyLockedModule) {
		// summary:
		//		Toggle the heading state of one or more rows.
		// callerAlreadyLockedModule: Boolean?
		//		Pass as true iff the calling function has already locked the
		//		module.  If not passed, or passed as false, this function will
		//		lock the module before performing headingness updates and
		//		unlock it afterward.
		// returns: Deferred
		//		Returns a deferred which will fire after the headingness
		//		changes (which affect only the module structure) have completed.

		var toggleDeferred;
		var resourceURIs = [],
			resourceURI = null,
			moduleStructure = this.moduleStructure;
			
		for (resourceURI in rowIds) {
			resourceURIs.push(resourceURI);
		}
		
		// UX request here is to only take the action taken by toggling
		// the artifact the menu has been launched on.  e.g. if we have
		// selected a mixture of headings and non headings, and we bring
		// up the menu on a heading - we are switching to non heading
		// so remove all the things from the selection that are already
		// not headings.
		var keyArtifact = moduleStructure.getArtifactByURI(key);
		
		resourceURIs = dojo.filter(resourceURIs, function (uri) {
			var artifact = moduleStructure.getArtifactByURI(uri);
			return artifact.isHeading !== isHeading;
		});
		
		if (resourceURIs.length === 0) {
			toggleDeferred = new dojo.Deferred();
			toggleDeferred.callback();
			return toggleDeferred;
		}
		
		var isStub = keyArtifact.isStub;
		
		var moduleProcessAreaURI = this._dashboardInterface.getModuleProcessAreaURI(),
			lockFailedFn = function() {},
			moduleURI = this.moduleStructure.moduleURI,
			currentFocusRowKey = this.grid.getFocusRowKey();
		
		var lockObtainedFn = dojo.hitch(this, function() {
			if (!callerAlreadyLockedModule) {
				this._editInterface.saveStarted(resourceURIs);
				StatusDisplay.showStatusMessage(this._messages.updatingArtifact);
			}
			toggleDeferred = this.moduleStructure.toggleHeading({
				resourceURIs: resourceURIs,
				key: key,
				isStub: isStub,
				onComplete: dojo.hitch(this, function(resource, isUnderEdit) {
					var oldRecord = this._dataCache.getRecordInfo(resource.getResourceURI());
					oldRecord.setAttribute(RRMTypes.WELL_KNOWN_IS_HEADING, resource.isHeading() ? "true" : false);
					this._editInterface.updateGridAfterHeaderToggle(oldRecord.isHeading(), oldRecord, isUnderEdit);
					this._editInterface.saveFinished([resource.getResourceURI()]);
				})
			});
			if (!callerAlreadyLockedModule) {
				toggleDeferred.addBoth(this, function(completeResponse){
					StatusDisplay.hideStatusMessage();
					LockManager.releaseLockedResource(moduleURI);
					this._editInterface.saveFinished(resourceURIs);
					if (currentFocusRowKey) {
						this.grid.moveFocusToCellOnRow(currentFocusRowKey);
					}
					return completeResponse;
				});
			}
		});

		if (callerAlreadyLockedModule) {
			lockObtainedFn();
		} else {
			this._implicitLockFunction(lockObtainedFn, lockFailedFn, moduleURI, this._projectURI, moduleProcessAreaURI);
		}

		return toggleDeferred;
	},

	executeOpenToggleHeadingAndType: function(rowIds, key) {
		var keyArtifact = this.moduleStructure.getArtifactByURI(key);
		var moduleTypeURI =
			this._dashboardInterface.getModuleArtifactTypeURI();
		var moduleType =
			this._typeCache.getProjectObjectType(moduleTypeURI);
		var headingTypeZ = this._typeCache.getHeadingObjectTypesForType(moduleTypeURI);
		var preferredHeadingTypeURIs =
			dojo.map(this._typeCache.getPreferredHeadingObjectTypesForType(moduleType), function(type) {
				return type.getResourceURI();
			});
		var nonHeadingTypes = this._typeCache.getNonHeadingObjectTypesForType(moduleTypeURI);
		var preferredNonHeadingTypeURIs =
			dojo.map(this._typeCache.getPreferredNonHeadingObjectTypesForType(moduleType), function(type) {
				return type.getResourceURI();
			});
		
		var addNonPreferred = function(types, otherTypes, filterURIs) {
			otherTypes = dojo.filter(otherTypes, function(type) {
				return dojo.indexOf(filterURIs, type.getResourceURI()) === -1;
			});
			return types.concat(otherTypes);
		};
		
		var typesToExclude, typeToPreselect;
		if (keyArtifact.isHeading) {
			typesToExclude = preferredNonHeadingTypeURIs.length === 0
				? headingTypeZ
				: addNonPreferred(headingTypeZ, nonHeadingTypes, preferredNonHeadingTypeURIs);
			typeToPreselect = preferredNonHeadingTypeURIs.length === 0 ? [] : preferredNonHeadingTypeURIs[0];
		} else {
			typesToExclude = preferredHeadingTypeURIs.length === 0
				? nonHeadingTypes
				: addNonPreferred(nonHeadingTypes, headingTypeZ, preferredHeadingTypeURIs);
			typeToPreselect = preferredHeadingTypeURIs.length === 0 ? [] : preferredHeadingTypeURIs[0];
		}
		var typeURIsToExclude = dojo.map(typesToExclude, function(type) {
			return type.getResourceURI();
		});
		var count = this._getNumberOfRowIds(rowIds),
			objectType = this._getTypeForResourceURI(key),
			resource,
			modifyArtifactPermissions;
		var showPickerForm = dojo.hitch(this, function(isModifyArtifactTypeAllowed){
			var form = new com.ibm.rdm.web.picker.ArtifactTypePickerForm({
				destroyContents: true,
				projectURI: this._projectURI,
				viewType: com.ibm.rdm.web.picker.ArtifactTypePickerForm.VIEW_TYPE.ToggleHeadingShortcut,
				title: this._messages.pickerDialogTitle,
				description: this._messages.pickerDialogDescription,
				artifactTypeUriToPreselect: typeToPreselect,
				artifactTypeUrisToOmitFromListing: typeURIsToExclude,
				onlyShowArtifactTypesOfModuleFormat: false,
				allowMultipleSelections: false,
				isModifyArtifactTypeAllowed: isModifyArtifactTypeAllowed,
				permissionType: ProcessOperationsCache.PERMISSION_TYPES.Modify,
				tooltipToUseForDisabledListItems: (
					isModifyArtifactTypeAllowed ?
					null :
					dojo.string.substitute(
						this._messages.cannotChangeArtifactTypeTooltip,
						[objectType.getTitle()]
					)
				),
				targetArtifactHeadingType: (
					keyArtifact.isHeading ?
					headingTypes.Heading :
					headingTypes.NonHeading
				),
				numberOfTargetArtifacts: count,
				onOk: dojo.hitch(this, function() {
					var typeAndHeadingInfo = form.getSelectedArtifactTypeAndHeadingType();
					var type = typeAndHeadingInfo.objectType || objectType;
					var isHeading = typeAndHeadingInfo.headingType === headingTypes.Heading;
					this._executeToggleHeadingAndType(type, isHeading, rowIds, key);
				})
			});
			form.show(dojo.hitch(this, this._onLaunchedDialogHasClosed));
		});
		if (count === 1) {
			// Handle the case of a single row selected.
			resource = this._dataCache.getResourceByUri(key);
			modifyArtifactPermissions =
				this._permissionsCache.getModifyArtifactPermission(resource);
			showPickerForm(modifyArtifactPermissions.isModifyArtifactTypeAllowed());
		} else {
			// Handle the case of multiple rows selected.
			showPickerForm(true);
		}
	},

	isAddArtifactsOrCreateEnabled: function(rowIds, count) {
		// We need to do a permissions check to make sure the user should
		// really be able to perform this action.
		// Although we return false, a subsequent call to the
		// getAddArtifactsOrCreateEnabledDeferred function will
		// result in appropriately enabling/disabling the menu item.
		return false;
	},

	getAddArtifactsOrCreateEnabledDeferred: function(rowIds, count) {
		var deferred = new dojo.Deferred();
		var projectDefered = ProjectCache.getProjectPermissions(this._projectURI);
		
		projectDefered.then(dojo.hitch(this, function() {
			var isAddArtifactsOrCreateEnabledWithoutConsideringTeamOperations =
				this._isAddArtifactsOrCreateEnabledWithoutConsideringTeamOperations(
					rowIds, count
				);
			if (isAddArtifactsOrCreateEnabledWithoutConsideringTeamOperations) {
				this.getIsModifyModuleContentAllowed().then(
					dojo.hitch(this, function(isModifyModuleContentAllowed) {
						deferred.callback(isModifyModuleContentAllowed);
					})
				);
			} else {
				deferred.callback(false);
			}
		}));
		
		return deferred;
	},

	_isAddArtifactsOrCreateEnabledWithoutConsideringTeamOperations: function(rowIds, count){
		if (!this.structuralOperationsAllowed()) {
			return false;
		}

		if (count > 0 && !(this._baselineURI || this._isInReviewMode || this._isInHistoryMode)) {
			var projectOperations = this._projectOperations;
			return projectOperations.item(ProjectCache.MODIFY_RESOURCE);
		}
		return false;


	},

	structuralOperationsAllowed: function() {
		//Check if in baseline/history/review....
		if (this._baselineURI || this._isInHistoryMode || this._isInReviewMode) {
			return false;
		}
		if (this.moduleStructure) {
			var sortOrder = this.moduleStructure.getPrimarySortBy();
			if ( (sortOrder.def === RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BookOrder) &&
					((sortOrder.order === "a") || (sortOrder.order === "")) ) {
			
				// so far so good...
				return !this.moduleStructure.hasViewFiltersApplied();
			}
		}
	},

	getRemoveArtifactEnablementDeferred: function(rowIds, count) {
		// summary:
		//		Returns a deferred which will be called back with a single
		//		param representing whether the "Remove Artifact" menu item
		//		should be enabled.
		// description:
		//		The user should only be able to "Remove Artifact" if his/her
		//		account has privileges to modify a Module based artifact type.
		//		This function returns a deferred which will be called back to
		//		tell you whether this privilege is enabled.
		// returns:
		//		A Deferred.
		var deferred = new dojo.Deferred();
		var projectDeferred = ProjectCache.getProjectPermissions(this._projectURI);
		
		projectDeferred.then(dojo.hitch(this, function() {
			if (this._isModifyArtifactsAllowedWithoutConsideringTeamOperations()) {
				this.getIsModifyModuleContentAllowed().then(
					dojo.hitch(this, function(isModifyModuleContentAllowed) {
						deferred.callback(isModifyModuleContentAllowed);
					})
				);
			} else {
				deferred.callback(false);
			}
		}));
		
		return deferred;
	},

	_isModifyArtifactsAllowedWithoutConsideringTeamOperations: function() {
		return (
			this._projectOperations.item(ProjectCache.MODIFY_RESOURCE) &&
			!( this.baselineURI || this._isInHistoryMode || this._isInReviewMode) &&
			this.structuralOperationsAllowed()
		);
	},

	getIsModifyModuleContentAllowed: function() {
		var deferred = new dojo.Deferred();
		var moduleArtifactTypeURI =
			this._dashboardInterface.getModuleArtifactTypeURI();
		var projectURI = this._projectURI;
		var projectSnapshotURI = this._baselineURI;
		var moduleProcessAreaURI =
			this._dashboardInterface.getModuleProcessAreaURI();
		var objectTypeDeferred = TypesManager.getObjectType({
			typeURI: moduleArtifactTypeURI,
			projectURI: projectURI,
			projectSnapshotURI: projectSnapshotURI
		});
		objectTypeDeferred.addCallback(this, function(objectType){
			var moduleArtifactTypeActionId = objectType.getActionId();
			ProcessOperationsCache.getModifyArtifactPermissions(
				moduleArtifactTypeActionId, moduleProcessAreaURI, projectURI
			).then(
				dojo.hitch(this, function(modifyArtifactPermissions){
					deferred.callback(
						modifyArtifactPermissions.isModifyContentAllowed()
					);
				})
			);
		});
		return deferred;
	},
	
	executeCopyStructure: function (rowIds, showCopy) {
		this.clearClipboard();
		
		// default to true for undefined othewise use truthiness of the parameter
		showCopy = (typeof showCopy === "undefined") ? true : showCopy;
		
		var forClipboard = [],
			totalCopy = {},	// keeps track of overlapping artifacts
			copyCount = 0,
			key, entry, children;
			
		
		for (key in rowIds) {
			forClipboard.push(key);
			if (showCopy) {
				totalCopy[key] = key; 
				entry = this.moduleStructure.getArtifactByURI(key);
				children = this.moduleStructure.getDescendants(entry);
				
				var thisRow = this.grid.getRowByKey(key);
				if (thisRow) {
					dojo.addClass(thisRow.node, com.ibm.rdm.web.grid.Module.ClassNames.ROW_COPY);
				}
				if (children) {
					dojo.forEach(children, dojo.hitch(this, function (child) {
						totalCopy[child.uri] = child.uri;
						var row = this.grid.getRowByKey(child.uri);
						if (row) {
							dojo.addClass(row.node, com.ibm.rdm.web.grid.Module.ClassNames.ROW_COPY);
						}
					}));
				}
			}
		}
		// count the entries
		for (var uri in totalCopy) {
			copyCount++;
		}
		
		var copyData = {
			action: COPY_ACTION,
			uris: forClipboard,
			projectURI: this._projectURI,
			moduleURI: this.moduleStructure.moduleURI,
			count: copyCount
		};
		
		// set the clipboard contents
		clipboard.put(copyData);
		
		// make sure focus stays within the module so that subsequent keyboard operations work
		this.grid.focus();
	},
	
	_isUnderCut: function() {
		var clipboardData = clipboard.get();
		return	(clipboardData && clipboardData.action === CUT_ACTION);
	},
	
	_isUnderCopy: function() {
		var clipboardData = clipboard.get();
		return	(clipboardData && clipboardData.action === COPY_ACTION);
	},
	
	_sameOriginClipboard: function() {
		var clipboardData = clipboard.get();
		return	(clipboardData && clipboardData.projectURI === this._projectURI);
	},
	
	_sameModuleClipboard: function() {
		var clipboardData = clipboard.get();
		return	(clipboardData && clipboardData.moduleURI === this.moduleStructure.moduleURI);
	},
	
	_isCopyEnabled: function (rowIds, count) {
		/*count === 1 && */
		return this._defaultIsVisible(rowIds, count) && !(this._isInReviewMode || this._isInHistoryMode);
	},
	
	_isPasteSpecialEnabledWithoutConsideringTeamOperations: function(rowIds, count) {
		var clipboardData = clipboard.get();
		return (
			(clipboardData
			&& this._defaultIsVisible(rowIds, count)
			&& this._isPasteEnabledWithoutConsideringTeamOperations(rowIds, count))
		);
	},

	getPasteSpecialEnablementDeferred: function(rowIds, count) {
		// summary:
		//		Returns a deferred which will be called back with a single
		//		param representing whether the "Paste Special" menu
		//		item should be enabled.
		// returns:
		//		A Deferred.
		var deferred = new dojo.Deferred();
		var projectDeferred = ProjectCache.getProjectPermissions(this._projectURI);
		
		projectDeferred.then(dojo.hitch(this, function() {
			if (this._isPasteSpecialEnabledWithoutConsideringTeamOperations(rowIds, count)) {
				this.getIsModifyModuleContentAllowed().then(
					dojo.hitch(this, function(isModifyModuleContentAllowed) {
						deferred.callback(isModifyModuleContentAllowed);
					})
				);
			} else {
				deferred.callback(false);
			}
		}));
		
		return deferred;
	},
	
	getHeadingEnablementDeferred: function(rowIds, count) {
		// summary:
		//		Returns a deferred which will be called back with a single
		//		bool param representing whether the "Toggle Heading" Menu Item
		//		should be enabled.
		// returns:
		//		A Deferred.
		if(!this.structuralOperationsAllowed()){
			var deferred = new dojo.Deferred();
			deferred.callback(false);
			return deferred;
		}
		
		var rowIdsArray = this._getUniqueKeysFromRowIds(rowIds),
			resourceInfos = this._getRecordInfos(rowIdsArray),
			promiseArray = [],
			deferredList,
			rowId,
			resourceInfo;
		for (var i = 0 ; i < rowIdsArray.length ; i++) {
			rowId = rowIdsArray[i];
			resourceInfo = resourceInfos[i];
			promiseArray.push(this.getIsModifyModuleContentAllowed().then(
					dojo.hitch(this, function(isModifyModuleContentAllowed) {
						return isModifyModuleContentAllowed;
					})
			));
		}
		
		deferredList = new dojo.DeferredList(promiseArray);
		// Add callback rather than use .then, because we need to change the return value of the deferredList.
		deferredList.addCallback(this, function(results){
			for (var i = 0 ; i < results.length; i++) {
				var curResult = results[i];
				if (!curResult[0]) {
					// operation failed - send the whole deferredList into error by returning the underlying error.
					return curResult[1];
				}
				if (!curResult[1]) {
					// got a result of false (no permission) for one of the rows.  Make the overall result false;
					return false; 
				}
			}
			// Permission granted on all rows.
			return true;
		});
		return deferredList;
	},
	
	///// Lock and Unlock Commands
	
	_mapRowIdsToBoundURI: function(rowIds) {
		// summary:
		//		Takes the rowIds and returns two arrays: an array of base artifact URIs and an array of keys.

		var keysArray = [];
		var baseArray = [];
		for(var item in rowIds) {
			keysArray.push(item);
			baseArray.push(this._dataCache.getBaseArtifactForResource(item));
		}
		
		return {keys: keysArray, base: baseArray};
	},
	
	_replaceRowIdsWithBaseURI: function(rowIds) {
		// summary:
		//		Takes the rowIds and returns a RowId with base artifact URIs.

		var newRowIds = [];
		for(var item in rowIds) {
			// skip this mapping if we are dealing with a stub
			if (item && (item.indexOf("STUB") === -1)) {
				var newUri = this._dataCache.getBaseArtifactForResource(item);
				newRowIds[newUri] = rowIds[item];
			} else {
				newRowIds[item] = rowIds[item];
			}
		}
		
		return newRowIds;
	},

	// Lock Commands
	
	//Just use the inherited function
	//lockText: function(num){

	isLockEnabled: function(rowIds, count){
		// summary:
		//		same as the non-module command, but mapped from moduleURIs to base artifact URIs
		return false;
	},

	isLockVisible: function(rowIds, count){
		// summary:
		//		same as the non-module command, but mapped from moduleURIs to base artifact URIs
		var mapped = this._replaceRowIdsWithBaseURI(rowIds);
		
		return this._isLockVisibleImpl(mapped, count);	
	},
	
	
	executeLock: function(rowIds){
		// summary:
		//		same as the non-module command, but mapped from moduleURIs to base artifact URIs
		var mapped = this._mapRowIdsToBoundURI(rowIds);
		
		this._executeLockImpl(mapped.keys, mapped.base);
	},
		
	////////// Unlock Command
	
	//unlockText: function(num){

	isUnlockEnabled: function(rowIds, count){
		// summary:
		//		same as the non-module command, but mapped from moduleURIs to base artifact URIs
		var mapped = this._replaceRowIdsWithBaseURI(rowIds);
		
		return this._isUnlockEnabledImpl(mapped, count);	
	},
	
	isUnlockVisible: function(rowIds, count){
		// summary:
		//		same as the non-module command, but mapped from moduleURIs to base artifact URIs
		var mapped = this._replaceRowIdsWithBaseURI(rowIds);
		if (this._defaultIsVisible(rowIds, count)) {
			return this._isUnlockVisibleImpl(mapped, count);
		}
		return false;
	
	},	

	executeUnlock: function(rowIds){
		// summary:
		//		same as the non-module command, but mapped from moduleURIs to base artifact URIs
		var mapped = this._mapRowIdsToBoundURI(rowIds);
		
		this._executeUnlockImpl(mapped.keys, mapped.base);
	},
	
	getReleaseLockEnablementDeferred: function(rowIds, count) {
		// summary:
		//		Returns a deferred which will be called back with a single param
		//		representing whether the "Release Lock" menu item should be enabled.
		//		This is for ModuleArtifactCommands, so it maps the moduleURIs to base URIs to verify them

		// returns:
		//		A Deferred.
		var mapped = this._replaceRowIdsWithBaseURI(rowIds);

		return this._getReleaseLockEnablementDeferred(mapped, count);
	},

///// End of Lock and Unlock Commands

	_getRelativeSiblingAndPosition: function(targetEntry, resourceURI, where) {
		// summary:
		//		For a given resource  and position (below, after, before),
		//		compute the relative sibling URI and map the position to fit
		//		the server API.  e.g. for a simple case such as after, there 
		//		is no mapping to do and the relative sibling is the target.
		//		
		//		For "below", we want to get the first child of the target
		//		and specify where as "before" that first child.  As per the
		//		server API, if there are no children we need pass neither
		//		relative sibling URI or where.
		//		
		// targetEntry and resourceURI represent the same thing in the case of
		// a "below", however for after and before targetEntry is the parent
		// of resourceURI.
		var relativeSiblingURI;
		
		if ((where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.AFTER) 
			|| (where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE)) {
			relativeSiblingURI = resourceURI;
		} else if (where === com.ibm.rdm.web.model.resources.modules.ModuleStructure.BELOW) {
			if (targetEntry && targetEntry.children.length > 0) {
				// place the new thing before the first child in this case...
				where = com.ibm.rdm.web.model.resources.modules.ModuleStructure.BEFORE;
				relativeSiblingURI = targetEntry.children[0].uri;
			} else {
				where = null;
				// do not assign a position, if we are adding below, we either
				// add it before the first child if one exists, or if it does not, we omit
				// the relativeTargetPosition and relativeTargetSiblingUrl
			}
				
			// Maybe we should consider making our internal API the same as the servers
			// for now, BELOW means as first child for the module structure.

		} 
		
		return {
			relativeSiblingURI: relativeSiblingURI,
			where: where
		};
	}

});

})();
