/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.collections.CompareResourceWidget");

dojo.require("dojo.i18n");
dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");

dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.ui.MessageBox");

dojo.require("com.ibm.rdm.web.collections.CollectionCompareUtil");
dojo.require("com.ibm.rdm.web.common.UserHover");
dojo.require("com.ibm.rdm.web.history.AuditHistoryRetriever");
dojo.require("com.ibm.rdm.web.server.client.utils.UserCache");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.viewer.viewerutils");

dojo.requireLocalization("com.ibm.rdm.web.collections", "CompareResourceWidget");
dojo.requireLocalization("com.ibm.rdm.web.history", "AuditHistoryParser");

var queryutils = com.ibm.rdm.web.util.queryutils;
var UserCache = com.ibm.rdm.web.server.client.utils.UserCache;
var UserHover = com.ibm.rdm.web.common.UserHover;
var viewerutils = com.ibm.rdm.web.viewer.viewerutils;

dojo.declare(
    "com.ibm.rdm.web.collections.CompareResourceWidget",
    [dijit._Widget, dijit._Templated],
{
	
	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "collections/templates/CompareResourceWidget.html"),
	rdmModulePath: dojo.moduleUrl("com.ibm.rdm.web"),
    widgetsInTemplate: true,

    ///////////////////////////////////////////////////////////////////////////////////
    // Attach points
    _resourceDiv: null,
    _expandNode: null,
    _iconNode: null,
    _linkNode: null,
    _loadingNode: null,
    _hideNode: null,
    _historyDiv: null,
    
    ///////////////////////////////////////////////////////////////////////////////////
    // Public fields
    compareViewer: null,
    resource: null,
    showHistory: null,
    projectURI: null,
    collectionOneName: null,
    versionOneInformation: null,
    collectionTwoName: null,
    versionTwoInformation: null,
	
    ///////////////////////////////////////////////////////////////////////////////////
    // Private fields
    _messages: null,
    _auditHistoryMessages: null,
    _toDestroy: null,
    _expanded: null,
    _haveFetchedHistory: null,
    _fetchedHistoryNode: null,
    _auditHistoryRetriever: null,
    _expand: null,
    _collapse: null,

    ///////////////////////////////////////////////////////////////////////////////////
    // API
    
    constructor: function() {
    	this._toDestroy = [];
    	this._expanded = false;
    	this._haveFetchedHistory = false;
	},
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.collections", "CompareResourceWidget");
		this._auditHistoryMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.history", "AuditHistoryParser");
	},

    postCreate: function() {
    	this.inherited(arguments);
    	
    	var historyNode = this._historyDiv, hideNode = this._hideNode;
		
		this._expand = dojo.fx.wipeIn({
			node: historyNode,
			duration: dijit.defaultDuration,
			beforeBegin: function() {
				dojo.removeClass(hideNode, "hidden");
			}
		});
		this._collapse = dojo.fx.wipeOut({
			node: historyNode,
			duration: dijit.defaultDuration,
			onEnd: function() {
				dojo.addClass(hideNode, "hidden");
			}
		});
    	
		if (this.showHistory) {
			this._auditHistoryRetriever = new com.ibm.rdm.web.history.AuditHistoryRetriever();
			dojo.addClass(this._resourceDiv, "resourceDiffPane");
			this.connect(this._resourceDiv, "onclick", dojo.hitch(this, this._handleBuildHistory));
			this.connect(this._resourceDiv, "onkeydown", dojo.hitch(this, this._handleBuildHistory));
			dojo.attr(this._expandNode, "innerHTML", "+");
		}
		
		var resourceURI = this.resource.getResourceURI();
		var revision = this.resource.getRevision();
		if(revision) {
			resourceURI = revision;
		}
		var title = this.resource.getTitle();
		
		var resourceLink = new jazz.ui.ResourceLink({
		    uri: resourceURI,
		    presentationUri: resourceURI,
		    label: dojo.string.substitute(this._messages.resourceLinkLabel, [this.resource.getID(), title]),
		    iconNode: this._iconNode,
		    lazyFetch: false
		});
		
		this._toDestroy.push(resourceLink);
		
		dojo.place(resourceLink.domNode, this._linkNode);
	},
	
	uninitialize: function() {
		dojo.forEach(this._toDestroy, function(object) {
			if (object.destroyRecursive && !object._beingDestroyed) {
				object.destroyRecursive();
			}
			else if (object.destroy && !object._beingDestroyed) {
				object.destroy();
			}
			object = null;
		});

		this._toDestroy = null;
		this.inherited(arguments);
	},
	
	_handleBuildHistory: function(event) {
		if (event && event.keyCode && event.keyCode !== 13) {
			return;
		}
		
		var animation = this[this._expanded ? "_collapse" : "_expand"];
		this._expanded = !this._expanded;
		var innerHTML = (this._expanded) ? "-&nbsp;" : "+";
		dojo.attr(this._expandNode, "innerHTML", innerHTML);
		dojo.toggleClass(this._resourceDiv, "selected");
		
		if (!this._haveFetchedHistory) {
			// Build up history URI
			var resourceURI = this.resource.getResourceURI();
			var startDate = null;
			var endDate = null;
			if (this.versionOneInformation.baseline.issued < this.versionTwoInformation.baseline.issued) {
				startDate = this.versionOneInformation.baseline.issued;
				if (!this.versionOneInformation.currentVersion && !this.versionTwoInformation.currentVersion) {
					endDate = this.versionTwoInformation.baseline.issued;
				}
			}
			else {
				startDate = this.versionTwoInformation.baseline.issued;
				if (!this.versionOneInformation.currentVersion && !this.versionTwoInformation.currentVersion) {
					endDate = this.versionOneInformation.baseline.issued;
				}
			}
			var historyURI = "?uri=" + resourceURI + "&startDate=" + startDate;
			(endDate) ? historyURI += "&endDate=" + endDate : historyURI += "";
			dojo.removeClass(this._loadingNode, "hidden");
			
			// Fetch history
			this._auditHistoryRetriever.getArtifactHistory(historyURI).then(
				dojo.hitch(this, function(auditHistoryResponse) {
					this._auditHistoryParser = auditHistoryResponse.doc;
					var changeLogURIs = this._auditHistoryParser.getChangeLogs();
					var historyContent = this._buildHistoryContent(changeLogURIs);
					dojo.addClass(this._loadingNode, "hidden");
					dojo.marginBox(this._historyDiv, { h: dojo.marginBox(this._historyDiv).h });
					dojo.place(historyContent, this._historyDiv);
					animation.play();
				}),
				dojo.hitch(this, function(errorResponse) {
					console.error(errorResponse);
					this.compareViewer.showXhrError(errorResponse);
				})
			);
			this._haveFetchedHistory = true;
		}
		else {
			animation.play();
		}
	},
	
	_buildHistoryContent: function(changeLogURIs) {
		var wrapperDiv = dojo.create("div");
		dojo.place(this._createHeaderTable(), wrapperDiv);
		var userURIs = this._getUserURIs(changeLogURIs);
		UserCache.getUsers(userURIs).addCallback(this, function(usersMap) {
			dojo.forEach(changeLogURIs, function(changeLogURI) {
				var changeLog = this._auditHistoryParser.getChangeLog(changeLogURI);
				var user = usersMap[changeLog.contributor];
				var modifiedDate = dojo.date.locale.format(dojo.date.stamp.fromISOString(changeLog.modified), {formatLength:"medium"});
				var userNode = dojo.create("span", {
					innerHTML: user.name,
					"class": "user"
				});
				var dateNode = dojo.create("span", {
					innerHTML: modifiedDate,
					"class": "date"
				});
				
				var userHover = new UserHover({projectURI: this.projectURI}, null);
	            userHover.createHoverPopup(userNode, user);
	            this._toDestroy.push(userHover);
	            
	            var historyTable = this._buildChangeEntryContent(changeLog.changeEntries, changeLogURI, userNode, dateNode);
				dojo.place(historyTable, wrapperDiv);
				
			}, this);
		});
		
		return wrapperDiv;
	},
	
	_buildChangeEntryContent: function(changeEntries, changeLog, userNode, dateNode) {
		var table = dojo.create("table");
		var tbody = dojo.create("tbody", null, table);
		
		dojo.forEach(changeEntries, function(entry, i) {
			var changeEntry = this._auditHistoryParser.getChangeEntry(entry);
			var changeSubject = changeEntry.changeSubjectID && this._auditHistoryParser.getChangeSubject(changeEntry.changeSubjectID);
			var displayName = changeSubject && changeSubject.displayName;
			
			// Create and style DOM nodes
			var tableRow = dojo.create("tr", {}, tbody);
			var infoCell = (i > 0) ? dojo.create("td", {style: {width: "35%"}}) : this._buildInfoCell(dateNode, userNode);
			dojo.place(infoCell, tableRow);
			var tdLeftCell = dojo.create("td", {style: {width: "11%"}, noWrap: "true", vAlign: "top"}, tableRow);
			var tdRightCell = dojo.create("td", {style: {width: "49%"}}, tableRow);
			
			// Find the display name (e.g. "Business Priority"), and put it in the left cell
			var leftCellDiv = dojo.create("div", null, tdLeftCell);
			if (changeEntry.event && changeEntry.event !== "ATTRIBUTE"){
				leftCellDiv.innerHTML = this._auditHistoryMessages[changeEntry.event];
			} else if (displayName){
				leftCellDiv.innerHTML = this._auditHistoryMessages[displayName] || displayName;
			}
			
			if (changeEntry.diff){
				dojo.create("div", {innerHTML: changeEntry.diff}, tdRightCell);
			} else if (displayName === this._auditHistoryMessages["Primary Text"]){
				var textDiffSpan = dojo.create("span", null, tdRightCell);
				var showChangesLink = dojo.create("a", {
					innerHTML: this._auditHistoryMessages.showchanges
				}, textDiffSpan);
				var clickHandle = dojo.connect(showChangesLink, "onclick", this, function(){
					dojo.disconnect(clickHandle);
					this._auditHistoryParser._getChangeEntryDetails(changeLog, changeEntry.identifier).addCallback(this, function(changeEntryParser) {
						var changeEntryPrimaryText = changeEntryParser.getChangeEntry();
						var diff = changeEntryPrimaryText.diff;
						textDiffSpan.innerHTML = diff || "";
						// Update embeds and images for appropriate display in history.  Do embeds first, because doing images
						// relies on everything in the DOM being loaded and settled in size.
						this._auditHistoryParser.substituteEmbedPlaceholders(textDiffSpan);
						this._auditHistoryParser.updateImagesForDisplay(textDiffSpan);
					});
				});
			} else {
				var isDisplayNameShown = displayName && changeEntry.event !== "ATTRIBUTE";
				var isEventTypeShown = isDisplayNameShown || (!changeEntry.previousValueID && changeEntry.valueID);
				if (isEventTypeShown){
					dojo.create("div", {innerHTML: this._auditHistoryMessages[changeEntry.eventType]}, tdRightCell);
				}
				if (isDisplayNameShown){
					dojo.create("div", {innerHTML: displayName}, tdRightCell);
				}
				if (changeEntry.valueID){
					if (changeEntry.previousValueID){
						var previousValue = this._auditHistoryParser.getValue(changeEntry.previousValueID);
						var previousValueSpan = dojo.create("span", null, tdRightCell);
						if (previousValue.displayName){
							previousValueSpan.innerHTML = previousValue.displayName;
						} else {
							this._auditHistoryParser._getDisplayName(previousValue.value, previousValue.dataType).addCallback(this, function(displayName) {
								previousValueSpan.innerHTML = displayName;
							});
						}
						var arrowSpan = dojo.create("span", {innerHTML: "&rarr;"}, tdRightCell);
						dojo.style(arrowSpan, "padding", "0 3px");
					}
					var currentValue = this._auditHistoryParser.getValue(changeEntry.valueID);
					var currentValueSpan = dojo.create("span", null, tdRightCell);
					if (currentValue.displayName){
						currentValueSpan.innerHTML = currentValue.displayName;
					} else if (changeEntry.event === "COMMENT"){
						this._auditHistoryParser._getCommentInfo(currentValue.value, this._auditHistoryParser.getChangeLog(changeLog).artifactId).addCallback(this, function(response) {
							currentValueSpan.innerHTML = response;
						});
					} else if (changeEntry.event === "ARTIFACT" || changeEntry.event === "LINK"){
						// For ARTIFACT and LINK cases, if currentValue.displayName is undefined, then the resource may have
						// been deleted, or it may be a CLM resource.
						currentValueSpan.innerHTML = this._auditHistoryMessages.resourceNameNotFound;
					} else {
						this._auditHistoryParser._getDisplayName(currentValue.value, currentValue.dataType).addCallback(this, function(displayName) {
							currentValueSpan.innerHTML = displayName;
						});
					}
				}
				if (changeEntry.eventType === "MODIFIED" && !tdRightCell.hasChildNodes()) {
					// Entry has been modified, but cannot be displayed
					var changedDiv = dojo.create("div", {innerHTML: this._auditHistoryMessages.changed}, tdRightCell);
					dojo.style(changedDiv, "color", "gray");
				}
			}
		}, this);
		
		return table;
	},
	
	_getUserURIs: function(changeLogURIs) {
		var userURIs = [];
		
		dojo.forEach(changeLogURIs, function(changeLogURI) {
			var changeLog = this._auditHistoryParser.getChangeLog(changeLogURI);
			var userURI = changeLog.contributor;
			if (dojo.indexOf(userURIs, userURI) === -1) {
				userURIs.push(userURI);
			}
		}, this);
		
		return userURIs;
	},
	
	_getCollectionNamesForHistory: function() {
		var collectionNames = [];
		
        if (this.versionOneInformation.currentVersion) {
        	collectionNames.push(this._messages.collectionTwo, this._messages.collectionOne);
        }
        else if (this.versionTwoInformation.currentVersion) {
        	collectionNames.push(this._messages.collectionOne, this._messages.collectionTwo);
        }
        else if (this.versionOneInformation.baseline.issued < this.versionTwoInformation.baseline.issued) {
        	collectionNames.push(this._messages.collectionOne, this._messages.collectionTwo);
        }
        else {
        	collectionNames.push(this._messages.collectionTwo, this._messages.collectionOne);
        }
        
        return collectionNames;
	},
	
	_createHeaderTable: function() {
		var collectionNames = this._getCollectionNamesForHistory();
		var headerTable = dojo.create("table");
		var th = dojo.create("th", {
			"class": "differencesHeader",
			innerHTML: dojo.string.substitute(this._messages.changes, collectionNames)
		}, headerTable);
		
		return headerTable;
	},
	
	_buildInfoCell: function(dateNode, userNode) {
		var infoCell = dojo.create("td", {
			style: {
				width: "40%"
			},
			innerHTML: this._messages.changedOn + " ", 
			vAlign: "top"
		});
		dojo.place(dateNode, infoCell);
		dojo.create("span", {
			innerHTML: " " + this._messages.by + " "
		}, infoCell);
		dojo.place(userNode, infoCell);
		
		return infoCell;
	}
		
});

})();
