/*
 Licensed Materials - Property of IBM - some unique copyrights (i.e., the ability to copy,  modify,
 distribute) may be licensed in certain circumstances (e.g., sample/template code  intended to be used by
 customers to build upon and distribute as their own products).

 (c) Copyright IBM Corporation 2007, 2015. All Rights Reserved.

 U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP
 Schedule Contract with IBM Corp.
*/
(function () {
dojo.provide("com.ibm.rdm.web.grid.ViewHeader");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rdm.web.artifactgrid.ArtifactGroup");
dojo.require("com.ibm.rdm.web.grid.Module");
dojo.require("com.ibm.rdm.web.grid.List");
dojo.require("com.ibm.rdm.web.grid.GridColumnsDialog");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");

dojo.requireLocalization("com.ibm.rdm.web.project", "DashboardTabPage");


var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
var DomUtils = com.ibm.rdm.web.util.domutils;

var SORT_NONE = '';
var SORT_ASCENDING = 'a';
var SORT_DESCENDING = 'd';


dojo.declare("com.ibm.rdm.web.grid.ViewHeader",
			[dijit._Widget, dijit._Templated], {


	templateString : '<div><div class="viewTopNode" dojoAttachPoint="viewTopNode"> </div><table dojoAttachPoint="viewTableNode"></table><div dojoAttachPoint="viewBottomNode" class="viewBottomNode"> </div></div>',

	rdmModulePath: dojo.moduleUrl("com.ibm.rdm.web"),

	// stored here to aid telling the listeners how wide I am without
	// needing to consult styles, perhaps not required.
	width : 0,
	
	// How wide is the scrollbar - Varies browser to browser.
	scrollBarWidth: 0,

	columnListeners : null,

	addedListeners : null,
	
	dojoConnectList: null,

	rendered : null,
	
	//The checkbox in the header.
	headerCheckbox: null,
	
	//The menu button in the header.
	headerMenuButton: null,
	
	//The grid model
	gridmodel: null,

	// groupingId: String
	//		The ID associated with the currently selected value in the
	//		"Group by" ComboBox (if one is present in grid associated
	//		with this header).
	groupingId: null,

	// showWithFindButton:
	//		If passed as true, you must override the onFind method.
	showWithFindButton: false,


	constructor: function () {
		this.columnListeners = [];
		this.addedListeners = [];
		this.dojoConnectList = new com.ibm.rdm.web.grid.List();
		this.rendered = false;
		this.resizeManager = new com.ibm.rdm.web.grid.ResizeManager();
		this.resizeManager.setOnHeaderCellResized(
			dojo.hitch(this, function(){
				window.setTimeout(
					dojo.hitch(this, function(){
						this._notifyColumnsModified();
					}),
					1
				);
			})
		);
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this._addFindButtonIfNeeded();
		this.scrollBarWidth = window.innerWidth - document.body.clientWidth;
	},

	onFocus: function() {
		// summary:
		//		This is an override of a superclass method.
		// description:
		//		We're overriding this function to make sure that when the user
		//		tabs into the header area the header buttons are shown with no
		//		transparency (as they're slightly transparent if the focus is
		//		either out of the header area or if the user isn't using the
		//		mouse to hover over the header area).

		dojo.addClass(this.domNode, com.ibm.rdm.web.grid.ViewHeader.ClassNames.HEADER_HOVER);
	},
	
	onBlur: function() {
		// summary:
		//		This is an override of a superclass method.
		// description:
		//		We're overriding this function to make sure that when the user
		//		tabs out of the header area the header buttons are shown with
		//		slight transparency (similar to the behavior we use if the user
		//		moves the mouse out of the header area).

		dojo.removeClass(this.domNode, com.ibm.rdm.web.grid.ViewHeader.ClassNames.HEADER_HOVER);
	},
	
	onFind: function(/*DOMNode*/findNode) {
		// summary:
		//		User defined function to handle the case of the user clicking
		//		on the Find button in this widget (note: the Find button will
		//		only be present if you've passed showWithFindButton as true
		//		when constructing this widget).
		// tags:
		//		callback
	},

	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.project", "DashboardTabPage");
	},
	
	setGridmodel: function(gridmodel) {
		this.gridmodel = gridmodel;
	},
	
	showColumnsDialog: function() {
		this._displayGridDialog();
	},

	_handleMouseOverHeader: function () {
		// summary:
		//		Make sure header buttons are shown without any transparency.

		dojo.addClass(this.domNode, com.ibm.rdm.web.grid.ViewHeader.ClassNames.HEADER_HOVER);
	},

	_handleMouseOutHeader: function () {
		// summary:
		//		Make sure header buttons are shown with slight transparency.

		dojo.removeClass(this.domNode, com.ibm.rdm.web.grid.ViewHeader.ClassNames.HEADER_HOVER);
	},

	_displayHeader: function () {
		if (this.rendered) {
			return;
		}

		// make some DOM nodes out of the JSON.
		var primaryRowTypeId = this.gridmodel.getPrimaryRowTypeId();
		var columnIds = this.gridmodel.getColumnIds(primaryRowTypeId);
		
		var size = columnIds.length;
		var row = this.viewTableNode.insertRow(0);
		
		var handle;
		handle = dojo.connect(this.domNode, "onmouseover", this, this._handleMouseOverHeader);
		this.dojoConnectList.addItem(handle);
		handle = dojo.connect(this.domNode, "onmouseleave", this, this._handleMouseOutHeader);
		this.dojoConnectList.addItem(handle);
		
		this.rendered = true;

		if (this.currentWidth) {
			this.resize(this.currentWidth);
		}
		
		var nextWillContainExpando = false,
			lastColumnId = this._getLastColumnId();

		for (var i=0; i<size; i++) {
			var columnId = columnIds[i],
				isLastColumn = columnId === lastColumnId;
			 
			if (columnId === com.ibm.rdm.web.grid.GridModel.COL_EXPANDO) {
				nextWillContainExpando = true;
			} else {
				if (this.gridmodel.isVisible(primaryRowTypeId, columnId)) {
					if (nextWillContainExpando) {
						this.gridmodel.setMergedColumn(columnId, com.ibm.rdm.web.grid.GridModel.COL_EXPANDO);
						nextWillContainExpando = false;
					}
					this._createHeaderCell(row, primaryRowTypeId, columnId, -1, isLastColumn);
				}
			}
		}
		//Removes extra white space due to header cells being the same width as the columns in the actual grid.
		//By creating an extra td to take up the rest of the space.
		var td = dojo.create("td", {
			className: "_lastCol"
		}, row);
		// We add a div to the td element so that the last visible column will appear with a divider
		// showing where this column's right hand side is (note: the "cell border" is on the internal div
		// rather than the outer td element).
		dojo.create("div", null, td);
		for (var i=0; i<this.addedListeners.length; i++) {
			this.addedListeners[i].addedToPage(this);
		}
	},
	
	_createSelectorCol: function(row, rowTypeId) {
		
		//Check whether to display the select box
		var hasSelectionBox = this.gridmodel.hasSelectionBox(rowTypeId);
		
		if (hasSelectionBox) {
			
			var td = row.insertCell(-1);	
			var div = document.createElement("div");
			
			var widthVal = this.gridmodel.getWidth(rowTypeId, com.ibm.rdm.web.grid.GridModel.COL_SELECTOR);
			var width = widthVal+"px";
			
			var style = {
					width : width
			};
			//Keep total right
			this.width += widthVal;

			dojo.style(td, style);
			dojo.addClass(td, com.ibm.rdm.web.grid.GridModel.COL_SELECTOR);
			dojo.style(div, style);
			
			dojo.setSelectable(div, false);

			if (hasSelectionBox) {
				var temp2 = dojo.create("input", {"class": "headerCheckbox"});
				dojo.attr(temp2, "type", "checkbox");
				dojo.attr(temp2, "focusable", true);
				dojo.attr(temp2, "title", this._messages.selectAll);
				this.dojoConnectList.addItem(dojo.connect(temp2, "onclick", this, "onCheck"));
				dojo.place(temp2, div, "only");
				this.headerCheckbox = temp2;
			}
			
			td.appendChild(div);
			td.contentDiv = div;
		}
	},

	_createMenuCol: function(row, rowTypeId) {

		//Check is the menu needs to be shown
		var self=this;
		var showMenu = false;
		var colIds = this.gridmodel.getColumnIds(rowTypeId);
		dojo.forEach(colIds, function (colId) {
			if (!self.gridmodel.isAlwaysVisible(rowTypeId, colId)) {
				showMenu = true;
			}
		});
		
		var td = row.insertCell(-1);	
		var div = document.createElement("div");
			
		var widthVal = this.gridmodel.getWidth(rowTypeId, com.ibm.rdm.web.grid.GridModel.COL_MENU);
		var width = widthVal+"px";
		
		var style = {
				width : width
		};
		//Keep total right
		this.width += widthVal;

		dojo.style(td, style);
		dojo.addClass(td, com.ibm.rdm.web.grid.GridModel.COL_MENU);
		dojo.style(div, style);
		
		dojo.setSelectable(div, false);

		if (showMenu) {
			var temp2 = document.createElement('img');
			dojo.attr(temp2, "role", "button");
			dojo.attr(temp2, "tabindex", "0");
			dojo.attr(temp2, "src", this._blankGif);
			dojo.attr(temp2, "focusable", "true");
			dojo.attr(temp2, "menuType", "2");
			dojo.addClass(temp2, "menuLauncher");
			dojo.attr(temp2, "title", this._messages["configurePageSettings"]);
			this.dojoConnectList.addItem(dojo.connect(temp2, "onclick", this, "_handleClick"));
			this.dojoConnectList.addItem(dojo.connect(temp2, "onkeydown", this, "_handleKeyPress"));
			dojo.place(temp2, div, "last");
			
			var handle;
			handle = dojo.connect(temp2, "onmouseover", function() {
				dojo.addClass(td, com.ibm.rdm.web.grid.ViewHeader.ClassNames.MENU_LAUNCHER_HOVER);
			});
			this.dojoConnectList.addItem(handle);

			handle = dojo.connect(temp2, "onmouseleave", function() {
				dojo.removeClass(td, com.ibm.rdm.web.grid.ViewHeader.ClassNames.MENU_LAUNCHER_HOVER);
			});
			this.dojoConnectList.addItem(handle);
			
			//Accessibility Change to allow image to appear wrt Focus
			handle = dojo.connect(temp2, "onfocus", function() {
				dojo.addClass(td, com.ibm.rdm.web.grid.ViewHeader.ClassNames.MENU_LAUNCHER_HOVER);
			});
			this.dojoConnectList.addItem(handle);

			handle = dojo.connect(temp2, "onblur", function() {
				dojo.removeClass(td, com.ibm.rdm.web.grid.ViewHeader.ClassNames.MENU_LAUNCHER_HOVER);
			});
			this.dojoConnectList.addItem(handle);
		}
		
		this.headerMenuButton = td;
		
		td.appendChild(div);
		td.contentDiv = div;
	},

	_setGroupingIdAttr: function(groupingId) {
		this.groupingId = groupingId;
	},

	_createHeaderCell: function (row, primaryRowTypeId, columnId, position, isLastColumn) {
		if (columnId === com.ibm.rdm.web.grid.GridModel.COL_MENU) {
			this._createMenuCol(row, primaryRowTypeId);
			return;
		} else if (columnId === com.ibm.rdm.web.grid.GridModel.COL_SELECTOR) {
			this._createSelectorCol(row, primaryRowTypeId);
			return;
		}
		
		var td = row.insertCell(position);
		
		//Accessibility Change to allow Grid to be read usefully.
		dojo.attr(this.viewTableNode, "role", "grid");
		dojo.attr(row, "role", "row");
		dojo.attr(td, "role", "columnheader");
		
		var div = document.createElement("div");
			
		var width = this.gridmodel.getWidth(primaryRowTypeId, columnId);

		var style = {
				width : width+"px"
		};
		dojo.style(td, style);

		/* This keeps the column at the width specified by the server
		 * in the instance where the column is too narrow to fit the
		 * column name in.
		 */
		dojo.style(div, style);
		
		dojo.setSelectable(div, false);
		dojo.attr(td, "column", columnId);
		var columnName =  this.gridmodel.getName(primaryRowTypeId, columnId);
		// For reference by aria-describedby attributes on cells, provide an ID for the element
		// that contains the column name.  Record it in the grid model.
		var nameId = this.id + "-" + columnId;
		this.gridmodel.registerColumnNameId(primaryRowTypeId, columnId, nameId);
		var colNameNode = dojo.create("span", {innerHTML: columnName, id: nameId}, div, "first");
				
		if (this.gridmodel.isSortable(primaryRowTypeId, columnId)) {
			this._createSortableHeaderCell(td, div, columnName, columnId);
		} else {
			//Add the title for columns that need it (not ones for sort, as they already have it)
			if (this.gridmodel.isShowFullTitleOnHover(primaryRowTypeId, columnId)) {
				dojo.attr(colNameNode, "title", columnName);
			}
		}
		
		td.appendChild(div);
		if (this.gridmodel.isColumnHeaderHideable(primaryRowTypeId, columnId)) {
			dojo.addClass(td, "hideableHeader");
			this._setColumnHeaderTextVisibility(colNameNode, primaryRowTypeId, columnId, width);
		}

		// add references to some inner nodes to be used when resizing
		td.contentDiv = div;
		td.nameSpan = colNameNode;

		this.width += parseInt(width);
		if (!this.gridmodel.isFixedWidth(primaryRowTypeId, columnId)) {
			this._resizeEnable(td);
		}
		
		this._invokeColumnListeners(primaryRowTypeId, columnId, width, this.width, isLastColumn);
	},

	isHeaderCellSortingEnabled: function(columnId) {
		// summary:
		//		Subclasses may wish to override this (perhaps by also consulting
		//		this class's current value for the groupingId field).
		return true;
	},

	_createSortableHeaderCell: function(td, div, columnName, columnId) {
		// summary:
		//		Create a sortable header cell.
		// td:
		//		The TableData element that represents the cell in the table.
		// div:
		//		The div that's found within this the td element.
		// columnName:
		//		The name of the column that this header cell represents.
		// columnId:
		//		The ID that we refer to this header cell by within the
		//		gridmodel.
		// tags:
		//		protected

		dijit.setWaiState(td, "sort", "none");
		dojo.addClass(td, "sortableHeader");
		dojo.style(div, "cursor", com.ibm.rdm.web.grid.Module.HAND_CURSOR);
		dojo.attr(td, "title", columnName);
		var handle = dojo.connect(div, "onclick", this, function(event){
			if (this.isHeaderCellSortingEnabled(columnId)) {
				this._handleSortClick(event);
			} else {
				dojo.stopEvent(event);
			}
		});
		this.dojoConnectList.addItem(handle);
		handle = dojo.connect(div, "onkeydown", this, function(event){
			if (this.isHeaderCellSortingEnabled(this.groupingId)) {
				this._handleSortKeyPress(event);
			} else {
				dojo.stopEvent(event);
			}
		});
		this.dojoConnectList.addItem(handle);
		
		//Accessibility Change to allow user to select sorting function, and hear description.
		dojo.attr(div, "tabindex", "0");
		dojo.attr(div, "role", "button");

		var sortImg = document.createElement("img");
		dojo.attr(sortImg, "src", this._blankGif);
		dojo.place(sortImg, div, "last");
	},

	_setHeaderCellSortEnablement: function(td, setAsEnabled) {
		var div = td.firstChild,
			sortImg = div ? div.lastChild : null,
			isCurrentlyEnabled = (
				sortImg ?
				dojo.style(sortImg, "display") !== "none" :
				false
			);

		// Only proceed if the header cell represented by the 'td' is really
		// a sortable header cell ...
		if (sortImg) {
			if (setAsEnabled && !isCurrentlyEnabled) {
				// Change the enablement state from not enabled to enabled.
				var sortValue = "none";
				if (dojo.hasClass(td, "descSort")) {
					sortValue = "descending";
				}
				else if (dojo.hasClass(td, "ascSort")) {
					sortValue = "ascending";
				}
				dijit.setWaiState(td, "sort", sortValue);
				dojo.addClass(td, "sortableHeader");
				dojo.style(div, "cursor", com.ibm.rdm.web.grid.Module.HAND_CURSOR);
				dojo.attr(div, "tabindex", "0");
				dojo.attr(div, "role", "button");
				dojo.style(sortImg, "display", "");
			}
			else if (!setAsEnabled && isCurrentlyEnabled) {
				// Change the enablement state from enabled to not enabled.
				dojo.removeAttr(td, "aria-sort");
				dojo.removeClass(td, "sortableHeader");
				dojo.style(div, "cursor", "");
				dojo.removeAttr(div, "tabindex");
				dojo.removeAttr(div, "role");
				dojo.style(sortImg, "display", "none");
			}
		}
	},

	_createShowColumnsGroup: function(menuOperations) {
		// summary:
		//		Returns a group object with properties that are appropriate for
		//		a group found within the 'layout' parameter in a call to the
		//		EditMenu constructor.  In addition, this function also adds
		//		appropriate new properties to the passed 'menuOperations'.

		var primaryRowTypeId = this.gridmodel.getPrimaryRowTypeId();
		var cols = this.gridmodel.getColumnIds(primaryRowTypeId);
		
		var columnsToShowGroup = {
			groupName: "Columns",
			separatorAbove: true,
			separatorLabel: this._messages.showColumns,
			isShown: false,
			operations: []
		};

		var self=this;
		
		var limit = 23;
		dojo.forEach(cols, function (colId) {
			if (limit-- > 0) {
				//Build the layout details
				var layoutEntry = {id: colId, separatorAbove: false};
				columnsToShowGroup.operations.push(layoutEntry);
				
				var isPrivate = self.gridmodel.isPrivate(primaryRowTypeId, colId);

				if (!isPrivate) {
					var isVisible = self.gridmodel.isVisible(primaryRowTypeId, colId);
					var isAlwaysVisible = self.gridmodel.isAlwaysVisible(primaryRowTypeId, colId);
					var name = self.gridmodel.getName(primaryRowTypeId, colId);
					
					//Build the operation
					//Params: _id, _iconClass, _displayText or method, _isEnabledMethod, _executeMethod
					var operation = new com.ibm.rdm.web.grid.Operation(colId, (isVisible)?"ticked":"unticked", 
							name, 
							function(rowIds) {return !isAlwaysVisible;},
							(isVisible) ? function() {self.hideCol(colId);} : function() {self.showCol(colId);}
							);
					menuOperations[colId] = operation;
				}
			}
		});

		return columnsToShowGroup;
	},

	_createColumnsToDisplayGroup: function(menuOperations) {
		// summary:
		//		Returns a group object with properties that are appropriate for
		//		a group found within the 'layout' parameter in a call to the
		//		EditMenu constructor.  In addition, this function also adds
		//		appropriate new properties to the passed 'menuOperations'.

		var self = this,
			colId = ".ConfigureColumnsGroupOperation",
			operation = new com.ibm.rdm.web.grid.Operation(
				colId,
				"configureColumnsToDisplayIcon",
				this._messages.configureColumnsToDisplay,
				function() {return true;},
				function() {self._displayGridDialog();}
			);
		menuOperations[colId] = operation;
		return {
			groupName:"ConfigureColumnsGroup",
			separatorAbove: false,
			isShown: false,
			operations: [{
				id: colId,
				separatorAbove: false
			}]
		};
	},

	_createMoreColumnGroup: function(menuOperations) {
		// summary:
		//		Create the "More..." item that appears at the bottom of the menu.
		// description:
		//		Returns a group object with properties that are appropriate for
		//		a group found within the 'layout' parameter in a call to the
		//		EditMenu constructor.  In addition, this function also adds
		//		appropriate new properties to the passed 'menuOperations'.

		var self = this,
			colId = ".MoreAttributesAndRelationsGroupOperation",
			operation = new com.ibm.rdm.web.grid.Operation(
				colId,
				"",
				this._messages.more,
				function() {return true;},
				function() {self._displayGridDialog();}
			);
		menuOperations[colId] = operation;
		return {
			groupName:"MoreAttributesAndRelations",
			separatorAbove: true,
			isShown: false,
			operations: [{
				id: colId,
				separatorAbove: false
			}]
		};
	},
	
	_handleKeyPress: function (event, node) {
		//If user hits enter or space, sort
		if(event.keyCode === dojo.keys.ENTER || event.charCode === dojo.keys.SPACE || event.which === dojo.keys.SPACE){
			node = node || event.target;
			//Needed for IE8 (kept trying to close the EditMenu right away)
			dojo.stopEvent(event);
			this._handleClick(null, node);
		}
	},
	
	_handleClick: function (event, node) {
		// summary:
		//		Generic event handler determines whether
		//		a node representing an object has been clicked
		//		(i.e. row selection) or a menu icon has been clicked.
		//		dojo.connect'ed to onClick on the parent div.

		//If _handleKeyPress triggered this call, node has already been defined.
		if(event){
			node = node || event.target;
		}

		var menuType = dojo.attr(node, com.ibm.rdm.web.grid.Module.AttributeNames.MENU_TYPE);
		var that = this;
		if (menuType) {
			var menuOperations = {},
				firstGroups = this.gridmodel.getFirstEditMenuGroups(menuOperations),
				menuLayout = [];
			// First add any custom user defined groups to menuLayout.
			if (firstGroups && firstGroups.length > 0) {
				menuLayout = menuLayout.concat(firstGroups);
			}
			// Now add any groups that this widget manages to menuLayout.
			menuLayout = menuLayout.concat([
				this._createColumnsToDisplayGroup(menuOperations),
				this._createShowColumnsGroup(menuOperations),
				this._createMoreColumnGroup(menuOperations)
			]);

			var editMenu = new com.ibm.rdm.web.grid.EditMenu(
				{
					launchGesture: event,
					node: node,
					menuType: menuType,
					module: that
				},
				menuOperations,
				menuLayout,
				"column-display-menu",
				16,
				16
			);
			
			dojo.addClass(editMenu.domNode, "view-header-editmenu");
			
			editMenu.show({ x: this.domNode.parentNode.scrollLeft,
							y: this.domNode.parentNode.scrollTop });
			return;
		}
	},
	
	_displayGridDialog: function() {
	
		var dialogFunction = this.gridmodel.columnSelectionDialogFunction();
		
		if (!dialogFunction) {
			//Use the default
			this._gridDialog = new com.ibm.rdm.web.grid.GridColumnsDialog();
		} else {
			//Use the custom dialog
			this._gridDialog = dialogFunction();
		}
		
		var griddata = [];
		//Build the column data for the grid dialog
		var primaryRowTypeId = this.gridmodel.getPrimaryRowTypeId();
		var cols = this.gridmodel.getColumnIds(primaryRowTypeId);
		var self = this;
		
		dojo.forEach(cols, function (colId) {
			//Build the layout details
			var isPrivate = self.gridmodel.isPrivate(primaryRowTypeId, colId);
			if (!isPrivate) {
				var isVisible = self.gridmodel.isVisible(primaryRowTypeId, colId);
				var isAlwaysVisible = self.gridmodel.isAlwaysVisible(primaryRowTypeId, colId);
				
				var name = self.gridmodel.getName(primaryRowTypeId, colId);
				var width = self.gridmodel.getWidth(primaryRowTypeId, colId);
				var isFixedWidth = self.gridmodel.isFixedWidth(primaryRowTypeId, colId);
				
				var entry = {	columnId: colId, 
								name: name, 
								visible: {checked: isVisible, disabled: isAlwaysVisible}, 
								width: {value: width, disabled: isFixedWidth}	};
				
				var customData = self.gridmodel.getCustomData(primaryRowTypeId, colId);
				if (customData) {
					entry.custom = customData;
				}
				//Build the entry
				griddata.push(entry); 
			}
		});
		var self = this;
		this._gridDialog.show(griddata, dojo.hitch(this, function(griddata, isModified) {
			self.reapplyGridDialogData(griddata);
			if (isModified) {
				this._notifyColumnsModified();
			}
		}));
	},
	
	reapplyGridDialogData: function(griddata) {
		//Check if any col was resized manually
		var manualResize = false;
		dojo.forEach(griddata, function(item) {
			if (item.width.set) {
				manualResize = true;
			}
		});
		if (manualResize) {
			this.resizeManager.hasManuallyResizedColumns = true;
		}
		this._gridDialog.destroyRecursive();
	},
	
	setSortCol: function(columnId, order) {
		// summary:
		//		Sets the sort order displayed.
		// columnNo:
		//		the column concerned
		// order:
		//		'a', 'd', or '' (ascending, descending, or remove all)
		
		//Remove existing orders
		var that = this,
			primaryRowTypeId = this.gridmodel.getPrimaryRowTypeId(),
			getTextContent = function (item) {
				 var columnId = dojo.attr(item, "column");
				 return that.gridmodel.getName(primaryRowTypeId, columnId);
			};
			
		dojo.query(".descSort", this.viewTableNode).forEach(function (item) {
			dojo.removeClass(item, "descSort");
			dojo.attr(item, "title",  getTextContent(item)+" - "+that._messages["clickToOrder"]);	
		});
		dojo.query(".ascSort", this.viewTableNode).forEach(function (item) {
			dojo.removeClass(item, "ascSort");
			dojo.attr(item, "title",  getTextContent(item)+" - "+that._messages["clickToOrder"]);
		});
					
		if (order===SORT_ASCENDING) {
			dojo.query("[column$=\""+columnId+"\"]", this.viewTableNode).forEach(function (item) { 
				dojo.addClass(item, "ascSort");
				dojo.attr(item, "title", getTextContent(item)+" - "+that._messages["orderAsc"]);
			});
		} else if (order === SORT_DESCENDING) {
			dojo.query("[column$=\""+columnId+"\"]", this.viewTableNode).forEach(function (item) { 
				dojo.addClass(item, "descSort");
				dojo.attr(item, "title", getTextContent(item)+" - "+that._messages["orderDesc"]);
			});	
		}
			
		
	},
	
	getSortCol: function() {
		// summary:
		//		Gets the sort order displayed.
		// returns:
		//		{columnId: "", order: 'a'}
		//		where the columnId is the column concerned
		//		and order is 'a', 'd', or '' (ascending, descending, or remove all)

		var columnId, order;
		dojo.query(".descSort", this.viewTableNode).forEach(function (item) {
			var colNode = DomUtils.findAncestorByAttr(item, 'column');
			columnId = dojo.attr(colNode, 'column');
			order = SORT_DESCENDING;
		});
		dojo.query(".ascSort", this.viewTableNode).forEach(function (item) {
			var colNode = DomUtils.findAncestorByAttr(item, 'column');
			columnId = dojo.attr(colNode, 'column');
			order = SORT_ASCENDING;
		});
		
		if (columnId) {
			return {columnId: columnId, order: order};
		}
		
	},
	
	showCol: function(columnId) {
	},

	hideCol: function(columnId) {
	},
	
	removeCol: function (columnId) {
		//Make the right col show the right order.
		dojo.query("[column$=\""+columnId+"\"]", this.viewTableNode).orphan();
	},
	
	addCol: function (rowType, columnId, position) {
		var row = this.viewTableNode.rows[0],
			lastColumnId = this._getLastColumnId();
		this._createHeaderCell(row, rowType, columnId, position, columnId === lastColumnId);
	},
	
	reapplyCols: function() {
		//Clear current cols
		if (this.viewTableNode.rows && this.viewTableNode.rows.length > 0){
			this.viewTableNode.deleteRow(0);
		}
		
		this.rendered = false;
		if (!this.gridmodel.hideHeader()) {
			this._displayHeader();
		}
		this.resizeManager.hasManuallyResizedColumns = false;
	},
	
	uninitialize: function(){
		this._clearHandles();
	},
	
	_clearHandles: function () {
		//Stop listening for events.
		this.dojoConnectList.forEach(function f(handle){
				dojo.disconnect(handle);
		});
		this.dojoConnectList.clear();
	},
	
	_handleSortKeyPress: function(event) {
		//If user hits enter or space, sort
		if(event.keyCode === dojo.keys.ENTER || event.charCode === dojo.keys.SPACE || event.which === dojo.keys.SPACE){
			this._handleSortClick(event);
		}
	},
	
	_handleSortClick: function (event) {
		// summary:
		//		Handles the click on a sortable column.
				
		var clickedColNode = DomUtils.findAncestorByAttr(event.target, 'column');
		var clickedColId = dojo.attr(clickedColNode, 'column');
		
		var currentSort = this.getSortCol();
		
		var order = SORT_ASCENDING;
		dijit.setWaiState(clickedColNode, "sort", "ascending");
		if (this.gridmodel.isDefaultSortOrderDescending(clickedColId)) {
			order = SORT_DESCENDING;
			dijit.setWaiState(clickedColNode, "sort", "descending");
		}
			
		if (currentSort && currentSort.columnId === clickedColId) {
			if (this.gridmodel.isDefaultSortOrderDescending(clickedColId)) {
				if (currentSort.order === SORT_DESCENDING) {
					order = SORT_ASCENDING;
					dijit.setWaiState(clickedColNode, "sort", "ascending");
				} else {
					//Must be SORT_ASCENDING, so go to no order.
					order = SORT_NONE;
					dijit.setWaiState(clickedColNode, "sort", "none");
				}
			} else {
				if (currentSort.order === SORT_ASCENDING) {
					order = SORT_DESCENDING;
					dijit.setWaiState(clickedColNode, "sort", "descending");
				} else {
					//Must be SORT_DESCENDING, so go to no order.
					order = SORT_NONE;
					dijit.setWaiState(clickedColNode, "sort", "none");
				}
			}
		}
		
		this.setSortCol(clickedColId, order);
		this.sortBy(clickedColId, order);

		this._notifyColumnsModified();
	},

	_getLastColumnId: function() {
		var primaryRowTypeId = this.gridmodel.getPrimaryRowTypeId(),
			columnIds = this.gridmodel.getColumnIds(primaryRowTypeId),
			i;

		for (i = columnIds.length-1; i >= 0; i--) {
			var columnId = columnIds[i];
			if (this.gridmodel.isVisible(primaryRowTypeId, columnId)) {
				return columnId;
			}
		}

		return null;
	},
	
	onCheck: function(event) {
		if (event && event.target && event.target.nodeName === "INPUT") {
			this.selectAll(event.target.checked);
		}
	},
	
	setHeaderCheckbox: function(/*boolean*/state) {
		// summary:
		//		Simply checks or unchecks the header checkbox, with no sideeffects.
		//		Intended to be called when the selection is changed to or from ALL.
		// state:
		//		true to set the checkbox, and false to clear it. 
		if (this.headerCheckbox) {
			this.headerCheckbox.checked = state;
		}
	},

	render: function() {
		if (!this.gridmodel.hideHeader()) {
			this._displayHeader();
		}
		this.rendered = true;
	},
	
	resize: function(width) {
		if (this._previousWidth && this._previousWidth === width && this._gridModelVersion && this.gridmodel.getModelVersion() === this._gridModelVersion) { return; }
		this.currentWidth = width;

		//This is added to each col by css - but not taken account of in the resize.
		var colPadding = com.ibm.rdm.web.grid.ViewHeader.COLUMN_PADDING;
		width -= this.scrollBarWidth;
		
		if (this.rendered) {
			this._previousWidth = width;

			var lastColumnId = this._getLastColumnId();

			//Resize All RowTypes
			var rowTypes = this.gridmodel.getRowTypeIds();

			dojo.forEach(rowTypes, function (rowTypeId) {
				//Find visible non-fixed width cols

				var columnIds = this.gridmodel.getColumnIds(rowTypeId);
				var primaryRowId = this.gridmodel.getPrimaryRowTypeId();
				var isPrimaryRow = (rowTypeId == primaryRowId);
				
				var fixedTotal = 0;
				//Extra width is provided on same proportion as current widths
				var varColTotal = 0.0;
				var size = columnIds.length;
				var colWidth = null;
				var totalColumns = 0;

				for (var i=0; i<size; i++) {
					var columnId = columnIds[i];
					if (this.gridmodel.isVisible(rowTypeId, columnId)) {
						colWidth = this.gridmodel.getWidth(rowTypeId, columnId);
						fixedTotal = fixedTotal + colWidth + colPadding;
						this._actionResizeCommand(
							this.resizeManager,
							dojo.query("[column="+columnId+"]", this.viewTableNode)[0],
							rowTypeId,
							columnId,
							colWidth,
							0,
							this.columnListeners,
							columnId === lastColumnId
						);
						totalColumns++;
					}
				}
			}, this);
			
			//Clear manually set rows
			dojo.forEach(rowTypes, function (rowTypeId) {
				var columnIds = this.gridmodel.getColumnIds(rowTypeId);
				var size = columnIds.length;
				
				for (var i=0; i<size; i++) {
					if(this.gridmodel.isManuallySet(rowTypeId, columnIds[i])) {
						this.gridmodel.clearManuallySet(rowTypeId, columnIds[i]);
					}
				}
			}, this);			
			
			//Remember the final version - as setWidth changes it.
			this._gridModelVersion = this.gridmodel.getModelVersion();
		}
	},

	/*
	 * Called to remove whatever content has been
	 * added. This should leave the widget in the same
	 * state as if "render" was never called.
	 */
	clear: function() {
		if (this.viewTableNode && this.viewTableNode.rows.length > 0) {
			this.viewTableNode.deleteRow(0);
		}
		this.rendered = false;
		this.width = 0;
	},

	addColumnListener: function (listener) {
	
		if ((typeof listener.setColumnWidth !== 'function')) {
			console.warn("Tried to add a listener without the correct 'interface' ", listener);
		}

		this.columnListeners.push(listener);

	},

	/*
	 * Add a listener who will be notified when the header widget
	 * draws itself on the page for the first time.
	 */
	addRenderedListener: function (pl) {

		if (typeof pl.addedToPage !== 'function') {
			console.warn("Tried to add a listener without the correct 'interface' ", pl);
		}

		this.addedListeners.push(pl);
	},


	onDocumentName: function (docName) {
	},

	_resizeEnable: function(elem) {
		// function argument is called when the element resize
		// is complete
		// args are
		// elem - the element resized
		// newWidth - the new width of the element
		// diff - the difference in size pre and post resize

		this.resizeManager.enable(elem, this.domNode.parentNode, dojo.hitch(this, function(elem, newWidth, diff) {

			if (!elem || !newWidth || (!diff && (diff !== 0))) {
				console.error("Cannot resize col", elem, newWidth, diff);
			} else {
				var colId = dojo.attr(elem, "column"),
					primaryRowTypeId = this.gridmodel.getPrimaryRowTypeId(),
					theResizeManager = this.resizeManager,
					lastColumnId = this._getLastColumnId();
				this._actionResizeCommand(
					theResizeManager, elem, primaryRowTypeId,
					colId, newWidth, diff, this.columnListeners,
					colId === lastColumnId
				);
				//Deal with any child rows
				var childRowIds = this.gridmodel.getChildRowTypeIds(primaryRowTypeId);

				dojo.forEach(childRowIds, dojo.hitch(this, function(childRowTypeId) {
					this._actionResizeCommand(
						theResizeManager, null, childRowTypeId,
						colId, newWidth, diff, this.columnListeners,
						colId === lastColumnId
					);
				}));
			}
		}), this.onColumnResize);
	},
	
	onColumnResize: function (eventX) {
		// summary:
		//		Called when a column is resized - this fires
		//		as the user drags the column header.
	},
	
	_actionResizeCommand: function(theResizeManager, headerColElem, rowTypeId, colId, newWidth, diff, columnListeners, isLastColumn) {
		//console.log("VH _actionResizeCommand", theResizeManager, headerColElem, rowTypeId, colId, newWidth, diff, columnListeners);
		if (!theResizeManager || !rowTypeId || !colId || !newWidth || !columnListeners) {
			console.error("Cannot resize col", theResizeManager, rowTypeId, colId, newWidth, diff, columnListeners);
		} else {
	
			theResizeManager.containerWidth += diff;
			if (headerColElem) {
				//Non primary rows won't have headerElems
				dojo.style(headerColElem, "width", newWidth + "px");
				dojo.style(headerColElem.contentDiv, "width", newWidth + "px");
				
				
				//To satisfy WI 54666 - comment and used in should not show header
				//text if the column is too small.
				//Applied to all for consistency
				if (this.gridmodel.isColumnHeaderHideable(rowTypeId, colId)) {
					this._setColumnHeaderTextVisibility(headerColElem.contentDiv.firstChild, rowTypeId, colId, newWidth);
				}
			}

			this._invokeColumnListeners(rowTypeId, colId, newWidth, theResizeManager.containerWidth, isLastColumn);
		}
	},

	_invokeColumnListeners: function(rowTypeId, columnId, newWidth, containerWidth, isLastColumn) {
		var primaryRowTypeId = this.gridmodel.getPrimaryRowTypeId();
		var getMeasurements = dojo.hitch(this, function(){
			var rowContentWidth = 0,
				headerContainer = this.domNode.parentNode.parentNode,
				width = dojo.position(headerContainer).w - this.scrollBarWidth,
				columnIds = this.gridmodel.getColumnIds(primaryRowTypeId),
				i;

			// This is added to the left side and right side for each column by css.
			var colPadding = com.ibm.rdm.web.grid.ViewHeader.COLUMN_PADDING;

			for (i = 0; i < columnIds.length; i++) {
				var columnId = columnIds[i];
				if (this.gridmodel.isVisible(primaryRowTypeId, columnId)) {
					var columnWidth = this.gridmodel.getWidth(primaryRowTypeId, columnId);
					rowContentWidth += colPadding + columnWidth + colPadding;
				}
			}

			return {
				gapOnRightInPixels: Math.max(width - rowContentWidth, 0),
				rowContentWidth: rowContentWidth
			};				
		});

		var invokeColumnListeners = dojo.hitch(this, function(rowTypeId, columnId, newWidth, isLastColumn) {
			var measurements = getMeasurements(),
				gapOnRightInPixels = measurements.gapOnRightInPixels,
				rowContentWidth = measurements.rowContentWidth,
				i;
			for (i = 0; i < this.columnListeners.length; i++) {
				this.columnListeners[i].setColumnWidth(
					rowTypeId,
					columnId,
					newWidth,
					rowContentWidth,
					isLastColumn ? gapOnRightInPixels : 0
				);
			}
		});

		invokeColumnListeners(rowTypeId, columnId, newWidth, isLastColumn);

		// If this isn't the last column, we want the last column to appropriately
		// "bleed into" the gap that might now be appearing on the right hand side
		// (i.e. the width between the right hand side of the last visible column
		// and the right hand side of the grid itself).
		if (!isLastColumn) {
			var lastColumnId = this._getLastColumnId(),
				width = this.gridmodel.getWidth(primaryRowTypeId, lastColumnId, false);
			invokeColumnListeners(primaryRowTypeId, lastColumnId, width, true);
		}
	},
	
	_setColumnHeaderTextVisibility: function(headerTextElem, rowTypeId, colId, width) {
		// summary:
		//		Set the text in a column header to be visible or not, depending
		//		on the width of the column.
		// headerTextElem:
		//		Element (span) containing the header text.
		// rowTypeId:
		//		row type that colum headers are derived from.
		// colId:
		//		column identifier
		// width:
		//		width that the column is being set to.
		
		//To satisfy WI 54666 and 59824 - comment and used in columns should not show header
		//text if the column is too small.
		//Applied to all for consistency.
		
		// make sure element is displayed so we can get a proper size. 
		dojo.style(headerTextElem, "display", "");

		var position = dojo.position(headerTextElem);
		if (position.w > width) {
			// Text is too wide for the column.  Remove it from display.
			// Toggle "display" rather than "visibility" so that if the text is not showing it does not push the
			// header larger in the vertical direction, and so that the image indicating column sort is in a
			// sensible place.
			dojo.style(headerTextElem,  "display", "none");
		}
	},
	
	selectAll: function(on) {
	},
	
	sortBy: function(columnId, order) {
	},

	_addFindButtonIfNeeded: function() {
		if (this.showWithFindButton) {
			var findButton = new jazz.ui.toolbar.Button({
				title: this._messages.findGoTo,
				imgClass: "findIcon",
				onClick: dojo.hitch(this, function(){
					this.onFind(findButton.domNode);
				})
			});
			dojo.addClass(findButton.domNode, "findButton");
			dojo.place(findButton.domNode, this.viewTableNode, "after");
		}
	},

	_notifyColumnsModified: function() {
		ChangeNotifier.notifyListeners({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.GRID_COLUMNS_MODIFIED
		});
	}
});

/*
 * Object to handle resizing of the column headers.
 * Defines a number of functions which are added to
 * the element and the supplied container as as required.
 * Both the element that can be resized and it's container
 * are given references to the instance of ResizeManager
 * when enable is called.
 */

// NOTE : - I'm not dojo.declare()'ing this "class" in it's own file since
// it only works with the ViewHeader widget at the moment.
com.ibm.rdm.web.grid.ResizeManager = function () {};


com.ibm.rdm.web.grid.ResizeManager.prototype = {
	
	// hasManuallyResizedColumns: Boolean
	//		This is true if the columns have been resized by dragging the edge of a column.
	hasManuallyResizedColumns: false,

	// onHeaderCellResized: Function
	//		If defined, this function will be called back whenever a column's width
	//		is changed (by dragging its right hand side).
	onHeaderCellResized: null,

	setOnHeaderCellResized: function(onHeaderCellResized) {
		// summary:
		//		Since this 'ResizeManager' isn't really a class we're using this
		//		approach to have the consumer be informed of header resize events.

		this.onHeaderCellResized = onHeaderCellResized;
	},

	/*
	 * Make "elem" resizable, container is the DOM node
	 * that contains elem and should be widened as elem
	 * is widened.  Callback is a function invoked whenever
	 * "elem" is resized.
	 */
	enable: function (elem, container, callback, onResize) {
		 // store a reference to the elements container,
		 // this could be gotten using .parent I suppose.
		 this.container = container;
		 this.elem = elem;

		 this.containerWidth = dojo.style(this.container, "width");
		 elem.onselectstart = function() {return false;};

		 elem.onmousemove = this.detectMouseOverBorder;
		 elem.resizeManager = this;
		 container.resizeManager = this;
		 elem.onResizeComplete = callback;
		 this.onResize = onResize;

	},

	/*
	 * Get the location of "elem" on the page.
	 */
	locate: function(elem) {
		 var p = dojo.position(elem, false);
		 return p;
	},

	 /* Styles the node and container with new width */
	resize: function(evt) {
		// extremely annoying but IE doesn't seem to get the parameter "evt"
		// hence below, must refer to window.event.
		// change it to "evt" and you get the message clientX is null or not
		// an object...
		evt = evt || window.event;
		var diff = ((evt.clientX || evt.pageX) - this.dragStartX);

		var newContainerWidth = this.containerWidth + diff;

		dojo.style(this.elem, "width", (this.originalWidth + diff)+"px");
		dojo.style(this.elem.contentDiv, "width", (this.originalWidth + diff)+"px");
		// make column title text visible during resize drag
		var nameSpan = this.elem.nameSpan;
		if (nameSpan) {
			dojo.style(this.elem.nameSpan, "display", "");
		}
		
		this.onResize((evt.clientX || evt.pageX));
	},

	/*
	 * The function added as a mouse move listener, detects when the
	 * mouse is over the right hand border of the element.
	 */
	detectMouseOverBorder: function(evt) {
		evt = evt || window.event;
		// work out when the mouse is directly over the right hand border...

		var coords = this.resizeManager.locate(this);

		var borderRight = coords.x + dojo.style(this, "width");
		var clientX = (evt.clientX || evt.pageX);
		// this defines a width of a few pixels to make
		// border less tricky to grab...

		// Check if this is a sortable column.
		var isSortableColumn = dojo.hasClass(this, "sortableHeader");

		if (clientX >= borderRight-8 && clientX <= borderRight+8) {
			this.style.cursor = "e-resize";
			if (isSortableColumn) {
				// If the node is a sortable column, then it has a child node
				// that also has a cursor style that needs to be set.
				dojo.style(this.firstChild, "cursor", "e-resize");
			}
			this.onmousedown = this.resizeManager.borderGrabbed;
		} else {
			this.style.cursor = "default";
			if (isSortableColumn) {
				// Use the cursor associated with a sortable column.				
				dojo.style(this.firstChild, "cursor", com.ibm.rdm.web.grid.Module.HAND_CURSOR);				
			}
			this.onmousedown = null;
		}
	},

	/*
	 * Function invoked when the used presses the mouse
	 * button over the right hand border of the element.
	 */
	borderGrabbed: function(evt) {
		evt = evt || window.event;
		this.resizeManager.elem = this;

		this.resizeManager.dragStartX = (evt.clientX || evt.pageX);

		this.resizeManager.originalWidth = dojo.style(this, "width");
		var that = this;
		dojo.addClass(this.resizeManager.container, "resizing");
		
		this.resizeManager.container.onmousemove = function (evt) {
			that.resizeManager.resize(evt);
		};
		
		// this event needs to be set on an element outwith the
		// one that triggered the resize event.
		//this.resizeManager.container.onmouseup = this.resizeManager.borderReleased;
		var con = dojo.connect(document.body, "onmouseup", this.resizeManager, function (evt) {
			dojo.disconnect(con);
			if (this.onHeaderCellResized) {
				this.onHeaderCellResized();
			}
			this.borderReleased(evt);
		});
		
		this.resizeManager.hasManuallyResizedColumns = true;
	},

	/*
	 * Function invoked when the used releases the mouse
	 * button after grabbing the right hand border of the
	 * element.
	 */
	borderReleased: function(evt) {
		evt = evt || window.event;

		// this is a bit nasty, the current function is defined
		// on the document so "this" refers to the document object.
		// "this" therefore has no knowledge of the element that started the drag.
		// TODO : investigate a nicer way to do this..
		dojo.removeClass(this.container, "resizing");
		var dragProps = this;

		var dragAmnt = ((evt.clientX || evt.pageX) - dragProps.dragStartX);

		// reset event listeners...

		dragProps.elem.onmousedown = null;
		this.onmouseup = null;
		this.onmousemove = null;
		this.container.onmousemove = null;
		dragProps.elem.style.cursor = "default";

		var newWidth = Math.floor(dragProps.originalWidth + dragAmnt);

		dragProps.elem.onResizeComplete(dragProps.elem, newWidth, dragAmnt);
	}
};
})();

com.ibm.rdm.web.grid.ViewHeader.ClassNames = {
	HEADER_HOVER  : "headerHover",
	MENU_LAUNCHER_HOVER : "hoverOverMenuLauncher"
};

// The padding that appears on one of the sides of a column.
com.ibm.rdm.web.grid.ViewHeader.COLUMN_PADDING = 4;
