/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.pages.ResourcesPage");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.layout.StackContainer");
dojo.require("dojo.i18n");

dojo.require("jazz.ui.SideNavigationLayout");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.core.loader");

dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.notifications.EventTopics");
dojo.require("com.ibm.rdm.web.common.UpdateContext");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.project.NewProjectWizard");
dojo.require("com.ibm.rdm.web.project.ProjectDashboard");
dojo.require("com.ibm.rdm.web.project.ProjectSnapshotsSection");
dojo.require("com.ibm.rdm.web.search.results.RecentCommentResults");
dojo.require("com.ibm.rdm.web.search.results.RecentResourceResults");
dojo.require("com.ibm.rdm.web.sidebar.SidebarLayout");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.artifact.AbstractResourceArtifactWidget");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.pages.ActionViewCache");
dojo.require("com.ibm.rdm.web.review.ReviewsViewer");
dojo.require("com.ibm.team.dashboard.web.ui.ProjectDashboard");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rdm.web.viewer.RecentlyViewedSection");
dojo.require("com.ibm.rdm.web.viewer.CollectionsMenu");
dojo.require("com.ibm.rdm.web.viewer.ProjectAreaAdminViewer");
dojo.require("com.ibm.rdm.web.common.MessageDisplayWidget");
dojo.require("com.ibm.rdm.web.server.client.utils.DashboardUtil");
dojo.require("com.ibm.rdm.web.server.client.utils.TypeServiceUtil");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.project.AllProjectsPage");
// Loaded via load_async
//dojo.require("com.ibm.rdm.web.artifact.SidebarArtifactLayout");
//dojo.require("com.ibm.rdm.web.viewer.AdminPageViewer");
//dojo.require("com.ibm.rdm.web.viewer.ReportsWelcome");

dojo.requireLocalization("com.ibm.rdm.web.pages", "ResourcesPage");
dojo.requireLocalization("com.ibm.rdm.web.pages", "ChangesSection");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");

var queryutils = com.ibm.rdm.web.util.queryutils;
var AllProjectsPage = com.ibm.rdm.web.project.AllProjectsPage;
var DashboardUtil = com.ibm.rdm.web.server.client.utils.DashboardUtil;
var ProjectDashboard = com.ibm.team.dashboard.web.ui.ProjectDashboard;
var TypeServiceUtil = com.ibm.rdm.web.server.client.utils.TypeServiceUtil;

var UriHelper = com.ibm.rdm.web.common.UriHelper;
var ActionViewCache = com.ibm.rdm.web.pages.ActionViewCache;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;

var ACTION_PREFIX = "com.ibm.rdm.web.pages.";
var ACTION_NAMES = [ "showArtifact", "showArtifactHistory", "showArtifactInReviewContext", "showProjectDashboard",
		"showProjectHistory", "showUserDashboard", "showFoundationProjectDashboard", "showAdminPage",
		"showReportsPage", "showProjectAreas", "showAppropriatePage", "showAllProjectsPage" ];
var ARTIFACT_ACTION_CACHE_SIZE = 4;
var TIMELINE_WIDGET_HEIGHT = 125;
var ARTIFACT_IN_REVIEW_ACTION_CACHE_SIZE = 2;
var ARTIFACT_HISTORY_ACTION_CACHE_SIZE = 2;
var FOUNDATION_DASHBOARD_ACTION_CACHE_SIZE = 1;
var PROJECT_DASHBOARD_CACHE_SIZE = 1;

function isMethod(object, method) {
	return object && method && dojo.isFunction(object[method]);
}

dojo.declare(
	"com.ibm.rdm.web.pages.ResourcesPage",
	[ dijit._Widget, dijit._Templated ],
{
	templateString: '<div dojoAttachPoint="_contentNode" class="rdm-global-styles"></div>',
	_messages: null,
	_messages2: null,
	_appMessages: null,
	// Keeps the last main content that was displayed
	_lastMainContent: null,
	_browsingHistory: [],
	// If this is true, _lastMainContent was not cached, and it must be destroyed
	_shouldLastMainContentBeDestroyed: true,
	_contentNode: null,
	domNode: document.createElement("DIV"),
	_footerHeight: 0,
	_userDashboardWidget: null,
	_recentCommentsSidebar: null,
	_recentRequirementsSidebar: null,
	_reviewsSidebar: null,
	_messageDisplayWidget: null,
	_previousLocationHash: null, 

	postCreate: function() {
		this.inherited(arguments);

		this._initializeCaching();
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.pages", "ResourcesPage");
		this._messages2 = dojo.i18n.getLocalization("com.ibm.rdm.web.viewer", "ServerAdministration");
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
		this._appMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.pages", "application");

		// Widget for display of error, warning and info messages.
		this._messageDisplayWidget = new com.ibm.rdm.web.common.MessageDisplayWidget({
			onResize: dojo.hitch(this, this.updateResize)
		});
		this.domNode.appendChild(this._messageDisplayWidget.domNode);
		
		jazz.app.currentApplication.ui&&jazz.app.currentApplication.ui.EnableCollapsibleBanner&&jazz.app.currentApplication.ui.EnableCollapsibleBanner(true);

		// For firefox, prevent escape key from prematurely stopping XHRs which results
		// in a 7556 error being displayed.
		if (dojo.isFF) {
			dojo.connect(document.body, "onkeydown", function(event) {
				if (event.keyCode === dojo.keys.ESCAPE) { event.preventDefault(); }
			});
		}

		dojo.addClass(this._contentNode, "net-jazz-ajax-SideNavigationLayout");

		var _ActionRegistry = net.jazz.ajax.ui.PlatformUI.getWorkbench().getActionRegistry();

		dojo.forEach(ACTION_NAMES, function(actionName) {
			_ActionRegistry.registerAction(ACTION_PREFIX + actionName, dojo.hitch(this, function(args) {
				// Project area hack for N-1 compatibility. Kindly remove me at > 5.0.6 
				this.__updateProject(args);
				this._dispatchAction(actionName, args);
			}));
		}, this);

		this._footerHeight = dojo.position(jazz.app.currentApplication._footer.domNode).h;

		ChangeNotifier.addListener({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_DELETED,
			callbackFunction: this._onResourceDelete,
			callbackScope: this,
			neverDelayCallback: true
		});
		ChangeNotifier.addListener({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.PROJECT_DELETED,
			callbackFunction: this._onProjectDelete,
			callbackScope: this,
			neverDelayCallback: true
		});

	},

	clearMenuHighlights: function(action) {
		var leftLinks = com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication._navbar._leftLinks.children;
		dojo.removeClass(leftLinks[0].firstChild, "selected");
		for ( var i = 1; i < leftLinks.length; i++) {
			dojo.removeClass(leftLinks[i], "selected");
		}
	},

	/**
	 * dispatchAction allowing any dirty resources to clean up before we do.
	 * 
	 * call onClose when we are being overwritten.
	 * 
	 */
	_dispatchAction: function(action, args) {
		if (!isMethod(this, action)) {
			console.warn("com.ibm.rdm.web.pages.ResourcesPage._dispatchAction: " + action + " unavailable");
			return;
		}
		var currentWidget = this._lastMainContent && dijit.byNode(this._lastMainContent);
		
		var callAction = dojo.hitch(this, function(action, args) {
			dojo.setObject("com.ibm.rdm.web.LAST_ARGS", args); // used by component menus to determine current item
			this.clearMenuHighlights(action);
			// 	When we are viewing/editing an artifact, we want to relax the min-width that Foundation specifies for
			//	the window.  See work item 65387: Horizontal scrollbar introduced too eagerly
			if (action.indexOf("showArtifact") === 0) {
				dojo.addClass(window.document.body, "rdm-artifact-viewer");
			} else {
				dojo.removeClass(window.document.body, "rdm-artifact-viewer");
			}
			this[action](args);
		});

		/**
		 * do I have a widget? does it need to be saved? can I save it?
		 */
		if (currentWidget && isMethod(currentWidget, "isDirty") && currentWidget.isDirty()) {
			var isDifferentTabInSameDashboard = dojo.hitch(this, function(){
				// Return true if and only if the newer hash represents the
				// same project dashboard as the last saved hash.
				if (currentWidget instanceof ProjectDashboard) {
					var queryObject1 = dojo.queryToObject(window.location.hash);
					var queryObject2 = dojo.queryToObject(this._previousLocationHash);
					// Note: the two query objects may have a 'tab' query param and
					// we need to make sure that we omit this 'tab' query param (if
					// present) from consideration when performing our "hash comparison".
					queryObject1.tab = "";
					queryObject2.tab = "";
					return dojo.objectToQuery(queryObject1) === dojo.objectToQuery(queryObject2);
				}
				return false;
			});

			// Bow out if the hash is the same as the last hash OR if the hash
			// represents a project dashboard and the last hash also represented
			// this same project dashboard.
			if (window.location.hash === this._previousLocationHash
				|| isDifferentTabInSameDashboard()) {
				return;
			}

			if(confirm(this._messages.unsavedNavigateConfirm)){
				isMethod(currentWidget, "onCancelButton") && currentWidget.onCancelButton(true);
				isMethod(currentWidget, "onClose") && currentWidget.onClose();
				callAction(action, args);				
			} else {
				window.location.hash = this._previousLocationHash;
				return;				
			}
		} else {
			this._previousLocationHash = window.location.hash;
			isMethod(currentWidget, "onClose") && currentWidget.onClose();
			callAction(action, args);
		}
	},

	handleRefresh: function() {
		com.ibm.rdm.web.common.UpdateContext.refreshHasOccurred();
		var widget = dijit.byNode(this._lastMainContent);
		if (widget.handleRefresh) {
			widget.handleRefresh();
		}
	},

	_onResourceDelete: function(eventParams) {
		// handle artifact deletion on a global scale
		this._removeResourceBrowsingHistoryEntry(eventParams.notifierParams.resourceURI);
		com.ibm.rdm.web.viewer.RecentlyViewedSection.removeResource(eventParams.notifierParams.resourceURI);
		com.ibm.rdm.web.viewer.CollectionsMenu.removeResource(eventParams.notifierParams.resourceURI);
		com.ibm.rdm.web.viewer.ModulesMenu.removeResource(eventParams.notifierParams.resourceURI);

		var escapedHash = decodeURIComponent(window.location.hash);
		if (escapedHash.indexOf(eventParams.notifierParams.resourceURI) > 0) {
			this._shouldLastMainContentBeDestroyed = true;
			this._destroyLastMainContent();
			if (eventParams.notifierParams.projectURI) {
				// 40518: User should be forwarded to project dashboard after deleting an artifact in the editor
				window.location.href = "#action=com.ibm.rdm.web.pages.showProjectDashboard&projectURI="
					+ encodeURIComponent(eventParams.notifierParams.projectURI);
			} else {
				window.location.href = this._getPreviousBrowsingHistoryEntry();
			}
		}
	},

	_onProjectDelete: function() {
		if (this._userDashboardWidget) {
			// destroy dashboard widget so all the viewlets are rebuild with current data
			this._userDashboardWidget.destroy();
			this._userDashboardWidget = null;
		}
	},

	_initializeCaching: function() {
		com.ibm.rdm.web.common.UpdateContext.isChangeContextAllowed();
	},

	showAppropriatePage: function(args) {
		// summary:
		//		This is the "default" action function that's called when an
		//		appropriate action query param has not been specified.
		//		description:
		//		An appropriate action query param has not been specified.
		//		Attempt to route to the appropriate show* function, with
		//		the default being to call showAllProjectsPage.
		//		example:
		//		https://spud.com:9443/rm/web/project-areas/_Y7PDQNAZEeCOVsmDP5Yg5Q
		//			==> Results in an appropriate call to showFoundationProjectDashboard
		//		example:
		//		https://spud.com:9443/rm/web
		//			==> Results in a call to showAllProjectsPage
		this._showNavBarAndSearch();
		var pathname = window.location.pathname;
		if (!args.action && pathname.indexOf("project-areas") !== -1) {
			var projectURI = (com.ibm.rdm.web.FRONTING_SERVICE_JFS_PROJECT_AREAS + "/" + queryutils
				.getLastPathSegment(pathname));
			var overriddenArgs = {};
			overriddenArgs.projectURI = projectURI;
			this.showFoundationProjectDashboard(overriddenArgs);
		} else {
			this.showAllProjectsPage();
		}
	},

	showArtifact: function(args) {
		this._showNavBarAndSearch();
		var cachedActionView = ActionViewCache.getCachedActionView("showArtifact", args.artifactURI);
		var commentURI = args.commentURI;
		var linkURI = args.linkURI;
		var revision = args.revision;
		var artifactInModule = args.artifactInModule;
		var baselineURI = args["rrc:baseline"] || args.baselineURI;

		var graphicalArtifact = cachedActionView ? cachedActionView.widget.artifactWidget.declaredClass === "com.ibm.rdm.web.artifact.ArtifactExtensionArtifactWidget" ||
								cachedActionView.widget.artifactWidget.declaredClass === "com.ibm.rdm.web.artifact.StoryboardArtifactWidget" : false;
		
		// WI 64346 Modified to not use graphical artifacts from the actionview cache
		if (cachedActionView && !graphicalArtifact) {
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
			this._setMainContent(cachedActionView.widget.domNode);
			cachedActionView.widget.updateBrowserTitle();
			
			if (commentURI != undefined) {
				cachedActionView.widget.setCommentToSelect(commentURI);
			}
			if (linkURI != undefined) {
				cachedActionView.widget.setLinkToSelect(linkURI);
			}
			if (artifactInModule != undefined) {
				cachedActionView.widget.setArtifactInModule(artifactInModule, false);
			}

			this._notifyViewRestoration(cachedActionView.widget);

			if (cachedActionView.widget.projectURI) {
				this._updateCurrentProject(cachedActionView.widget.projectURI, null, cachedActionView.widget.baselineURI);
			}
			// Artifact view changed but artifactwidget was cached... pass the breadcrumbs
			if(cachedActionView.widget.artifactWidget && cachedActionView.widget.artifactWidget._breadcrumbInfo){
				com.ibm.rdm.web.viewer.RecentlyViewedSection.setFolderBreadcrumb(cachedActionView.widget.artifactWidget._breadcrumbInfo);
			} else {
				com.ibm.rdm.web.viewer.RecentlyViewedSection.resetFolderReferences();
			}
			return;
		}
		this._createModuleAsync("com.ibm.rdm.web.artifact.SidebarArtifactLayout", {
			resourceURI: args.artifactURI,
			bottomOffset: this._footerHeight,
			viewType: com.ibm.rdm.web.artifact.AbstractResourceArtifactWidget.viewType.ShowArtifact,
			editMode: args.editMode,
			revision: revision,
			baselineURI: baselineURI,
			filterId: args.filterId,
			setCurrentMainContentWidgetId: this._setCurrentMainContentWidgetId
		}).then(dojo.hitch(this, function(sidebarArtifactLayout){
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
			
			ActionViewCache
				.cacheActionView("showArtifact", args.artifactURI, sidebarArtifactLayout, ARTIFACT_ACTION_CACHE_SIZE);
			
			this._setMainContent(sidebarArtifactLayout.domNode);

			if (commentURI != undefined) {
				sidebarArtifactLayout.setCommentToSelect(commentURI);
			}
			if (linkURI != undefined) {
				sidebarArtifactLayout.setLinkToSelect(linkURI);
			}
			if (artifactInModule != undefined) {
				sidebarArtifactLayout.setArtifactInModule(artifactInModule, true);
			}
			sidebarArtifactLayout.startup();
		}));
	},

	showArtifactHistory: function(args) {
		this._showNavBarAndSearch();
		//Remove the baseline from the URI
		var baselineInUrl = null;
		if (args.artifactURI.indexOf('?baseline=') > 0) {
			var split = args.artifactURI.split('?baseline=');
			args.artifactURI = split[0];
			baselineInUrl = split[1];
		}
		var parameterHashCode = (args.artifactURI);
		var cachedActionView = ActionViewCache
			.getCachedActionView("showArtifactHistory", parameterHashCode);
		var revision = args.revision;
		var baselineURI = args["rrc:baseline"] || args.baselineURI;
		if (!baselineURI) {
			//baseline in args has priortiy, but if none use the one in the URI (if any)
			baselineURI = baselineInUrl;
		}
		var linkURI = args.linkURI;
		var artifactInModule = args.artifactInModule;

		// Note: The way this is implemented, we re-use an existing view ONLY
		// if we can find one that corresponds to the same resource at the
		// same "enveloping revision level" (i.e. where the headRevision query
		// param matches). In this case we re-use this view and inform it
		// that it a context change is needed in order to select the passed
		// revision query param.
		if (cachedActionView) {
			this._setMainContent(cachedActionView.widget.domNode);
			if (linkURI !== undefined) {
				cachedActionView.widget.setLinkToSelect(linkURI);
			}
			if (artifactInModule != undefined) {
				cachedActionView.widget.setArtifactInModule(artifactInModule, false);
			}
			cachedActionView.widget.updateBrowserTitle();
			cachedActionView.widget.showRevision(revision, baselineURI);
			if (cachedActionView.widget.projectURI) {
				this._updateCurrentProject(cachedActionView.widget.projectURI, null,
					cachedActionView.widget.baselineURI);
			}
			
			return;
		}

		this._createModuleAsync("com.ibm.rdm.web.artifact.SidebarArtifactLayout", {
			resourceURI: args.artifactURI,
			bottomOffset: this._footerHeight,
			viewType: com.ibm.rdm.web.artifact.AbstractResourceArtifactWidget.viewType.ShowArtifactHistory,
			revision: revision,
			baselineURI: baselineURI,
			setCurrentMainContentWidgetId: this._setCurrentMainContentWidgetId
		}).then(dojo.hitch(this, function(sidebarArtifactLayout){
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
			if (linkURI !== undefined) {
				sidebarArtifactLayout.setLinkToSelect(linkURI);
			}
			if (artifactInModule != undefined) {
				sidebarArtifactLayout.setArtifactInModule(artifactInModule, true);
			}
			
			ActionViewCache
				.cacheActionView("showArtifactHistory", parameterHashCode, sidebarArtifactLayout, ARTIFACT_HISTORY_ACTION_CACHE_SIZE);
			this._setMainContent(sidebarArtifactLayout.domNode);
			sidebarArtifactLayout.startup();
		}));
	},

	showArtifactInReviewContext: function(args) {
		this._showNavBarAndSearch();
		var artifactURI = args.artifactURI;
		var revision = args.revision;
		var reviewId = args.reviewId;
		var commentURI = args.commentURI;
		var baselineURI = args["rrc:baseline"] || args.baselineURI;
		var linkURI = args.linkURI;
		var href = UriHelper.getResourceInReviewContextJafLink(artifactURI, revision, reviewId, null, baselineURI);
		var cacheKeyCode = href.substring(href.indexOf("&") + 1);
		var cachedActionView = ActionViewCache.getCachedActionView("showArtifactInReviewContext", cacheKeyCode);
		if (cachedActionView) {
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
			this._setMainContent(cachedActionView.widget.domNode);
			cachedActionView.widget.updateBrowserTitle();

			if (commentURI) {
				cachedActionView.widget.setCommentToSelect(commentURI);
			}
			if (linkURI) {
				cachedActionView.widget.setLinkToSelect(linkURI);
			}
			
			this._notifyViewRestoration(cachedActionView.widget);
			
			// Artifact view changed but artifactwidget was cached... pass the breadcrumbs
			if(cachedActionView.widget.artifactWidget && cachedActionView.widget.artifactWidget._breadcrumbInfo){
				com.ibm.rdm.web.viewer.RecentlyViewedSection.setFolderBreadcrumb(cachedActionView.widget.artifactWidget._breadcrumbInfo);
			} else {
				com.ibm.rdm.web.viewer.RecentlyViewedSection.resetFolderReferences();
			}
			return;
		}
		this._createModuleAsync("com.ibm.rdm.web.artifact.SidebarArtifactLayout", {
			resourceURI: artifactURI,
			baselineURI: baselineURI,
			bottomOffset: this._footerHeight,
			viewType: com.ibm.rdm.web.artifact.AbstractResourceArtifactWidget.viewType.ShowArtifactInReviewContext,
			revision: revision,
			reviewId: reviewId,
			setCurrentMainContentWidgetId: this._setCurrentMainContentWidgetId
		}).then(dojo.hitch(this, function(sidebarArtifactLayout){
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
			
			ActionViewCache
				.cacheActionView("showArtifactInReviewContext", cacheKeyCode, sidebarArtifactLayout, ARTIFACT_IN_REVIEW_ACTION_CACHE_SIZE);
			
			this._setMainContent(sidebarArtifactLayout.domNode);
			
			if (commentURI) {
				sidebarArtifactLayout.setCommentToSelect(commentURI);
			}
			if (linkURI) {
				sidebarArtifactLayout.setLinkToSelect(linkURI);
			}
			sidebarArtifactLayout.startup();
		}));
	},

	showProjectDashboard: function(args) {
		this._showNavBarAndSearch();
		var links = com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication._navbar._leftLinks;
		if (args.viewType === "modules") {
			dojo.addClass(links.children[3], "selected");
		} else if (args.viewType === "collection") {
			dojo.addClass(links.children[2], "selected");
		} else {
			dojo.addClass(links.children[1], "selected");
		}
		var projectURI = encodeURI(args.projectURI);
		var folderId = args.folderId;
		var filterId = args.filterId;

		this._updateCurrentProject(projectURI);

		var taskProgressURI = args.taskProgressURI ? encodeURI(args.taskProgressURI) : "";

		var cachedActionView = null;

		var cacheKey = projectURI;
		if (!folderId && !filterId) {
			cachedActionView = ActionViewCache.getCachedActionView("showProjectDashboard", cacheKey);
		} else {
			cachedActionView = ActionViewCache.getCachedActionView("showProjectDashboardFiltering", cacheKey);
		}

		com.ibm.rdm.web.common.ProjectCache.getProject(projectURI)
			.addCallback(this, function(project) {
				var projectPageString = dojo.string.substitute(this._messages["projectDashboard"], [project.name]);
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(projectPageString);
			});

		if (cachedActionView) {
			this._setMainContent(cachedActionView.widget.domNode);
			this._notifyViewRestoration(cachedActionView.widget, {
				viewType: args.viewType,
				folderId: folderId,
				filterId: filterId
			});
			return;
		}
		
		com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
		var projectDashboard = new com.ibm.rdm.web.project.ProjectDashboard({
			projectURI: projectURI,
			folderId: folderId,
			filterId: filterId,
			taskProgressURI: taskProgressURI,
			footerHeight: this._footerHeight,
			viewType: args.viewType,
			resourcesPage: this
		});
		var sidebarLayout = new com.ibm.rdm.web.sidebar.SidebarLayout({
			bottomOffset: this._footerHeight,
			contentNode: projectDashboard.domNode,
			setCurrentMainContentWidgetId: this._setCurrentMainContentWidgetId,
			isProjectDashboard: true
		});
		var projectSnapshots = new com.ibm.rdm.web.project.ProjectSnapshotsSection({
			projectURI: projectURI,
			width: sidebarLayout.getSidebarSectionWidth(),
			navigateAwayFunction: function(arg) {
				return projectDashboard.navigateAwayFunction(arg);
			}
		});
		var recentComments = new com.ibm.rdm.web.search.results.RecentCommentResults({
			scopingProjectURIs: [ projectURI ],
			viewType: com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.RECENT_COMMENTS,
			displayContext: com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.SIDEBAR_SECTION,
			resourcesPage: this,
			initialTitle: this._messages["recentComments"]
		});
		var recentRequirements = new com.ibm.rdm.web.search.results.RecentResourceResults({
			scopingProjectURIs: [ projectURI ],
			viewType: com.ibm.rdm.web.search.results.RecentResourceResults.viewTypes.SHOW_REQUIREMENTS,
			displayContext: com.ibm.rdm.web.search.results.RecentResourceResults.displayContext.SIDEBAR_SECTION,
			resourcesPage: this,
			initialTitle: this._messages["recentArtifacts"]

		});
		var reviewsViewer = new com.ibm.rdm.web.review.ReviewsViewer({
				scopingProjectURIs: [ projectURI ],
				resourcesPage: this,
				// YUK
				myReviewsViewlet: true,
				canCreateReviews: true,
				pageSize: 10
			// May need to add scopingProjectURIs, viewType and displayContext, to make it work the same way for dashboard viewlet
			});

			this._recentRequirementsSidebar = sidebarLayout.addSidebarSection(
				this._messages["recentArtifacts"], recentRequirements, true);
			
			sidebarLayout.addSidebarSection(this._messages["projectSnapshots"], projectSnapshots, false);
			this._reviewsSidebar = sidebarLayout.addSidebarSection(this._messages["reviewsViewer"],
				reviewsViewer, false);

			var setSidebarTitle = dojo.hitch(this, function(title) {
				this._reviewsSidebar.setSectionTitleInfo(title);
			});

			var that = this;

			// WI: 32719 - gives the header of the SidebarSection the same interface
			// as a viewlets "site", whatever that exactly is.
			reviewsViewer.site = {
				_isLoading: false,
				_getLoadingMessage: function() {
					return " (" + that._commonMessages["loading"] + ")";
				},
				setTitle: function(title) {
					this._title = title;
					title = this._isLoading ? (title + this._getLoadingMessage()) : title;
					setSidebarTitle(title);
				},
				setSecondaryTitle: function(title) {
				},
				setLoading: function(isLoading) {
					this._isLoading = isLoading;
				}
			};

		this._recentCommentsSidebar = sidebarLayout.addSidebarSection(this._messages["recentComments"],
			recentComments, false);

		if (!folderId && !filterId) {
			ActionViewCache
				.cacheActionView("showProjectDashboard", cacheKey, sidebarLayout, PROJECT_DASHBOARD_CACHE_SIZE);
		} else {
			ActionViewCache
				.cacheActionView("showProjectDashboardFiltering", cacheKey, sidebarLayout, PROJECT_DASHBOARD_CACHE_SIZE);
		}

		sidebarLayout.startup();
		projectDashboard._createDeferred.addCallback(this, function() {
			this._setMainContent(sidebarLayout.domNode);
		});
	},

	showReportsPage: function(args) {
		this._showNavBarAndSearch();
		dojo.addClass(
			com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication._navbar._leftLinks.children[4],
			"selected");
		var doCreateReportsPage = dojo.hitch(this, function() {
			this._createModuleAsync("com.ibm.rdm.web.viewer.ReportsWelcome", {
				setCurrentMainContentWidgetId: this._setCurrentMainContentWidgetId
			}).then(dojo.hitch(this, function(reportsWelcome){
				com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
				this._setMainContent(reportsWelcome.domNode);
				reportsWelcome.startup();
			}));
		});

		if (!com.ibm.rdm.web.CURRENT_PROJECT) {
			com.ibm.rdm.web.common.ProjectCache.getUserMemberProjects().addCallback(this,
				function(myProjects) {
					for ( var projectURI in myProjects) {
						this._updateCurrentProject(projectURI);
						break;
					}
					doCreateReportsPage();
				});
		} else {
			doCreateReportsPage();
		}
	},

	showAdminPage: function(args) {
		this._showNavBarAndSearch();
		// Ensure the type system cache is cleared.
		TypeServiceUtil.clearCache();
		var cachedActionView = ActionViewCache.getCachedActionView("showAdminPage", args.projectURI);
		var projectURI = args.projectURI;

		this._updateCurrentProject(projectURI);

		com.ibm.rdm.web.common.ProjectCache.getProject(projectURI).addCallback(this, function(project) {
			var adminPageString = dojo.string.substitute(this._messages["adminPage"], [ project.name ]);
			net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(adminPageString);
		});
		if (cachedActionView) {
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
			this._setMainContent(cachedActionView.widget.domNode);
			return;
		}
		this._createModuleAsync("com.ibm.rdm.web.viewer.AdminPageViewer", {
			projectURI: args.projectURI,
			setCurrentMainContentWidgetId: this._setCurrentMainContentWidgetId
		}).then(dojo.hitch(this, function(adminPageViewer){
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
			ActionViewCache.cacheActionView("showAdminPage", args.projectURI, adminPageViewer);
			this._setMainContent(adminPageViewer.domNode);
			adminPageViewer.startup();
		}));
	},

	showProjectAreas: function(args) {
		this._showNavBarAndSearch();
		this._genericProjectManagement("showProjectAreas", "manageProjectAreas",
			"applicationAdministrationProjects");

	},

	_genericProjectManagement: function(actionName, dataProperty, bannerTitle) {
		var cachedActionView = ActionViewCache.getCachedActionView(actionName, actionName);

		this._updateCurrentProject(null, this._messages2[bannerTitle]);

		if (cachedActionView) {
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
			this._setMainContent(cachedActionView.widget.domNode);
		}
		else {
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
			var projectAreaAdminViewer = new com.ibm.rdm.web.viewer.ProjectAreaAdminViewer({
				dataProperty: dataProperty,
				setCurrentMainContentWidgetId: this._setCurrentMainContentWidgetId
			});
			ActionViewCache.cacheActionView(actionName, actionName, projectAreaAdminViewer);
			this._setMainContent(projectAreaAdminViewer.domNode);
			projectAreaAdminViewer.startup();
		}
	},

	showProjectHistory: function(args) {
		this._showNavBarAndSearch();
		var links = com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication._navbar._leftLinks;
		if (args.viewType === "collection") {
			dojo.addClass(links.children[2], "selected");
		} else {
			dojo.addClass(links.children[1], "selected");
		}
		var projectURI = args.projectURI,
			baselineURI = args.baselineURI;

		this._updateCurrentProject(projectURI, null, baselineURI);

		var folderId = args.folderId,
			filterId = args.filterId;

		if (baselineURI === undefined) {
			baselineURI = null;
		}
		var cacheKey = projectURI;
		if (baselineURI !== null) {
			cacheKey += "&" + baselineURI;
		}

		if (folderId) {
			cacheKey = null;
		}
		else if (filterId) {
			cacheKey = cacheKey + "." + filterId;
		}

		var cachedActionView = ActionViewCache.getCachedActionView("showProjectHistory", cacheKey);
		com.ibm.rdm.web.common.ProjectCache.getProject(projectURI)
			.addCallback(this, function(project) {
				var projectPageString = dojo.string.substitute(this._messages["projectDashboard"], [project.name]);
				net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(projectPageString);
			});
		if (cachedActionView) {
			this._notifyViewRestoration(cachedActionView.widget, {
				viewType: args.viewType
			});
			this._setMainContent(cachedActionView.widget.domNode);
			cachedActionView.widget._contentWidget._snapshotsWidget
				.selectThumbnail(cachedActionView.widget.baselineURI);
			return;
		}

		com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
		var projectDashboard = new com.ibm.rdm.web.project.ProjectDashboard({
			projectURI: projectURI,
			baselineURI: baselineURI,
			folderId: folderId,
			filterId: filterId,
			footerHeight: this._footerHeight + TIMELINE_WIDGET_HEIGHT,
			viewType: args.viewType,
			resourcesPage: this
		});
		var sidebarLayout = new com.ibm.rdm.web.sidebar.SidebarLayout({
			bottomOffset: this._footerHeight,
			contentNode: projectDashboard.domNode,
			baselineURI: baselineURI,
			setCurrentMainContentWidgetId: this._setCurrentMainContentWidgetId
		});

		ActionViewCache.cacheActionView("showProjectHistory", cacheKey, sidebarLayout);

		this._setMainContent(sidebarLayout.domNode);
		sidebarLayout.startup();
	},

	showFoundationProjectDashboard: function(args) {
		this._showNavBarAndSearch();
		dojo.addClass(
			com.ibm.rdm.web.pages.WebUIinitializer._jazzApplication._navbar._leftLinks.children[0].firstChild,
			"selected"
		);
		var projectURI = args.projectURI;
		this._updateCurrentProject(projectURI);

		var cachedActionView =
			ActionViewCache.getCachedActionView("showFoundationProjectDashboard", args.projectURI);
		if (cachedActionView) {
			com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
			this._setMainContent(cachedActionView.widget.domNode);
		} else {
			var deferred = DashboardUtil.getOrCreateProjectDashboard(projectURI);
			deferred.addCallback(this, function(projectDashboardURI) {
				var dashboard = new ProjectDashboard({
					url: projectDashboardURI,
					page: args.tab
				});
				this._setMainContent(dashboard.domNode);
				this._showInitialWorkbechTitleForProjectDashboard(dashboard);
				this._listenForProjectDashboardTitleChanges(dashboard);
				this._listenForProjectDashboardPageChanges(dashboard);
				com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
				ActionViewCache.cacheActionView(
					"showFoundationProjectDashboard", args.projectURI,
					dashboard, FOUNDATION_DASHBOARD_ACTION_CACHE_SIZE
				);
				return projectDashboardURI;
			});
		}
		// Loading the dashboard -> remove references to folder filter and breadcrumbs
		// in recent viewed section
		com.ibm.rdm.web.viewer.RecentlyViewedSection.resetFolderReferences();
	},

	showAllProjectsPage: function(args) {
		this._hideNavBarAndSearch();
		dojo.setObject("com.ibm.rdm.web.CURRENT_PROJECT", null);
		dojo.setObject("com.ibm.rdm.web.BANNER_DISPLAY_NAME", this._messages["allProjects"]);
		ChangeNotifier.notifyListeners({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.BANNER_UPDATE
		});
		com.ibm.rdm.web.common.UpdateContext.redirectHasOccurred();
		var allProjectsPage = new AllProjectsPage();
		this._setMainContent(allProjectsPage.domNode);
		allProjectsPage.startup();
		
		if(args && args.createProjectArea === "true") {
			this._createNewProjectArea();
		}
	},
	
	_createNewProjectArea: function() {
		var wizard = new com.ibm.rdm.web.project.NewProjectWizard();
		wizard.show();
	},

	updateResize: function() {
		// summary:
		//		This function should be called whenever a previously orphaned DOM
		//		node is re-parented back into the active document's DOM. In
		//		practice we do this whenever we bring a previously cached page
		//		back into view.
		if (this._lastMainContent) {
			var contentNodeWidget = dijit.byNode(this._lastMainContent);
			if (contentNodeWidget && contentNodeWidget.handleWindowResize) {
				contentNodeWidget.handleWindowResize();
			}
		}
	},

	// /////////////////////////////////////////////////////////////////////////////////
	// Helpers

	_setMainContent: function(contentNode, destroyLastMainContent) {
		// destroyLastMainContent:
		// If the action view (contentNode) is cached so that a refresh does not happen with back/forward buttons
		// this should be set to false, so this._lastMainContent will not destroyed.

		// calling this function simulates focusing out of the element which forces all opened menus to hide
		// when a user navigates away from the current page
		// 27682: JTS: djit and foundation menus are not collapsed when browser's back and forward buttons are pressed
		dijit.focus(document.body);

		this._saveCurrentBrowsingHistoryEntry();
		this._destroyLastMainContent();

		var currentWidget = this._lastMainContent ? dijit.byNode(this._lastMainContent) : undefined;
		var canBeDetached = true;
		// Give the widget that is about to be replaced a chance to indicate
		// whether it needs to remain in the DOM (in which case it is hidden)
		// or whether it can be detached. Note: this is added for WI: 28551
		// as IFRAME contents appear to be lost when the tree is detached.
		if (currentWidget && typeof currentWidget.canBeDetached === "function") {
			canBeDetached = currentWidget.canBeDetached();
		}

		if (this._lastMainContent && this._lastMainContent.parentNode) {
			if (canBeDetached) {
				this._contentNode.removeChild(this._lastMainContent);
			} else {
				dojo.style(this._lastMainContent, "display", "none");
			}
		}

		if (contentNode.parentNode !== this._contentNode) {
			this._contentNode.appendChild(contentNode);
		} else {
			dojo.style(contentNode, "display", "block");
		}

		// Note: We take care here to scroll the window back up to the top if
		// it's currently been scrolled down (so that the top area of the new
		// content doesn't "go missing").
		if (dojo.pageYOffset > 0) {
			window.scrollTo(0, 0);
		}

		this._lastMainContent = contentNode;
		this._shouldLastMainContentBeDestroyed = destroyLastMainContent || false;

		this.updateResize();
		this._setCurrentMainContentWidgetId(contentNode);

		ChangeNotifier.notifyDelayedListeners();
	},
	
	_createModuleAsync: function(moduleName, args) {
		var deferred = new dojo.Deferred();	
		try {
			jazz.core.loader.load_async(moduleName, function() {
		        var constructor = dojo.getObject(moduleName);
		        if(!constructor) { throw "No constructor found"; }
		        var module = new constructor(args);
		        deferred.resolve(module);
		    });
		} catch (e) {
		    console.error("Failed to load Module["+moduleName+"] : " + e.message);
		    deferred.reject();
		}
		return deferred;
	},

	_setCurrentMainContentWidgetId: function(widget) {
		// required for ChangeNotifier
		dojo.setObject("com.ibm.rdm.web.CURRENT_MAIN_CONTENT_WIDGET_ID", widget.id);
	},

	_destroyLastMainContent: function() {
		// check if we need to destroy a previous DOM node of main content
		if (this._lastMainContent && this._shouldLastMainContentBeDestroyed) {
			var widget = (this._lastMainContent.nodeType === 1 && dijit.byNode(this._lastMainContent));
			if (widget) {
				widget._removeSelfFromCachedActionView();
				this._lastMainContent = null;
			}
		}
	},

	_saveCurrentBrowsingHistoryEntry: function() {
		this._browsingHistory.push(window.location.href);
	},

	_removeResourceBrowsingHistoryEntry: function(resourceURI) {
		this._browsingHistory = dojo.filter(this._browsingHistory, function(value) {
			if(value.search(resourceURI) !== -1) { return false; }
			if(value.search(encodeURIComponent(resourceURI)) !== -1) { return false; }
			return true;
		}, this);
	},

	_getPreviousBrowsingHistoryEntry: function() {
		var previousBrowsingHistoryEntry = null;
		for ( var i = this._browsingHistory.length; i >= 0; i--) {
			if (this._browsingHistory[i - 1]) {
				previousBrowsingHistoryEntry = this._browsingHistory[i - 1];
				break;
			}
		}
		if (!previousBrowsingHistoryEntry) {
			previousBrowsingHistoryEntry = "#action=com.ibm.rdm.web.pages.showUserDashboard";
		}
		return previousBrowsingHistoryEntry;
	},

	_updateCurrentProject: function(newProjectURI, /*optional */bannerDisplay, newBaselineURI) {
		DashboardUtil.updateBannerProjectDashboardLink(newProjectURI);

		dojo.setObject("com.ibm.rdm.web.CURRENT_PROJECT", newProjectURI);
		
		dojo.setObject("com.ibm.rdm.web.CURRENT_PROJECT_BASELINE", newBaselineURI);
		if (bannerDisplay) {
			dojo.setObject("com.ibm.rdm.web.BANNER_DISPLAY_NAME", bannerDisplay);
			jazz.app.context.set({
				name: bannerDisplay,
				uri: newProjectURI
			});
			ChangeNotifier.notifyListeners({
				eventTopic: com.ibm.rdm.web.notifications.EventTopics.BANNER_UPDATE
			});
		}
		else {
			ProjectCache.getProject(com.ibm.rdm.web.CURRENT_PROJECT).addCallback(this, function(project) {
				dojo.setObject("com.ibm.rdm.web.BANNER_DISPLAY_NAME", project.name);
				jazz.app.context.set({
					name: project.name,
					uri: newProjectURI
				});
				ChangeNotifier.notifyListeners({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.BANNER_UPDATE
				});
			});
		}
	},

	_notifyViewRestoration: function(widget, args) {
		if ((widget.viewHasBeenRestored) && (typeof widget.viewHasBeenRestored === "function")) {
			widget.viewHasBeenRestored(args);
		}
	},

	_updateDisplayedProjectDashboardTitle: function(dashboard) {
		var title = dojo.string.substitute(this._messages["projectDashboardWorkbenchTitle"], [ dashboard
			.getTitle() ]);
		net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(title);
	},

	_showInitialWorkbechTitleForProjectDashboard: function(dashboard) {
		// summary:
		//		Wait for the foundation dashboard to initialize and THEN
		//		set the workbench title to an appropriate value.
		var handle = this.connect(dashboard, "onLoad", function() {
			this._updateDisplayedProjectDashboardTitle(dashboard);
			this.disconnect(handle);
		});
	},

	_listenForProjectDashboardTitleChanges: function(dashboard) {
		// summary:
		//		React to user changes to the dashboard title by making an
		//		appropriate update to the displayed workbench title.
		this.connect(dashboard, "onTitleChange", dojo.partial(this._updateDisplayedProjectDashboardTitle, dashboard));
	},

	_listenForProjectDashboardPageChanges: function(dashboard) {
		// summary:
		//		Tack on a "tab" query param to the current URL in use whenever
		//		the user changes tab pages so that users are able to bookmark
		//		our project dashboard tab pages.
		// description:
		//		It MAY be the case that users have added additional tabs (and
		//		even re-ordered these tabs after adding them).  We want users
		//		to be able to copy the URL found in the browser and have it
		//		"capture" the currently selected tab page ... if we didn't
		//		do this users would have to capture the URL and then say "use
		//		this URL and _then_ select Tab <target-tab-name>".
		this.connect(dashboard, "onPageChange", function(pageId){
			var uri = window.location.href,
				hashIndex = uri.indexOf("#");
			if (hashIndex !== -1) {
				var queryObject = dojo.queryToObject(uri.substring(hashIndex+1));
				if (!queryObject.tab) {
					window.location.href += "&tab=" + pageId;
				} else {
					// Note: it MAY be the case that the queryObject.tab value is
					// not the same as pageId (due to our ActionViewCache) ... so
					// we just override it here by setting queryObject.tab to the
					// pageId.
					queryObject.tab = pageId;
					window.location.href = (
						uri.substring(0, hashIndex+1)
						+ dojo.objectToQuery(queryObject)
					);
				}
			}
		});
	},

	_hideNavBarAndSearch: function() {
		// summary:
		//		Calling this function results in hiding the navigation drop
		//		down menus and the Search control.
		var pageTemplate = jazz.app.currentApplication.ui;
		pageTemplate.get("navbar").domNode.style.display = "none";
		pageTemplate.get("_searchArea").style.display = "none";
		
		// The banner height has changed, therefore we need to 'poke'.
		jazz.util.ResizeNotifier.poke();
	},

	_showNavBarAndSearch: function() {
		// summary:
		//		Calling this function results in showing the navigation drop
		//		down menus and the Search control.
		var pageTemplate = jazz.app.currentApplication.ui;
		pageTemplate.get("navbar").domNode.style.display = "";
		pageTemplate.get("_searchArea").style.display = "";
		
		//pageTemplate.set("searchProvider", searchProvider);
		// The banner height has changed, therefore we need to 'poke'.
		jazz.util.ResizeNotifier.poke();
	},
	
	__updateProject: function(args) {
		// Check if we have a project url that references a non rm url
		if(args && args.projectURI
				&& args.projectURI.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) === -1) {
			var processAreaIdx = args.projectURI.indexOf('/process/project-areas/');
			if(processAreaIdx !== -1) {
				args.projectURI = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL
					+ args.projectURI.substring(processAreaIdx, args.projectURI.length);
				
				dojo.hash(dojo.objectToQuery(args), /*replace*/true);
			}
		}
	}
});

com.ibm.rdm.web.pages.ResourcesPage.DELIMETER = " > ";

})();
