/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2015.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function(){
dojo.provide("com.ibm.rdm.web.picker.ArtifactPickerForm");

dojo.require("dijit.form.Button");
dojo.require("dijit.form.Form");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.rdm.web.common.Dialog");
dojo.require("com.ibm.rdm.web.common.DialogForm");
dojo.require("com.ibm.rdm.web.picker.ArtifactPickerPane");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.project.UploadArtifactForm");

dojo.requireLocalization("com.ibm.rdm.web.common", "DialogForm");
dojo.requireLocalization("com.ibm.rdm.web.picker", "ArtifactPickerForm");

var ArtifactPickerPane = com.ibm.rdm.web.picker.ArtifactPickerPane;
var MessageHandler = com.ibm.rdm.web.common.MessageHandler;
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;

var DEFAULT_HIDE_TIMEOUT = 1500; // milliseconds

dojo.declare(
	"com.ibm.rdm.web.picker.ArtifactPickerForm",
	com.ibm.rdm.web.common.DialogForm,
{
	// summary:
	//		Provides a common form that can be used to present an "artifact
	//		picker dialog".
	// description:
	//		The dialog that this form presents shows picker-based filter
	//		controls such as an artifact name input field, a folder section
	//		and an artifact selection area that displays the artifacts matching
	//		the currently specified filters.  Beneath these picker related
	//		controls are the following buttons: "Add & Close", "Add" and
	//		"Close".  Consumers of this class should hook up for receipt of the
	//		following callback defined within this class:
	//		    onAddArtifacts: function(arrayOfIndexedPropertiesForSelectedArtifacts) {
	//		    ...
	//		[Please see this function's comment block for further details.]

	templatePath: dojo.moduleUrl("com.ibm.rdm.web",
		"picker/templates/ArtifactPickerForm.html"),
	widgetsInTemplate: true,
	rdmModulePath: dojo.moduleUrl("com.ibm.rdm.web"),
	
	///////////////////////////////////////////////////////////////////////////
	// Public fields
	
	// title: [const] String
	//		The title to use when this form is presented in a dialog.
	title: null,

	// description: [const] String
	//		The description to use when this form is presented in a dialog.
	description: null,

	// style: enum type
	//		Enumerated type that determines the button selections avaiable in
	//		this form. 
	style: null, 

	// projectURI: [const] String
	//		The project URI representing the project domain from which to
	//		enable artifact picking.
	projectURI: null,
	
	displayAllProjects: false,

	// baselineURI: [const] String
	//		Baseline to pull artifacts from for the picker
	baselineURI: null,

	// resourceURI: [const] String
	//		URI of the resource loading the artifact. 
	resourceURI: null,

	// filterArtifacts: [const] String
	//		array of resources to filter from the picker selection display
	filterArtifacts: [],

	// showModuleRowStyleIcons: [const] Boolean
	//		When passed as true the results listing section of the dialog will
	//		show with a "row style" icon just to the right of the artifact type
	//		icon for every artifact listed.
	showModuleRowStyleIcons: null,

	enableFilterByWhereUsed: true,
	selectionOptions: null,
	allowUploadNewArtifact: false,
	onAddCallbackScope: null,
	surrenderClose: false,
	onDialogOpen: null,
	onDialogClose: null,
	defaultFolderForArtifactCreation: null,
	pauseOnAddAndClose: false,
	errorState: false,
	showModuleViews: false,

	// autoDisableAddedEntries: [const] Boolean
	//		When set to true the picker will automatically disable the selected
	//		artifact entries when you click on the "Add" button.  The default
	//		is true.
	autoDisableAddedEntries: true,
		
	///////////////////////////////////////////////////////////////////////////
	// Private fields
	
	_messages: null,
	_commonMessages: null,
	_dialog: null,
	_artifactPickerPane: null,
	_isBusy: false,
	_onAddAndCloseDisplayTimeout: DEFAULT_HIDE_TIMEOUT,

	///////////////////////////////////////////////////////////////////////////
	// Attach points

	_artifactPickerDiv: null,
	_addAndCloseButton: null,
	_addButton: null,
	_closeButton: null,
	_statusImageNode: null,
	_messagesSection: null,
	
	///////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args){
		this._isBusy = false;
		this.title = args.title;
		this.description = args.description;
		this.style = (args.style)?args.style:com.ibm.rdm.web.picker.ArtifactPickerForm.formStyle.OK_CANCEL;
		this.resourceURI = (args.resourceURI)?args.resourceURI:null;
		this.projectURI = args.projectURI;
		this.baselineURI = (args.baselineURI)?args.baselineURI:null;
		this.filterArtifacts = (args.filterArtifacts)?args.filterArtifacts:[];
		this.displayAllProjects = (args.displayAllProjects)?args.displayAllProjects:false;
		this.selectionOptions = (args.selectionOptions)?args.selectionOptions:null;
		this.allowUploadNewArtifact = (args.allowUploadNewArtifact)?args.allowUploadNewArtifact:false;
		this.onDialogOpen = (args.onDialogOpen)?args.onDialogOpen:null;
		this.onDialogClose = (args.onDialogClose)?args.onDialogClose:null;
		this.enableFilterByWhereUsed = args.enableFilterByWhereUsed;
		this.onCancel = args.onCancel;
		this.showModuleRowStyleIcons = args.showModuleRowStyleIcons ? true : false;
		this.disablePickerModuleReuseMode = args.disablePickerModuleReuseMode;
		this.showModuleViews = args.showModuleViews;

		if(args.pauseOnAddAndClose) {
			this.pauseOnAddAndClose = true;
			if(args.onAddAndCloseDisplayTimeout) {
				this._onAddAndCloseDisplayTimeout =
					parseInt(args.onAddAndCloseDisplayTimeout);	
			}
		}

		//Do not display the current resource in the picker. For this add this resource to the
		//list of artifacts to filter
		if(this.resourceURI){
			var resourceURIFound = false;
			for(var i=0; i < this.filterArtifacts.length; i++){			
				if(this.filterArtifacts[i] == this.resourceURI) {
					resourceURIFound=true;
				}
			}
			if(resourceURIFound != true){
				this.filterArtifacts.push(this.resourceURI);
			}
		}
	},
	
	postMixInProperties: function() {
		this._messages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.picker",
				"ArtifactPickerForm");
		this._commonMessages =
			dojo.i18n.getLocalization("com.ibm.rdm.web.common",
				"DialogForm");
		if (!this.title) {
			this.title = this._messages["title"];
		}
		if (!this.description) {
			this.description = this._messages["description"];
		}
	},

	postCreate: function() {
		this.inherited("postCreate", arguments);
		
		this._messageBox = new jazz.ui.MessageBox().placeAt(this._messagesSection);

		//Create the picker pane widget to add inside the form
		var pickerSelectionOptions = this.selectionOptions;
		this._artifactPickerPane = new ArtifactPickerPane({
			projectURI: this.projectURI,
			baselineURI: this.baselineURI,
			displayAllProjects: this.displayAllProjects,
			showModuleRowStyleIcons: this.showModuleRowStyleIcons,
			disablePickerModuleReuseMode: this.disablePickerModuleReuseMode,
			showModuleViews: this.showModuleViews,
			filterArtifacts: this.filterArtifacts,
			selectionOptions: pickerSelectionOptions,
			defaultFolderForArtifactCreation: this.defaultFolderForArtifactCreation,
			onArtifactSelectionChanged: dojo.hitch(this, this._handleArtifactSelectionChanged),
			onItemSelected: dojo.hitch(this, this._handleItemSelected),
			doNotSetInitialFocusOnFolders: true,
			enableFilterByWhereUsed: this.enableFilterByWhereUsed,
			enterKeyHandler: dojo.hitch(this, function() {							
				// If the Add and Close button is enabled,
				// then click the button.
				if (! dojo.attr(this._addAndCloseButton, "disabled")) {					 
					this._addAndCloseButton.click();
				}				
			})
		});
		
		//Add the picker widget to the form
		this._artifactPickerDiv.appendChild(this._artifactPickerPane.domNode);
		
		var launchUploadArtifactDialog = dojo.hitch(this, function() {
			var projectSelect = this._artifactPickerPane._projectListSelect;
			// Use the resource's folder if the resource's project is the currently selected project;
			// otherwise use null value and the UploadArtifactForm will default to the root folder for
			// the selected project.
			var folder = null;
			if (this.projectURI === projectSelect.options[projectSelect.selectedIndex].value) {
				folder = this.folderURI;
			} 
			var uploadArtifactDialog = new com.ibm.rdm.web.project.UploadArtifactForm({
				artifactFormat: MimeTypes.getArtifactFormat(MimeTypes.WRAPPER_RESOURCE_NAME, projectSelect.options[projectSelect.selectedIndex].value),
				projectURI: projectSelect.options[projectSelect.selectedIndex].value,
				folderURI: folder,
				hideOpen: true
			});
			
			var selectNewFileInPicker = dojo.hitch(this, function(artifactList, openOnOk, wrappedProperties) {

				var artifactsToSelect = [];
				
				dojo.forEach(artifactList, function(artifact) {
					if (wrappedProperties) {
						// 77603: If multiple artifacts have been passed in, the wrapped properties have already
						//   been merged.
						dojo.mixin(artifact, wrappedProperties);
					}
					
					artifactsToSelect.push(artifact);
				}, this);
				if (artifactsToSelect.length > 0) {
					var folderURI = uploadArtifactDialog.folderURI;
					var projectURI = uploadArtifactDialog.projectURI;
					this.selectArtifactList(artifactsToSelect, folderURI, projectURI);
				}
				
				dojo.disconnect(uploadArtifactCompleteConnection);
			});
			
			var uploadArtifactCompleteConnection = 
				dojo.connect(uploadArtifactDialog, "onCreate", this, selectNewFileInPicker);

			uploadArtifactDialog.show();
		});
		
		var uploadIcon = dojo.create("img", {
			src: this._blankGif,
			className: "rdmImageUpload",
			title: this._messages["uploadFileFromDrive"]
		});
		
		if(this.allowUploadNewArtifact === true){
			var uploadLink = dojo.create("a", {
				innerHTML:this._messages["uploadFileFromDrive"],
				href: "#",
				title: this._messages["uploadFileFromDrive"]
			});
			dojo.place(uploadIcon, uploadLink, "first");
			
			dojo.empty(this._uploadFileDiv);
			this._uploadFileDiv.appendChild(uploadLink);
			
			this.connect(uploadLink, "onclick", dojo.hitch(this, function(event){
				dojo.stopEvent(event);
				launchUploadArtifactDialog();
			}));
		}
		
		//Handle the display of button selections based on the style of the
		//form
		switch(this.style){
		case com.ibm.rdm.web.picker.ArtifactPickerForm.formStyle.ADD_CLOSE:	
			
			
			this._addAndCloseButton = this._createButton(this._messages.addAndClose, true);
			this._addButton= this._createButton(this._messages.add);
			this._closeButton = this._createButton(this._messages.close);
			
			this._buttonSectionDiv.appendChild(this._addAndCloseButton);
			this._buttonSectionDiv.appendChild(this._addButton);
			this._buttonSectionDiv.appendChild(this._closeButton);
			
			this.connect(this._addAndCloseButton, "click", this._onAddAndClose);
			this.connect(this._addButton, "click", this._onAdd);
			this.connect(this._closeButton, "click", this._onCancel);

			
			dojo.attr(this._addAndCloseButton, "disabled", true);
			dojo.attr(this._addButton, "disabled", true);	
			
		
			break;

		case com.ibm.rdm.web.picker.ArtifactPickerForm.formStyle.OK_CANCEL:
			
			this._addAndCloseButton = this._createButton(this._messages.ok, true);
			this._closeButton = this._createButton(this._messages.cancel);
			
			this._buttonSectionDiv.appendChild(this._addAndCloseButton);
			this._buttonSectionDiv.appendChild(this._closeButton);
			
			this.connect(this._addAndCloseButton, "click", this._onAddAndClose);
			this.connect(this._closeButton, "click", this._onCancel);
	
			
			dojo.attr(this._addAndCloseButton, "disabled", true);

			break;
		}
	},

	uninitialize: function() {
		this._messageBox.destroyRecursive();
		this._artifactPickerPane.destroyRecursive();
		this.inherited(arguments);
	},
		
	_createButton: function(label, /*Boolean?*/isSubmit) {
		var button = dojo.create("button", {
			innerHTML: label,
			type: (isSubmit ? "submit" : "button")
		});
		return button;
	},

	show: function(/*Callback?*/onDialogClose) {
		var form = this;
		form.domNode.title = "";
		this._dialog = new com.ibm.rdm.web.common.Dialog({
			// maintain reference to the parent div that contains
			// the templates elements
			contentNode: this.domNode,
			primaryTitle: this.title,
			destroyContents: true,
			width: "804px",
			onOpen: function() {
				if (form.onDialogOpen) {
					form.onDialogOpen();
				}
			},
			onClose: function() {
				if (form.onDialogClose) {
					form.onDialogClose();
				}
			},
			onDialogClose: onDialogClose
		});
		this._artifactPickerPane.startup();
		this._artifactPickerPane.setFocusOnSearchBox();
	},

	onAddArtifacts: function(resources) {
		// summary:
		//		Callback to handle the case of the user asking for one or more
		//		artifacts to be added.
		// resources:
		//		An array of resource objects of type 
		//		com.ibm.rdm.web.model.resources.requirements.Requirement
		// tags:
		//		callback
	},
	
	showBusy: function(){
		this._isBusy = true;
		dojo.style(this._statusImageNode, "display", "inline");
		dojo.attr(this._addAndCloseButton, "disabled", true);
		dojo.attr(this._addButton, "disabled", true);
		dojo.attr(this._closeButton, "disabled", true);
	},
	
	hideBusy: function(){
		this._isBusy = false;
		dojo.style(this._statusImageNode, "display", "none");
		dojo.attr(this._addAndCloseButton, "disabled", false);
		dojo.attr(this._addButton, "disabled", false);
		dojo.attr(this._closeButton, "disabled", false);
	},
	
	selectArtifact : function(artifact, folderURI, projectURI) {
		var pickerPane = this._artifactPickerPane._artifactSelectionPane[projectURI];
		pickerPane._displaySingleArtifact(artifact, folderURI);
	},
	
	selectArtifactList : function(artifactList, folderURI, projectURI) {
		var pickerPane = this._artifactPickerPane._artifactSelectionPane[projectURI];
		pickerPane._displayArtifactList(artifactList, folderURI);
	},
	
	_handleItemSelected: function(/* Object */ selectedItem){
		// summary:
		//		Callback when exactly one item is selected.  This callback occurs after
		//		onArtifactSelectionChanged and passes along an object, not resource URIs.
	},
	
	displayInfoMessage: function(message, params) {
		// summary:
		//		Display an info (blue) message on the dialog.
		// message:
		//		String message which may contain substitution parameter indicators like ${0}.
		// params:
		//		Optional array of substitution parameters.
		if (params) {
			message = dojo.string.substitute(message, params);
		}
		this._messageBox.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: message,
			details: [],
			closable: true
		});
	},
	
	displayOkMessage: function(message, params) {
		// summary:
		//		Display an OK (green) message on the dialog.
		// message:
		//		String message which may contain substitution parameter indicators like ${0}.
		// params:
		//		Optional array of substitution parameters.
		if (params) {
			message = dojo.string.substitute(message, params);
		}
		this._messageBox.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: message,
			details: [],
			closable: true
		});
	},
	
	displayMessage: function(messageNumber, params){
		// summary:
		//		Displays a numbered error or warning message.
		// messageNumber:
		//		Number for a message, as defined in NumberedMessages
		// params:
		//		Optional substitution parameters to add to the message.
		var messageInfo = MessageHandler.getMessageBoxInfo(messageNumber, params);
		if(params.type === jazz.ui.MessageBox.STATUS_TYPES.ERROR) {
			this.errorState = true;
		}
		this._messageBox.showMessage(messageInfo);
	},
	
	hideMessage: function() {
		// summary:
		//		Hide the message box.
		this._messageBox.hideMessage();
	},
	
	///////////////////////////////////////////////////////////////////////////
	// Helpers

	_handleArtifactSelectionChanged: function(numArtifactsSelected) {		
		dojo.attr(this._addAndCloseButton, "disabled", (this._isBusy || numArtifactsSelected.length === 0));
		if(this.style == com.ibm.rdm.web.picker.ArtifactPickerForm.formStyle.ADD_CLOSE){
			dojo.attr(this._addButton, "disabled", (this._isBusy || numArtifactsSelected.length === 0));
		}		
	},

	_mixinAdditionalProperties: function (arrayOfIndexedPropertiesForSelectedArtifacts) {
		// summary:
		//		Invoked just prior to the dialogs callback.  Gives subclasses
		//		a chance to attach any properties they are additional for managing
		//		the collection of.
	},

	_onAddAndClose: function(event) {
		var arrayOfIndexedPropertiesForSelectedArtifacts =
			this._artifactPickerPane.getIndexedPropertiesForSelectedArtifacts();
		if (!this.onAddCallbackScope){
			this.onAddCallbackScope = this;
		}
		
		this._mixinAdditionalProperties(arrayOfIndexedPropertiesForSelectedArtifacts);
		
		this.onAddArtifacts.call(this.onAddCallbackScope, arrayOfIndexedPropertiesForSelectedArtifacts, event);
		if (!this.surrenderClose) {
			this._onOk(event);
		}
	},

	_onAdd: function(event) {
		dojo.stopEvent(event);
		var arrayOfIndexedPropertiesForSelectedArtifacts =
			this._artifactPickerPane.getIndexedPropertiesForSelectedArtifacts();
			
		if (!this.onAddCallbackScope){
			this.onAddCallbackScope = this;
		}
		this.onAddArtifacts.call(this.onAddCallbackScope, arrayOfIndexedPropertiesForSelectedArtifacts);
		if (this.autoDisableAddedEntries) {
			this._artifactPickerPane.removeSelectedArtifacts();
		} else {
			this._artifactPickerPane.clearSelection();
		}
	},

	_onClose: function(event) {
		this._onCancel(event);
	},
	
	_ok: function() {
		
		var doHide = dojo.hitch(this, function() {
			if(!this.errorState) {
				this.hide();
			}
		});

		dojo.attr(this._addAndCloseButton, "disabled", true);
		if (this._addButton) {
			dojo.attr(this._addButton, "disabled", true);
		}

		if (this.pauseOnAddAndClose && this._onAddAndCloseDisplayTimeout) {
			setTimeout(doHide, this._onAddAndCloseDisplayTimeout);
		}
		else {
			doHide();
		}
	}
});

})();

com.ibm.rdm.web.picker.ArtifactPickerForm.formStyle = {
		ADD_CLOSE: 0,
		OK_CANCEL: 1
};
