/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.query.ViewServiceClient");

dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");

dojo.require("com.ibm.rdm.web.calm.CalmConstants");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.query.artifactFormatUtils");
dojo.require("com.ibm.rdm.web.query.linkTypeUtils");
dojo.require("com.ibm.rdm.web.query.stringMatchUtils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.suspicion.model.types");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.locks.LockManager");

var artifactFormatUtils = com.ibm.rdm.web.query.artifactFormatUtils;
var linkTypeUtils = com.ibm.rdm.web.query.linkTypeUtils;
var stringMatchUtils = com.ibm.rdm.web.query.stringMatchUtils;
var queryutils = com.ibm.rdm.web.util.queryutils;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var CalmConstants = com.ibm.rdm.web.calm.CalmConstants;
//var ResourceManager = com.ibm.rdm.web.model.resources.ResourceManager;

var ARTIFACT_VARIABLE_NAME = "R1";
var LINKED_ARTIFACT_ROOT_NAME = "R";
var FIRST_LINKED_ARTIFACT_INDEX = 2;


com.ibm.rdm.web.query.ViewServiceClient = {
	// summary:
	//		This singleton is used to make requests for the "Views" service.

	///////////////////////////////////////////////////////////////////////////
	// Public API
	
	getViewContents: function(params) {
		// summary:
		//		Performs a request to the /rdm/views service.  This function
		//		returns a deferred.
		// params:
		//		A property list in the following format:
		//		{
		//			projectURI: <projectURI>,
		//			baselineURI: <baselineURI>,
		//			collectionURI: <collectionURI>,
		//			moduleURI: <moduleURI>,
		//			folderURIs: [ <folderURI>* ],
		//			tagOperator: <string>
		//			tagURIs: [ <tagURI>* ],
		//			fullTextCondition: 
		//			  {
		//				searchText: <lucene search string>,
		//				searchField: <one of ViewServiceClient.LUCENE_SEARCH_FIELDS>
		//			  },
		//			attributeConditionInfos:
		//			  [
		//				{
		//				  attributeDefinitionURI: <attributeDefinitionURI>
		//				  attributeType: <a type from RRMTypes.ATTRIBUTE_TYPES>
		//				  attributeValues: [ <operator>, <attributeValue> * ],
		//				  isEnum: <true or false>
		//				},
		//				...
		//			  ],
		//			linkedArtifactConditionInfos:
		//			  [
		//				{
		//				  linkConditionInfo:
		//					{
		//					  attributeValues: [ <operator>, <attributeValue> * ],
		//					},
		//				  artifactTypeConditionInfo:
		//					{
		//					   attributeValues: [ <attributeValue> * ],
		//					},
		//				  attributeConditionInfos:
		//					[
		//					  {
		//						attributeDefinitionURI: <attributeDefinitionURI>
		//						attributeType: <a type from RRMTypes.ATTRIBUTE_TYPES>
		//						attributeValues: [ <operator>, <attributeValue> * ],
		//						isEnum: <true or false>
		//					  },
		//					  ...
		//					]
		//				}
		//				...
		//			  ],
		//			artifactTypeConditionInfo:
		//			  {
		//				attributeValues: [ <attributeValue> * ],
		//			  },
		//			pageSize: <the page size to use>,
		//			forceNoCount: <true to force count=false>,
		//			currentPageDetails: details of the current page info to be asserted even though we are not asking for the count,
		//			isCoreView: <true only when not viewing a module>,
		//			pageRequested: <the page you want returned>,
		//			orderBy: an array of the attributes to order the results by, as objs like {def: <definition Uri>, order: <"a" or "d">}.
		//			displayLinks: true if the links for the objects are to be returned.
		//			includedLinks: an array of {uri: link type uri, dir: sub|obj} objects to include in the request. (Not yet Implemented)
		//			requestedAttributeUris: [] of the Attribute URIs you want in the response. These attributes will be returned as bound values. 
		//			requestedLinkAttributeUris: [] of the Attribute URIs you want in the response on Links. These attributes will be returned as bound values. 
		//			artifactUris: [] of the actual artifact URIs you want.
		//			capQueryExecutionTime: boolean flag to force query to stop running after a certain timeout - 36582: Cap query execution time
		//			multiValuesOnlyUri: if present, the uri of an attribute and only artifacts with multiple values for that attribute will be returned. 
		//			moduleBoundArtifact: Artifact which may be bound to a module to query. View service will only honor 1 (ebcraig - defect 65289)
		//			displayBaseProperties: true if "show base properties" mode in the saved view of a module
		//			processLockInfo: true if this request should ask for 'lock' information (and supply that direct to the LockManager)
		//			fetchLarge: true if this request should fetch the 'full' HTML primary text.
		//		
		//			expansionRules: an [] of rules, each of which is {
		//				sourceArtifacts: an [] of source ArtifactType URIs
		//				linkTypes: an [] of source LinkType URIs
		//				targetArtifacts: an [] of target ArtifactType URIs
		//			}. 
		//		  When expansionRules are provided, a drilldown query will be constructed using the artifactUris given (and which 
		//		  does NOT use the selection/filtering criteria provided to choose the rows). The count will be switched off for drilldown queries.
		//		Notes:
		//		- For details on the attributeConditionInfos, please see the
		//		  getConditionInfo function of such classes as StringConditionSection,
		//		  IntegerConditionSection, etc.
		//		- For details on the artifactTypeConditionInfo, please see the
		//		  getConditionInfo function of the ArtifactTypeConditionSection
		//		  class.
		//		- For details on the linkConditionInfo, please see the
		//		  getConditionInfo function of the LinkConditionSection class.
		// returns:
		//		This function returns a deferred.
		//		When completion is called on the deferred, the parameter will be an
		//		object with {
		//			resources: [] an array of resources, in the correct display order.
		//			uriOnly: true if you want the results as URIs only
		//			currentPage: the number of the current page
		//			maxPages: the total number of pages.
		//			pageSize: the pagesize used. If not provided, results are not paged.
		//			displayLinks: true if the links for the objects are to be returned.
		//		}
		var projectURI = params.projectURI;
		var uri = com.ibm.rdm.web.FRONTING_SERVICE_VIEWS + "?execute=true";
		var postData;
		if (!params.postData) {
			postData = this.composeViewsPostData(params);
		} else {
			postData = params.postData;
		}

		//We still need to add these 'command line' entries to the uri - others are added as 'ViewState'.
		//(page needs to be here as it is parsed out of the 'self' link uri in the results).
		var baselineURI = params.baselineURI;
		var uriOnly = params.uriOnly;
		var displayLinks = params.displayLinks;
		
		var displayBaseProperties = params.displayBaseProperties;
		
		var getFullObjects = !uriOnly;
		if (params.requestedAttributeUris && params.requestedAttributeUris.length > 0) {
			//If we are asking for attributes, then switch off full object
			getFullObjects = false;
		}
		if (params.summary) {
			//If we are asking for a summary drill down, then switch off full object
			getFullObjects = false;
		}
		uri += "&fullObject="+getFullObjects;

		if (params.pageSize) {
			uri += "&size=" + params.pageSize
				+ "&count=" + (params.forceNoCount || params.clmFilters ? "false" : "true");
		} else {
			// Need to explicitly state this, as the default behavior of the
			// view service is to perform the count query. See comments for
			// Defect 35265.
			uri += "&count=false";
		}
		
		if (params.offset) {
			uri += "&offset=" + params.offset;
		}
		
		if (params.pageRequested) {
			uri += "&page="+params.pageRequested ;
		}
		if (baselineURI) {
			uri += "&baseline="+encodeURIComponent(baselineURI);
		}
		if (displayLinks) {
			uri += "&displayLinks="+params.displayLinks ;
		}
		
		if (com.ibm.rdm.web.MODULE_SUPPORT) {
			if (params.disableModuleReuseMode || params.whereUsedConditionInfo || params.isCoreView) {
				uri += "&reuse=false";
			}
					
			if(params.disableModulefactBasePropertiesUnion || params.isCoreView){
					uri += "&extrinsicReuse=false";
			}
		}
		
		if(params.displayBaseProperties && !params.isCoreView) {
			uri += "&displayBaseProperties="+params.displayBaseProperties;
		}
		
		if (((params.moduleURI) && !(params.summary)) || params.fetchLarge) {
			uri += "&fetchLarge=true";
		}
		
		//if (params.multiValuesOnlyUri) {
		//	uri += "&multiValueFetch=false&minCount=2";
		//}
		
		//Links need to wait for indexing to happen
		var delayCallbackUntilDataIndexed = false;
		if(params.delayCallbackUntilDataIndexed === true){
			delayCallbackUntilDataIndexed = true;
		}
		
		var viewsDeferred = com.ibm.rdm.web.server.client.RDMClient.doPost({
			uri: uri,
			projectURI: projectURI,
			delayCallbackUntilDataIndexed: delayCallbackUntilDataIndexed,
			xhrArgs: {
				headers: {
					"Accept": "text/json"
				},
				handleAs: "json"
			},
			postData: postData,
			skipErrors: params.skipErrors,
			capQueryExecutionTime: params.capQueryExecutionTime,
			correlationData: {processLockInfo: params.processLockInfo, currentPageDetails: params.currentPageDetails, isModuleView: !params.isCoreView}
		});
		return viewsDeferred;
	},

	composeViewsPostData: function(viewRequestParams, /*optional*/args) {
		// summary:
		//		Create the RDF content to be POST'd to the views service.
		// viewRequestParams:
		//		Please see the documentation for getViewContents's 'params'
		//		argument.
		// args:
		//		Optional arguments that contain the following:
		//		   args = {
		//				title: "a title",
		//				description: "a description",
		//				scope: "public",
		//				singleModuleScope: true, //Denotes scope to a particular modules
		//				gridColumns: data from the grid
		//				gridColumns.expansionLevel: the expansion level set
		//			   }
		
		if (!viewRequestParams){
			return null;
		}
		
		if (!args) {
			args = {};
		}
		
		var artifactVariableName = ARTIFACT_VARIABLE_NAME;
		var rdfHeader = (
			// TODO Use constants like queryutils.RDF_QUALIFIER rather than
			// hardcoding them as we see here ... however, in the next milestone
			// we might be able to POST with JSON instead of RDF so I'm holding
			// off on this for now.
			'<rdf:RDF\n'
			+ '	xmlns:dcterms="http://purl.org/dc/terms/"\n'
			+ '	xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"\n'
			+ '	xmlns:rs="http://www.w3.org/2001/sw/DataAccess/tests/result-set#"\n'
			+ '	xmlns:rrmNav="http://com.ibm.rdm/navigation#"\n'
			+ '	xmlns:rrmViewdata="http://com.ibm.rdm/viewdata#"\n'
			+ '	xmlns:rt="' + com.ibm.rdm.web.FRONTING_SERVICE_VIEWS + '"\n'
			+ '	xmlns:rm="http://www.ibm.com/xmlns/rdm/rdf/"\n'
			+ '	xmlns:xsd="http://www.w3.org/2001/XMLSchema#"\n'
			+ '	xmlns:rql="http://www.ibm.com/xmlns/rdm/rql/"\n'
			+ '	xmlns:owl="http://www.w3.org/2002/07/owl#">\n'
			+ '  <rm:View rdf:about="">\n'
			+ '	<rm:rowquery rdf:parseType="Resource">\n'
			+ '	  <rql:select rdf:parseType="Resource">\n'
			+ '		<rdf:_1 rdf:parseType="Resource">\n'
			+ '		  <rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		</rdf:_1>\n'
		);
			
		var rdfPostSelectHeader = (
			'	  </rql:select>\n'
			+ '	  <rql:where rdf:parseType="Resource">\n'
		);
		var rdfWhereFooter = (
			'	  </rql:where>\n'
		);
		var rdfRowQueryFooter = (
			'	  </rm:rowquery>\n'
		);
		
		var rdfFooter = (
			  '	<rrmNav:scope>' + (args.scope ? args.scope : 'public') + '</rrmNav:scope>\n'
			+ '	<rm:ofType>GridView</rm:ofType>\n'				
			+ '	<dcterms:description>' + (args.description ? args.description : ' ') + '</dcterms:description>\n'
			+ '	<dcterms:title>' + (args.title ? args.title : 'Grid View 1') + '</dcterms:title>\n'
			+ '  </rm:View>\n'
			+ '</rdf:RDF>\n'
		);
		var buf = [];
		buf.push(rdfHeader);
		
		var numForRdfFragment = 1;
		
		var appendFragmentsAndBoostFragmentCounter = function(fragments) {
			for (var i = 0; i < fragments.length; i++) {
				buf.push(fragments[i]);
				numForRdfFragment++;
			}
		};
		
		if (com.ibm.rdm.web.MODULE_SUPPORT) {
			// If needed, add an attribute condition clause to filter out modulefacts.
			// We only do this if the query doesn't involve either a folder or a
			// collection (as the query is more performant without this clause and
			// this clause is not really needed for these two cases).
			if (viewRequestParams.isCoreView
				&& !(
					(viewRequestParams.folderURIs && viewRequestParams.folderURIs.length > 0)
					|| viewRequestParams.collectionURI
					|| viewRequestParams.whereUsedConditionInfo
					)
				) {
				if (!viewRequestParams.attributeConditionInfos) {
					viewRequestParams.attributeConditionInfos = [];
				}
				//Checking for parent folder existing is far faster than checking for module relationship NOT existing
				// 	-ebcraig
				viewRequestParams.attributeConditionInfos.push({
					isEnum: true,
					attributeDefinitionURI: RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Parent,
					attributeValues: [com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.Exists, true]
				});
			}
		}

		if (viewRequestParams.whereUsedConditionInfo) {
			viewRequestParams.requestedAttributeUris.push(
				com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title
			);
			viewRequestParams.requestedAttributeUris.push(
				com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier
			);
		}

		//Add in the attributes we want
		var rdfPart = 2;
		if (viewRequestParams.requestedAttributeUris) {
			for (var i = 0; i < viewRequestParams.requestedAttributeUris.length; i++) {
				var attribUri = viewRequestParams.requestedAttributeUris[i];
				buf.push('		<rdf:_' + rdfPart + ' rdf:parseType="Resource">\n');
				buf.push('		  <rql:object>' + artifactVariableName + '</rql:object>\n');
				if (dojo.isArray(attribUri)) {
					// We have an array of URIs to combine as a list in the field selector
					buf.push('		  <rql:field>\n');
					buf.push('		    <rdf:Seq>\n');
					for (var j = 0 ; j < attribUri.length ; j++) {
						buf.push('		      <rdf:li rdf:resource="' + attribUri[j] + '"/>\n');
					}
					buf.push('		    </rdf:Seq>\n');
					buf.push('		  </rql:field>\n');
				} else {
					// The normal case - a single URI
					// But for Module system attrs we need the base artifact's one
					if(!viewRequestParams.isCoreView && 
						(attribUri === com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Creator ||
						attribUri === com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Created ||
						attribUri === com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Contributor ||
						attribUri === com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ModifiedOn)) {
						
						buf.push('		  <rql:field>\n');
						buf.push('		    <rdf:Seq>\n');
						buf.push('		      <rdf:li rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/boundArtifact"/>\n');
						buf.push('		      <rdf:li rdf:resource="' + attribUri + '"/>\n');
						buf.push('		    </rdf:Seq>\n');
						buf.push('		  </rql:field>\n');						
					} else {
						buf.push('		  <rql:field rdf:resource="' + attribUri + '"/>\n');
					}
				}
				if (attribUri == com.ibm.rdm.web.suspicion.model.types.SUSPECT_ENTRIES &&
						viewRequestParams.suspicionProfileURIs &&
							viewRequestParams.suspicionProfileURIs.length > 0) {
					this._composeSuspicionHavingClause(artifactVariableName, viewRequestParams, buf);
				}
				buf.push('		</rdf:_' + rdfPart + '>\n');

				rdfPart = rdfPart + 1;
			}
		}
		
		//Get the lock information is requested
		if (viewRequestParams.processLockInfo) {
			// figure out if we are already asking for lock creator and or lock
			// creation date.  This will be the case if the locked by and/or locked on
			// columns are added to the view, if so there is no point adding them in twice.
			var arrays = dojo.filter(viewRequestParams.requestedAttributeUris, 
				function(thing) {
					return dojo.isArray(thing);
				}),
				alreadyAskingForLockCreator = false,
				alreadyAskingForLockCreated = false;
			
			dojo.forEach(arrays, function(array) {
				alreadyAskingForLockCreator = ((array.length == 2) 
						&& (array[0] === RRMTypes.RDF_LOCK)
						&& (array[1] === RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Creator));
						
				alreadyAskingForLockCreated	= ((array.length == 2) 
						&& (array[0] === RRMTypes.RDF_LOCK)
						&& (array[1] === RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Created));					
			});

			rdfPart = this._addLockingInfoRequest(rdfPart, buf, !!viewRequestParams.moduleURI, alreadyAskingForLockCreator, alreadyAskingForLockCreated);
		}
		
		//Add in the Link Attributes we want
		if (viewRequestParams.requestedLinkAttributeUris) {
			for (i = 0; i < viewRequestParams.requestedLinkAttributeUris.length; i++) {
				attribUri = viewRequestParams.requestedLinkAttributeUris[i];

				buf.push('		<rdf:_'+rdfPart+' rdf:parseType="Resource">\n');
				buf.push('		  <rql:object>L1</rql:object>\n');
				buf.push('		  <rql:field rdf:resource="'+attribUri+'"/>');
				buf.push('		</rdf:_'+rdfPart+'>\n');
				
				rdfPart = rdfPart + 1;
			}
		}
		
		buf.push(rdfPostSelectHeader);

		var projectURI = viewRequestParams.projectURI;
		var linkedArtifactConditionInfos = viewRequestParams.linkedArtifactConditionInfos;
		var queryDoesInvolveLinkFollowing =
			(linkedArtifactConditionInfos && linkedArtifactConditionInfos.length > 0);
		
		// Append direct resource requests
		var artifactsRequestedUris = viewRequestParams.artifactUris;
		if (artifactsRequestedUris && artifactsRequestedUris.length > 0) {
			//We are directly requesting the artifacts we need 
			buf.push(
				this._composeArtifactsRequestClause(
					artifactVariableName, numForRdfFragment, artifactsRequestedUris, args.relative));
			numForRdfFragment++;
		}
		
		// Append a tags clause if needed.
		var tagOperator = viewRequestParams.tagOperator;
		var tagURIs = viewRequestParams.tagURIs;
		if (tagURIs && tagURIs.length > 0) {
			buf.push(
				this._composeTagsClause(
					artifactVariableName, numForRdfFragment, tagOperator, tagURIs, args.relative));
			numForRdfFragment++;
		}
		// Append a folder clause if needed.
		var folderURIs = viewRequestParams.folderURIs;
		if (folderURIs && folderURIs.length > 0) {
			buf.push(
				this._composeFoldersClause(
					artifactVariableName, numForRdfFragment, folderURIs, args.relative));
			numForRdfFragment++;
		}
		// Append a collection clause if needed.
		var collectionURI = viewRequestParams.collectionURI;
		if (collectionURI) {
			buf.push(
				this._composeCollectionClause(
					artifactVariableName, numForRdfFragment, collectionURI, args.relative));
			numForRdfFragment++;
		}
		// Append a module clause if needed.
		var moduleURI = viewRequestParams.moduleURI;
		if (moduleURI && args.scope !== "modulePublic" && args.scope !== "modulePrivate") {
			buf.push(
				this._composeModuleClause(
					artifactVariableName, numForRdfFragment, moduleURI, args.relative));
			numForRdfFragment++;
		}
		var moduleBoundArtifact = viewRequestParams.moduleBoundArtifact;
		if (moduleBoundArtifact) {
			buf.push(
				this._composeModuleBoundArtifactsClause(
					artifactVariableName, numForRdfFragment, [moduleBoundArtifact], args.relative));
			numForRdfFragment++;
		}
		// Append a clause for artifact type if needed.
		var artifactTypeConditionInfo = viewRequestParams.artifactTypeConditionInfo;
		if (artifactTypeConditionInfo) {
			buf.push(this._composeArtifactTypeClause(artifactVariableName,
				numForRdfFragment, artifactTypeConditionInfo, args.relative));
			numForRdfFragment++;
		}
		// Append a clause for where used (i.e. core artifacts used in modules) if needed.
		var whereUsedConditionInfo = viewRequestParams.whereUsedConditionInfo;
		if (whereUsedConditionInfo) {
			buf.push(this._composeWhereUsedClause(
				artifactVariableName, numForRdfFragment, whereUsedConditionInfo));
			numForRdfFragment++;
		}

		// Check first for fullTextCondition
		if(viewRequestParams.fullTextCondition) {
			if(viewRequestParams.fullTextCondition.searchText) {
				var luceneSearchText = viewRequestParams.fullTextCondition
					.searchText;
				var luceneSearchField;
				if (viewRequestParams.fullTextCondition.searchField) {
					luceneSearchField =
						viewRequestParams.fullTextCondition.searchField;
				}
				var clauseFragment = 
					this._composeLuceneSearchClause(artifactVariableName,
						numForRdfFragment, luceneSearchText, luceneSearchField);
				appendFragmentsAndBoostFragmentCounter(clauseFragment);
			}
		}
		
		// Append clauses for attributes if needed (the "More attributes"
		// filter content is dealt with here).
		var attributeConditionInfos = viewRequestParams.attributeConditionInfos,
			attributeValues, 
			operator;
		
		if (attributeConditionInfos && attributeConditionInfos.length > 0) {
			
			for (i = 0; i < attributeConditionInfos.length; i++) {
				
				attributeValues = attributeConditionInfos[i].attributeValues;
				operator = attributeValues[0];
				
				if (this._isLockedByCurrentUserOperator(operator)) {
					appendFragmentsAndBoostFragmentCounter([this._composeLockedByCurrentUserClause(artifactVariableName, numForRdfFragment)]);
				} else {
					var clauseFragments =
						this._composeAttributeClause(
							artifactVariableName, numForRdfFragment,
							attributeConditionInfos[i], projectURI, !viewRequestParams.isCoreView);
					appendFragmentsAndBoostFragmentCounter(clauseFragments);
				}
			}
		}
		// Append 'link' relationship clauses if needed (the "Linked artifacts"
		// filter content is dealt with here).
		var linkedArtifactConditionInfo;
		if (queryDoesInvolveLinkFollowing) {
			// Append all the link clauses in one section.
			for (i = 0; i < linkedArtifactConditionInfos.length; i++) {
				linkedArtifactConditionInfo = linkedArtifactConditionInfos[i];
				var linkConditionInfo = linkedArtifactConditionInfo.linkConditionInfo;
				var linkedArtifactVariableName =
					(LINKED_ARTIFACT_ROOT_NAME + (FIRST_LINKED_ARTIFACT_INDEX + i));
				// TODO deal with "any" when we're supporting "any" for the link type.
				buf.push(
					this._composeLinkClauses(
						artifactVariableName, linkedArtifactVariableName,
						numForRdfFragment, linkConditionInfo
					)
				);
				numForRdfFragment++;
			}
			// Append clauses for constraining the linked artifacts just after the
			// link clauses.
			for (i = 0; i < linkedArtifactConditionInfos.length; i++) {
				linkedArtifactConditionInfo = linkedArtifactConditionInfos[i];
				artifactTypeConditionInfo = linkedArtifactConditionInfo.artifactTypeConditionInfo;
				attributeConditionInfos = linkedArtifactConditionInfo.attributeConditionInfos;
				linkedArtifactVariableName =
					(LINKED_ARTIFACT_ROOT_NAME + (FIRST_LINKED_ARTIFACT_INDEX + i));
				// Append a clause constraining the linked artifact's type.
				if (artifactTypeConditionInfo
					&& artifactTypeConditionInfo.attributeValues.length > 1
					) {
					// Please note that the above 'if' clause is appropriately
					// excluding the case of an artifactTypeConditionInfo in
					// which the attributeValues.length is 1 ... as this means
					// that the value is "(any)" (meaning we don't need to add
					// a query clause for this condition info).
					buf.push(
						this._composeArtifactTypeClause(
							linkedArtifactVariableName, numForRdfFragment,
							artifactTypeConditionInfo, args.relative));
					numForRdfFragment++;
				}
				// Append clauses for the linked artifact's attribute values.
				if (attributeConditionInfos) {
					for (j = 0; j < attributeConditionInfos.length; j++) {
						
						attributeValues = attributeConditionInfos[j].attributeValues;
						operator = attributeValues[0];
				
						if (this._isLockedByCurrentUserOperator(operator)) {
							appendFragmentsAndBoostFragmentCounter([this._composeLockedByCurrentUserClause(linkedArtifactVariableName, numForRdfFragment)]);
						} else {
							clauseFragments =
								this._composeAttributeClause(
									linkedArtifactVariableName, numForRdfFragment,
									attributeConditionInfos[j], projectURI, !viewRequestParams.isCoreView);
							appendFragmentsAndBoostFragmentCounter(clauseFragments);
						}
					}
				}
			}
		}
		
		buf.push(rdfWhereFooter);
		
		// Append clauses for sorting if needed.
		var orderBy = viewRequestParams.orderBy;
		if (orderBy && orderBy.length > 0) {
			buf.push(this._composeSortClause(artifactVariableName, orderBy, !viewRequestParams.isCoreView));
		}
		
		buf.push(rdfRowQueryFooter);
		buf.push(this._composeLinkQueryBlock(viewRequestParams));
		
		buf.push(this._composeViewStateBlock(viewRequestParams));

		if (viewRequestParams.displayLinks) {
			var linkedArtifactVariableName =
				(LINKED_ARTIFACT_ROOT_NAME + FIRST_LINKED_ARTIFACT_INDEX);
			buf.push(
				this._composeLinksClauses(
					artifactVariableName,
					linkedArtifactVariableName,
					viewRequestParams.ignoredLinkTypes));
		}


		//gridColumns is an optional argument used in the case of storing the view for later
		//recalling. For the immediate we 'convert' the simple names used for system attributes into URIs to make things
		//more friendly/understandable on server for the future.
		//This Conversion behavior should go away in 2011 and resultSetGridView should handle exact URIs appropriately.
		if (args.gridColumns || viewRequestParams.clmFilters){
			var currentId = 1;
			buf.push("<rrmViewdata:clientData rdf:parseType=\"Resource\">\n");
			if (args.gridColumns) {
				for (var column in args.gridColumns.columns) {
					var columnURI = args.gridColumns.columns[column].colURI;
					var traceAttributeData =
						args.gridColumns.columns[column].format;
					if (args.gridColumns.columns[column].colType === "s") {
						switch (args.gridColumns.columns[column].colURI) {
							case "id-col":
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier;
								break;
							case "name":
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title;
								break;
							case "description":
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Description;
								break;
							case "alternateSpelling":
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.AlternateSpelling;
								break;
							case "obj_type_col":
								columnURI = RRMTypes.WELL_KNOWN_OBJECT_TYPE;
								break;
							case "format":
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ArtifactFormat;
								break;
							case "mod-user":
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Contributor;
								break;
							case "creator":
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Creator;
								break;
							case "mod-date":
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ModifiedOn;
								break;
							case "created_date_col":
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Created;
								break;
							case "module-section-col":
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Section;
								break;
							case "imp-by":
								columnURI = com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["implementedBy"];
								break;
							case "track-by":
								columnURI = com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["trackedBy"];
								break;
							case "val-by":
								columnURI = com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["validatedBy"];
								break;
							case "aff-by":
								columnURI = com.ibm.rdm.web.calm.CalmConstants.CalmLinkTypeURI["affectedBy"];
								break;
							case "tags":
								columnURI = "http://com.ibm.rdm/navigation#tag";
								break;
							case com.ibm.rdm.web.artifactgrid.ArtifactConstants.MODULE_CONTENT_COL:
								columnURI = "http://com.ibm.rdm/viewdata#moduleContentColumn";
								break;
							case com.ibm.rdm.web.artifactgrid.ArtifactConstants.COMMENTS_COL:
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.CommentCount;
								break;
							case com.ibm.rdm.web.artifactgrid.ArtifactConstants.REUSEARTIFACTS_COL:
								columnURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ReuseCount;
								break;
							case com.ibm.rdm.web.artifactgrid.ArtifactConstants.SUSPICION_COL:
								if (viewRequestParams.suspicionProfileURIs.length === 0) {
									continue;
								}
								columnURI = com.ibm.rdm.web.suspicion.model.types.SUSPECT_ENTRIES;
								break;
							case "lockedBy":
								columnURI = RRMTypes.RDF_LOCKED_BY;
								break;
							case "lockedOn":
								columnURI = RRMTypes.RDF_LOCKED_ON;
								break;
							default:
								break;
						}
					}
					
					buf.push("<rdf:_" + currentId + " rdf:parseType=\"Resource\">\n");
					buf.push("<rrmViewdata:gridColumn rdf:parseType=\"Resource\">\n");
					buf.push("<rrmViewdata:colType rdf:datatype=\"http://www.w3.org/2001/XMLSchema#string\">" + args.gridColumns.columns[column].colType + "</rrmViewdata:colType>\n");
					buf.push("<rrmViewdata:colURI rdf:resource=\"" + columnURI + "\"/>\n");
					buf.push("<rrmViewdata:colWidth rdf:datatype=\"http://www.w3.org/2001/XMLSchema#int\">" + args.gridColumns.columns[column].width + "</rrmViewdata:colWidth>\n");

					// Format for trace attribute data:
					// traceAttributeData: {
					//   attributes: [ 
					//	   "http://localhost/attrUri1",
					//     ...
					//   ],
					//   showIcon: boolean,
					//   showTitle: boolean,
					//   showAttributeName: boolean
					// }

					// Only add this segment if there is at least one attribute
					if(traceAttributeData && traceAttributeData.attributes
							&& traceAttributeData.attributes.length) {

						var traceRDFSegId = 1;
						buf.push("<rrmViewdata:traceAttributeData rdf:parseType=\"Resource\">\n");
						// First, add the attribute segments:
						dojo.forEach(traceAttributeData.attributes,
							function(attributeURI) {
							
							buf.push("<rdf:_" + traceRDFSegId + " rdf:parseType=\"Resource\">\n");
							buf.push("<rrmViewdata:traceAttribute rdf:parseType=\"Resource\">\n");
							buf.push("<rrmViewdata:attrURI rdf:resource=\"");
							buf.push(attributeURI);
							buf.push("\"/>");
							buf.push("</rrmViewdata:traceAttribute>\n");
							buf.push("</rdf:_" + traceRDFSegId + ">\n");
							traceRDFSegId++;
						}, this);

						// Second, add the formatting segment
						buf.push("<rdf:_" + traceRDFSegId + " rdf:parseType=\"Resource\">\n");
						buf.push("<rrmViewdata:traceAttributeFormat rdf:parseType=\"Resource\">\n");
						buf.push("<rrmViewdata:showIcon rdf:datatype=\"http://www.w3.org/2001/XMLSchema#boolean\">");
						buf.push(!!traceAttributeData.showIcon + "");
						buf.push("</rrmViewdata:showIcon>\n");
						buf.push("<rrmViewdata:showTitle rdf:datatype=\"http://www.w3.org/2001/XMLSchema#boolean\">");
						buf.push(!!traceAttributeData.showTitle + "");
						buf.push("</rrmViewdata:showTitle>\n");
						buf.push("<rrmViewdata:showAttributeName rdf:datatype=\"http://www.w3.org/2001/XMLSchema#boolean\">");
						buf.push(!!traceAttributeData.showAttributeName + "");
						buf.push("</rrmViewdata:showAttributeName>\n");
						buf.push("</rrmViewdata:traceAttributeFormat>\n");
						buf.push("</rdf:_" + traceRDFSegId + ">\n");
						
						// Close the attribute data segment
						buf.push("</rrmViewdata:traceAttributeData>\n");
					}
					
					buf.push("</rrmViewdata:gridColumn>\n");
					buf.push("</rdf:_" + currentId + ">\n");
					currentId++;
				}
			}
			if (viewRequestParams.clmFilters){
				for (var calmFilter in viewRequestParams.clmFilters){
					var linkType = viewRequestParams.clmFilters[calmFilter].getLinkType();
					var filterId = viewRequestParams.clmFilters[calmFilter].getFilterId();
					
					for (var key in CalmConstants.CalmLinkTypeURI) {
						if (CalmConstants.CalmLinkTypeURI[key] == linkType) {
							filterId = CalmConstants.CalmLinkType[key] + "#" + filterId;
							break;
						}
					}
					
					buf.push("<rdf:_" + currentId + " rdf:parseType=\"Resource\">\n");
					buf.push("<rrmViewdata:clmFilter rdf:parseType=\"Resource\">\n");
					buf.push("<rrmViewdata:linkType rdf:resource=\"" + linkType + "\"/>\n");
					buf.push("<rrmViewdata:filterId rdf:datatype=\"http://www.w3.org/2001/XMLSchema#string\">" + filterId + "</rrmViewdata:filterId>\n");
					buf.push("</rrmViewdata:clmFilter>\n");
					buf.push("</rdf:_" + currentId + ">\n");
					currentId++;
				}
			}
			if (viewRequestParams.includedLinks && viewRequestParams.includedLinks.length > 0){
				for (var link in viewRequestParams.includedLinks) {
					var currentLink = viewRequestParams.includedLinks[link];
					var linkURI = currentLink.linkUri;
					
					buf.push("<rdf:_" + currentId + " rdf:parseType=\"Resource\">\n");
					buf.push("<rrmViewdata:relationship rdf:parseType=\"Resource\">\n");
					buf.push("<rrmViewdata:linkURI rdf:resource=\"" + linkURI + (currentLink.dir == "S" ? "/in" : "/out")+"\"/>\n");
					buf.push("</rrmViewdata:relationship>\n");
					buf.push("</rdf:_" + currentId + ">\n");
					currentId++;
				}
			}
			
			//Now preserve the expansionLevel
			if (args.gridColumns && (typeof args.gridColumns.expansionLevel != "undefined") && args.gridColumns.expansionLevel != null) {
				buf.push("<rdf:_" + currentId + " rdf:parseType=\"Resource\">\n");
				buf.push("<rrmViewdata:expansionLevel rdf:datatype=\"http://www.w3.org/2001/XMLSchema#int\">"+args.gridColumns.expansionLevel+"</rrmViewdata:expansionLevel>\n");
				buf.push("</rdf:_" + currentId + ">\n");
				currentId++;
			}

			buf.push("</rrmViewdata:clientData>\n");
		}

		if (args.singleModuleScope && moduleURI) {
			buf.push('<rm:module rdf:resource="' + moduleURI + '"/>');
		}
		// else if (!args.singleModuleScope && (args.scope === "modulePublic" || args.scope === "modulePrivate")) {
		// 	buf.push('<rm:module rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/module"/>');
		// }
		
		//Now preserve the extrinsic base properties of a module
		if(viewRequestParams.displayBaseProperties != undefined){
			buf.push("<rm:displayBaseProperties rdf:datatype=\"http://www.w3.org/2001/XMLSchema#boolean\">" + viewRequestParams.displayBaseProperties + "</rm:displayBaseProperties>\n");
		}
		
		buf.push(rdfFooter);
		return buf.join("");
	},

	getSetAttributeConditionInfosProperties: function(conditionInfos, projectURI, isModuleView) {
		// summary:
		//		Return an object that can be used to determine how to persist the current
		//		"Filter by Attribute" filters to JSON so that they can be subsequently
		//		resurrected.

		var titleAttributeURI = RRMTypes.getWellKnownAttributeURI(projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Title),
			modifiedOnAttributeURI = RRMTypes.getWellKnownAttributeURI(projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ModifiedOn),
			modifiedByAttributeURI = RRMTypes.getWellKnownAttributeURI(projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Contributor),
			attributes = [],
			users,
			dateRange,
			search;
		var conditionInfosToUse = dojo.map(
			conditionInfos,
			dojo.hitch(this, function(conditionInfo){
				var json = this._composeAttributeJson(conditionInfo, projectURI, isModuleView);
				if (conditionInfo.attributeDefinitionURI === titleAttributeURI) {
					search = dojo.clone(json);
				}
				else if (conditionInfo.attributeDefinitionURI === modifiedOnAttributeURI) {
					dateRange = dojo.clone(json);
				}
				else if (conditionInfo.attributeDefinitionURI === modifiedByAttributeURI) {
					users = dojo.clone(json);
				}
				else {
					attributes.push(dojo.clone(json));
				}
				return json;
			})
		);
		return {
			attributeConditionInfos: conditionInfosToUse,
			search: search,
			users: users,
			dateRange: dateRange,
			attributes: attributes
		};
	},
	
	_addLockingInfoRequest: function(rdfPart, buf, isModuleView, alreadyAskingForLockCreator, alreadyAskingForLockCreated) {
		
		if (!alreadyAskingForLockCreated) {
			this._addLockPart(rdfPart++, buf, "http://purl.org/dc/terms/created", isModuleView);
		}
		
		if (!alreadyAskingForLockCreator) {
			this._addLockPart(rdfPart++, buf, "http://purl.org/dc/terms/creator", isModuleView);
		}
		
		this._addLockPart(rdfPart++, buf, null, isModuleView);
		
		return rdfPart;
	},
	
	
	_addLockPart: function(rdfPart, buf, part, isModuleView) {

		buf.push('       <rdf:_'+rdfPart+' rdf:parseType="Resource">\n');
		if (part) {
			buf.push('       	<rql:field>\n');
			buf.push('       		<rdf:Seq>\n');
			if (isModuleView) {
				buf.push('       			<rdf:li rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/boundArtifact"/>\n');
			}
			buf.push('       			<rdf:li rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/lock"/>\n');	
			buf.push('       			<rdf:li rdf:resource="'+part+'"/>\n');
			buf.push('       		 </rdf:Seq>\n');
			buf.push('       	</rql:field>\n');
		} else {
			if (isModuleView) {
				buf.push('       	<rql:field>\n');
				buf.push('       		<rdf:Seq>\n');
				buf.push('       			<rdf:li rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/boundArtifact"/>\n');
				buf.push('       			<rdf:li rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/lock"/>\n');	
				buf.push('       		 </rdf:Seq>\n');
				buf.push('       	</rql:field>\n');
			} else {
				buf.push('			<rql:field rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/lock"/>\n');
			}
		}
		buf.push('       	<rql:object>R1</rql:object>\n');
		buf.push('       </rdf:_'+rdfPart+'>\n');
	},
	
	parseViewContentsResponse: function(response, projectURI, namedQueryAttributesMap, fullResults) {
		//Summary: handles the response from the View Service.
		//	If the response contains full object details, will pass back an array of Requirement (or sub-class) objects.
		//	If the response only includes specific attributes, will (at the moment) only pass back objects with the relevant attributes populated.
		// response: the response as received from the ViewService.
		// projectURI: the current projectURI to use.
		// namedQueryAttributesMap: a map of AttrName -> Attribute URI. If this exists, results will be as a PartialRequirement, not as
		//						full resources. If returned attributes on the results exist in this map, they will be returned as 
		//						primary attributes (eg result.AttrName).
		//						Note that for attributes on the result object (ie R1.<definition uri>) these
		//                      will be returned on the resource. For those found on a link (ie L1.<..>), they are currently
		//                      only returned for External and Calm links. 
		//						All other attributes found in the results will be available from the PartialRequirement getAttributeValue
		//						method using their attribute definition uri as key.
		//						NOTE: if not using named attributes, but still wanting partial requirements, then an empty namedQueryAttributeMap
		//						Must be provided.
		//
		// return: an object with {
		//			  resources: an array of the resources recieved, 
		//			  fromLinkDetails: a one-to-one array showing from which resource and through which link a resource was identified, for drill down.
		//			  currPage: the current page number,
		//			  lastPage: the last page of the current request,
		//			  nextPage: the next page number (Only needed for CLM filters),
		//			  nextPageOffset: the offset for the query to get the next page  (Only needed for CLM filters),
		//			  count: the number of items return
		//		 };

		var artifactVariableName = ARTIFACT_VARIABLE_NAME;
		var artifactVariableName2 = "R2";
	
		var results = response.doc.feed;
		
		if (fullResults == true || fullResults == undefined) {
			//Only set full results this way if it is not specified in arguments
			fullResults = true;
			
			if (namedQueryAttributesMap != null) {
				//If we are expecting attributes, so by default we are not expecting full objects
				fullResults = false;
			}
		}
		
		//Did the request include lockInfo?
		var processLockInfo = response.correlationData && response.correlationData.processLockInfo;
		
		var isModuleView = response.correlationData && response.correlationData.isModuleView;
		
		var returnedObjects = [];
		//This gives the link details for each object in the same order
		var fromLinkDetails = [];
		// hash of tag information.  key is resourceURI.  value is array of {uri: taguri, title: tagtitle, scope: "private"/"public"}. 
		var resourceTagDetails = {};
		var suspectEntryDetails = {};
		var clmFilterLinkDetails = {};
		var clmFilterLinkLabelsDetails = {};
		var projectURIs = [];
		var totalResultCount = 0;

		//Pick out the uri and either the xml, or specific attributes, from the returned results
		for (var key in results) {
			//Capture the total number of results for all pages
			if (key == "title") {				
				var queryResultValue = results[key].value;
				//Check if a count is in the string. This result is located at the end of the string, after the ":" character
				var stringToMatch = "Query Results:";
				var index = queryResultValue.lastIndexOf(stringToMatch);
				if(index != -1){
					totalResultCount = dojo.trim(queryResultValue.substring(index + stringToMatch.length));
				}				
			}
			var href, loc;
			if (key.substring(0,4) === "link" && results[key].xmlAttributes.rel === "last") {
				href = results[key].xmlAttributes.href;
				//Nasty, but we have to parse the href to get the last page
				loc = href.lastIndexOf("page=");
				//Get the number after here
				var pageStr = href.slice(loc+5);
				var lastPage = parseInt(pageStr);
			}
			if (key.substring(0,4) === "link" && results[key].xmlAttributes.rel === "self") {			
				href = results[key].xmlAttributes.href;
				
				//Nasty, but we have to parse the href to get the last page
				loc = href.lastIndexOf("page=");
				//Get the number after here
				//If there is no 'page=' in self, then we are on page 1
				var currPage;
				if (loc < 0) {
					currPage = 1;
				} else {
					var pageStr = href.slice(loc+5);
					currPage = parseInt(pageStr);
				}
			}
			if (key.substring(0,4) === "link" && results[key].xmlAttributes.rel === "next") {			
				href = results[key].xmlAttributes.href;
				loc = href.lastIndexOf("page=");
				var nextPage = null;
				if (loc >= 0) {
					var pageStr = href.slice(loc+5);
					nextPage = parseInt(pageStr);
				}
				
				loc = href.lastIndexOf("offset=");
				var nextPageOffset = null;
				if (loc >= 0) {
					var pageStr = href.slice(loc+7);
					nextPageOffset = parseInt(pageStr);
					if (isNaN(nextPageOffset)) {
						nextPageOffset = null;
					}
				}
			}
			if (key.substring(0,5)==="entry") {
				//We want to check it
				if (results[key].content && results[key].content.result) {
					var object = {};
					var etag = null;
					var linkDetails = {};
					// tag info for the object - each entry is {uri: taguri, title: tagtitle, scope: "public"/"private"}
					var tagDetails = [];
					//lockInfo for object, if any
					var lockInfo = {};
					var lockInfoFound = false;
					var suspectEntries = [];
					var clmFilterLinks = {};
					var clmFilterLinkLabels = {};
					var resourceContextURI = null;
					object._namedAttribs = {};
					
					for (var binding in results[key].content.result) {
						if (binding.substring(0,7) === "binding") {
							var bindVal = results[key].content.result[binding];
							if (bindVal.xmlAttributes) {
								if (bindVal.xmlAttributes.name === (artifactVariableName + "_obj")) {
									object.resourceXml = bindVal.literal.value;
								} else if (bindVal.xmlAttributes.name === artifactVariableName) {
									object.resourceURI = bindVal.uri.value;
								} else if (bindVal.xmlAttributes.name === artifactVariableName + "_resourceContext") {
									resourceContextURI = bindVal.uri.value;
									object.projectURI = bindVal.uri.value;
								} else if (bindVal.xmlAttributes.name === artifactVariableName2) {
									linkDetails.fromResourceURI = bindVal.uri.value;
								} else if (bindVal.xmlAttributes.name === "linkType") {
									linkDetails.linkType = bindVal.uri.value;
								} else if (bindVal.xmlAttributes.name === "linkDir") {
									linkDetails.linkDir = bindVal.literal.value; //'out' or 'in'
								}else if(bindVal.xmlAttributes.name === "L1.<http://www.w3.org/1999/02/22-rdf-syntax-ns#type>"){
									linkDetails.isDiscoveredLink = true;								
								} else if (bindVal.xmlAttributes.name === (artifactVariableName + "_ETag")) {
									object.etag = bindVal.literal.value.replace(/\"/g, ""); //Need to strip quotes from the etag
								} else if (bindVal.xmlAttributes.name.indexOf(artifactVariableName + '.<http://com.ibm.rdm/navigation#tag>') == 0) {
									this._addTagField(bindVal, tagDetails);
								} else if (bindVal.xmlAttributes.name.indexOf(artifactVariableName +
										'.<' + com.ibm.rdm.web.suspicion.model.types.SUSPECT_ENTRIES + '>') == 0) {
									this._addSuspectEntry(bindVal, suspectEntries);
								} else if (bindVal.xmlAttributes.name.indexOf(artifactVariableName +
										'.<http://com.ibm.rdm/viewdata#clmFilter>.') == 0) {
									this._addCLMFilterLink(bindVal, clmFilterLinks);
								} else if (bindVal.xmlAttributes.name.indexOf(artifactVariableName +
										'.<http://com.ibm.rdm/viewdata#clmFilterLabel>.') == 0) {
									this._addCLMFilterLinkLabel(bindVal, clmFilterLinkLabels);
								} else if (bindVal.xmlAttributes.name.indexOf('<http://www.ibm.com/xmlns/rdm/rdf/lock>') >= 0) {
									lockInfoFound = true;
									this._addLockInfoEntry(bindVal, lockInfo);
								} else if (isModuleView && bindVal.xmlAttributes.name.indexOf('http://www.ibm.com/xmlns/rdm/rdf/boundArtifact>.') >= 0) {
									//Check for system attributes from bound artifact
									this._decodeSystemAttributeEntry(bindVal, object);
								} else if (bindVal.xmlAttributes.name.substring(0,4) === (artifactVariableName + ".<") ||
												bindVal.xmlAttributes.name.substring(0,4) === ("L1.<")) {
									var value = bindVal.literal ? bindVal.literal.value : bindVal.uri.value;
									var name = bindVal.xmlAttributes.name;

									//Remove the <...> around the URL
									var valKey = name.substring(name.indexOf("<")+1, name.lastIndexOf(">"));
											
									var found = false;
									for (var desiredAttrib in namedQueryAttributesMap) {
										var attribUri = namedQueryAttributesMap[desiredAttrib];
										if (valKey === attribUri) {
											var currValue = object._namedAttribs[desiredAttrib];
											object._namedAttribs[desiredAttrib] = this._insertValueToArray(currValue, value);
											
											found = true;
										}
									}
									
									if (!found) {
										var currValue;
										
										//Process a module entry
										if(valKey === "http://www.ibm.com/xmlns/rdm/rdf/module"){
											//Add the module URI which is found if this artifact entry is re-used in
											//a module.
											if (com.ibm.rdm.web.MODULE_SUPPORT) {
												object[RRMTypes.WELL_KNOWN_MODULE_TYPE] = bindVal.uri.value;
											}
										}else{
											currValue = object[valKey];
											object[valKey] = this._insertValueToArray(currValue, value);
										}										
									}
								} else if (bindVal.xmlAttributes.name === "L1") {
									linkDetails.linkUri = bindVal.literal ? bindVal.literal.value : bindVal.uri.value;
								} else {
									//console.log("VSC No Match For", bindVal.xmlAttributes.name, bindVal.xmlAttributes);
								}
							}
						}
					}
					if (object.resourceURI === null || (fullResults && object.resourceXml === null)) {
						com.ibm.rdm.web.common.ErrorHandler.error("Unable to identify uri and xml in view response", object);
					} else if (object.resourceURI.charAt(0) == '-') {
						com.ibm.rdm.web.common.ErrorHandler.error("Illegal URI provided as target artifact:", object.resourceURI, " from: ", linkDetails.fromResourceURI, " Dir: ", linkDetails.linkDir, " linkType: ", linkDetails.linkType, " linkURI ", linkDetails.linkUri);
					} else {
						returnedObjects.push(object);
						fromLinkDetails.push(linkDetails);
						resourceTagDetails[object.resourceURI] = tagDetails;
						suspectEntryDetails[object.resourceURI] = suspectEntries;
						clmFilterLinkDetails[object.resourceURI] = clmFilterLinks;
						clmFilterLinkLabelsDetails[object.resourceURI] = clmFilterLinkLabels;
						projectURIs.push(resourceContextURI);

						if (processLockInfo) {
							var lockingResourceURI = object.resourceURI;
							var boundArtifactURI = object[RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BoundArtifact];
							if (boundArtifactURI) {
								//If we are in a module, we have a BoundArtifact - and that is the one we need to unlock
								lockingResourceURI = boundArtifactURI;
							}						
						
							if (lockInfoFound) {
								com.ibm.rdm.web.locks.LockManager.foundLockInfo(lockingResourceURI, lockInfo);
							} else {
								com.ibm.rdm.web.locks.LockManager.noLockFound(lockingResourceURI);
							}
						}
					}
				}
			}
		}
		
		if (processLockInfo) {
			com.ibm.rdm.web.locks.LockManager.getEtagsForMyFoundItems();
		}

		var resources = null;

		//Now parse the results if we are expecting full results.
		resources = [];
		var self = this;
		
		//Process backwards as we remove elements from corresponding arrays if there are issues
		for(var count = returnedObjects.length-1; count >= 0; count--) {
			var resource = returnedObjects[count];
			var projectURItoUse = projectURI || projectURIs[count];
			if (projectURItoUse === null) {
				//No resource context, so must be an external link
				resource.externalLink = true;
				resource.uri = resource.resourceURI;
				resources.push(resource);
			} else {
				try {
					var resourceObject;
					if (fullResults) {
						resourceObject = com.ibm.rdm.web.model.resources.ResourceManager.getFromXml(projectURItoUse, resource.resourceURI, resource.resourceXml, resource.revision);
					} else {
						resourceObject = com.ibm.rdm.web.model.resources.ResourceManager.getFromPartial(projectURItoUse, resource, isModuleView);
					}
					if (resourceObject) {
						// Add tags to the resource object - they don't really belong in a resource, but it is the
						// best way of ensuring they stay with the resource information.
						resourceObject.tags = resourceTagDetails[resource.resourceURI];
						resourceObject.suspectEntries = suspectEntryDetails[resource.resourceURI];
						resourceObject.clmFilterLinks = clmFilterLinkDetails[resource.resourceURI];
						resourceObject.clmFilterLinkLabels = clmFilterLinkLabelsDetails[resource.resourceURI];
						
						// successfully created full or partial resource.
						resources.push(resourceObject);
					}
				} catch (e) {
					//Error instantiating the resource - note it and move on.
					com.ibm.rdm.web.common.ErrorHandler.error("Cannot instantiate resource", resource, count);
					//Remove the corresponding linkdetails and projectURI
					fromLinkDetails.splice(count, 1);
					projectURIs.splice(count, 1);
					totalResultCount--;
				}
			}
		}
		
		//Reverse order of results, as we processed backwards
		resources.reverse();
			
		var results = {
			resources: resources,
			fromLinkDetails: fromLinkDetails,
			currPage: currPage,
			lastPage: lastPage,
			nextPage: nextPage,
			nextPageOffset: nextPageOffset,
			count:totalResultCount
		}; 
		
		return results;
	},
	
	getResourcesOrderedByOriginalRequestOrdering: function(resources, originalRequestUris) {
		// summary:
		//		Return an array that represents the 'resources', but ordered in such
		//		a manner that they match the ordering found in 'originalRequestUris'.
		// description:
		//		When you call the getViewContents method AND pass an 'artifactUris'
		//		property to it, when the resulting deferred fires, the passed 'resources'
		//		property won't necessarily be ordered in the same manner as the
		//		'artifactUris' property that you passed.  This method can be used to
		//		patch things up by providing you with resources ordered so they match
		//		the original ordering (found in 'originalRequestUris').

		var orderedResources = [],
			mapUriToResource = {},
			i;
		// Fill mapUriToResource.
		for (i = 0; i < resources.length; i++) {
			var resource = resources[i];
			mapUriToResource[resource.getResourceURI()] = resource;
		}
		// Fill orderedResources.
		for (i = 0; i < originalRequestUris.length; i++) {
			var uri = originalRequestUris[i];
			orderedResources.push(mapUriToResource[uri]);
		}
		return orderedResources;
	},

	_insertValueToArray: function(currentValue, valueToAdd) {
		var newValue = currentValue;
		if (currentValue) {
			if (dojo.isArray(currentValue)) {
				currentValue.push(valueToAdd);
			} else {
				newValue = [currentValue, valueToAdd];
			}
		} else {
			newValue = valueToAdd;
		}
		return newValue;
	},
	
	_addTagField: function(bindVal, tagDetails) {
		// summary:
		//		Add a tag field into the tag details array.  Called once for each binding
		//		that looks like it relates to tag information, so the fields for each tag
		//		are accumulated over several calls as the bindings are traversed.  If there
		//		are more than one tag, the fields for each tag are identified by binding names
		//		with a _1, _2 etc. suffix.  The fields for each tag are added to the array at
		//		the index corresponding to this suffix.  There is no _0 suffix, so absence of
		//		the suffix is treated as _0.
		// bindVal:
		//		A binding with a name and value.
		// tagDetails:
		//		Array to which entries are added for the tags.
		
		// Starting with something like: "R1.<http://com.ibm.rdm/navigation#tag>.<http://purl.org/dc/terms/title>_1"
		// or "R1.<http://com.ibm.rdm/navigation#tag>" (for the tag URI)
		var name = bindVal.xmlAttributes.name;
		var index = name.indexOf('>.<');
		var fieldUri, suffix;
		if (index < 0) {
			// no second field - this field is the tag URI
			// leave fieldUri undefined
			index = name.lastIndexOf('>');
			suffix = name.substring(index+1);
		} else {
			var tagTrailingField = name.substring(index+3);			// e.g.  "http://purl.org/dc/terms/title>_1"
			index = tagTrailingField.indexOf('>');
			var fieldUri = tagTrailingField.substring(0, index);	// e.g. "http://purl.org/dc/terms/title"
			var suffix = tagTrailingField.substring(index+1);		// e.g. "_1"
		}
		
		// now set index to the array index to use in tagDetails, and select or create the correct entry in tagDetails array
		var index = (suffix.length == 0) ? 0 : suffix.substring(1);
		
		var tagEntry = tagDetails[index];
		if (!tagEntry) {
			// first field for this tag - create a new entry
			tagEntry = {};
			tagDetails[index] = tagEntry;
		}

		// Set the appropriate field in tagEntry, based on fieldUri	
		if (fieldUri) {
			switch (fieldUri) {
				case "http://purl.org/dc/terms/title":
					tagEntry.title = bindVal.literal.value;
					break;
				case "http://com.ibm.rdm/navigation#scope":
					tagEntry.scope = bindVal.literal.value;
					break;
				case "http://www.w3.org/1999/02/22-rdf-syntax-ns#subject":
					tagEntry.subject = bindVal.uri.value;
					break;	
					
			}
		} else {
			tagEntry.uri = bindVal.uri.value;
		}
	},
	
	_addSuspectEntry: function(bindVal, suspectEntries) {
		var value = bindVal.literal ? bindVal.literal.value : bindVal.uri.value;
		suspectEntries.push(value);
	},
	
	_addCLMFilterLink: function(bindVal, clmFilterLinks) {
		var value = bindVal.literal ? bindVal.literal.value : bindVal.uri.value;
		var regExp = /<http:\/\/com.ibm.rdm\/viewdata#clmFilter>.<([^>]+)>/g;
		var results = regExp.exec(bindVal.xmlAttributes.name);
		if (results && results.length == 2) {
			var linkType = results[1];
			if (!clmFilterLinks[linkType]) {
				clmFilterLinks[linkType] = [];
			}
			clmFilterLinks[linkType].push(value);
		}
	},
	
	_addCLMFilterLinkLabel: function(bindVal, clmFilterLinkLabels) {
		var value = bindVal.literal ? bindVal.literal.value : bindVal.uri.value;
		var regExp = /<http:\/\/com.ibm.rdm\/viewdata#clmFilterLabel>.<([^>]+)>/g;
		var results = regExp.exec(bindVal.xmlAttributes.name);
		if (results && results.length == 2) {
			var uri = results[1];
			clmFilterLinkLabels[uri] = value;
		}
	},
	
	_addLockInfoEntry: function(bindVal, lockInfo) {
		//Parses one of
		//"R1.<http://www.ibm.com/xmlns/rdm/rdf/lock>.<http://purl.org/dc/terms/created>"
		//"R1.<http://www.ibm.com/xmlns/rdm/rdf/lock>.<http://www.ibm.com/xmlns/rdm/rdf/persistent>"
		//"R1.<http://www.ibm.com/xmlns/rdm/rdf/lock>.<http://purl.org/dc/terms/creator>"
		// or the same but with
		//"R1.<http://www.ibm.com/xmlns/rdm/rdf/boundArtifact>.<http://www.ibm.com/xmlns/rdm/rdf/lock>...
		if (bindVal.xmlAttributes.name.indexOf("http://purl.org/dc/terms/created") > 0) {
			lockInfo.utcDate = bindVal.literal.value;
		} else if (bindVal.xmlAttributes.name.indexOf("http://purl.org/dc/terms/creator") > 0) {
			lockInfo.userURI = bindVal.uri.value;
		} else {
			//Assumed to be the URI
			lockInfo.lockURI = bindVal.uri.value;
		}
	},	
	
	_decodeSystemAttributeEntry: function(bindVal, object) {
		//Parses one of
		//"R1.<http://www.ibm.com/xmlns/rdm/rdf/boundArtifact>.<http://purl.org/dc/terms/created>"
		//"R1.<http://www.ibm.com/xmlns/rdm/rdf/boundArtifact>.<http://purl.org/dc/terms/contributor>"
		//"R1.<http://www.ibm.com/xmlns/rdm/rdf/boundArtifact>.<http://purl.org/dc/terms/creator>"
		//"R1.<http://www.ibm.com/xmlns/rdm/rdf/boundArtifact>.<http://purl.org/dc/terms/modified>"
		if (bindVal.xmlAttributes.name.indexOf("http://purl.org/dc/terms/created") > 0) {
			object["http://purl.org/dc/terms/created"] = bindVal.literal.value;
		} else if (bindVal.xmlAttributes.name.indexOf("http://purl.org/dc/terms/contributor") > 0) {
			object["http://purl.org/dc/terms/contributor"] = bindVal.uri.value;
		} else if (bindVal.xmlAttributes.name.indexOf("http://purl.org/dc/terms/creator") > 0) {
			object["http://purl.org/dc/terms/creator"] = bindVal.uri.value;
		} else if (bindVal.xmlAttributes.name.indexOf("http://purl.org/dc/terms/modified") > 0) {
			object["http://purl.org/dc/terms/modified"] = bindVal.literal.value;
		} 
	},
	
	parseViewSummaryResponse: function(response) {
		//Summary: handles the response from the View Service when it is a drilldown summary.
		// response: the response as received from the ViewService.
		//
		// return: an object with {
		//			  resources: a map of {resource uri -> [{linkType: <the type>, linkDir: <in|out>, count: <the number>}]
		//		   }

		var artifactVariableName = ARTIFACT_VARIABLE_NAME;
		
		var results = response.doc.feed;		
		var resultMap = {};

		//Pick out the uri and either the xml, or specific attributes, from the returned results
		for (var key in results) {
			if (key.substring(0,5)==="entry") {
				//We want to check it
				if (results[key].content && results[key].content.result) {
					var resourceURI = null;
					var linkDetails = {};
					
					for (var binding in results[key].content.result) {
						if (binding.substring(0,7) === "binding") {
							var bindVal = results[key].content.result[binding];
							if (bindVal.xmlAttributes && bindVal.xmlAttributes.name
										=== artifactVariableName) {
								resourceURI = bindVal.uri.value;
							} else if (bindVal.xmlAttributes && bindVal.xmlAttributes.name
										=== "linkType") {
								linkDetails.linkType = bindVal.uri.value;
							} else if (bindVal.xmlAttributes && bindVal.xmlAttributes.name
										=== "linkDir") {
								linkDetails.linkDir = bindVal.literal.value; //'out' or 'in'
							} else if (bindVal.xmlAttributes && bindVal.xmlAttributes.name
										=== "count") {
								linkDetails.count = bindVal.literal.value;
							}
						}
					}
					
					var resourceArray = resultMap[resourceURI];
					if (!resourceArray) {
						resourceArray = [];
						resultMap[resourceURI] = resourceArray;
					}
					resourceArray.push(linkDetails);
				}
			}
		}

		
		var results = {resources: resultMap};
		
		return results;
	},
	
	///////////////////////////////////////////////////////////////////////////
	// Private API
	
	_composeLinkQueryBlock: function(params) {
		//Summary: This composes the linkQuery block, used only in drill down requests, and queries a set of rows for the links they have.
		//		 Only drillDown queries have non-null expansionRules.
		// params.expansionRules are 
		var buf = [];

		var rules = params.expansionRules;
		if (rules) {
			if (rules.length === 0) {
				//No rules - so just get every linktype we have.
				buf.push('<rm:linkquery rdf:parseType="Resource"><rql:where rdf:parseType="Resource">\n');
				buf.push('	<rdf:_1 rdf:parseType="Resource">\n');
				buf.push('	  <rql:from>R1</rql:from>\n');
				buf.push('	  <rql:to>R2</rql:to>\n');
				buf.push('	  <rql:op>links</rql:op>\n');
				buf.push('	</rdf:_1>\n');
				buf.push('</rql:where></rm:linkquery>\n');
			} else {
				//We need to get
				buf.push('<rm:linkquery rdf:parseType="Resource"><rql:where rdf:parseType="Resource">\n');
				buf.push('	<rdf:_1 rdf:parseType="Resource">\n');
				buf.push('	  <rql:from>R1</rql:from>\n');
				buf.push('	  <rql:to>R2</rql:to>\n');
				buf.push('	  <rql:op>links</rql:op>\n');
				buf.push('	  <rql:opts>\n');

				buf.push('	 <rdf:Seq>\n');				
				//Add each rule
				var defaultDefined = false;
				for(var i=0; i<rules.length; i++) {
					buf.push('	 <rdf:li rdf:parseType="Resource">\n');				
					defaultDefined = this._addOnRulePart(buf, rules[i], defaultDefined, params.summary);
					buf.push('	 </rdf:li>\n');
				}
				//Now add a 'default' rule to switch off all links not covered by these rules - if no other default defined
				if (!defaultDefined) {
					buf.push('	 <rdf:li rdf:parseType="Resource">\n');				
					this._addOnRulePart(buf, {sourceArtifacts: [], targetArtifacts: [], 
							linkTypes: ['http://com.ibm.rdm/unmatchable_link#subject'] }, false);
					buf.push('	 </rdf:li>\n');
				}
				buf.push('	 </rdf:Seq>\n');
				
				buf.push('	</rql:opts>\n');
				buf.push('	</rdf:_1>\n');
				buf.push('</rql:where></rm:linkquery>\n');
			}
		}
		
		var result = buf.join("");
		return result;	
	},
	
	_addOnRulePart: function(buf, rule, defaultDefined, isSummary) {
		//Summary: adds a rule to the server RDF.
		//	buf: the buffer array used to concatenate the request string.
		//   rule: a rule object, consisting of 
		//			{sourceArtifacts: [] of source type uris, 
		//			 linkTypes: [] of link type URIs with #object and #subject suffixes, 
		//			 targetArtifacts: [] of source type uris}
		//			The artifacts can be null (or an empty array), which means *ANY*.
		//			The linkTypes - at the moment - must have at least one linkType.
		//   defaultDefined: true if the default rule - the one with no source/target artifacts - has already been defined.
		//   isSummary: true if this is the summary rule
		
		var hasSource = rule.sourceArtifacts && rule.sourceArtifacts.length > 0;
		var hasTarget = rule.targetArtifacts && rule.targetArtifacts.length > 0;
		var hasLinks = rule.linkTypes && rule.linkTypes.length > 0;
		if (!hasLinks) {
			com.ibm.rdm.web.common.ErrorHandler.error("A Rule must have linktypes defined", rule);
			return;
		}
		var hasArtifactTypes = hasSource || hasTarget;
		if (!hasArtifactTypes) {
			if (defaultDefined) {
				com.ibm.rdm.web.common.ErrorHandler.error("We cannot have two default link rules", rule);
				return;
			} else {
				defaultDefined = true;
			}
		}
		
		var sourceR, targetR;
		if (isSummary) {
			//Reverse the direction for the summary
			sourceR = 'R1';
			targetR = 'R2';
		} else {
			sourceR = 'R2';
			targetR = 'R1';
		}
		
		buf.push('<rdf:value rdf:parseType="Resource">\n');
		if (hasArtifactTypes) {
			buf.push(' <rql:then rdf:parseType="Resource">\n');
		}
		buf.push('  <rql:e1>\n');
		buf.push('	<rdf:Seq>\n');
		
		//Add in the types 
		for (var i = 0; i < rule.linkTypes.length; i++) {
			var uri = rule.linkTypes[i];
			var uriBits = uri.split('#');
			var uriDir;
			var linkName = uriBits[0];
			var bit = 1;
			while ((bit+1) < uriBits.length) {
				linkName+='#'+uriBits[bit];
				bit++;
			}
			
			if (isSummary) {
				//Reverse the direction for the summary
				uriDir = (uriBits[1]!=="object")?"out":"in";
			} else {
				uriDir = (uriBits[1]==="object")?"out":"in";
			}
			
			buf.push('	  <rdf:li rdf:parseType="Resource">\n');
			buf.push('		<rql:linktype rdf:resource="'+linkName+'"/>\n');
			buf.push('		<rql:linkdir>'+uriDir+'</rql:linkdir>\n');
			buf.push('	  </rdf:li>\n');
		}
		
		buf.push('	</rdf:Seq>\n');
		buf.push('  </rql:e1>\n');
		buf.push('  <rql:op>roles</rql:op>\n');
		if (hasArtifactTypes) {
			buf.push('</rql:then>\n');
		}
		
		//Add the source and target types (R2 is source, R1 is target)
		if (hasTarget && hasSource) {
			buf.push('   <rql:if rdf:parseType="Resource">\n');
			
			//R2 is in fact the source rows
			buf.push('	<rdf:_2 rdf:parseType="Resource">\n');
			buf.push('	 <rql:e1>'+sourceR+'</rql:e1>\n');
			buf.push('	 <rql:e2>\n');
			buf.push('	  <rdf:Seq>\n');
			//Write the source types
			for (var i = 0; i < rule.sourceArtifacts.length; i++) {
				buf.push('		<rdf:li rdf:resource="'+rule.sourceArtifacts[i]+'"/>\n');
			}
			buf.push('	  </rdf:Seq>\n');
			buf.push('	 </rql:e2>\n');
			buf.push('	 <rql:op>is</rql:op>\n');
			buf.push('	</rdf:_2>\n');
			
			//And R1 really is our target rows - because that is what we specify the results in terms of.
			buf.push('	<rdf:_1 rdf:parseType="Resource">\n');
			buf.push('	 <rql:e1>'+targetR+'</rql:e1>\n');
			buf.push('	 <rql:e2>\n');
			buf.push('	  <rdf:Seq>\n');
			//Write the target types
			for (var i = 0; i < rule.targetArtifacts.length; i++) {
				buf.push('		<rdf:li rdf:resource="'+rule.targetArtifacts[i]+'"/>\n');
			}
			buf.push('	  </rdf:Seq>\n');
			buf.push('	 </rql:e2>\n');
			buf.push('	 <rql:op>is</rql:op>\n');
			buf.push('	</rdf:_1>\n');
	
			buf.push('	<rql:op>and</rql:op>\n');
			buf.push('   </rql:if>\n');
		} else if (hasTarget) {
			buf.push('   <rql:if rdf:parseType="Resource">\n');
 
			buf.push('	<rdf:_1 rdf:parseType="Resource">\n');			
			buf.push('	 <rql:e1>'+targetR+'</rql:e1>\n');
			buf.push('	 <rql:e2>\n');
			buf.push('	  <rdf:Seq>\n');
			//Write the target types
			for (var i = 0; i < rule.targetArtifacts.length; i++) {
				buf.push('		<rdf:li rdf:resource="'+rule.targetArtifacts[i]+'"/>\n');
			}
			buf.push('	  </rdf:Seq>\n');
			buf.push('	 </rql:e2>\n');
			buf.push('	 <rql:op>is</rql:op>\n');
			buf.push('	</rdf:_1>\n');			
			buf.push('	<rql:op>and</rql:op>\n');
			buf.push('   </rql:if>\n');
		} else if (hasSource) {
			buf.push('   <rql:if rdf:parseType="Resource">\n');
			
			buf.push('	<rdf:_1 rdf:parseType="Resource">\n');			
			buf.push('	 <rql:e1>'+sourceR+'</rql:e1>\n');
			buf.push('	 <rql:e2>\n');
			buf.push('	  <rdf:Seq>\n');
			//Write the source types
			for (var i = 0; i < rule.sourceArtifacts.length; i++) {
				buf.push('		<rdf:li rdf:resource="'+rule.sourceArtifacts[i]+'"/>\n');
			}
			buf.push('	  </rdf:Seq>\n');
			buf.push('	 </rql:e2>\n');
			buf.push('	 <rql:op>is</rql:op>\n');
			buf.push('	</rdf:_1>\n');
			buf.push('	<rql:op>and</rql:op>\n');
				
			buf.push('   </rql:if>\n');
		}

		if (hasArtifactTypes) {
			buf.push('  <rql:op>ifthen</rql:op>\n');
		}
		buf.push(' </rdf:value>\n');
		buf.push('<rql:type>constraintLinkOption</rql:type>\n');
		
		return defaultDefined;
	},
	
	
	_composeViewStateBlock: function (params) {
		//Summary: This composes the parameters of the request into a separate block POSTed with the request, rather than using ?x=y items on the URL.
		// params.expansionRules are 
		var buf = [];
		
		if ((params.expansionRules && params.artifactUris) || params.multiValuesOnlyUri) {

			//We only need to add the requested artifacts are to the view state if we are doing drill down.
			buf.push('<rm:viewState rdf:parseType="Resource"><rm:params><rdf:Seq>\n');
		
			if (params.artifactUris) {
				this._addListItem(buf, "selObject", params.artifactUris);
			}
			if (params.summary || params.multiValuesOnlyUri) {
				this._addListItem(buf, "summary", "true");
			}
			if (params.multiValuesOnlyUri) {
				this._addListItem(buf, "multiValueFetchEnabled", "false");
				this._addListItem(buf, "minCount", "2");
			}
			if (!params.multiValuesOnlyUri) {
				this._addListItem(buf, "count", "false");
			}
			buf.push('</rdf:Seq></rm:params></rm:viewState>\n');
			
		}
		
		var result = buf.join("");
		return result;
	},
	
	_addListItem: function(buf, key, value) {
		buf.push(' <rdf:li rdf:parseType="Resource">\n<rdf:value>');
		if (dojo.isArray(value)) {
			  //<rdf:Seq>
			  //  <rdf:li>https://tjhn-ibm-laptop:9443/rm/resources/_OGzWgegNEd-ONbru39XHaA</rdf:li>
			  //  <rdf:li>https://tjhn-ibm-laptop:9443/rm/resources/_O0jNUegNEd-ONbru39XHaA</rdf:li>
			  //</rdf:Seq>
			buf.push('  <rdf:Seq>\n');
			for (var i = 0; i < value.length; i++) {
				buf.push('   <rdf:li>'+value[i]+'</rdf:li>\n');
			}
			buf.push('  </rdf:Seq>\n');			
		} else {
			buf.push(value);
		}
		buf.push('</rdf:value>\n<dcterms:name>'+key+'</dcterms:name>\n');
		buf.push(' </rdf:li>\n');
	},

	_composeArtifactsRequestClause:  function(
			artifactVariableName, numForRdfFragment, artifactURIs) {
		var buf = [];
		buf.push('		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n');
		buf.push('		  <rql:e1 rdf:parseType="Resource">\n');
		buf.push('		  <rql:object>' + artifactVariableName + '</rql:object>\n');
		buf.push('		  </rql:e1>\n');
		buf.push('		  <rql:e2>\n');
		buf.push('			<rdf:Seq>\n');
		
		for (var i = 0; i < artifactURIs.length; i++) {
			buf.push('			<rdf:li rdf:resource="' + artifactURIs[i]+ '"/>\n');
		}
		
		buf.push('			</rdf:Seq>\n');
		buf.push('		  </rql:e2>\n');
		buf.push('		  <rql:op>in</rql:op>\n');
		buf.push('		  <rql:all>false</rql:all>\n');
		buf.push('		</rdf:_' + numForRdfFragment + '>\n');

		var result = buf.join("");
		return result;
	},
	
	_composeTagsClause: function(
			artifactVariableName, numForRdfFragment, tagOperator, tagURIs) {
		var liFragmentsString = "";
		var allClause = (
			(
				tagOperator === com.ibm.rdm.web.query.ViewServiceClient.TAG_OPERATORS.IsAllOf
				|| tagOperator === com.ibm.rdm.web.query.ViewServiceClient.TAG_OPERATORS.IsNotAllOf
			) ?
			'		  <rql:all>true</rql:all>\n' :
			''
		);
		var notClause = (
			(
				tagOperator === com.ibm.rdm.web.query.ViewServiceClient.TAG_OPERATORS.IsNotAnyOf
				|| tagOperator === com.ibm.rdm.web.query.ViewServiceClient.TAG_OPERATORS.IsNotAllOf
			) ?
			'		  <rql:not>true</rql:not>\n' :
			''
		);
		for (var i = 0; i < tagURIs.length; i++) {
			liFragmentsString += '			  <rdf:li rdf:resource="' + tagURIs[i]+ '"/>\n';
		}
		return (
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ '		  <rql:e1 rdf:parseType="Resource">\n'
			+ '			<rql:field rdf:resource="http://com.ibm.rdm/navigation#tag"/>\n'
			+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  </rql:e1>\n'
			+ '		  <rql:e2>\n'
			+ '			<rdf:Seq>\n'
			+ liFragmentsString
			+ '			</rdf:Seq>\n'
			+ '		  </rql:e2>\n'
			+ '		  <rql:op>in</rql:op>\n'
			+ notClause
			+ allClause
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		);
	},

	_composeFoldersClause: function(
			artifactVariableName, numForRdfFragment, folderURIs) {
		// TODO support >1 URI in folderURIs.
		
		var folders = [];
		folders.push(
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ '		  <rql:e1 rdf:parseType="Resource">\n'
			+ '			<rql:field rdf:resource="http://com.ibm.rdm/navigation#parent"/>\n'
			+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  </rql:e1>\n'
			+ '		  <rql:e2>\n'
			+ '			<rdf:Seq>\n');
			dojo.forEach(folderURIs, function(thisFolderURI){
				folders.push('			  <rdf:li rdf:resource="' + thisFolderURI + '"/>\n');
			});
			
			folders.push('			</rdf:Seq>\n'
			+ '		  </rql:e2>\n'
			+ '		  <rql:op>in</rql:op>\n'
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		);
		
		return folders.join('');
	},

	_composeCollectionClause: function(
			artifactVariableName, numForRdfFragment, collectionURI) {
		return (
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ '		  <rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  <rql:collection rdf:resource="' + collectionURI + '"/>\n'
			+ '		  <rql:op>member</rql:op>\n'
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		);
	},
	
	_composeModuleClause: function(
			artifactVariableName, numForRdfFragment, moduleURI) {
		return (
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ '			<rql:op>=</rql:op>\n'
			+ '			<rql:e1 rdf:parseType="Resource">\n'
			+ '				<rql:field rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/module"/>\n'
			+ '		  		<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  	</rql:e1>\n'
			+ '		  	<rql:e2>&lt;' + moduleURI + '&gt;</rql:e2>\n'
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		);
	},
	
	_composeModuleBoundArtifactsClause: function(
			artifactVariableName, numForRdfFragment, resourceURIs) {
		var folders = [];
		folders.push(
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ '		  <rql:e1 rdf:parseType="Resource">\n'
			+ '			<rql:field rdf:resource="' + RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BoundArtifact + '"/>\n'
			+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  </rql:e1>\n'
			+ '		  <rql:e2>\n'
			+ '			<rdf:Seq>\n');
			dojo.forEach(resourceURIs, function(thisResourceURI){
				folders.push('			  <rdf:li rdf:resource="' + thisResourceURI + '"/>\n');
			});
			
			folders.push('			</rdf:Seq>\n'
			+ '		  </rql:e2>\n'
			+ '		  <rql:op>in</rql:op>\n'
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		);
		
		return folders.join('');
	},

	_composeWhereUsedClause: function(
			artifactVariableName, numForRdfFragment, artifactFormatInstancesConditionInfo) {
		var attributeValues = artifactFormatInstancesConditionInfo.attributeValues;
		var operator = attributeValues[0];
		var allClause = (
			(
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAll
				|| operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAll
			) ?
			'		  <rql:all>true</rql:all>\n' :
			''
		);
		var notClause = (
			(
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAny
				|| operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAll
			) ?
			'		  <rql:not>true</rql:not>\n' :
			''
		);
		var array = [];
		array.push(
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ '		  <rql:e1 rdf:parseType="Resource">\n'
			+ '			<rql:field>\n'
			+ '				<rdf:Seq>\n'
			+ '					<rdf:li rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/binding"/>\n'
			+ '					<rdf:li rdf:resource="' + RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Module + '"/>\n'
			+ '				</rdf:Seq>\n'
			+ '			</rql:field>\n'
			+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  </rql:e1>\n'
			+ '		  <rql:e2>\n'
			+ '			<rdf:Seq>\n'
		);
		for (var i = 1; i < attributeValues.length; i++) {
			array.push('			  <rdf:li rdf:resource="' + attributeValues[i] + '"/>\n');
		}
		array.push('			</rdf:Seq>\n'
			+ '		  </rql:e2>\n'
			+ '		  <rql:op>in</rql:op>\n'
			+ notClause
			+ allClause
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		);
		return array.join('');
	},

	_composeArtifactTypeClause: function(
			artifactVariableName, numForRdfFragment, artifactTypeConditionInfo) {
		var attributeValues = artifactTypeConditionInfo.attributeValues;
		var operator = attributeValues[0];
		var notClause = ((
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNot ||
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAny ||
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAll
			) ?
			'		  <rql:not>true</rql:not>\n' :
			''
		);
		var allClause = ((
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAll || 
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAll 
			) ?
			'		  <rql:all>true</rql:all>\n' :
			''
		);
		var isOrInClause = ((
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAll || 
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAll || 
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAny ||
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAny
			) ?
			'		  <rql:op>in</rql:op>\n' :
			'		  <rql:op>is</rql:op>\n'
		);		
		var liFragmentsString = "";
		for (var i = 1; i < attributeValues.length; i++) {
			
			liFragmentsString +=
				'			  <rdf:li rdf:resource="' + attributeValues[i] + '"/>\n';
		}
		return (
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ '		  <rql:e1>' + artifactVariableName + '</rql:e1>\n'
			+ '		  <rql:e2>\n'
			+ '			<rdf:Seq>\n'
			+ liFragmentsString
			+ '			</rdf:Seq>\n'
			+ '		  </rql:e2>\n'
			+ notClause
			+ allClause
			+ '		  <rql:op>is</rql:op>\n'
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		);
	},
	
	_composeSuspicionHavingClause: function(artifactVariableName, viewRequestParams, buf) {
		buf.push('        <rql:having rdf:parseType="Resource">\n');
		buf.push('           <rdf:_1 rdf:parseType="Resource">\n');
		buf.push('              <rql:all>false</rql:all>\n');
		buf.push('              <rql:e1 rdf:parseType="Resource">\n');
		buf.push('                 <rql:field>\n');
		buf.push('                    <rdf:Seq>\n');
		buf.push('                       <rdf:li rdf:resource="' + com.ibm.rdm.web.suspicion.model.types.SUSPECT_ENTRIES + '"/>\n');
		buf.push('                       <rdf:li rdf:resource="' + com.ibm.rdm.web.suspicion.model.types.SUSPECT_PROFILES + '"/>\n');
		buf.push('                    </rdf:Seq>\n');
		buf.push('                 </rql:field>\n');
		buf.push('                 <rql:object>' + artifactVariableName + '</rql:object>\n');
		buf.push('              </rql:e1>\n');
		buf.push('              <rql:e2>\n');
		buf.push('                 <rdf:Seq>\n');
		dojo.forEach(viewRequestParams.suspicionProfileURIs, function(profileURI) {
			buf.push('                <rdf:li rdf:resource="' + profileURI + '"/>\n');
		});
		buf.push('                 </rdf:Seq>\n');
		buf.push('              </rql:e2>\n');
		buf.push('              <rql:op>in</rql:op>\n');
		buf.push('           </rdf:_1>\n');
		buf.push('        </rql:having>\n');
	},

	_getLinkSubjectObjectInfo: function(linkedArtifactConditionInfo) {
		var subjectObjectInfo = {
			subjectSideLinkResourceURIs: [],
			objectSideLinkResourceURIs: []
		};
		var subjectSideLinkResourceURIs = subjectObjectInfo.subjectSideLinkResourceURIs;
		var objectSideLinkResourceURIs = subjectObjectInfo.objectSideLinkResourceURIs;
		var attributeValues = linkedArtifactConditionInfo.attributeValues;
		for (var i = 1; i < attributeValues.length; i++) {
			var enumAbout = attributeValues[i];
			var linkResourceURI;
			var linkDirectionality =
				linkTypeUtils.enumAboutToLinkDirectionality(enumAbout);
			if (linkDirectionality === linkTypeUtils.linkDirectionality.Bidirectional) {
				linkResourceURI = linkTypeUtils.enumAboutToLinkResourceURI(enumAbout);
				subjectSideLinkResourceURIs.push(linkResourceURI);
				objectSideLinkResourceURIs.push(linkResourceURI);
			}
			else if (linkDirectionality === linkTypeUtils.linkDirectionality.SubjectSide) {
				linkResourceURI = linkTypeUtils.enumAboutToLinkResourceURI(enumAbout);
				subjectSideLinkResourceURIs.push(linkResourceURI);
			}
			else if (linkDirectionality === linkTypeUtils.linkDirectionality.ObjectSide) {
				linkResourceURI = linkTypeUtils.enumAboutToLinkResourceURI(enumAbout);
				objectSideLinkResourceURIs.push(linkResourceURI);
			}
		}
		return subjectObjectInfo;
	},

	_composeLinkClauses: function(
			artifactVariableName, linkedArtifactVariableName,
			numForRdfFragment, linkConditionInfo) {
		// example:
		//	  If the user uses the following filter:
		//		Link type exists Link To, Link From
		//	  the following RDF clawould be produced:
		//		<rdf:_1 rdf:parseType="Resource">
		//		  <rql:op>links</rql:op>
		//		  <rql:opts>
		//			<rdf:Seq>
		//			  <rdf:li rdf:parseType="Resource">
		//				<rql:type>constraintLinkOption</rql:type>
		//				<rdf:value rdf:parseType="Resource">
		//				  <rql:op>roles</rql:op>
		//				  <rql:e1>
		//					<rdf:Seq>
		//					  <rdf:li rdf:parseType="Resource">
		//						<rql:linkdir>out</rql:linkdir>
		//						<rql:linktype rdf:resource="https://spud.com:9443/rm/types/_ShuHAegrEd-sNKtFrtHD8g"/>
		//					  </rdf:li>
		//					  <rdf:li rdf:parseType="Resource">
		//						<rql:linkdir>in</rql:linkdir>
		//						<rql:linktype rdf:resource="https://spud.com:9443/rm/types/_ShuHAegrEd-sNKtFrtHD8g"/>
		//					  </rdf:li>
		//					</rdf:Seq>
		//				  </rql:e1>
		//				</rdf:value>
		//			  </rdf:li>
		//			</rdf:Seq>
		//		  </rql:opts>
		//		  <rql:to>R2</rql:to>
		//		  <rql:from>R1</rql:from>
		//		</rdf:_1>
		var composeListItemFragmentString = function(linkURI, linkDirection) {
			return (
				'					  <rdf:li rdf:parseType="Resource">\n'
				+ '						  <rql:linktype rdf:resource="' + linkURI + '"/>\n'
				+ '						  <rql:linkdir>' + linkDirection + '</rql:linkdir>\n'
				+ '					  </rdf:li>\n'
			);
		};
		var liFragmentsString = "";
		var operator = linkConditionInfo.attributeValues[0];
		var subjectObjectInfo = this._getLinkSubjectObjectInfo(linkConditionInfo);
		var subjectSideLinkResourceURIs = subjectObjectInfo.subjectSideLinkResourceURIs;
		var objectSideLinkResourceURIs = subjectObjectInfo.objectSideLinkResourceURIs;
		// Add "rdf:li" elements for all of the linkdir "out" links.
		// (i.e. the relationship role is subject to object).
		for (var i = 0; i < subjectSideLinkResourceURIs.length; i++) {
			liFragmentsString +=
				composeListItemFragmentString(
					subjectSideLinkResourceURIs[i], "out");
		}
		// Add "rdf:li" elements for all of the linkdir "in" links.
		// (i.e. the relationship role is object to subject).
		for (var i = 0; i < objectSideLinkResourceURIs.length; i++) {
			liFragmentsString +=
				composeListItemFragmentString(
					objectSideLinkResourceURIs[i], "in");
		}
		var notClause = (
			operator === com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist?
			'		  <rql:not>true</rql:not>\n' :
			''
		);
		var optsFragment = (
			liFragmentsString.length > 0 ?
			(
				'		  <rql:opts>\n'
				+ '			<rdf:Seq>\n'
				+ '			  <rdf:li rdf:parseType="Resource">\n'
				+ '				<rdf:value rdf:parseType="Resource">\n'
				+ '				  <rql:e1>\n'
				+ '					<rdf:Seq>\n'
				+ liFragmentsString
				+ '					</rdf:Seq>\n'
				+ '				  </rql:e1>\n'
				+ '				  <rql:op>roles</rql:op>\n'
				+ '				</rdf:value>\n'
				+ '				<rql:type>constraintLinkOption</rql:type>\n'
				+ '			  </rdf:li>\n'
				+ '			</rdf:Seq>\n'
				+ '		  </rql:opts>\n'
			) :
			""
		);
		return (
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ notClause
			+ '		  <rql:from>' + artifactVariableName + '</rql:from>\n'
			+ '		  <rql:to>' + linkedArtifactVariableName + '</rql:to>\n'
			+ optsFragment
			+ '		  <rql:op>links</rql:op>\n'
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		);
	},

	_composeArtifactExistenceClause: function(artifactVariableName, numForRdfFragment) {
		return [
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ '		  <rql:e1 rdf:parseType="Resource">\n'
			+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  </rql:e1>\n'
			+ '		  <rql:op>exists</rql:op>\n'
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		];
	},

	_composeAttributeClause: function(
			artifactVariableName, numForRdfFragment, attributeConditionInfo, projectURI, isModuleView) {
		var clauseFragment = null;
		if (attributeConditionInfo.isEnum) {
			var attributeDefinitionURI = attributeConditionInfo.attributeDefinitionURI;
			var isCrossProjectQuery = (projectURI === null || projectURI === undefined);
			var isWellKnownArtifactFormatsClause = (
				isCrossProjectQuery ?
				(attributeDefinitionURI === "http://www.ibm.com/xmlns/rdm/types/ArtifactFormat") :
				(attributeDefinitionURI
					=== RRMTypes.getWellKnownAttributeURI(
							projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ArtifactFormat))
			);
			var isCustomArtifactFormatsClause = (
				(!isWellKnownArtifactFormatsClause)
				&& (attributeConditionInfo.attributeType
					=== RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType)
			);
			clauseFragment =
				this._composeEnumOrArtifactFormatsAttributeClause(
					artifactVariableName, numForRdfFragment, attributeConditionInfo, projectURI,
					isWellKnownArtifactFormatsClause, isCustomArtifactFormatsClause, isModuleView
				);
		} else {
			// Note: this switch statement does NOT contain RRMTypes.ATTRIBUTE_TYPES.UserType
			// and RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType as they're both handled
			// above in the if clause (as their 'isEnum' is true).
			switch (attributeConditionInfo.attributeType) {
				case RRMTypes.ATTRIBUTE_TYPES.StringType:
					clauseFragment =
						this._composeStringAttributeClause(
							artifactVariableName, numForRdfFragment, attributeConditionInfo);
					break;
				case RRMTypes.ATTRIBUTE_TYPES.DateTimeType:
				case RRMTypes.ATTRIBUTE_TYPES.DateType:
				case RRMTypes.ATTRIBUTE_TYPES.TimeType:
					clauseFragment =
						this._composeDateOrDateTimeOrTimeAttributeClause(
							artifactVariableName, numForRdfFragment, attributeConditionInfo, projectURI, isModuleView);
					break;
				case RRMTypes.ATTRIBUTE_TYPES.IntegerType:
				case RRMTypes.ATTRIBUTE_TYPES.FloatType:
					clauseFragment =
						this._composeIntegerOrFloatAttributeClause(
							artifactVariableName, numForRdfFragment, attributeConditionInfo);
					break;
				case RRMTypes.ATTRIBUTE_TYPES.BooleanType:
					clauseFragment =
						this._composeBooleanAttributeClause(
							artifactVariableName, numForRdfFragment, attributeConditionInfo);
					break;
				case RRMTypes.ATTRIBUTE_TYPES.DurationType:
					clauseFragment =
						this._composeDurationAttributeClause(
							artifactVariableName, numForRdfFragment, attributeConditionInfo);
					break;
			}
		}
		return clauseFragment;
	},

	_composeEnumOrArtifactFormatsAttributeJson: function(
			attributeConditionInfo, projectURI,
			isWellKnownArtifactFormatsClause, isCustomArtifactFormatsClause, isModuleView)
	{
		// summary:
		//		This is similar to the _composeEnumOrArtifactFormatsAttributeClause
		//		method, but is for persisting a view as JSON rather than RDF.

		var operator = attributeConditionInfo.attributeValues[0],
			enumAttributeClause = "",
			// although locked by is potentially an enum, the operator "is any user"
			// will test for the existence of a lock held by any user.
			isLockExistenceOperator = this._isLockExistenceOperator(operator);
		var operator = (isWellKnownArtifactFormatsClause ? 'format' : operator),
			artifactFormats = (projectURI ? RRMTypes.getArtifactFormats(projectURI) : {}),
			originalAttributeValues = attributeConditionInfo.attributeValues,
			attributeValues = [operator];
		for (var i = 1; i < originalAttributeValues.length; i++) {
			var value = originalAttributeValues[i];
			var attributeURI = 
				(isWellKnownArtifactFormatsClause ?
				 artifactFormatUtils.enumAboutToFormatValue(value) :
				 (isCustomArtifactFormatsClause ?
				  artifactFormats[value] :
				  value
				 )
				);
			attributeValues.push(attributeURI);
		}
		return {
			attributeDefinitionURI: attributeConditionInfo.attributeDefinitionURI,
			attributeType: attributeConditionInfo.attributeType,
			attributeValues: attributeValues,
			isEnum: true
		};
	},
	
	_composeStringAttributeJson: function(attributeConditionInfo) {
		// summary:
		//		This is similar to the _composeStringAttributeClause method,
		//		but is for persisting a view as JSON rather than RDF.

		var attributeValues = attributeConditionInfo.attributeValues,
			operator = attributeValues[0],
			inputText = attributeValues[1],
			javaRegexpEncodedInputText;
		if (inputText){
			javaRegexpEncodedInputText = stringMatchUtils.inputTextToJavaRegexpEncodedInputText(inputText);
		}
		return {
			attributeDefinitionURI: attributeConditionInfo.attributeDefinitionURI,
			attributeType: attributeConditionInfo.attributeType,
			attributeValues: [
				operator,
				javaRegexpEncodedInputText
			],
			isEnum: false
		};
	},

	_composeDateOrDateTimeOrTimeAttributeJson: function(
		attributeConditionInfo, projectURI, isModuleView)
	{
		// summary:
		//		This is similar to the _composeDateOrDateTimeOrTimeAttributeClause
		//		method, but is for persisting a view as JSON rather than RDF.

		var attributeValues = attributeConditionInfo.attributeValues,
			isDateTime = attributeConditionInfo.attributeType === RRMTypes.ATTRIBUTE_TYPES.DateTimeType,
			operator = attributeValues[0];
		var date1 = (
			attributeValues.length > 0 ?
			attributeValues[1] :
			null
		);
		var date2 = (
			this._isBetweenOperator(operator, attributeConditionInfo) ?
			attributeValues[2] :
			null
		);
		// Reorder the dates, if needed, so that date1 is earlier than date2.
		if (date2 && date1 && date2 < date1) {
			var tempDate = date2;
			date2 = date1;
			date1 = tempDate;
		}
		var getISOString = function(attributeConditionInfo, date) {
			var isoString = "";
			switch (attributeConditionInfo.attributeType) {
			case RRMTypes.ATTRIBUTE_TYPES.DateTimeType:
				isoString = dojo.date.stamp.toISOString(date);
				break;
			case RRMTypes.ATTRIBUTE_TYPES.DateType:
				isoString = dojo.date.stamp.toISOString(date, {selector:"date"});
				break;
			case RRMTypes.ATTRIBUTE_TYPES.TimeType:
				isoString =
					dojo.date.locale.format(
						date, {timePattern: "HH:mm:ss", selector: "time"});
				break;
			}
			return isoString;
		};
		var attributeValuesToUse = [operator];
		if (date2) {
			// The user is either asking for a query bounded by start date and end date
			// or is asking for query bounded to a particular date.
			if (this._isIsOperator(operator, attributeConditionInfo)) {
				// Use the same logic (i.e. use of null) found in SavedFilterUtils._composeDateAttribute().
				attributeValuesToUse.push(date1);
				attributeValuesToUse.push(null);
			} else {
				attributeValuesToUse.push(date1);
				attributeValuesToUse.push(date2);
			}
		}
		else if (date1) {
			// The user has asked for a query involving only one date.
			attributeValuesToUse.push(date1);
		}
		else {
			// The operator must be 'exists' or 'does not exist' ... so do nothing.
		}
		return {
			attributeDefinitionURI: attributeConditionInfo.attributeDefinitionURI,
			attributeType: attributeConditionInfo.attributeType,
			attributeValues: attributeValuesToUse
		};
	},

	_composeIntegerOrFloatAttributeJson: function(attributeConditionInfo) {
		// summary:
		//		This is similar to the _composeIntegerOrFloatAttributeClause method,
		//		but is for persisting a view as JSON rather than RDF.

		var attributeValues = attributeConditionInfo.attributeValues,
			operator = attributeValues[0],
			inputText = attributeValues[1]+'', // Make sure it's a string (not a number).
			isExistenceOperator = this._isExistenceOperator(operator);
		// We take care here to account for the fact that the user may have
		// entered something like "1" as the value for a float-based filter
		// and this needs to be turned into "1.0" in order for the view
		// service to formulate the appropriate SPARQL.
		var normalizedInputText = (
			attributeConditionInfo.attributeType === RRMTypes.ATTRIBUTE_TYPES.FloatType ?
			((inputText.indexOf(".") >= 0) ? inputText : (inputText + ".0")) :
			inputText
		);
		var attributeValuesToUse = [operator];
		if (attributeValues.length > 1) {
			attributeValuesToUse.push(normalizedInputText);
		}
		return {
			attributeDefinitionURI: attributeConditionInfo.attributeDefinitionURI,
			attributeType: attributeConditionInfo.attributeType,
			attributeValues: attributeValuesToUse
		};
	},

	_composeBooleanAttributeJson: function(attributeConditionInfo) {
		// summary:
		//		This is similar to the _composeBooleanAttributeClause method,
		//		but is for persisting a view as JSON rather than RDF.

		var attributeValues = attributeConditionInfo.attributeValues,
			operator = attributeValues[0],
			isExistenceOperator = this._isExistenceOperator(operator);
		return {
			attributeDefinitionURI: attributeConditionInfo.attributeDefinitionURI,
			attributeType: attributeConditionInfo.attributeType,
			attributeValues: (
				isExistenceOperator ?
				[operator] :
				[operator, attributeValues[1]]
			)
		};
	},

	_composeDurationAttributeJson: function(attributeConditionInfo) {
		// summary:
		//		This is similar to the _composeDurationAttributeClause method,
		//		but is for persisting a view as JSON rather than RDF.

		var attributeValues = attributeConditionInfo.attributeValues,
			operator = attributeValues[0];
		return {
			attributeDefinitionURI: attributeConditionInfo.attributeDefinitionURI,
			attributeType: attributeConditionInfo.attributeType,
			attributeValues: (
				this._isExistenceOperator(operator) ?
				[operator] :
				(
					this._isBetweenOperator(operator, attributeConditionInfo) ?
					[operator, attributeValues[1], attributeValues[2]] :
					[operator, attributeValues[1]]
				)
			)
		};
	},
	
	_composeAttributeJson: function(attributeConditionInfo, projectURI, isModuleView) {
		// summary:
		//		This is similar to the _composeAttributeClause method, but is for persisting
		//		a view as JSON rather than RDF.

		var attributeJson = null;
		if (attributeConditionInfo.isEnum) {
			var attributeDefinitionURI = attributeConditionInfo.attributeDefinitionURI;
			var isCrossProjectQuery = (projectURI === null || projectURI === undefined);
			var isWellKnownArtifactFormatsClause = (
				isCrossProjectQuery ?
				(attributeDefinitionURI === "http://www.ibm.com/xmlns/rdm/types/ArtifactFormat") :
				(attributeDefinitionURI
					=== RRMTypes.getWellKnownAttributeURI(
							projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ArtifactFormat))
			);
			var isCustomArtifactFormatsClause = (
				(!isWellKnownArtifactFormatsClause)
				&& (attributeConditionInfo.attributeType
					=== RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType)
			);
			attributeJson =
				this._composeEnumOrArtifactFormatsAttributeJson(
					attributeConditionInfo, projectURI,
					isWellKnownArtifactFormatsClause, isCustomArtifactFormatsClause, isModuleView
				);
		} else {
			// Note: this switch statement does NOT contain RRMTypes.ATTRIBUTE_TYPES.UserType
			// and RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType as they're both handled
			// above in the if clause (as their 'isEnum' is true).
			switch (attributeConditionInfo.attributeType) {
				case RRMTypes.ATTRIBUTE_TYPES.StringType:
					attributeJson =
						this._composeStringAttributeJson(attributeConditionInfo);
					break;
				case RRMTypes.ATTRIBUTE_TYPES.DateTimeType:
				case RRMTypes.ATTRIBUTE_TYPES.DateType:
				case RRMTypes.ATTRIBUTE_TYPES.TimeType:
					attributeJson =
						this._composeDateOrDateTimeOrTimeAttributeJson(
							attributeConditionInfo, projectURI, isModuleView);
					break;
				case RRMTypes.ATTRIBUTE_TYPES.IntegerType:
				case RRMTypes.ATTRIBUTE_TYPES.FloatType:
					attributeJson =
						this._composeIntegerOrFloatAttributeJson(attributeConditionInfo);
					break;
				case RRMTypes.ATTRIBUTE_TYPES.BooleanType:
					attributeJson =
						this._composeBooleanAttributeJson(attributeConditionInfo);
					break;
				case RRMTypes.ATTRIBUTE_TYPES.DurationType:
					attributeJson =
						this._composeDurationAttributeJson(attributeConditionInfo);
					break;
			}
		}
		return attributeJson;
	},

	_composeLuceneSearchClause: function(
		artifactVariableName, numForRdfFragment, textSearchString,
		/*?Optional*/ luceneSearchField)
	{
		// summary:
		//		Compose an rql:where clause to utilize the
		//		Lucene-SPARQL join functionality in the View Service.
		// description:
		//		Constructs a where clause with the following format:
		//		<rdf:_1 rdf:parseType="Resource">
		//			<rql:op>matches</rql:op>
		//			<rql:e1 rdf:parseType="Resource">
		//				<rql:object>R1</rql:object>
		//			</rql:e1>
		//			<rql:e2>"(content:search string)"</rql:e2>
		//			<rql:flags>"t"</rql:flags>
		//		</rdf:_1>
		//		
		// artifactVariableName:
		//		The name of the RDF variable (ex, R1)
		// numForRdfFragment:
		//		The number for this RDF where fragment
		// textSearchString:
		//		The lucene search string to pass to the view service.
		// luceneSearchField:
		//		Which field lucene should search. Should be one of 
		//		com.ibm.rdm.web.query.ViewServiceClient.LUCENE_SEARCH_FIELDS.
		//		Defaults to FullContent.

		var operatorClause = '		  <rql:op>matches</rql:op>\n';
		var searchField = searchField ||
			com.ibm.rdm.web.query.ViewServiceClient.LUCENE_SEARCH_FIELDS.FullContent;

		textSearchString = this._sanitizeLuceneSearchString(textSearchString);
		var luceneQuery = '(' + searchField + ':' + textSearchString + ')';

		return [
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ operatorClause
			+ '		  <rql:e1 rdf:parseType="Resource">\n'
			+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  </rql:e1>\n'
			+ '		  <rql:e2>"' + luceneQuery + '"</rql:e2>\n'
			+ '		  <rql:flags>"t"</rql:flags>\n'
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		];
	},

	_sanitizeLuceneSearchString: function(searchString) {
		
		// Enhancement 68329. Support phrase search with search syntax using ""s around the phrase.
		// If the search string has ""s around, we will use the exact phrase for searching.
		if (searchString.charAt(0) === "\"" && searchString.charAt(searchString.length-1) === "\"") {
			searchString = searchString.substring(1, searchString.length-1);
			searchString = queryutils.encodeLuceneSearchString(searchString);
			searchString = "\\\"" + searchString + "\\\"";
		}
		else {
			var r1 = /[:\"\']|([ \t]){2,}/g, r2 = /([\\])?\*$/g;
			searchString = queryutils.encodeLuceneSearchString(searchString);
			searchString = dojo.map(searchString.replace(r1, "").split(" "), function(s) {
				return s.replace(r2, function(m1,m2){ return m2?m1:''; }) + "*";
			}).join(" ");
		}
		return searchString;
	},

	_composeStringAttributeClause: function(
			artifactVariableName, numForRdfFragment, attributeConditionInfo) {
		var attributeValues = attributeConditionInfo.attributeValues;
		var operator = attributeValues[0];
		// TODO Break out the input text as mentioned in rrc9447 (e.g. "ux, ui"
		// means perform matches against either "ux" OR "ui").
		var inputText = attributeValues[1];
		var javaRegexpEncodedInputText;
		if (inputText){
			javaRegexpEncodedInputText = stringMatchUtils.inputTextToJavaRegexpEncodedInputText(inputText);
		}
		var matchTextToUse = "";
		var doUseNotClause = false;
		// Initialize matchTextToUse and doUseNotClause.
		switch (operator) {
			case com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.Contains:
				matchTextToUse =
					'^.*' + javaRegexpEncodedInputText + '.*$';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.DoesNotContain:
				matchTextToUse =
					'^.*' + javaRegexpEncodedInputText + '.*$';
				doUseNotClause = true;
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.StartsWith:
				matchTextToUse =
					'^' + javaRegexpEncodedInputText + '.*$';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.EndsWith:
				matchTextToUse =
					'^.*' + javaRegexpEncodedInputText + '$';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS.Is:
				matchTextToUse =
					'^' + javaRegexpEncodedInputText + '$';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist:
				doUseNotClause = true;
				break;
		}
		var isExistenceOperator = this._isExistenceOperator(operator);
		var valueClause = (
			isExistenceOperator ?
			'' :
			'		  <rql:e2>"' + matchTextToUse + '"</rql:e2>\n'
		);
		var operatorClause = (
			isExistenceOperator ?
			'		  <rql:op>exists</rql:op>\n' :
			'		  <rql:op>matches</rql:op>\n'
		);
		var notClause = (
			doUseNotClause ?
			'		  <rql:not>true</rql:not>\n' :
			''
		);
		var flagsClause = (
			isExistenceOperator ?
			'' :
			'		  <rql:flags>"im"</rql:flags>\n' // Ignore case and match multi-line ...
		);
		return [
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ flagsClause
			+ '		  <rql:e1 rdf:parseType="Resource">\n'
			+ '			<rql:field rdf:resource="' + attributeConditionInfo.attributeDefinitionURI + '"/>\n'
			+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  </rql:e1>\n'
			+ valueClause
			+ operatorClause
			+ notClause
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		];
	},

	_isExistenceOperator: function(operator) {
		return (
			operator === com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.Exists
			|| operator === com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist
		);
	},
	
	_isLockExistenceOperator: function(operator) {
		// summary:
		//		Returns true if the operator is "is any user"
		//		used in the locked by attribute condition.  
		//		This operator require a lock exists clause to be added.
		return (operator === com.ibm.rdm.web.query.ViewServiceClient.LOCKEDBY_OPERATORS.IsAnyUser);
			
	},
	
	_isLockedByCurrentUserOperator: function(operator) {
		return (operator === com.ibm.rdm.web.query.ViewServiceClient.LOCKEDBY_OPERATORS.IsCurrentUser);
	},
	
	_composeLockedByCurrentUserClause: function(linkedArtifactVariableName, numForRdfFragment) {
		// summary:
		//		Builds the RDF expression for locked by current user.
		return ''+
		'<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'+
		'	<rql:op>=</rql:op>\n'+
		'	<rql:e1 rdf:parseType="Resource">\n'+
		this._composeLockCreatorField() +
		'		<rql:object>'+linkedArtifactVariableName+'</rql:object>\n'+
		'	</rql:e1>\n'+
		'	<rql:e2>&lt;'+RRMTypes.RDF_CURRENT_USER+'&gt;</rql:e2>\n'+
		'</rdf:_' + numForRdfFragment +'>\n';
	},

	_composeIntegerOrFloatAttributeClause: function(
			artifactVariableName, numForRdfFragment, attributeConditionInfo) {
		var attributeValues = attributeConditionInfo.attributeValues;
		var operator = attributeValues[0];
		var inputText = attributeValues[1]+''; // Make sure it's a string (not a number).
		var operatorToUse = "";
		var doUseNotClause = false;
		// Initialize operatorToUse and doUseNotClause.
		switch (operator) {
			case com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.Is:
				operatorToUse = '=';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsNot:
				operatorToUse = '!=';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsGreaterThanOrEqualTo:
				operatorToUse = '&gt;=';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsLessThanOrEqualTo:
				operatorToUse = '&lt;=';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist:
				doUseNotClause = true;
				break;
		}
		var isExistenceOperator = this._isExistenceOperator(operator);
		// We take care here to account for the fact that the user may have
		// entered something like "1" as the value for a float-based filter
		// and this needs to be turned into "1.0" in order for the view
		// service to formulate the appropriate SPARQL.
		var normalizedInputText = (
			attributeConditionInfo.attributeType === RRMTypes.ATTRIBUTE_TYPES.FloatType ?
			((inputText.indexOf(".") >= 0) ? inputText : (inputText + ".0")) :
			inputText
		);
		var valueClause = (
			isExistenceOperator ?
			'' :
			'		  <rql:e2>' + normalizedInputText + '</rql:e2>\n'
		);
		var operatorClause = (
			isExistenceOperator ?
			'		  <rql:op>exists</rql:op>\n' :
			'		  <rql:op>' + operatorToUse + '</rql:op>\n'
		);
		var notClause = (
			doUseNotClause ?
			'		  <rql:not>true</rql:not>\n' :
			''
		);
		return [
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ operatorClause
			+ '		  <rql:e1 rdf:parseType="Resource">\n'
			+ '			<rql:field rdf:resource="' + attributeConditionInfo.attributeDefinitionURI + '"/>\n'
			+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  </rql:e1>\n'
			+ valueClause
			+ notClause
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		];
	},

	_isBetweenOperator: function(operator, attributeConditionInfo) {
		var isBetweenOperator = false;
		switch (attributeConditionInfo.attributeType) {
			case RRMTypes.ATTRIBUTE_TYPES.DateTimeType:
				isBetweenOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.IsBetween
				);
				break;
			case RRMTypes.ATTRIBUTE_TYPES.DateType:
				isBetweenOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsBetween
				);
				break;
			case RRMTypes.ATTRIBUTE_TYPES.TimeType:
				isBetweenOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.TIME_OPERATORS.IsBetween
				);
				break;
			case RRMTypes.ATTRIBUTE_TYPES.DurationType:
				isBetweenOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsBetween
				);
				break;
		}
		return isBetweenOperator;
	},

	_isAfterOperator: function(operator, attributeConditionInfo) {
		var isAfterOperator = false;
		switch (attributeConditionInfo.attributeType) {
			case RRMTypes.ATTRIBUTE_TYPES.DateTimeType:
				isAfterOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.IsAfter
				);
				break;
			case RRMTypes.ATTRIBUTE_TYPES.DateType:
				isAfterOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsAfter
				);
				break;
			case RRMTypes.ATTRIBUTE_TYPES.TimeType:
				isAfterOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.TIME_OPERATORS.IsAfter
				);
				break;
		}
		return isAfterOperator;
	},

	_isBeforeOperator: function(operator, attributeConditionInfo) {
		var isBeforeOperator = false;
		switch (attributeConditionInfo.attributeType) {
			case RRMTypes.ATTRIBUTE_TYPES.DateTimeType:
				isBeforeOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.IsBefore
				);
				break;
			case RRMTypes.ATTRIBUTE_TYPES.DateType:
				isBeforeOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsBefore
				);
				break;
			case RRMTypes.ATTRIBUTE_TYPES.TimeType:
				isBeforeOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.TIME_OPERATORS.IsBefore
				);
				break;
		}
		return isBeforeOperator;
	},

	_isIsOperator: function(operator, attributeConditionInfo) {
		var isIsOperator = false;
		switch (attributeConditionInfo.attributeType) {
			case RRMTypes.ATTRIBUTE_TYPES.DateTimeType:
				isIsOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS.Is
				);
				break;
			case RRMTypes.ATTRIBUTE_TYPES.DateType:
				isIsOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.Is
				);
				break;
		}
		return isIsOperator;
	},

	_isIsNotOperator: function(operator, attributeConditionInfo) {
		var isIsNotOperator = false;
		switch (attributeConditionInfo.attributeType) {
			case RRMTypes.ATTRIBUTE_TYPES.DateType:
				isIsNotOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsNot
				);
				break;
			case RRMTypes.ATTRIBUTE_TYPES.IntegerType:
			case RRMTypes.ATTRIBUTE_TYPES.FloatType:
				isIsNotOperator = (
					operator
					=== com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS.IsNot
				);
				break;
		}
		return isIsNotOperator;
	},

	_composeDateOrDateTimeOrTimeAttributeClause: function(
			artifactVariableName, numForRdfFragment, attributeConditionInfo, projectURI, isModuleView) {
		var attributeValues = attributeConditionInfo.attributeValues;
		var isDateTime = attributeConditionInfo.attributeType === RRMTypes.ATTRIBUTE_TYPES.DateTimeType;
		var operator = attributeValues[0];
		var date1 = (
			attributeValues.length > 0 ?
			attributeValues[1] :
			null
		);
		var date2 = (
			this._isBetweenOperator(operator, attributeConditionInfo) ?
			attributeConditionInfo.attributeValues[2] :
			null
		);
		// Reorder the dates, if needed, so that date1 is earlier than date2.
		if (date2 && date1 && date2 < date1) {
			var tempDate = date2;
			date2 = date1;
			date1 = tempDate;
		}
		var fragments = [];
		var getISOString = function(attributeConditionInfo, date) {
			var isoString = "";
			switch (attributeConditionInfo.attributeType) {
				case RRMTypes.ATTRIBUTE_TYPES.DateTimeType:
					isoString = dojo.date.stamp.toISOString(date);
					break;
				case RRMTypes.ATTRIBUTE_TYPES.DateType:
					isoString = dojo.date.stamp.toISOString(date, {selector:"date"});
					break;
				case RRMTypes.ATTRIBUTE_TYPES.TimeType:
					isoString =
						dojo.date.locale.format(
							date, {timePattern: "HH:mm:ss", selector: "time"});
					break;
			}
			return isoString;
		};
		var doUseNotClause = (
			operator === com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS.IsNot
			|| operator === com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist
		);
		var composeDateComparisonRdfFragment = dojo.hitch(this, function(
				numForRdfFragment, attributeConditionInfo, date, operatorToUse, doUseNotClause, projectURI, isModuleView) {
			return (
				'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
				+ '		  <rql:op>' + operatorToUse + '</rql:op>\n'
				+ (doUseNotClause ? '		  <rql:not>true</rql:not>\n' : '')
				+ '		  <rql:e1 rdf:parseType="Resource">\n'
				+ this._composeFieldClause(projectURI, attributeConditionInfo.attributeDefinitionURI, isModuleView)
				+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
				+ '		  </rql:e1>\n'
				+ '		  <rql:e2>' + getISOString(attributeConditionInfo, date) + '</rql:e2>\n'
				+ '		</rdf:_' + numForRdfFragment + '>\n'
			);
		});
		var composeDateExistenceRdfFragment = function(
				numForRdfFragment, attributeConditionInfo, operatorToUse) {
			return (
				'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
				+ '		  <rql:op>' + operatorToUse + '</rql:op>\n'
				+ (doUseNotClause ? '		  <rql:not>true</rql:not>\n' : '')
				+ '		  <rql:e1 rdf:parseType="Resource">\n'
				+ '			<rql:field rdf:resource="' + attributeConditionInfo.attributeDefinitionURI + '"/>\n'
				+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
				+ '		  </rql:e1>\n'
				+ '		</rdf:_' + numForRdfFragment + '>\n'
			);
		};
		var isDateTimeWithNoSpecifiedEndTime = function(attributeConditionInfo, date) {
			return (
				attributeConditionInfo.attributeType === RRMTypes.ATTRIBUTE_TYPES.DateTimeType
				&& date.getHours() === 0
				&& date.getMinutes() === 0
				&& date.getSeconds() === 0
				&& date.getMilliseconds() === 0
			);
		};
		if (date2
			|| (this._isIsOperator(operator, attributeConditionInfo)
				&& isDateTime)
			) {
			// The user is either asking for a query bounded by start date and end date
			// or is asking for query bounded to a particular date.
			if (this._isIsOperator(operator, attributeConditionInfo)) {
				date2 = date1;
				// Note: we add 1 day to endDate to account for the 24 hours that
				// are found (but not represented) in this endDate.
				date2 = dojo.date.add(date2, "day", 1);
			}
			// TODO Paul has added support for a new 'between' rql:op ... so I
			// shouldn't need to use two clauses any more.  Convert this over
			// to using this this new between operator.
			fragments.push(
				composeDateComparisonRdfFragment(
					numForRdfFragment,
					attributeConditionInfo,
					date1,
					"&gt;=",
					doUseNotClause,
					projectURI,
					isModuleView
				)
			);
			numForRdfFragment++;
			fragments.push(
				composeDateComparisonRdfFragment(
					numForRdfFragment,
					attributeConditionInfo,
					date2,
					"&lt;=",
					doUseNotClause,
					projectURI,
					isModuleView
				)
			);
		}
		else if (date1) {
			// The user has asked for a query involving only one date.
			var rqlOperatorToUse;
			if (this._isAfterOperator(operator, attributeConditionInfo)) {
				rqlOperatorToUse = "&gt;=";
			}
			else if (this._isBeforeOperator(operator, attributeConditionInfo)) {
				rqlOperatorToUse = "&lt;=";
			}
			else if (this._isIsNotOperator(operator, attributeConditionInfo)
					 || this._isIsOperator(operator, attributeConditionInfo)) {
				rqlOperatorToUse = "=";
			}
			fragments.push(
				composeDateComparisonRdfFragment(
					numForRdfFragment, attributeConditionInfo, date1,
					rqlOperatorToUse, doUseNotClause, projectURI,
					isModuleView)
			);
		}
		else {
			// The operator must be 'exists' or 'does not exist'.
			fragments.push(
				composeDateExistenceRdfFragment(
					numForRdfFragment, attributeConditionInfo, "exists",
					doUseNotClause)
			);
		}
		return fragments;
	},

	_composeEnumOrArtifactFormatsAttributeClause: function(
			artifactVariableName, numForRdfFragment, attributeConditionInfo, projectURI,
			isWellKnownArtifactFormatsClause, isCustomArtifactFormatsClause, isModuleView) {
		var attributeValues = attributeConditionInfo.attributeValues;
		var operator = attributeValues[0];
		var notClause = ((
			operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNot ||
			operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAny ||
			operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAll ||
			operator === com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist) ?
			'		  <rql:not>true</rql:not>\n' :
			''
		);
		var allClause = ((
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAll || 
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAll 
			) ?
			'		  <rql:all>true</rql:all>\n' :
			''
		);
		var isOrInOperator = ((
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAll || 
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAll || 
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsNotAny ||
				operator === com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS.IsAny
			) ?
			'in' : 'is'
		);		
		var enumAttributeClause = "",
			// although locked by is potentially an enum, the operator "is any user"
			// will test for the existence of a lock held by any user.
			isLockExistenceOperator = this._isLockExistenceOperator(operator);
		if (this._isExistenceOperator(operator) || isLockExistenceOperator) {
			enumAttributeClause = [
				'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
				+ '		  <rql:e1 rdf:parseType="Resource">\n'
				+ this._composeFieldClause(projectURI, attributeConditionInfo.attributeDefinitionURI, isModuleView, isLockExistenceOperator)
				+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
				+ '		  </rql:e1>\n'
				+ notClause
				+ '		  <rql:op>exists</rql:op>\n'
				+ '		</rdf:_' + numForRdfFragment + '>\n'
			];
		} else {
			var operator = (isWellKnownArtifactFormatsClause ? 'format' : isOrInOperator);
			var e1Clause = (
				isWellKnownArtifactFormatsClause ?
				'<rql:e1>' + artifactVariableName + '</rql:e1>' :
				(
					'		  <rql:e1 rdf:parseType="Resource">\n'
					+ this._composeFieldClause(projectURI, attributeConditionInfo.attributeDefinitionURI, isModuleView)
					+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
					+ '		  </rql:e1>\n'
				)
			);
			var artifactFormats = (projectURI ? RRMTypes.getArtifactFormats(projectURI) : {});
			var liFragmentsString = "";
			for (var i = 1; i < attributeValues.length; i++) {
				var attributeURI = 
						(isWellKnownArtifactFormatsClause ?
							artifactFormatUtils.enumAboutToFormatValue(attributeValues[i]) :
						(isCustomArtifactFormatsClause ?
							artifactFormats[attributeValues[i]] :
							attributeValues[i]
						  )
						);
				liFragmentsString += (
					isWellKnownArtifactFormatsClause ?
					(
						'			  <rdf:li>"'
						+ attributeURI
						+ '"</rdf:li>\n'
					) :
					(
						'			  <rdf:li rdf:resource="'
						+ attributeURI
						+ '"/>\n'
					)
				);
			}
			enumAttributeClause = [
				'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
				+ e1Clause
				+ '		  <rql:e2>\n'
				+ '			<rdf:Seq>\n'
				+ liFragmentsString
				+ '			</rdf:Seq>\n'
				+ '		  </rql:e2>\n'
				+ notClause
				+ allClause
				+ '		  <rql:op>' + operator + '</rql:op>\n'
				+ '		</rdf:_' + numForRdfFragment + '>\n'
			];
		}
		return enumAttributeClause;
	},

	_composeFieldClause: function(projectURI, attributeDefinitionURI, isModuleView, isLockExistenceQuery) {
		//Composes the clause, taking into account that, for modules, certain system attributes
		//must be pointed at the base artifact value rather than the modufact
		
		var buf = [];
		if (isModuleView) {
			if (!projectURI) {
				//If none defined, use the current project
				projectURI = com.ibm.rdm.web.CURRENT_PROJECT;
			}
			var wellKnown = RRMTypes.getWellKnownAttributeObjectFromURI(attributeDefinitionURI, projectURI);
			
			if (wellKnown) {
				if (wellKnown.sameAs === com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Creator ||
					wellKnown.sameAs === com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Created ||
					wellKnown.sameAs === com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Contributor ||
					wellKnown.sameAs === com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ModifiedOn) {
							
					buf.push('		  <rql:field>\n');
					buf.push('		    <rdf:Seq>\n');
					buf.push('		      <rdf:li rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/boundArtifact"/>\n');
					buf.push('		      <rdf:li rdf:resource="' + attributeDefinitionURI + '"/>\n');
					buf.push('		    </rdf:Seq>\n');
					buf.push('		  </rql:field>\n');
					return buf.join("");
				}
			}
		}
		
		if (attributeDefinitionURI === com.ibm.rdm.web.model.types.RRMTypes.RDF_LOCKED_BY) {
			if (isLockExistenceQuery) {
				buf.push('		<rql:field rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/lock"/>\n');
			} else {
				buf.push(this._composeLockCreatorField());
			}
			return buf.join("");
		} else {
			return ('		  <rql:field rdf:resource="' + attributeDefinitionURI + '"/>\n');
		}
	},
	
	_composeLockCreatorField: function() {
		return '		  <rql:field>\n'+
			   '		    <rdf:Seq>\n'+
			   '		      <rdf:li rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/lock"/>\n'+
			   '		      <rdf:li rdf:resource="http://purl.org/dc/terms/creator"/>\n'+
			   '		    </rdf:Seq>\n'+
			   '		  </rql:field>\n';
	},

	_composeBooleanAttributeClause: function(
			artifactVariableName, numForRdfFragment, attributeConditionInfo) {
		var attributeValues = attributeConditionInfo.attributeValues;
		var operator = attributeValues[0];
		var trueOrfalseString = (
			attributeValues.length > 0 ?
			attributeValues[1] :
			null
		);
		var isExistenceOperator = this._isExistenceOperator(operator);
		var valueClause = (
			isExistenceOperator ?
			'' :
			'		  <rql:e2>' + trueOrfalseString + '</rql:e2>\n'
		);
		var operatorClause = (
			isExistenceOperator ?
			'		  <rql:op>exists</rql:op>\n' :
			'		  <rql:op>=</rql:op>\n'
		);
		var notClause = (
			(operator === com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist) ?
			'		  <rql:not>true</rql:not>\n' :
			''
		);
		return [
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ operatorClause
			+ notClause
			+ '		  <rql:e1 rdf:parseType="Resource">\n'
			+ '			<rql:field rdf:resource="' + attributeConditionInfo.attributeDefinitionURI + '"/>\n'
			+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  </rql:e1>\n'
			+ valueClause
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		];
	},

	_composeDurationAttributeClause: function(
			artifactVariableName, numForRdfFragment, attributeConditionInfo) {
		var fragments = [];
		var attributeDefinitionURI = attributeConditionInfo.attributeDefinitionURI;
		var attributeValues = attributeConditionInfo.attributeValues;
		var operator = attributeValues[0];
		var operatorToUse = "";
		var doUseNotClause = false;
		// Initialize operatorToUse and doUseNotClause.
		switch (operator) {
			case com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.Is:
				operatorToUse = '=';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsGreaterThan:
				operatorToUse = '&gt;';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsBetween:
				operatorToUse = '&gt;=';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsLessThan:
				operatorToUse = '&lt;';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.Exists:
				operatorToUse = 'exists';
				break;
			case com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS.DoesNotExist:
				operatorToUse = 'exists';
				doUseNotClause = true;
				break;
		}
		var valueClause = (
			this._isExistenceOperator(operator) ?
			'' :
			'		  <rql:e2>' + attributeValues[1] + '</rql:e2>\n'
		);
		var notClause = (
			doUseNotClause ?
			'		  <rql:not>true</rql:not>\n' :
			''
		);
		fragments.push(
			'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
			+ '		  <rql:op>' + operatorToUse + '</rql:op>\n'
			+ notClause
			+ '		  <rql:e1 rdf:parseType="Resource">\n'
			+ '			<rql:field rdf:resource="' + attributeDefinitionURI + '"/>\n'
			+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
			+ '		  </rql:e1>\n'
			+ valueClause
			+ '		</rdf:_' + numForRdfFragment + '>\n'
		);
		if (operator === com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS.IsBetween) {
			numForRdfFragment++;
			var duration = attributeValues[2];
			operatorToUse = '&lt;=';
			fragments.push(
				'		<rdf:_' + numForRdfFragment + ' rdf:parseType="Resource">\n'
				+ '		  <rql:op>' + operatorToUse + '</rql:op>\n'
				+ '		  <rql:e1 rdf:parseType="Resource">\n'
				+ '			<rql:field rdf:resource="' + attributeDefinitionURI + '"/>\n'
				+ '			<rql:object>' + artifactVariableName + '</rql:object>\n'
				+ '		  </rql:e1>\n'
				+ '		  <rql:e2>' + duration + '</rql:e2>\n'
				+ '		</rdf:_' + numForRdfFragment + '>\n'
			);
		}
		return fragments;
	},
	
	_composeSortClause: function(artifactVariableName, orderBy, isModule) {
		//Summary: Applies an ordering relation to the query, so the order the results come back in is defined.
		//
		var result = "";
		if (orderBy && orderBy.length > 0) {
			var buf = [];
			buf.push('      <rql:sort rdf:parseType="Resource">\n');

			var count = 1;
			dojo.forEach(orderBy, function (order) {
				if (order.order && order.order === "d") { var sortOrder = "desc"; } else { var sortOrder = "asc"; }

				buf.push('        <rdf:_'+count+' rdf:parseType="Resource">\n');
				buf.push('          <rql:objField rdf:parseType="Resource">\n');
				
				// If it's a module and the column to sort on is the modified date, then use the module's artifacts' modified date not the module's modified date.
				if (isModule && 
						(order.def === com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ModifiedOn ||
						order.def === com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Created)) {						
					buf.push('            <rql:field>\n');
					buf.push('              <rdf:Seq>\n');
					buf.push('                <rdf:li rdf:resource="http://www.ibm.com/xmlns/rdm/rdf/boundArtifact"/>\n');
					buf.push('                <rdf:li rdf:resource="' + order.def + '"/>\n');
					buf.push('              </rdf:Seq>\n');
					buf.push('            </rql:field>\n');											
				}
				else {
					buf.push('            <rql:field rdf:resource="'+order.def+'"/>\n');
				}
				
				buf.push('            <rql:object>' + artifactVariableName + '</rql:object>\n');
				buf.push('          </rql:objField>\n');
				buf.push('          <rql:order>'+sortOrder+'</rql:order>\n');
				if (order.type == "g") {
					buf.push('          <rql:not>true</rql:not>\n');
				}
				buf.push('        </rdf:_'+count+'>\n');
				
				count++;
			});
			
			buf.push('      </rql:sort>\n');
			result = buf.join("");
		}

		return result;
	},
	
	_composeLinksClauses: function(
			artifactVariableName, linkedArtifactVariableName, ignoredLinkTypes) {
		// summary:
		//		This method builds the LinkFromQuery (and LinkToQuery) sections.
		//		These are used to report back the existence of links for the given
		//		rows. (As we do not want to distinguish between the directions of links,
		//		we always build the two to/from queries the same).
		
		//TODO: ignore the links in ignoredLinksTypes
		var result =	'	<rm:linktoquery rdf:parseType="Resource">\n' +
						'	 <rql:where rdf:parseType="Resource">\n' +
						'	  <rdf:_1 rdf:parseType="Resource">\n' +
						'	   <rql:from>' + linkedArtifactVariableName + '</rql:from>\n' +
						'	   <rql:link>L1</rql:link>\n' +
						'	   <rql:to>' + artifactVariableName + '</rql:to>\n' +
						'	   <rql:depth>1</rql:depth>\n' +
						'	   <rql:op>link</rql:op>\n' +
						'	  </rdf:_1>\n' +
						'	 </rql:where>\n' +
						'	</rm:linktoquery>\n' +
						'	<rm:linkfromquery rdf:parseType="Resource">\n' +
						'	 <rql:where rdf:parseType="Resource">\n' +
						'	  <rdf:_1 rdf:parseType="Resource">\n' +
						'	   <rql:from>' + artifactVariableName + '</rql:from>\n' +
						'	   <rql:link>L1</rql:link>\n' +
						'	   <rql:to>' + linkedArtifactVariableName + '</rql:to>\n' +
						'	   <rql:depth>1</rql:depth>\n' +
						'	   <rql:op>link</rql:op>\n' +
						'	  </rdf:_1>\n' +
						'	 </rql:where>\n' +
						'	</rm:linkfromquery>\n';
		return result;
	}
};


com.ibm.rdm.web.query.ViewServiceClient.EXISTENCE_OPERATORS = {
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

com.ibm.rdm.web.query.ViewServiceClient.BOOLEAN_OPERATORS = {
	Is: "is",
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

com.ibm.rdm.web.query.ViewServiceClient.NUMBER_OPERATORS = {
	Is: "is",
	IsNot: "isNot",
	IsGreaterThanOrEqualTo: "isGreaterThanOrEqualTo",
	IsLessThanOrEqualTo: "isLessThanOrEqualTo",
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

com.ibm.rdm.web.query.ViewServiceClient.DATE_OPERATORS = {
	Is: "is",
	IsNot: "isNot",
	IsAfter: "isAfter",
	IsBefore: "isBefore",
	IsBetween: "isBetween",
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

com.ibm.rdm.web.query.ViewServiceClient.DATETIME_OPERATORS = {
	Is: "is",
	IsAfter: "isAfter",
	IsBefore: "isBefore",
	IsBetween: "isBetween",
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

com.ibm.rdm.web.query.ViewServiceClient.STRING_OPERATORS = {
	Is: "is",
	Contains: "contains",
	DoesNotContain: "doesNotContain",
	StartsWith: "startsWith",
	EndsWith: "endsWith",
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

com.ibm.rdm.web.query.ViewServiceClient.TIME_OPERATORS = {
	IsAfter: "isAfter",
	IsBefore: "isBefore",
	IsBetween: "isBetween",
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

com.ibm.rdm.web.query.ViewServiceClient.DURATION_OPERATORS = {
	Is: "is",
	IsGreaterThan: "isAfter",
	IsLessThan: "isBefore",
	IsBetween: "isBetween",
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

com.ibm.rdm.web.query.ViewServiceClient.DURATION_UNITS = {
	Hours: "hours",
	Days: "days",
	Weeks: "weeks",
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

com.ibm.rdm.web.query.ViewServiceClient.ARTIFACT_TYPE_OPERATORS = {
	IsAny: "isAny",
	IsNotAny: "isNotAny",
	// WARNING: Do NOT use the following two operators (Is and IsNot).
	// They're still here ONLY to account for the fact that migration code
	// is not currently migrating "is" to "isAny" and "isNot" to "isNotAny"
	// and instead client code is accommodating the fact that this migration
	// has not occurred.
	Is: "is",
	IsNot: "isNot"
};

com.ibm.rdm.web.query.ViewServiceClient.ENUM_OPERATORS = {
	IsAny: "isAny",
	IsAll: "isAll",
	IsNotAny: "isNotAny",
	IsNotAll: "isNotAll",
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

com.ibm.rdm.web.query.ViewServiceClient.LOCKEDBY_OPERATORS = {
	IsCurrentUser: "isCurrentUser",
	IsAnyUser: "anyUser"
};

com.ibm.rdm.web.query.ViewServiceClient.TAG_OPERATORS = {
	IsAnyOf: "isAnyOf",
	IsAllOf: "isAllOf",
	IsNotAnyOf: "isNotAnyOf",
	IsNotAllOf: "isNotAllOf"
};

com.ibm.rdm.web.query.ViewServiceClient.LINK_OPERATORS = {
	Exists: "exists",
	DoesNotExist: "doesNotExist"
};

// Note: This exists to allow for future expansion, despite the
// fact that - as of 3.0 - there is only one option. There were loose
// plans to expand this functionality in server, so if you are
// reading this, check with them and see if it's happened.
// --Christian
com.ibm.rdm.web.query.ViewServiceClient.LUCENE_SEARCH_FIELDS = {
	FullContent: "content"
};


})();
