/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//console.debug("Loading: com.ibm.rqm.configuration.web.ui.internal.view.configuration.AbstractWizard");
dojo.provide("com.ibm.rqm.configuration.web.ui.internal.view.configuration.AbstractWizard");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("jazz.ui.MessageBox");

(function(){
	
	var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
	var MessageBox = jazz.ui.MessageBox;
	
    dojo.declare("com.ibm.rqm.configuration.web.ui.internal.view.configuration.AbstractWizard", [dijit._Widget, dijit._Templated], {
        templateString: "<div dojoAttachPoint=\"_wizardRootNode\" class=\"com-ibm-rqm-configuration-web-ui-internal-view-configuration-GenerateEWIWizard\"></div>",
        
		width: "95%",
		height: "90%",
		
        constructor: function(args, frag){
            if (args.parentController == null) {
                throw new Error("AbstractWizard#constructor: Missing required parameter 'args.parentController'");
            }
            this.controller = args.parentController;
            this.callback = args.callback ? args.callback : null;
			this.title = "";
        },
        
        postCreate: function(){
			this.setTitle(this.title);
            this.showCurrentStep();
        },
        
        //@Abstract - override if necessary, but default behavior should be sufficient
		doPrevious: function(params){
            this.currentStep -= 1;
            this.showCurrentStep(params);
        },
        
		//@Abstract - override if necessary, but default behavior should be sufficient
        doNext: function(params){
            this.currentStep += 1;
            this.showCurrentStep(params);
        },
        
		//@Abstract - override if necessary, but default behavior should be sufficient
        doFinish: function(args){
			this.destroyRecursive();
			if(this.artifactType){
				args.artifactType = this.artifactType;
			}
			this._wizardDialog.close();
            if (this.callback) {
                if (dojo.isObject(this.callback.context) && dojo.isFunction(this.callback.method)) {
                    this.callback.method.apply(this.callback.context, [args]);
                }
            }
        },
		
		//@Abstract - override if necessary, for example, to unsubscribe dojo event.
		uninitialize: function(){
			return;
		},
		
		//@Abstract - override it, set every step heading title of the dialog window
        setTitle: function(title){
			return;
        },
		
		getTitle: function(){
			return this.title;
		},
        
		//@Abstract - override it, set every step panel of the dialog
        showCurrentStep: function(args){
			return;
        },
        
        _setDialogContent: function(node, /*optional*/focusNode){
			if (!this._wizardDialog) {
				var dialogArgs = {
					isClosable: true,
					showActions: false,
					style: "z-index: 1000;",
					zIndex: 999,
					primaryTitle:this.getTitle()
				};
				
				this._wizardDialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
				this._wizardDialog.setWidth(this.width);
				this._wizardDialog.setHeight(this.width);
				this._wizardDialog.newCreated = true;
	            this._wizardDialog.open();
			}
			dojox.xml.parser.removeChildren(this._wizardDialog.contentContainer);
			
			var msgBoxNode = dojo.create("div");
			this.messageBox = new jazz.ui.MessageBox();
			msgBoxNode.appendChild(this.messageBox.domNode);
			
			this._wizardDialog.contentContainer.appendChild(msgBoxNode);
            this._wizardDialog.contentContainer.appendChild(node);

            // Update the position of the wizard dialog when just created. The new page data
            // may change the wizard dialog size but don't reposition to avoid shifting.
            setTimeout(dojo.hitch(this, function(){
            	  if (this._wizardDialog.newCreated) {
                      this._wizardDialog.updatePosition();
                      delete this._wizardDialog.newCreated;
                  }
                if(dojo.isIE){
                	dijit.focus(this._wizardDialog._headerText);
                } else {
                	var ffNode = focusNode || node;
                	dijit.focus(ffNode);
                }
            }), 600);
            
        },
            
	    showMessage: function(message, statusType) {
	    	if (this.messageBox) {
	    		this.messageBox.showMessage({ 
				   type: statusType,
				   summary: message,
				   closable: true
				});
	    	} else {
	    		// Safe fail
	    		var alertDialog = new AlertDialog({
					message: message,
					width: "40%",
					primaryTitle: statusType // ERROR, WARNING, etc.
				});
				alertDialog.open(true);
	    	}
	    },
	    
	    showErrorMessage: function(message) {
	    	this.showMessage(message, MessageBox.STATUS_TYPES.ERROR);
	    },
	
	    showWarningMessage: function(message) {
	    	this.showMessage(message, MessageBox.STATUS_TYPES.WARNING);
	    },
	    
	    showOkMessage: function(message) {
	    	this.showMessage(message, MessageBox.STATUS_TYPES.OK);
	    },
	    
	    showInfoMessage: function(message) {
	    	this.showMessage(message, MessageBox.STATUS_TYPES.INFO);
	    }
    });
    
})();
