/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryEditor");

dojo.require("com.ibm.asq.common.web.ui.internal.model.SaveContext");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.client.internal.option");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");


dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.CategoryClient");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryConstants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategorySaveParticipant");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreeModel");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreePanel");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTermPanel");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

dojo.require("jazz.app.context");

(function(dojo) {
	
	var CSHelp         				    = com.ibm.asq.common.web.ui.internal.util.CSHelp;
	var GlobalEvents   				    = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
	var SaveContext 					= com.ibm.asq.common.web.ui.internal.model.SaveContext;
	
	var Constants 	   				    = com.ibm.rqm.planning.web.client.internal.Constants;
	var CategoryClient 				    = com.ibm.rqm.planning.web.client.internal.CategoryClient;
	
	var CategoryConstants 				= com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryConstants;
	var CategoryTreeModel 				= com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreeModel;
	var CategoryTreePanel 				= com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreePanel;
	var CategoryTermPanel 				= com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTermPanel;
	var ServiceResponseHandler		    = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	
	var getCurrentProjectArea 			= jazz.app.context.get;
	var Utility 						= com.ibm.rqm.planning.web.ui.internal.util.Utility;
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryEditor", [dijit._Widget, dijit._Templated], {
		
		templateString: ['<div class="com-ibm-rqm-planning-category-editor" tabIndex="-1" dojoAttachPoint="domNode"  role="presentation" >',
		                 '  <table class="content-table" role="presentation" dojoAttachPoint="table" width="99%">',
		                 '    <tr  role="presentation" >',
		                 '      <td dojoAttachPoint="_treePanelNode"  role="presentation"></td>',
		                 '      <td class="cell-separator"  role="presentation" style="width: 10px;"></td>',
		                 '      <td dojoAttachPoint="_valuesPanelNode"  role="presentation" style="width: 300px;"></td>',
		                 '    </tr>',
		                 '  </table>',
		                 '</div>'].join(''),
		                 
		doPublish: true,
		
		// Holds the value on the tree panel's view mode to restore between saves.
		viewMode: CategoryConstants.VIEW_MODE_SHOW_ALL,
		
	    postCreate: function() {
	    	this._createModel();
	    	var valuesPanel = new CategoryTermPanel({});
	    	var treePanel = new CategoryTreePanel({
	    		artifactType: this.artifactType,
	    		typeName: this.typeName,
	    		valuesPanel: valuesPanel,
	    		setDirty: dojo.hitch(this, "setDirty"), 
	    		model: this.model,
	    		isDialog: this.isDialog
	    	});
	    	this._treePanelNode.appendChild(treePanel.domNode);
	    	this.treePanel = treePanel;
	    	this._valuesPanelNode.appendChild(valuesPanel.domNode);
	    	this.valuesPanel = valuesPanel;
	    	
	    	this.openDialogIfRequired();
	    },
	    
	    openDialogIfRequired: function() {
	    	//if this is being shown as a dialog, init the dialog
	        if (this.isDialog && !this._dialog) {
	            this.show();
	        }
	    },
	    
	    setDirty: function(dirty) {
			this._dirty = dirty;
			if (this._dialog) {
				this._dialog.getOkButton().disabled = !dirty;
			}
		},
	    
	    isDirty: function() {
			return this._dirty;
		},
	    
	    startup: function() {
	    	if (this.valuesPanel) {
	    		this.valuesPanel.startup();
	    	}
	    },	    
	    
	    _createModel: function(){
			if(!this.model){
				this.model = new CategoryTreeModel({
					artifactType: this.artifactType,
					typeHierarchyOnly: false,
					orderPrefEntry: this.orderPrefEntry
				});
			}
		},
	    
		show: function() {
	        if (!this._dialog) {           
	            var dialogArgs = {                    
					primaryTitle: Constants.getArtifactTypeMessage("labelManageCategories", this.typeName),
					isClosable: true,
					okCallback: dojo.hitch(this, "_doSave"),
					cancelCallback: dojo.hitch(this, "hide"),
					showActions: true
	            };                
	            this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
				var contextID = null;
				switch (this.artifactType) {
					case Constants.MODEL_TYPE_CASE: contextID = "rqm_manage_tc_categories.html"; break;
					case Constants.MODEL_TYPE_PLAN: contextID = "rqm_manage_tp_categories.html"; break;
					case Constants.MODEL_TYPE_SUITE: contextID = "rqm_manage_tes_categories.html"; break;
		            case Constants.MODEL_TYPE_SCRIPT: contextID = "rqm_manage_ts_categories.html"; break;
		            case Constants.MODEL_TYPE_TESTCASE_EXECUTION_RECORD: contextID = "rqm_manage_ter_categories.html"; break;
		            case Constants.MODEL_TYPE_SUITE_EXECUTION_RECORD: contextID = "rqm_manage_ser_categories.html"; break;					
		            case Constants.MODEL_TYPE_RESULT: contextID = "rqm_manage_result_categories.html"; break;
		            case Constants.MODEL_TYPE_SUITE_LOG: contextID = "rqm_manage_suite_result_categories.html"; break;
		            case Constants.MODEL_TYPE_DATAPOOL: contextID = "rqm_manage_test_data_categories.html"; break;
		            case Constants.MODEL_TYPE_KEYWORD: contextID = "rqm_manage_keyword_categories.html"; break;
				}
				var csHelp = new CSHelp({
					contextID: contextID 
				});
	            csHelp.setButtonLabel(Constants.getMessage("labelManageCategoriesCSHelp"));
	            this._dialog.setSecondaryTitleNode(csHelp.domNode);

	            this.table.setAttribute("align", "center");
	            this._dialog.contentContainer.appendChild(this.domNode);                     
	            
	            // create a horizontal resize, the height of the table is fixed
	            var handle = new dojox.layout.ResizeHandle({
	                activeResize: true,
	                resizeAxis: "x",
	                targetId: this._dialog.domNode.id // resize function of this will be called
	            });
	            handle.placeAt(this._dialog.containerNode);
	            this.connect(this._dialog, "resize", dojo.hitch(this, function (sizeHint) {
	                // we don't want default resize handling which would set a height
	                // on the dialog which would prevent it from self adjusting when page size is change
	                var dlgWidth = sizeHint.w + "px";
	                dojo.style(this._dialog.domNode, "width", dlgWidth);
	            }));
	        }
	        this._dialog.open();
		},
		
		hide: function() {
			// fire onClose callback, if specified
			if (dojo.isFunction(this.onClose)) {
				this.onClose(this.categoryTypes);
			}
	        // we could be called from OK, which does not close the dialog
	        // while we save the content. Close the dialog if it's 
	        // still open.
	        if (this._dialog && this._dialog.opened) {
	            this._dialog.close();
	        }
		},
		
	    _doSave: function() {
	    	
	    	this.viewMode = this.treePanel.getViewMode();
	    	
			var dirtyCategoryTypes = this.model.getDirtyTypes();
			
			var categoryTypesLen = dirtyCategoryTypes.length;
			var context = new SaveContext();
			
			var toCreateSubTypeReference = [];
			var newCategoryTypeParticipants = [];
			var categoryValueSetParticipants = [];
			var newCategoryParticipants = [];
			
			for(var catTypeCont=0;catTypeCont<categoryTypesLen;catTypeCont++){
				//add save participant to context (if its dirty)
				var categoryType = dirtyCategoryTypes[catTypeCont];
				var typeParticipant = null;
				if (categoryType.isDirty) {
					typeParticipant = new com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategorySaveParticipant.CategoryTypeSaveParticipant({						                  
						categoryType: categoryType,
						artifactType: this.artifactType
					});
					context.addParticipant(typeParticipant);
					if (categoryType.dependsOnNewType) {
						toCreateSubTypeReference.push(typeParticipant);
					}
					if (categoryType.isNew) {
						newCategoryTypeParticipants.push(typeParticipant);
					}
				}
				
				var categories =  this.model.getAllCategoryValuesForType(categoryType.itemId || categoryType.name) ;
				var catLength = categories.length;
	            	          
				if (categoryType.hasDirtyCategories) {
	                
	                for (var j = 0; j < catLength; j++) {
	                    //add dirty save participants to context
	                    var category = categories[j];
	                
	                    var categorySaveParticipant = null;
	                    if (category.isDirty) {
	                        categorySaveParticipant = new com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategorySaveParticipant({
	                            category: category,
	                            parentType: categoryType
	                        });
	                        context.addParticipant(categorySaveParticipant);
	                        //create new reference if this is a new category item within a new category type item
	                        if (category.isNew && categoryType.isNew) {
	                            //add relationship to parent Category Type	
	                            context.createReference(categorySaveParticipant, "typeItemId", typeParticipant);
	                        }
	                        if (category.isNew) 
	                            newCategoryParticipants.push(categorySaveParticipant);
	                    }
	                    	                    
	                    if (category.subTypesModified) {
	                    	var subTypesModified  =  this.model.organizeSubCategoriesByType(category);
	                    	
	                    	var subTypesLen = subTypesModified?subTypesModified.length:0;
	                    	for(var subTypesCont=0;subTypesCont<subTypesLen; subTypesCont++){
	                    		var subType = subTypesModified[subTypesCont];
	                    		
	                    		var participant = new com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategorySaveParticipant.CategoryValueSetParticipant({
		                            category: category,
		                            subType: subType
		                        });
		                        context.addParticipant(participant);
		                        if (category.isNew) {
		                            context.createReference(participant, "key", categorySaveParticipant);
		                        }
		                        //need to create save participants for new values and added values both only if there are new values
		                        categoryValueSetParticipants.push(participant);
		                        var categoryValuesAdded = subType.addedValues || [];
		                        var subCatLength = categoryValuesAdded.length;
		                        	                        	                        
		                        for (var k = 0; k < subCatLength; k++) {
		                        	var subCategory = categoryValuesAdded[k];
		                        	var itemType = subCategory.categoryType;
		                        	var subType = this.model.getCategoryTypeFromBase(itemType.itemId || itemType.name);	                        		
		                            var itemId = subCategory.itemId;	                            
		                            
		                            var saveParticipant = context.findParticipant(itemId);
		                            if (saveParticipant == null) {
		                            	
		                                saveParticipant = new com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategorySaveParticipant({
		                                    category: subCategory,
		                                    parentType: subType
		                                });
		                            }
		                            context.createReference(participant, "valuesAdded", saveParticipant);		                            		                           
		                        }
		                        
		                        var removedChildrenItemIds = this._getValueItemIds(category.removedChildren);
		                        var removedCatLength =removedChildrenItemIds.length;		                        		                        
		                        for (var p = 0; p < removedCatLength; p++) {
		                            context.removeReference(participant, "valuesRemoved", removedChildrenItemIds[p]);
		                        }		                        		                        		                        
	                    		
	                    	}	                    		                        
	                    }
	                }	                
	            }
				
			}
		
			if(toCreateSubTypeReference.length >0){
				var len2 = toCreateSubTypeReference.length;
				for(var j=0;j<len2;j++){
					var dependsOn = toCreateSubTypeReference[j].categoryType.dependsOn;
					var parentParticipant = context.findParticipant(dependsOn);
					if(parentParticipant){
						context.createReference(toCreateSubTypeReference[j],"dependsOn",parentParticipant);
					}
				}
			}
			if(categoryValueSetParticipants.length > 0){
				var len3 = categoryValueSetParticipants.length;
				var len4 = newCategoryParticipants.length;
				
				
				for(var k=0;k<len3;k++){															
					var parentCategory = categoryValueSetParticipants[k].category;
					var subType =  categoryValueSetParticipants[k].subType;
					var len5 = subType.newValues?subType.newValues.length:0;					
					for(var l = 0;l<len5;l++){								
						var categoryAdded = subType.newValues[l];
						var itemIdOrName = categoryAdded.itemId || categoryAdded.name;						
						var subTypeBase = this.model.getCategoryTypeFromBase(subType.itemId || subType.name);
						if(subTypeBase.isNew){
							var subTypeParticipant = context.findParticipant(subTypeBase.itemId || subTypeBase.name);
							context.createReference(categoryValueSetParticipants[k],"categoryType",subTypeParticipant);
						} 
						
						for(var m=0;m<len4;m++){
							if(newCategoryParticipants[m].category.itemId == itemIdOrName || newCategoryParticipants[m].category.name == itemIdOrName){
								context.createReference(categoryValueSetParticipants[k],"valuesAdded",newCategoryParticipants[m]);
								break;
							}
						}
					}
					
				}
			}			if(newCategoryTypeParticipants.length > 0 ){
				var typesLength = newCategoryTypeParticipants.length;
				for(var p=0;p<typesLength;p++){
					newCategoryTypeParticipants[p].setItemId(null);
				}
			}		
			
			//commit the save context
			context.setOnSaveFinished(dojo.hitch(this, "_saveOrder"));
			var status = context.commit();
			
			//handle status code from save context
			if (status == Constants.STATUS_NOT_NECESSARY) {
				this._saveOrder({ok: true});
			} else if (status != Constants.STATUS_SAVING) {
				console.error("Bad Status from SaveContext.commit() ==> " + status);
			}
		},
		
		_getValueItemIds: function(values){
			var valuesItemIds =  [];
			var len = values? values.length:0;
			for(var i=0; i<len; i++){
				valuesItemIds.push(values[i].itemId);
			}
			return valuesItemIds;
		},
        _saveOrder: function(status){	    	
			var option = new com.ibm.asq.common.web.client.internal.option();
			//this.parent.addSaveParticipantNum();
			var self = this;
			var handler = {
				success: function(results){                    
                    self.orderPrefEntry = results && results.value ? dojo.fromJson(results.value) : {};
					self.orderPrefEntry.itemId = results.itemId;
					self._saveFinished(status); 					
				},
				error: function(e){
					console.error("Unable to save categories order", e.message);					
                    self.parentContainer.onSaveFinished(status);
				}
			};						
			var context = Utility.getCategoryOrderContext(this.artifactType);
			var responsehandler = new ServiceResponseHandler(handler, "success", "error");
			var orderPrefEntry = this.model.orderPrefEntry;
			var res = this.model.getRootCategoryTypesOrder();
			if(orderPrefEntry && orderPrefEntry.itemId){
				option.updateProjectPreferences(responsehandler, getCurrentProjectArea().itemId, orderPrefEntry.itemId, dojo.toJson(res));
			}else{
				option.createProjectPreferences(responsehandler, getCurrentProjectArea().itemId, context, dojo.toJson(res));
			}
	    },	    	 
		
		
		_saveFinished:function(status){
			//fire onSaved callback, if specified
			if (dojo.isFunction(this.onSaved)) {
				this.onSaved(status);
			}
			var ok = status.ok;
			if (ok != true) {
			    // display the error if we're the primary display.
			    if(this._dialog) {
			        this.showWarning(status.message, true);
			        this._dialog.getCancelButton().disabled = false;
			    }
				return;
			}
			this.doPublish = true;
			this.model = null;
			
			if (this._dialog) {
				this._updateCategories();
			} else {				
				this._createModel();
				this.treePanel.reloadTreeModel(this.model, /*notCreateTree */true);
				this.treePanel.setViewMode(this.viewMode);
			}
		},
		
		_updateCategories:function(){
			var handler = {
				success: dojo.hitch(this, function(categoryTypes) {
					this.categoryTypes = dojo.isArray(categoryTypes) ? categoryTypes : [];
					//publish event 
					this.doPublish = true;
					this._publishChanged();
					this.hide();
	                if (this.onOKCompleted) {
	                	this.onOKCompleted(this.categoryTypes);
	                }
				}),
				error: dojo.hitch(this, function(e) {
					this._error(e);
	                if (this.onOKCompleted) {
	                    this.onOKCompleted(this.categoryTypes);
	                }
					this.hide();
				})
			};
	        var responseHandler = new ServiceResponseHandler(handler, "success", "error");
	        CategoryClient.getCategoryTypes(responseHandler, this.artifactType, false);
		},
		
		_publishChanged: function() {
			if (this.doPublish) {
				var artifactType = this.artifactType;
				var type = null;
				if (artifactType == Constants.MODEL_TYPE_CASE) {
					type = Constants.TYPE_TC_CATEGORY;
				} else if (artifactType == Constants.MODEL_TYPE_PLAN) {
					type = Constants.TYPE_TP_CATEGORY;
				} else if (artifactType == Constants.MODEL_TYPE_SUITE) {
					type = Constants.TYPE_TES_CATEGORY;
	            } else if (artifactType == Constants.MODEL_TYPE_SCRIPT) {
	                type = Constants.TYPE_TS_CATEGORY;
	            } else if (artifactType == Constants.MODEL_TYPE_TESTCASE_EXECUTION_RECORD) {
	                type = Constants.TYPE_TER_CATEGORY;
				} else if (artifactType == Constants.MODEL_TYPE_SUITE_EXECUTION_RECORD) {
					type = Constants.TYPE_SER_CATEGORY;
				} else if (artifactType == Constants.MODEL_TYPE_RESULT) {
	                type = Constants.TYPE_ER_CATEGORY;
				} else if (artifactType == Constants.MODEL_TYPE_SUITE_LOG) {
	                type = Constants.TYPE_SL_CATEGORY;
				} else if(artifactType == Constants.MODEL_TYPE_DATAPOOL) {
					type = Constants.TYPE_DATAPOOL_CATEGORY;
				} else if(artifactType == Constants.MODEL_TYPE_KEYWORD) {
					type = Constants.TYPE_KEYWORD_CATEGORY;
				}
				if (type) {
					dojo.publish(GlobalEvents.ArtifactsUpdated, [type, this.categoryTypes]);
				}
			}
	    }
	    
	
	});
	
})(dojo);

