/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreePanel");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTree");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryConstants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryDialog");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryDndListener");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.Deferred");

(function(dojo) {
	
	var ImageCache        = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
	
	var CategoryTree        = com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTree;
	var CategoryDialog      = com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryDialog;
	var CategoryConstants   = com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryConstants;
	var CategoryDndListener = com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryDndListener;
	
	var getMessage = dojo.hitch(com.ibm.rqm.planning.web.client.internal.Constants, "getMessage");

    var CategoryTreePanelMessages = {};
	dojo.forEach([
		"labelCreateEllipsis",
		"labelCategories",
		"labelShowValues",
		"labelHideValues",
		"labelCollapseAll",
		"labelCategories_GettingStarted",
		"labelCreatingTemplate",
		"labelCategoryType",
		"labelCategoryTypes",
		"labelCategoryValueLowercase",
		"labelCategoryValues",
		"labelCreatedTemplate",
		"labelLoading"
	], function(key) {
		CategoryTreePanelMessages[key] = getMessage(key);
	});
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreePanel", [dijit._Widget, dijit._Templated], {
		
		templateString: ['<div class="com-ibm-rqm-planning-category-panel" tabindex="-1" dojoAttachPoint="domNode" role="presentation" style="position: relative; float: left; left: 0px; top: 0px; width: 99%">',  
		                 '  <div class="com-ibm-rqm-planning-category-panel-header" tabindex="-1" role="presentation">',
		                 '		<div class="border" role="presentation"></div>',
		                 '		<div class="corner" role="presentation"></div>',
		                 '		<span dojoAttachPoint="actions" class="com-ibm-rqm-planning-category-panel-header-actions" role="toolbar">',
		                 '			<a dojoAttachEvent="ondijitclick:_handleCollapseClicked" tabindex="0" dojoAttachPoint="" role="button"', 
		                 '                class="dijit dijitButtonNode com-ibm-rqm-planning-category-panel-action" aria-label="${labelCollapseAll}" ', 
		                 '                href="javascript:{}" id="" title="${labelCollapseAll}">',  
		                 '				<img dojoAttachPoint="iconLabelNode" class="com-ibm-rqm-planning-category-panel-icon collapsse-all-tree-node-icon-image" src="${imgCollapseAll}"', 
		                 '                    alt="${labelCollapseAll}" role="presentation">',
		                 '				<span class="com-ibm-rqm-planning-category-panel-action-high-contrast-label">',
		                 '					${labelCollapseAll}',
		                 '				</span>',
		                 '			</a>',
		                 '			<a dojoAttachPoint="createbutton" dojoAttachEvent="ondijitclick:_handleCreateClicked" tabindex="0" dojoAttachPoint="" role="button"', 
		                 '                class="com-ibm-rqm-planning-category-panel-action"',
		                 '                aria-label="${labelCreateEllipsis}" href="javascript:{}" id="" title="${labelCreateEllipsis}">',  
		                 '				<img dojoAttachPoint="addIcon" class="com-ibm-rqm-planning-category-panel-icon add-icon-image com-ibm-rqm-planning-category-panel-action-glow-icon" src="${imgBlank}"', 
		                 '                    alt="${labelCreateEllipsis}" role="presentation">',
		                 '				<span class="com-ibm-rqm-planning-category-panel-action-high-contrast-label">',
		                 '					${labelCreateEllipsis}',
		                 '				</span>',
		                 '			</a>',
		                 '		</span>  ',
		                 '		<img style="display:none" dojoAttachPoint="iconLabelNode" class="com-ibm-rqm-planning-category-panel-icon" src="${imgAdd}" role="presentation">',
		                 '		<div dojoAttachPoint="titleNode" class="com-ibm-rqm-planning-category-panel-header-label category-type-label" ', 
		                 '            role="presentation" title="${labelCategories}">${labelCategories}',
		                 '		</div>',
		                 '		<div class="corner" role="presentation"></div>',
		                 '		<div class="border" role="presentation"></div>',
		                 '      <div dojoAttachPoint="container" class="panel-messages"><span class="message" dojoAttachPoint="messageNode" role="status"></span></div>',
		                 '      <div dojoAttachPoint="container" class="panel-text-actions">',
		                 '          <a class="text-action" dojoAttachPoint="textActionNode" dojoAttachEvent="ondijitclick:_handleTextActionClicked" href="javascript:{}">',
		                 '              ${labelHideValues}',
		                 '          </a>',
		                 '      </div>',
		                 '      <div dojoAttachPoint="container" class="panel-container" role="presentation" style="width: 99%;"></div>',
		                 '  </div>',
		                 '</div>'].join(''),
		                 
		imgBlank: ImageCache.BLANK,
		
		imgAdd: ImageCache.ADD_ICON_ENABLED,
		
		imgCollapseAll: ImageCache.COLLAPSE_ALL_ICON,
		
		// isTreeLoaded: Boolean
		//    Whether or not the complete categories tree has been loaded
		//    i.e. all node expansions have been restored from its persisted
		//    state.
		//
		isTreeLoaded: false,
		
		// deferredSetValuesItem: Model object item
		//    When the tree is loading, every node loaded sends an onClick
		//    event, which triggers the setValues method. This is costly,
		//    so, this variable will hold a deferred item to be processed 
		//    only after the complete tree has been loaded (see isTreeLoaded)
		//
		deferredSetValuesItem: undefined,
		
		// currentViewMode: String
		//    Will specify in which mode we are, either outline or complete 
		//    (show all) mode.
		//
		currentViewMode: CategoryConstants.VIEW_MODE_SHOW_ALL,
		
		// valuesPanel: CategoryTermPanel
		//    The panel that will display the values to be available for 
		//    category value creation.
		//
		valuesPanel: null,
		
		postMixInProperties: function() {
			// We need to have the labels mixed in before buildRendering
			dojo.mixin(this, CategoryTreePanelMessages);
		},
		
	    postCreate: function() {	    	
	    	var hitch_this = dojo.hitch(dojo, "hitch", this);	    	
	    	this.tree= new CategoryTree({
				artifactType: this.artifactType,
                typeName: this.typeName,
				treePanel: this,
				//nonDialogWidth: "95%",				
				showWarning: hitch_this("setMessage"),
				canShowHidden: true,
				onClick: hitch_this("_handleTreeNodeClicked"),
				model: this.model,
				isDialog: this.isDialog
			});
	    	var tree = this.tree;
			this.connect(tree, "setDirty", "setDirty");
			this.connect(tree, "onLoad", hitch_this("updateGettingStartedState"));
			// onUserSelect... actually used for 'onClick' purposes
			this.connect(tree, "onUserSelect", hitch_this("onUserSelect"));
			this.connect(this.valuesPanel, "onViewModeSwitched", hitch_this("syncSelectionWithValuesPanel"));
			
            this.container.appendChild(tree.domNode);
            this.tree = tree;
            
            // Plug a listener to update the notification area
            this.dndListener = new CategoryDndListener({
            	treePanel: this
            });
            // Initially, we'll display a loading animation. We'll also plug a handler
            // to remove the animation when the tree gets loaded.
            this.showLoadingMessage();
            var handler = this.connect(tree, "onTreeLoaded", hitch_this(function() {
            	this.disconnect(handler);

        		this.isTreeLoaded = true;
            	// Initial message
                this.setMessage(this.labelCategories_GettingStarted);
                // If there's any selection which values need to be send to the
                // values panel, proceed
                if (this.deferredSetValuesItem) {
                	this.setValues(this.deferredSetValuesItem);
                	this.deferredSetValuesItem = undefined;
                }
            }));                      
	    },
	    
	    reloadTreeModel: function(model,notCreateTree/*Optional if true, we will not create tree*/){ 
	    	this.tree.reloadTreeModel(model,notCreateTree);
	    },
	    
	    // Summary:
	    //    When there's no category types in the model, we display a message to tell
	    //    the user to "start creating some types" and we make the "add..." icon glow
	    //    this method is in charge of setting that up.
	    //
	    updateGettingStartedState: function() {
	    	var model = this.tree.model;
            if (model.isEmpty()) {
	            this.glowingTimer = setInterval(dojo.hitch(this, function() {
	            	if (this.addIcon) {
		            	dojo.toggleClass(this.addIcon, "active");
		            	if (!model.isEmpty()) {
		            		dojo.removeClass(this.addIcon, "active");
		            		clearTimeout(this.glowingTimer);
		            	}
	            	}
	            }), 1000);
            }
	    },
	    
	    setDirty: function(dirty) {
	    	// No-Op, interested parties must use connect.
	    },
	    
	    
	    // Summary:
	    //    Action handler for the collapse all button...
	    //
	    _handleCollapseClicked: function(event) {
	    	this.stopEvent(event);
	    	this.tree.collapseAll();
	    },
	    
	    // Summary:
		//    Will handle the event of the + button clicked... invoke the creation dialog.
		//
		_handleCreateClicked: function(event) {
			this.stopEvent(event);
			if(this.currentViewMode == CategoryConstants.VIEW_MODE_OUTLINE){
				return;
			}
			var item = this.getSelectedItem();
			var model = this.tree.model;
			var categoryDialog = new CategoryDialog({
				onOk: dojo.hitch(this, "_handleCreateItems"),
				parent: item,
				model: model,
				onlyAllowRoot: true
			});
			categoryDialog.show();
		},
		
		// Summary:
		//    Category creation callback for the creation dialog.
		//
		_handleCreateItems: function(values, isCategoryType, createAsRoot, selectedItem) {
			var model = this.tree.model;
			// Selected item can be provided by the calling context
			selectedItem = selectedItem || this.getSelectedItem();
			var parentItem = createAsRoot ? null : selectedItem;
			var isSubCategoryType = isCategoryType && parentItem && model.isCategoryType(parentItem);
			// yield to allow UI to reflow, dialog be dismissed, loading message show up, etc...
			setTimeout(dojo.hitch(this, "_createItemsContinuation", 1, values.length, parentItem, values, model, isCategoryType, isSubCategoryType));
		},
		
		// Summary:
		//    Will continue the creation process as left by _handleCreateItems...
		//    --- Do not call this method. ---
		//
		_createItemsContinuation: function(n, total, parentItem, values, model, isCategoryType, isSubCategoryType) {
			if (values.length === 0) {
				var typesName = null;
				if (total === 1) {
					// singular
					typesName = isCategoryType ? this.labelCategoryType : this.labelCategoryValueLowercase;
				} else {
					// plural
					typesName = isCategoryType ? this.labelCategoryTypes : this.labelCategoryValues;
				}
				this.setMessage(dojo.string.substitute(this.labelCreatedTemplate, [total, typesName]));
				this.setDirty(true);
				// Raise the node clicked event (not really a node and not really an event :P)
				// to trigger the setValues update of the Values Panel.
				setTimeout(dojo.hitch(this, "_handleTreeNodeClicked", parentItem));
				return;
			}
			this.setCreatingMessage(n, total, isCategoryType);
			var value = values.shift();
			var changedItems = model.newItem(value, parentItem, isSubCategoryType);
			this.tree.refreshItemsChildren(changedItems);
			
			// yield to allow UI to reflow, dialog be dismissed, loading message show up, etc...
			setTimeout(dojo.hitch(this, "_createItemsContinuation", n + 1, total, parentItem, values, model, isCategoryType, isSubCategoryType));
		},
		
		
		// Summary:
		//    Will continue the creation process as left by _handleCreateItems...
		//
		//        --- Do not call this method. ---
		//
		//    Initial implementation with reflow issues, leaving here for reference
		//
		//_createItemsContinuation: function(parentItem, values, model, isCategoryType) {
		//    dojo.forEach(values, function(value) {
		//        model.newItem(value, parentItem, false);
		//    }, this);
		//    this.tree.tree.onItemChildrenChange(parentItem, parentItem.children);
		//    var typesName = isCategoryType ? this.labelCategoryTypes : this.labelCategoryValues;
		//    this.setMessage(dojo.string.substitute(this.labelCreatedTemplate, [values.length, typesName]));
		//},
		
		// Summary:
		//    will handle the click events on the "text action" link inside the
		//    messages area (basically, for show all/see outline.)
		//
		_handleTextActionClicked: function(event) {
			this.stopEvent(event);
			var newViewMode =   null;
			if(this.currentViewMode == CategoryConstants.VIEW_MODE_SHOW_ALL){
				newViewMode = CategoryConstants.VIEW_MODE_OUTLINE;
            	dojo.style(this.createbutton, {
            		opacity: 0.4,
            		filter: "alpha(opacity=40)"
            	});
            } else {
            	newViewMode = CategoryConstants.VIEW_MODE_SHOW_ALL;
            	dojo.style(this.createbutton, {
            		opacity: "",
            		filter: ""
            	});
            }			
			this.setViewMode(newViewMode);
			
			  
		},
		
		getViewMode: function() {
			return this.currentViewMode;
		},
		
		setViewMode: function(viewMode) {
			this.currentViewMode = viewMode;
			// Update state and text action label
			if (this.currentViewMode === CategoryConstants.VIEW_MODE_OUTLINE) {
				this.textActionNode.innerHTML = this.labelShowValues;
			} else {
				this.textActionNode.innerHTML = this.labelHideValues;
			}
			// Do the actual tree change...
			this.switchTree(viewMode);
		},
		
		// Summary:
		//    Will handle the node clicked and expanded events... the tree
		//    triggers different events for clicking a leaf node and expanding
		//    or contracting a parent node, so, we hook up the same event listener
		//    for both.
		//
		_handleTreeNodeClicked: function(item, node, event) {
			this._expandNodeIfNeeded(node, event);
			
			this.selectedItem = item;
			var model = this.tree.model;
			var isCategoryType = model.isCategoryType(item);
			this.valuesPanel.setAssociationsMode(isCategoryType && !item.isRoot);
			
	        if (isCategoryType) {
	        	if (!this.isTreeLoaded) {
					this.deferredSetValuesItem = item;
				} else {
					this.setValues(item);
				}
	        } else if (model.isCategoryValue(item)) {
	        	
	        }
		},
		
		expandNode: function(node) {
			if (!node) {
				// Returns a promise ready for managing any callback
				return new dojo.Deferred().resolve();
			}
			return this.tree.tree.tree._expandNode(node);
		},
		
		_expandNodeIfNeeded: function(node, event) {
			if (!event) {
				return;
			}
			var domElement = event.target;
			// tree: CategoryTree > TreeView > ActionableTree
			var tree = this.tree.tree.tree;
			if (!tree) {
				return;
			}
			var isExpandoClick = tree.isExpandoNode(domElement, node);
			// Check if the click wasn't made on a expansion node (the one with a '+'
			// or '-' otherwise, honor the default behavior.
			if (!isExpandoClick) {
				this.expandNode(node);
			}
		},
		
		getSelectedItem: function() {
			return this.tree.getSelectedItem();
		},
		
		// Summary:
		//    Handles the user selection event. This handler is plugged into
		//    the dnd select source of the tree and was necessary to workaround
		//    the fact that the onClick wasn't beeing triggered by clicking on
		//    the nodes, just clicking on the expansion nodes (the little squares
		//    with a '+' inside them.
		//
		onUserSelect: function() {
			var item = this.getSelectedItem();
			// Avoid processing repeatedly the same item.
			if (item !== this.lastSelectedItem) {
				// Delegate to the TreeNode click handler...
				// It's assumed the method doesn't care about the
				// actual node.
				this._handleTreeNodeClicked(item, null);
				this.lastSelectedItem = item;
			}
		},
		
		// Summary:
		//    Sets the contextual values in the values panel.
		//    This method assumes the item is a category type.
		//
		setValues: function(item) {
        	// item.categories have all values associated
        	var associatedValues = dojo.map(item.categories || [], function(value) {
        		return {
        			value: value.name,
        			name: value.name,
        			itemId: value.itemId,
        			categoryType: item,
        			isAssociated: true
        		};
        	}, this);
        	// Remove archived ones...
        	var allValues = dojo.filter(this.tree.model.getAllCategoryValuesForType(item.itemId || item.name), function(value) {
        		return !value.archived;
        	});
        	// We also need to include all other values, just not flagged as associated
        	allValues = dojo.map(allValues, function(value) {
        		return {
        			value: value.name,
        			name: value.name,
        			itemId: value.itemId,
        			categoryType: item
        		};
        	});
        	
        	// We might have duplicates... just keep the associated ones.
        	var values = this.removeDuplicates(item.isRoot ? allValues : associatedValues.concat(allValues));
        	this.valuesPanel.setValues(values);
		},
		
		syncSelectionWithValuesPanel: function() {
			var item = this.getSelectedItem();
			// Delegate to the TreeNode click handler...
			// It's assumed the method doesn't care about the
			// actual node.
			this._handleTreeNodeClicked(item, null);
		},
		
		// Summary:
		//    Removes the duplicates from the data array. If duplicates are
		//    found, we keep the one with a associated status of true.
		//
		removeDuplicates: function(data) {
			// Use an object key set to cache uniques
			var map = {};
			dojo.forEach(data, function(item) {
				if (item.value in map) {
					var cachedItem = map[item.value];
					if (!cachedItem.isAssociated && item.isAssociated) {
						// Replace with the one associated
						map[item.value] = item;
					}
				} else {
					map[item.value] = item;
				}
			});
			var results = [];
			for (var key in map) {
				if (map.hasOwnProperty(key)) {
					results.push(map[key]);
				}
			}
			return results;
		},
		
		// Summary:
		//    Will switch the tree from outline mode to show all mode and the
		//    other way around.
		//
		switchTree: function(viewMode) {
			this.tree.switchTreeMode(viewMode);
		},
		
		// Summary:
		//    Will display the animated "Loading..." message.
		//
		showLoadingMessage: function() {
			var message = ['<span class="loading-animation">',
			               '    <span style="padding-left: 18px;">', this.labelLoading, '</span>',
			               '</span>'].join('');
			this.setMessage(message);
		},
		
		// Summary:
		//    Will change the message displayed in the messages area at the 
		//    top of the tree view.
		//
		setMessage: function(message) {
			if (this.messageNode) {
				this.messageNode.innerHTML = message;
			}
		},
		
		clearMessage: function() {
			this.setMessage('');
		},
		
		// Summary:
		//    Will display a message and a progress icon to notify the user the 
		//    creation process is taking place.
		//
		//   TODO: Style this.
		//
		setWarningMessage: function(message) {
			var text = ['<span class="tbd">',
			            '    <span style="tbd">', message, '</span>',
			            '</span>'].join('');
			this.setMessage(text);
		},
		
		// Summary:
		//    Will display a message and a progress icon to notify the user the 
		//    creation process is taking place.
		//
		setCreatingMessage: function(num, total, isCategoryType) {
			var itemTypeLabel = isCategoryType ? this.labelCategoryType : this.labelCategoryValueLowercase;
			// We're leveraging the innerHTML property, which allows for easy DOM creation
			// this way, we can easily include the progress indicator next to the message.
			var message = ['<span class="loading-animation">',
			               '    <span style="padding-left: 18px;">', dojo.string.substitute(this.labelCreatingTemplate, [num, total, itemTypeLabel]), '</span>',
			               '</span>'].join('');
			this.setMessage(message);
		},
		
		destroy: function() {
			if (this.glowingTimer) {
				clearTimeout(this.glowingTimer);
			}
		},
		
		// Summary:
	    //    Convenience method to turn the calls to dojo.stopEvent 
	    //    when the event argument is set, into one-liners.
	    //
		stopEvent: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
		}
		
	});
	
})(dojo);

